@@ -71,9 +71,8 @@ wxString ExpandTextVars( const wxString& aSource, const PROJECT* aProject, int a
 wxString ExpandTextVars( const wxString& aSource,
                          const std::function<bool( wxString* )>* aResolver, int aFlags )
 {
-    static wxRegEx userDefinedWarningError( wxS( "^(ERC|DRC)_(WARNING|ERROR).*$" ) );
-    wxString       newbuf;
-    size_t         sourceLen = aSource.length();
+    wxString newbuf;
+    size_t   sourceLen = aSource.length();
 
     newbuf.Alloc( sourceLen );  // best guess (improves performance)
 
@@ -94,7 +93,10 @@ wxString ExpandTextVars( const wxString& aSource,
             if( token.IsEmpty() )
                 continue;
 
-            if( ( aFlags & FOR_ERC_DRC ) == 0 && userDefinedWarningError.Matches( token ) )
+            if( ( aFlags & FOR_ERC_DRC ) == 0 && (   token.StartsWith( wxS( "ERC_WARNING" ) )
+                                                  || token.StartsWith( wxS( "ERC_ERROR" ) )
+                                                  || token.StartsWith( wxS( "DRC_WARNING" ) )
+                                                  || token.StartsWith( wxS( "DRC_ERROR" ) ) ) )
             {
                 // Only show user-defined warnings/errors during ERC/DRC
             }
@@ -118,11 +120,12 @@ wxString ExpandTextVars( const wxString& aSource,
 }
 
 
-wxString GetTextVars( const wxString& aSource )
+wxString GetGeneratedFieldDisplayName( const wxString& aSource )
 {
     std::function<bool( wxString* )> tokenExtractor =
             [&]( wxString* token ) -> bool
             {
+                *token = *token;    // token value is the token name
                 return true;
             };
 
@@ -130,9 +133,10 @@ wxString GetTextVars( const wxString& aSource )
 }
 
 
-bool IsTextVar( const wxString& aSource )
+bool IsGeneratedField( const wxString& aSource )
 {
-    return aSource.StartsWith( wxS( "${" ) );
+    static wxRegEx expr( wxS( "^\\$\\{\\w*\\}$" ) );
+    return expr.Matches( aSource );
 }
 
 
@@ -431,7 +435,7 @@ wxString EnsureFileExtension( const wxString& aFilename, const wxString& aExtens
     // extension, such as "Schematic_1.1".
     if( newFilename.Lower().AfterLast( '.' ) != aExtension )
     {
-        if( newFilename.Last() != '.' )
+        if( !newFilename.EndsWith( '.' ) )
             newFilename.Append( '.' );
 
         newFilename.Append( aExtension );
