@@ -2,7 +2,8 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
- * Copyright (C) 2012-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2023 CERN
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +24,7 @@
  */
 
 #include <dialog_shim.h>
-#include <ignore.h>
+#include <core/ignore.h>
 #include <kiway_player.h>
 #include <kiway.h>
 #include <pgm_base.h>
@@ -41,31 +42,6 @@
 
 #include <algorithm>
 
-/// Toggle a window's "enable" status to disabled, then enabled on destruction.
-class WDO_ENABLE_DISABLE
-{
-    wxWindow* m_win;
-
-public:
-
-    WDO_ENABLE_DISABLE( wxWindow* aWindow ) :
-        m_win( aWindow )
-    {
-        if( m_win )
-            m_win->Disable();
-    }
-
-    ~WDO_ENABLE_DISABLE()
-    {
-        if( m_win )
-        {
-            m_win->Enable();
-            m_win->Raise(); // let's focus back on the parent window
-        }
-    }
-};
-
-
 BEGIN_EVENT_TABLE( DIALOG_SHIM, wxDialog )
     EVT_CHAR_HOOK( DIALOG_SHIM::OnCharHook )
 END_EVENT_TABLE()
@@ -74,16 +50,19 @@ END_EVENT_TABLE()
 DIALOG_SHIM::DIALOG_SHIM( wxWindow* aParent, wxWindowID id, const wxString& title,
                           const wxPoint& pos, const wxSize& size, long style,
                           const wxString& name ) :
-            wxDialog( aParent, id, title, pos, size, style, name ),
-            KIWAY_HOLDER( nullptr, KIWAY_HOLDER::DIALOG ),
-            m_units( EDA_UNITS::MILLIMETRES ),
-            m_useCalculatedSize( false ),
-            m_firstPaintEvent( true ),
-            m_initialFocusTarget( nullptr ),
-            m_qmodal_loop( nullptr ),
-            m_qmodal_showing( false ),
-            m_qmodal_parent_disabler( nullptr ),
-            m_parentFrame( nullptr )
+        wxDialog( aParent, id, title, pos, size, style, name ),
+        KIWAY_HOLDER( nullptr, KIWAY_HOLDER::DIALOG ),
+        m_units( EDA_UNITS::MM ),
+        m_useCalculatedSize( false ),
+        m_firstPaintEvent( true ),
+        m_initialFocusTarget( nullptr ),
+        m_isClosing( false ),
+        m_qmodal_loop( nullptr ),
+        m_qmodal_showing( false ),
+        m_qmodal_parent_disabler( nullptr ),
+        m_parentFrame( nullptr ),
+        m_userPositioned( false ),
+        m_userResized( false )
 {
     KIWAY_HOLDER* kiwayHolder = nullptr;
     m_initialSize = size;
@@ -124,6 +103,8 @@ DIALOG_SHIM::DIALOG_SHIM( wxWindow* aParent, wxWindowID id, const wxString& titl
 
     Bind( wxEVT_CLOSE_WINDOW, &DIALOG_SHIM::OnCloseWindow, this );
     Bind( wxEVT_BUTTON, &DIALOG_SHIM::OnButton, this );
+    Bind( wxEVT_SIZE, &DIALOG_SHIM::OnSize, this );
+    Bind( wxEVT_MOVE, &DIALOG_SHIM::OnMove, this );
 
 #ifdef __WINDOWS__
     // On Windows, the app top windows can be brought to the foreground (at least temporarily)
@@ -139,32 +120,37 @@ DIALOG_SHIM::DIALOG_SHIM( wxWindow* aParent, wxWindowID id, const wxString& titl
 
 DIALOG_SHIM::~DIALOG_SHIM()
 {
+    m_isClosing = true;
+
     Unbind( wxEVT_CLOSE_WINDOW, &DIALOG_SHIM::OnCloseWindow, this );
     Unbind( wxEVT_BUTTON, &DIALOG_SHIM::OnButton, this );
     Unbind( wxEVT_PAINT, &DIALOG_SHIM::OnPaint, this );
+    Unbind( wxEVT_SIZE, &DIALOG_SHIM::OnSize, this );
+    Unbind( wxEVT_MOVE, &DIALOG_SHIM::OnMove, this );
 
-    std::function<void( wxWindowList& )> disconnectFocusHandlers = [&]( wxWindowList& children )
-    {
-        for( wxWindow* child : children )
-        {
-            if( wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( child ) )
+    std::function<void( wxWindowList& )> disconnectFocusHandlers =
+            [&]( wxWindowList& children )
             {
-                textCtrl->Disconnect( wxEVT_SET_FOCUS,
-                                      wxFocusEventHandler( DIALOG_SHIM::onChildSetFocus ), nullptr,
-                                      this );
-            }
-            else if( wxStyledTextCtrl* scintilla = dynamic_cast<wxStyledTextCtrl*>( child ) )
-            {
-                scintilla->Disconnect( wxEVT_SET_FOCUS,
-                                       wxFocusEventHandler( DIALOG_SHIM::onChildSetFocus ), nullptr,
-                                       this );
-            }
-            else
-            {
-                disconnectFocusHandlers( child->GetChildren() );
-            }
-        }
-    };
+                for( wxWindow* child : children )
+                {
+                    if( wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( child ) )
+                    {
+                        textCtrl->Disconnect( wxEVT_SET_FOCUS,
+                                              wxFocusEventHandler( DIALOG_SHIM::onChildSetFocus ),
+                                              nullptr, this );
+                    }
+                    else if( wxStyledTextCtrl* scintilla = dynamic_cast<wxStyledTextCtrl*>( child ) )
+                    {
+                        scintilla->Disconnect( wxEVT_SET_FOCUS,
+                                               wxFocusEventHandler( DIALOG_SHIM::onChildSetFocus ),
+                                               nullptr, this );
+                    }
+                    else
+                    {
+                        disconnectFocusHandlers( child->GetChildren() );
+                    }
+                }
+            };
 
     disconnectFocusHandlers( GetChildren() );
 
@@ -288,20 +274,44 @@ bool DIALOG_SHIM::Show( bool show )
                          std::max( wxDialog::GetSize().y, savedDialogRect.GetSize().y ),
                          0 );
             }
+#ifdef __WXMAC__
+            if( m_parent != nullptr )
+            {
+                if( wxDisplay::GetFromPoint( m_parent->GetPosition() )
+                    != wxDisplay::GetFromPoint( savedDialogRect.GetPosition() ) )
+                    Centre();
+            }
+#endif
         }
         else if( m_initialSize != wxDefaultSize )
+        {
             SetSize( m_initialSize );
+            Centre();
+        }
 
         // Be sure that the dialog appears in a visible area
         // (the dialog position might have been stored at the time when it was
         // shown on another display)
         if( wxDisplay::GetFromWindow( this ) == wxNOT_FOUND )
             Centre();
+
+        m_userPositioned = false;
+        m_userResized = false;
     }
     else
     {
-        // Save the dialog's position & size before hiding, using classname as key
-        class_map[ hash_key ] = wxRect( wxDialog::GetPosition(), wxDialog::GetSize() );
+        // Save the dialog's position & size before hiding, using classname as key.
+        // Be careful of rounding errors: only re-save if the user modified the value or
+        // it has not yet been saved.
+        wxRect rect = class_map[ hash_key ];
+
+        if( m_userPositioned || rect.GetPosition() == wxPoint() )
+            rect.SetPosition( wxDialog::GetPosition() );
+
+        if( m_userResized || rect.GetSize() == wxSize() )
+            rect.SetSize( wxDialog::GetSize() );
+
+        class_map[ hash_key ] = rect;
 
 #ifdef __WXMAC__
         if ( m_eventLoop )
@@ -309,10 +319,10 @@ bool DIALOG_SHIM::Show( bool show )
 #endif
 
         ret = wxDialog::Show( show );
-    }
 
-    if( m_parent )
-        m_parent->SetFocus();
+        if( m_parent )
+            m_parent->SetFocus();
+    }
 
     return ret;
 }
@@ -343,6 +353,20 @@ void DIALOG_SHIM::resetSize()
 }
 
 
+void DIALOG_SHIM::OnSize( wxSizeEvent& aEvent )
+{
+    m_userResized = true;
+    aEvent.Skip();
+}
+
+
+void DIALOG_SHIM::OnMove( wxMoveEvent& aEvent )
+{
+    m_userPositioned = true;
+    aEvent.Skip();
+}
+
+
 bool DIALOG_SHIM::Enable( bool enable )
 {
     // so we can do logging of this state change:
@@ -353,7 +377,7 @@ bool DIALOG_SHIM::Enable( bool enable )
 // Recursive descent doing a SelectAll() in wxTextCtrls.
 // MacOS User Interface Guidelines state that when tabbing to a text control all its
 // text should be selected.  Since wxWidgets fails to implement this, we do it here.
-void DIALOG_SHIM::selectAllInTextCtrls( wxWindowList& children )
+void DIALOG_SHIM::SelectAllInTextCtrls( wxWindowList& children )
 {
     for( wxWindow* child : children )
     {
@@ -382,14 +406,19 @@ void DIALOG_SHIM::selectAllInTextCtrls( wxWindowList& children )
         else if( wxStyledTextCtrl* scintilla = dynamic_cast<wxStyledTextCtrl*>( child ) )
         {
             m_beforeEditValues[ scintilla ] = scintilla->GetText();
-            scintilla->Connect( wxEVT_SET_FOCUS, wxFocusEventHandler( DIALOG_SHIM::onChildSetFocus ),
+            scintilla->Connect( wxEVT_SET_FOCUS,
+                                wxFocusEventHandler( DIALOG_SHIM::onChildSetFocus ),
                                 nullptr, this );
 
             if( !scintilla->GetSelectedText().IsEmpty() )
             {
                 // Respect an existing selection
             }
-            else if( !scintilla->GetText().Contains( wxT( "\n") ) )
+            else if( scintilla->GetMarginWidth( 0 ) > 0 )
+            {
+                // Don't select-all in Custom Rules, etc.
+            }
+            else if( scintilla->IsEditable() )
             {
                 scintilla->SelectAll();
             }
@@ -413,7 +442,7 @@ void DIALOG_SHIM::selectAllInTextCtrls( wxWindowList& children )
 #endif
         else
         {
-            selectAllInTextCtrls( child->GetChildren() );
+            SelectAllInTextCtrls( child->GetChildren() );
         }
     }
 }
@@ -425,7 +454,7 @@ void DIALOG_SHIM::OnPaint( wxPaintEvent &event )
     {
         KIPLATFORM::UI::FixupCancelButtonCmdKeyCollision( this );
 
-        selectAllInTextCtrls( GetChildren() );
+        SelectAllInTextCtrls( GetChildren() );
 
         if( m_initialFocusTarget )
             KIPLATFORM::UI::ForceFocus( m_initialFocusTarget );
@@ -446,6 +475,24 @@ void DIALOG_SHIM::OnModify()
 }
 
 
+void DIALOG_SHIM::ClearModify()
+{
+    if( GetTitle().StartsWith( wxS( "*" ) ) )
+        SetTitle( GetTitle().AfterFirst( '*' ) );
+}
+
+int DIALOG_SHIM::ShowModal()
+{
+    // Apple in its infinite wisdom will raise a disabled window before even passing
+    // us the event, so we have no way to stop it.  Instead, we must set an order on
+    // the windows so that the modal will be pushed in front of the disabled
+    // window when it is raised.
+    KIPLATFORM::UI::ReparentModal( this );
+
+    // Call the base class ShowModal() method
+    return wxDialog::ShowModal();
+}
+
 /*
     Quasi-Modal Mode Explained:
 
@@ -471,15 +518,7 @@ void DIALOG_SHIM::OnModify()
 
 int DIALOG_SHIM::ShowQuasiModal()
 {
-    // This is an exception safe way to zero a pointer before returning.
-    // Yes, even though DismissModal() clears this first normally, this is
-    // here in case there's an exception before the dialog is dismissed.
-    struct NULLER
-    {
-        void*&  m_what;
-        NULLER( void*& aPtr ) : m_what( aPtr ) {}
-        ~NULLER() { m_what = nullptr; }   // indeed, set it to NULL on destruction
-    } clear_this( (void*&) m_qmodal_loop );
+    NULLER raii_nuller( (void*&) m_qmodal_loop );
 
     // release the mouse if it's currently captured as the window having it
     // will be disabled when this dialog is shown -- but will still keep the
@@ -495,19 +534,19 @@ int DIALOG_SHIM::ShowQuasiModal()
                                                   "window?" ) );
 
     // quasi-modal: disable only my "optimal" parent
-    m_qmodal_parent_disabler = new WDO_ENABLE_DISABLE( parent );
+    m_qmodal_parent_disabler = new WINDOW_DISABLER( parent );
 
     // Apple in its infinite wisdom will raise a disabled window before even passing
     // us the event, so we have no way to stop it.  Instead, we must set an order on
     // the windows so that the quasi-modal will be pushed in front of the disabled
     // window when it is raised.
-    KIPLATFORM::UI::ReparentQuasiModal( this );
+    KIPLATFORM::UI::ReparentModal( this );
 
     Show( true );
 
     m_qmodal_showing = true;
 
-    WX_EVENT_LOOP event_loop;
+    wxGUIEventLoop event_loop;
 
     m_qmodal_loop = &event_loop;
 
@@ -522,6 +561,20 @@ int DIALOG_SHIM::ShowQuasiModal()
 }
 
 
+void DIALOG_SHIM::PrepareForModalSubDialog()
+{
+    if( m_qmodal_parent_disabler )
+        m_qmodal_parent_disabler->SuspendForTrueModal();
+}
+
+
+void DIALOG_SHIM::CleanupAfterModalSubDialog()
+{
+    if( m_qmodal_parent_disabler )
+        m_qmodal_parent_disabler->ResumeAfterTrueModal();
+}
+
+
 void DIALOG_SHIM::EndQuasiModal( int retCode )
 {
     // Hook up validator and transfer data from controls handling so quasi-modal dialogs
@@ -538,14 +591,14 @@ void DIALOG_SHIM::EndQuasiModal( int retCode )
         return;
     }
 
+    TearDownQuasiModal();
+
     if( m_qmodal_loop )
     {
         if( m_qmodal_loop->IsRunning() )
             m_qmodal_loop->Exit( 0 );
         else
             m_qmodal_loop->ScheduleExit( 0 );
-
-        m_qmodal_loop = nullptr;
     }
 
     delete m_qmodal_parent_disabler;
@@ -610,10 +663,13 @@ void DIALOG_SHIM::onChildSetFocus( wxFocusEvent& aEvent )
 {
     // When setting focus to a text control reset the before-edit value.
 
-    if( wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( aEvent.GetEventObject() ) )
-        m_beforeEditValues[ textCtrl ] = textCtrl->GetValue();
-    else if( wxStyledTextCtrl* scintilla = dynamic_cast<wxStyledTextCtrl*>( aEvent.GetEventObject() ) )
-        m_beforeEditValues[ scintilla ] = scintilla->GetText();
+    if( !m_isClosing )
+    {
+        if( wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( aEvent.GetEventObject() ) )
+            m_beforeEditValues[ textCtrl ] = textCtrl->GetValue();
+        else if( wxStyledTextCtrl* scintilla = dynamic_cast<wxStyledTextCtrl*>( aEvent.GetEventObject() ) )
+            m_beforeEditValues[ scintilla ] = scintilla->GetText();
+    }
 
     aEvent.Skip();
 }
@@ -629,9 +685,50 @@ void DIALOG_SHIM::OnCharHook( wxKeyEvent& aEvt )
             return;
         }
     }
-    // shift-return (Mac default) or Ctrl-Return (GTK) for OK
-    else if( ( aEvt.GetKeyCode() == WXK_RETURN || aEvt.GetKeyCode() == WXK_NUMPAD_ENTER )
-             && ( aEvt.ShiftDown() || aEvt.ControlDown() ) )
+    // shift-return (Mac default) or Ctrl-Return (GTK) for new line input
+    else if( ( aEvt.GetKeyCode() == WXK_RETURN || aEvt.GetKeyCode() == WXK_NUMPAD_ENTER ) && aEvt.ShiftDown() )
+    {
+        wxObject* eventSource = aEvt.GetEventObject();
+
+        if( wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( eventSource ) )
+        {
+            // If the text control is not multi-line, we want to close the dialog
+            if( !textCtrl->IsMultiLine() )
+            {
+                wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
+                return;
+            }
+
+#if defined( __WXMAC__ ) || defined( __WXMSW__ )
+            wxString eol = "\r\n";
+#else
+            wxString eol = "\n";
+#endif
+
+            long pos = textCtrl->GetInsertionPoint();
+            textCtrl->WriteText( eol );
+            textCtrl->SetInsertionPoint( pos + eol.length() );
+            return;
+        }
+        else if( wxStyledTextCtrl* scintilla = dynamic_cast<wxStyledTextCtrl*>( eventSource ) )
+        {
+            wxString eol = "\n";
+            switch( scintilla->GetEOLMode() )
+            {
+            case wxSTC_EOL_CRLF: eol = "\r\n"; break;
+            case wxSTC_EOL_CR: eol = "\r"; break;
+            case wxSTC_EOL_LF: eol = "\n"; break;
+            }
+
+            long pos = scintilla->GetCurrentPos();
+            scintilla->InsertText( pos, eol );
+            scintilla->GotoPos( pos + eol.length() );
+            return;
+        }
+        return;
+    }
+    // command-return (Mac default) or Ctrl-Return (GTK) for OK
+    else if( ( aEvt.GetKeyCode() == WXK_RETURN || aEvt.GetKeyCode() == WXK_NUMPAD_ENTER ) && aEvt.ControlDown() )
     {
         wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
         return;
