@@ -619,18 +619,27 @@ void EDA_DRAW_FRAME::AddStandardSubMenus( TOOL_MENU& aToolMenu )
 
 void EDA_DRAW_FRAME::DisplayToolMsg( const wxString& msg )
 {
+    if( m_isClosing )
+        return;
+
     SetStatusText( msg, 6 );
 }
 
 
 void EDA_DRAW_FRAME::DisplayConstraintsMsg( const wxString& msg )
 {
+    if( m_isClosing )
+        return;
+
     SetStatusText( msg, 7 );
 }
 
 
 void EDA_DRAW_FRAME::DisplayGridMsg()
 {
+    if( m_isClosing )
+        return;
+
     wxString msg;
 
     GRID_SETTINGS& gridSettings = m_toolManager->GetSettings()->m_Window.grid;
@@ -645,6 +654,9 @@ void EDA_DRAW_FRAME::DisplayGridMsg()
 
 void EDA_DRAW_FRAME::DisplayUnitsMsg()
 {
+    if( m_isClosing )
+        return;
+
     wxString msg;
 
     switch( GetUserUnits() )
@@ -712,6 +724,9 @@ void EDA_DRAW_FRAME::updateStatusBarWidths()
 
 void EDA_DRAW_FRAME::UpdateStatusBar()
 {
+    if( m_isClosing )
+        return;
+
     SetStatusText( GetZoomLevelIndicator(), 1 );
 
     // Absolute and relative cursor positions are handled by overloading this function and
@@ -1236,7 +1251,7 @@ void EDA_DRAW_FRAME::ShowChangedLanguage()
 
 void EDA_DRAW_FRAME::UpdateProperties()
 {
-    if( !m_propertiesPanel || !m_propertiesPanel->IsShownOnScreen() )
+    if( m_isClosing || !m_propertiesPanel || !m_propertiesPanel->IsShownOnScreen() )
         return;
 
     m_propertiesPanel->UpdateData();
