@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2021 Ola Rinta-Koski
- * Copyright (C) 2021-2022 Kicad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * Font abstract base class
  *
@@ -24,7 +24,11 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#include <wx/font.h>
+#include <list>
+#include <mutex>
+#include <unordered_map>
+
+#include <macros.h>
 #include <string_utils.h>
 #include <gal/graphics_abstraction_layer.h>
 #include <font/stroke_font.h>
@@ -32,10 +36,9 @@
 #include <trigo.h>
 #include <markup_parser.h>
 
-// The "official" name of the building Kicad stroke font (always existing)
+// The "official" name of the Kicad stroke font (always existing)
 #include <font/kicad_font_name.h>
-#include "macros.h"
-
+#include <wx/tokenzr.h>
 
 // markup_parser.h includes pegtl.hpp which includes windows.h... which leaks #define DrawText
 #undef DrawText
@@ -43,9 +46,86 @@
 
 using namespace KIFONT;
 
+METRICS g_defaultMetrics;
+
+
+const METRICS& METRICS::Default()
+{
+    return g_defaultMetrics;
+}
+
+
 FONT* FONT::s_defaultFont = nullptr;
 
-std::map< std::tuple<wxString, bool, bool>, FONT*> FONT::s_fontMap;
+std::map< std::tuple<wxString, bool, bool, bool>, FONT*> FONT::s_fontMap;
+
+class MARKUP_CACHE
+{
+public:
+    struct ENTRY
+    {
+        std::string source;
+        std::unique_ptr<MARKUP::NODE> root;
+    };
+
+    MARKUP_CACHE( size_t aMaxSize ) :
+            m_maxSize( aMaxSize )
+    {
+    }
+
+    ENTRY& Put( const wxString& aQuery, ENTRY&& aResult )
+    {
+        auto it = m_cache.find( aQuery );
+
+        m_cacheMru.emplace_front( std::make_pair( aQuery, std::move( aResult ) ) );
+
+        if( it != m_cache.end() )
+        {
+            m_cacheMru.erase( it->second );
+            m_cache.erase( it );
+        }
+
+        m_cache[aQuery] = m_cacheMru.begin();
+
+        if( m_cache.size() > m_maxSize )
+        {
+            auto last = m_cacheMru.end();
+            last--;
+            m_cache.erase( last->first );
+            m_cacheMru.pop_back();
+        }
+
+        return m_cacheMru.begin()->second;
+    }
+
+    ENTRY* Get( const wxString& aQuery )
+    {
+        auto it = m_cache.find( aQuery );
+
+        if( it == m_cache.end() )
+            return nullptr;
+
+        m_cacheMru.splice( m_cacheMru.begin(), m_cacheMru, it->second );
+
+        return &m_cacheMru.begin()->second;
+    }
+
+    void Clear()
+    {
+        m_cacheMru.clear();
+        m_cache.clear();
+    }
+
+private:
+    size_t                                                                        m_maxSize;
+    std::list<std::pair<wxString, ENTRY>>                                         m_cacheMru;
+    std::unordered_map<wxString, std::list<std::pair<wxString, ENTRY>>::iterator> m_cache;
+};
+
+
+static MARKUP_CACHE s_markupCache( 1024 );
+static std::mutex s_markupCacheMutex;
+static std::mutex s_defaultFontMutex;;
 
 
 FONT::FONT()
@@ -55,6 +135,8 @@ FONT::FONT()
 
 FONT* FONT::getDefaultFont()
 {
+    std::lock_guard lock( s_defaultFontMutex );
+
     if( !s_defaultFont )
         s_defaultFont = STROKE_FONT::LoadFont( wxEmptyString );
 
@@ -62,17 +144,22 @@ FONT* FONT::getDefaultFont()
 }
 
 
-FONT* FONT::GetFont( const wxString& aFontName, bool aBold, bool aItalic )
+FONT* FONT::GetFont( const wxString& aFontName, bool aBold, bool aItalic,
+                     const std::vector<wxString>* aEmbeddedFiles, bool aForDrawingSheet )
 {
     if( aFontName.empty() || aFontName.StartsWith( KICAD_FONT_NAME ) )
         return getDefaultFont();
 
-    std::tuple<wxString, bool, bool> key = { aFontName, aBold, aItalic };
+    std::tuple<wxString, bool, bool, bool> key = { aFontName, aBold, aItalic, aForDrawingSheet };
 
-    FONT* font = s_fontMap[key];
+    FONT* font = nullptr;
+
+    if( s_fontMap.find( key ) != s_fontMap.end() )
+        font = s_fontMap[key];
 
     if( !font )
-        font = OUTLINE_FONT::LoadFont( aFontName, aBold, aItalic );
+        font = OUTLINE_FONT::LoadFont( aFontName, aBold, aItalic, aEmbeddedFiles,
+                                       aForDrawingSheet );
 
     if( !font )
         font = getDefaultFont();
@@ -93,20 +180,21 @@ bool FONT::IsStroke( const wxString& aFontName )
 
 void FONT::getLinePositions( const wxString& aText, const VECTOR2I& aPosition,
                              wxArrayString& aTextLines, std::vector<VECTOR2I>& aPositions,
-                             std::vector<VECTOR2I>& aExtents, const TEXT_ATTRIBUTES& aAttrs ) const
+                             std::vector<VECTOR2I>& aExtents, const TEXT_ATTRIBUTES& aAttrs,
+                             const METRICS& aFontMetrics ) const
 {
     wxStringSplit( aText, aTextLines, '\n' );
     int lineCount = aTextLines.Count();
     aPositions.reserve( lineCount );
 
-    int interline = GetInterline( aAttrs.m_Size.y, aAttrs.m_LineSpacing );
+    int interline = GetInterline( aAttrs.m_Size.y, aFontMetrics ) * aAttrs.m_LineSpacing;
     int height = 0;
 
     for( int i = 0; i < lineCount; i++ )
     {
         VECTOR2I pos( aPosition.x, aPosition.y + i * interline );
         VECTOR2I end = boundingBoxSingleLine( nullptr, aTextLines[i], pos, aAttrs.m_Size,
-                                              aAttrs.m_Italic );
+                                              aAttrs.m_Italic, aFontMetrics );
         VECTOR2I bBox( end - pos );
 
         aExtents.push_back( bBox );
@@ -120,11 +208,21 @@ void FONT::getLinePositions( const wxString& aText, const VECTOR2I& aPosition,
     VECTOR2I offset( 0, 0 );
     offset.y += aAttrs.m_Size.y;
 
+    if( IsStroke() )
+    {
+        // Fudge factors to match 6.0 positioning
+        offset.x += aAttrs.m_StrokeWidth / 1.52;
+        offset.y -= aAttrs.m_StrokeWidth * 0.052;
+    }
+
     switch(  aAttrs.m_Valign )
     {
     case GR_TEXT_V_ALIGN_TOP:                            break;
     case GR_TEXT_V_ALIGN_CENTER: offset.y -= height / 2; break;
     case GR_TEXT_V_ALIGN_BOTTOM: offset.y -= height;     break;
+    case GR_TEXT_V_ALIGN_INDETERMINATE:
+        wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+        break;
     }
 
     for( int i = 0; i < lineCount; i++ )
@@ -136,9 +234,12 @@ void FONT::getLinePositions( const wxString& aText, const VECTOR2I& aPosition,
 
         switch( aAttrs.m_Halign )
         {
-        case GR_TEXT_H_ALIGN_LEFT:                                   break;
-        case GR_TEXT_H_ALIGN_CENTER: lineOffset.x = -lineSize.x / 2; break;
-        case GR_TEXT_H_ALIGN_RIGHT:  lineOffset.x = -lineSize.x;     break;
+        case GR_TEXT_H_ALIGN_LEFT:                                              break;
+        case GR_TEXT_H_ALIGN_CENTER: lineOffset.x = -lineSize.x / 2;            break;
+        case GR_TEXT_H_ALIGN_RIGHT:  lineOffset.x = -( lineSize.x + offset.x ); break;
+        case GR_TEXT_H_ALIGN_INDETERMINATE:
+            wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+            break;
         }
 
         aPositions.push_back( aPosition + lineOffset );
@@ -146,18 +247,9 @@ void FONT::getLinePositions( const wxString& aText, const VECTOR2I& aPosition,
 }
 
 
-/**
- * Draw a string.
- *
- * @param aGal
- * @param aText is the text to be drawn.
- * @param aPosition is the text object position in world coordinates.
- * @param aCursor is the current text position (for multiple text blocks within a single text
- *                object, such as a run of superscript characters)
- * @param aAttrs are the styling attributes of the text, including its rotation
- */
 void FONT::Draw( KIGFX::GAL* aGal, const wxString& aText, const VECTOR2I& aPosition,
-                 const VECTOR2I& aCursor, const TEXT_ATTRIBUTES& aAttrs ) const
+                 const VECTOR2I& aCursor, const TEXT_ATTRIBUTES& aAttrs,
+                 const METRICS& aFontMetrics ) const
 {
     if( !aGal || aText.empty() )
         return;
@@ -169,7 +261,7 @@ void FONT::Draw( KIGFX::GAL* aGal, const wxString& aText, const VECTOR2I& aPosit
     std::vector<VECTOR2I> positions;
     std::vector<VECTOR2I> extents;
 
-    getLinePositions( aText, position, strings_list, positions, extents, aAttrs );
+    getLinePositions( aText, position, strings_list, positions, extents, aAttrs, aFontMetrics );
 
     aGal->SetLineWidth( aAttrs.m_StrokeWidth );
 
@@ -177,20 +269,23 @@ void FONT::Draw( KIGFX::GAL* aGal, const wxString& aText, const VECTOR2I& aPosit
     {
         drawSingleLineText( aGal, nullptr, strings_list[i], positions[i], aAttrs.m_Size,
                             aAttrs.m_Angle, aAttrs.m_Mirrored, aPosition, aAttrs.m_Italic,
-                            aAttrs.m_Underlined );
+                            aAttrs.m_Underlined, aFontMetrics );
     }
 }
 
 
 /**
- * @return position of cursor for drawing next substring
+ * @return position of cursor for drawing next substring.
  */
 VECTOR2I drawMarkup( BOX2I* aBoundingBox, std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
-                     const std::unique_ptr<MARKUP::NODE>& aNode, const VECTOR2I& aPosition,
+                     const MARKUP::NODE* aNode, const VECTOR2I& aPosition,
                      const KIFONT::FONT* aFont, const VECTOR2I& aSize, const EDA_ANGLE& aAngle,
-                     bool aMirror, const VECTOR2I& aOrigin, TEXT_STYLE_FLAGS aTextStyle )
+                     bool aMirror, const VECTOR2I& aOrigin, TEXT_STYLE_FLAGS aTextStyle,
+                     const METRICS& aFontMetrics )
 {
     VECTOR2I nextPosition = aPosition;
+    bool     drawUnderline = false;
+    bool     drawOverbar = false;
 
     if( aNode )
     {
@@ -199,30 +294,78 @@ VECTOR2I drawMarkup( BOX2I* aBoundingBox, std::vector<std::unique_ptr<GLYPH>>* a
         if( !aNode->is_root() )
         {
             if( aNode->isSubscript() )
-                textStyle = TEXT_STYLE::SUBSCRIPT;
+                textStyle |= TEXT_STYLE::SUBSCRIPT;
             else if( aNode->isSuperscript() )
-                textStyle = TEXT_STYLE::SUPERSCRIPT;
+                textStyle |= TEXT_STYLE::SUPERSCRIPT;
 
             if( aNode->isOverbar() )
-                textStyle |= TEXT_STYLE::OVERBAR;
+                drawOverbar = true;
 
             if( aNode->has_content() )
             {
                 BOX2I bbox;
 
                 nextPosition = aFont->GetTextAsGlyphs( &bbox, aGlyphs, aNode->asWxString(), aSize,
-                                                       aPosition, aAngle, aMirror, aOrigin,
+                                                       nextPosition, aAngle, aMirror, aOrigin,
                                                        textStyle );
 
                 if( aBoundingBox )
                     aBoundingBox->Merge( bbox );
             }
         }
+        else if( aTextStyle & TEXT_STYLE::UNDERLINE )
+        {
+            drawUnderline = true;
+        }
 
         for( const std::unique_ptr<MARKUP::NODE>& child : aNode->children )
         {
-            nextPosition = drawMarkup( aBoundingBox, aGlyphs, child, nextPosition, aFont, aSize,
-                                       aAngle, aMirror, aOrigin, textStyle );
+            nextPosition = drawMarkup( aBoundingBox, aGlyphs, child.get(), nextPosition, aFont,
+                                       aSize, aAngle, aMirror, aOrigin, textStyle, aFontMetrics );
+        }
+    }
+
+    if( drawUnderline )
+    {
+        // Shorten the bar a little so its rounded ends don't make it over-long
+        double barTrim = aSize.x * 0.1;
+        double barOffset = aFontMetrics.GetUnderlineVerticalPosition( aSize.y );
+
+        VECTOR2D barStart( aPosition.x + barTrim, aPosition.y - barOffset );
+        VECTOR2D barEnd( nextPosition.x - barTrim, nextPosition.y - barOffset );
+
+        if( aGlyphs )
+        {
+            STROKE_GLYPH barGlyph;
+
+            barGlyph.AddPoint( barStart );
+            barGlyph.AddPoint( barEnd );
+            barGlyph.Finalize();
+
+            aGlyphs->push_back( barGlyph.Transform( { 1.0, 1.0 }, { 0, 0 }, false, aAngle, aMirror,
+                                                    aOrigin ) );
+        }
+    }
+
+    if( drawOverbar )
+    {
+        // Shorten the bar a little so its rounded ends don't make it over-long
+        double barTrim = aSize.x * 0.1;
+        double barOffset = aFontMetrics.GetOverbarVerticalPosition( aSize.y );
+
+        VECTOR2D barStart( aPosition.x + barTrim, aPosition.y - barOffset );
+        VECTOR2D barEnd( nextPosition.x - barTrim, nextPosition.y - barOffset );
+
+        if( aGlyphs )
+        {
+            STROKE_GLYPH barGlyph;
+
+            barGlyph.AddPoint( barStart );
+            barGlyph.AddPoint( barEnd );
+            barGlyph.Finalize();
+
+            aGlyphs->push_back( barGlyph.Transform( { 1.0, 1.0 }, { 0, 0 }, false, aAngle, aMirror,
+                                                    aOrigin ) );
         }
     }
 
@@ -233,20 +376,33 @@ VECTOR2I drawMarkup( BOX2I* aBoundingBox, std::vector<std::unique_ptr<GLYPH>>* a
 VECTOR2I FONT::drawMarkup( BOX2I* aBoundingBox, std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
                            const wxString& aText, const VECTOR2I& aPosition, const VECTOR2I& aSize,
                            const EDA_ANGLE& aAngle, bool aMirror, const VECTOR2I& aOrigin,
-                           TEXT_STYLE_FLAGS aTextStyle ) const
+                           TEXT_STYLE_FLAGS aTextStyle, const METRICS& aFontMetrics ) const
 {
-    MARKUP::MARKUP_PARSER         markupParser( TO_UTF8( aText ) );
-    std::unique_ptr<MARKUP::NODE> root = markupParser.Parse();
+    std::lock_guard<std::mutex> lock( s_markupCacheMutex );
 
-    return ::drawMarkup( aBoundingBox, aGlyphs, root, aPosition, this, aSize, aAngle, aMirror,
-                         aOrigin, aTextStyle );
+    MARKUP_CACHE::ENTRY* markup = s_markupCache.Get( aText );
+
+    if( !markup || !markup->root )
+    {
+        MARKUP_CACHE::ENTRY& cached = s_markupCache.Put( aText, {} );
+
+        cached.source = TO_UTF8( aText );
+        MARKUP::MARKUP_PARSER markupParser( &cached.source );
+        cached.root = markupParser.Parse();
+        markup = &cached;
+    }
+
+    wxASSERT( markup && markup->root );
+
+    return ::drawMarkup( aBoundingBox, aGlyphs, markup->root.get(), aPosition, this, aSize, aAngle,
+                         aMirror, aOrigin, aTextStyle, aFontMetrics );
 }
 
 
 void FONT::drawSingleLineText( KIGFX::GAL* aGal, BOX2I* aBoundingBox, const wxString& aText,
                                const VECTOR2I& aPosition, const VECTOR2I& aSize,
                                const EDA_ANGLE& aAngle, bool aMirror, const VECTOR2I& aOrigin,
-                               bool aItalic, bool aUnderline ) const
+                               bool aItalic, bool aUnderline, const METRICS& aFontMetrics ) const
 {
     if( !aGal )
         return;
@@ -262,14 +418,14 @@ void FONT::drawSingleLineText( KIGFX::GAL* aGal, BOX2I* aBoundingBox, const wxSt
     std::vector<std::unique_ptr<GLYPH>> glyphs;
 
     (void) drawMarkup( aBoundingBox, &glyphs, aText, aPosition, aSize, aAngle, aMirror, aOrigin,
-                       textStyle );
+                       textStyle, aFontMetrics );
 
     aGal->DrawGlyphs( glyphs );
 }
 
 
 VECTOR2I FONT::StringBoundaryLimits( const wxString& aText, const VECTOR2I& aSize, int aThickness,
-                                     bool aBold, bool aItalic ) const
+                                     bool aBold, bool aItalic, const METRICS& aFontMetrics ) const
 {
     // TODO do we need to parse every time - have we already parsed?
     BOX2I            boundingBox;
@@ -281,13 +437,13 @@ VECTOR2I FONT::StringBoundaryLimits( const wxString& aText, const VECTOR2I& aSiz
     if( aItalic )
         textStyle |= TEXT_STYLE::ITALIC;
 
-    (void) drawMarkup( &boundingBox, nullptr, aText, VECTOR2I(), aSize, ANGLE_0, false,
-                       VECTOR2I(), textStyle );
+    (void) drawMarkup( &boundingBox, nullptr, aText, VECTOR2I(), aSize, ANGLE_0, false, VECTOR2I(),
+                       textStyle, aFontMetrics );
 
     if( IsStroke() )
     {
         // Inflate by a bit more than thickness/2 to catch diacriticals, descenders, etc.
-        boundingBox.Inflate( KiROUND( aThickness * 1.25 ) );
+        boundingBox.Inflate( KiROUND( aThickness * 1.5 ) );
     }
     else if( IsOutline() )
     {
@@ -300,7 +456,7 @@ VECTOR2I FONT::StringBoundaryLimits( const wxString& aText, const VECTOR2I& aSiz
 
 VECTOR2I FONT::boundingBoxSingleLine( BOX2I* aBBox, const wxString& aText,
                                       const VECTOR2I& aPosition, const VECTOR2I& aSize,
-                                      bool aItalic ) const
+                                      bool aItalic, const METRICS& aFontMetrics ) const
 {
     TEXT_STYLE_FLAGS textStyle = 0;
 
@@ -308,15 +464,17 @@ VECTOR2I FONT::boundingBoxSingleLine( BOX2I* aBBox, const wxString& aText,
         textStyle |= TEXT_STYLE::ITALIC;
 
     VECTOR2I extents = drawMarkup( aBBox, nullptr, aText, aPosition, aSize, ANGLE_0, false,
-                                   VECTOR2I(), textStyle );
+                                   VECTOR2I(), textStyle, aFontMetrics );
 
     return extents;
 }
 
 
-/*
- * Break marked-up text into "words".  In this context, a "word" is EITHER a run of marked-up
- * text (subscript, superscript or overbar), OR a run of non-marked-up text separated by spaces.
+/**
+ * Break marked-up text into "words".
+ *
+ * In this context, a "word" is EITHER a run of marked-up text (subscript, superscript or
+ * overbar), OR a run of non-marked-up text separated by spaces.
  */
 void wordbreakMarkup( std::vector<std::pair<wxString, int>>* aWords,
                       const std::unique_ptr<MARKUP::NODE>& aNode, const KIFONT::FONT* aFont,
@@ -376,20 +534,22 @@ void wordbreakMarkup( std::vector<std::pair<wxString, int>>* aWords,
         }
         else
         {
-            wxString      textRun = aNode->asWxString();
-            wxArrayString words;
+            wxString              textRun = aNode->asWxString();
+            wxStringTokenizer     tokenizer( textRun, " ", wxTOKEN_RET_DELIMS );
+            std::vector<wxString> words;
 
-            wxStringSplit( textRun, words, ' ' );
+            while( tokenizer.HasMoreTokens() )
+                words.emplace_back( tokenizer.GetNextToken() );
 
-            if( textRun.EndsWith( wxS( " " ) ) )
-                words.Add( wxS( " " ) );
-
-            for( size_t ii = 0; ii < words.size(); ++ii )
+            for( const wxString& word : words )
             {
-                int w = aFont->GetTextAsGlyphs( nullptr, nullptr, words[ii], aSize, { 0, 0 },
+                wxString chars = word;
+                chars.Trim();
+
+                int w = aFont->GetTextAsGlyphs( nullptr, nullptr, chars, aSize, { 0, 0 },
                                                 ANGLE_0, false, { 0, 0 }, textStyle ).x;
 
-                aWords->emplace_back( std::make_pair( words[ii], w ) );
+                aWords->emplace_back( std::make_pair( word, w ) );
             }
         }
     }
@@ -409,17 +569,6 @@ void FONT::wordbreakMarkup( std::vector<std::pair<wxString, int>>* aWords, const
 }
 
 
-/*
- * This is a highly simplified line-breaker.  KiCad is an EDA tool, not a word processor.
- *
- * 1) It breaks only on spaces.  If you type a word wider than the column width then you get
- *    overflow.
- * 2) It treats runs of formatted text (superscript, subscript, overbar) as single words.
- * 3) It does not perform justification.
- *
- * The results of the linebreaking are the addition of \n in the text.  It is presumed that this
- * function is called on m_shownText (or equivalent) rather than the original source text.
- */
 void FONT::LinebreakText( wxString& aText, int aColumnWidth, const VECTOR2I& aSize, int aThickness,
                           bool aBold, bool aItalic ) const
 {
@@ -441,35 +590,70 @@ void FONT::LinebreakText( wxString& aText, int aColumnWidth, const VECTOR2I& aSi
 
     for( size_t ii = 0; ii < textLines.Count(); ++ii )
     {
-        bool virginLine = true;
-        int  lineWidth = 0;
+        std::vector<std::pair<wxString, int>> markup;
         std::vector<std::pair<wxString, int>> words;
 
-        wordbreakMarkup( &words, textLines[ii], aSize, textStyle );
+        wordbreakMarkup( &markup, textLines[ii], aSize, textStyle );
 
-        for( size_t jj = 0; jj < words.size(); /* advance in loop */ )
+        for( const auto& [ run, runWidth ] : markup )
         {
-            if( virginLine )
+            if( !words.empty() && !words.back().first.EndsWith( ' ' ) )
             {
-                // First word is always placed, even when wider than columnWidth.
-                aText += words[jj].first;
-                lineWidth += words[jj].second;
-                jj++;
-
-                virginLine = false;
-            }
-            else if( lineWidth + spaceWidth + words[jj].second < aColumnWidth - aThickness )
-            {
-                aText += " " + words[jj].first;
-                lineWidth += spaceWidth + words[jj].second;
-                jj++;
+                words.back().first += run;
+                words.back().second += runWidth;
             }
             else
             {
-                aText += '\n';
+                words.emplace_back( std::make_pair( run, runWidth ) );
+            }
+        }
 
+        bool     buryMode = false;
+        int      lineWidth = 0;
+        wxString pendingSpaces;
+
+        for( const auto& [ word, wordWidth ] : words )
+        {
+            int  pendingSpaceWidth = (int) pendingSpaces.Length() * spaceWidth;
+            bool overflow = lineWidth + pendingSpaceWidth + wordWidth > aColumnWidth - aThickness;
+
+            if( overflow && pendingSpaces.Length() > 0 )
+            {
+                aText += '\n';
                 lineWidth = 0;
-                virginLine = true;
+                pendingSpaces = wxEmptyString;
+                pendingSpaceWidth = 0;
+                buryMode = true;
+            }
+
+            if( word == wxS( " " ) )
+            {
+                pendingSpaces += word;
+            }
+            else
+            {
+                if( buryMode )
+                {
+                    buryMode = false;
+                }
+                else
+                {
+                    aText += pendingSpaces;
+                    lineWidth += pendingSpaceWidth;
+                }
+
+                if( word.EndsWith( ' ' ) )
+                {
+                    aText += word.Left( word.Length() - 1 );
+                    pendingSpaces = wxS( " " );
+                }
+                else
+                {
+                    aText += word;
+                    pendingSpaces = wxEmptyString;
+                }
+
+                lineWidth += wordWidth;
             }
         }
 
