@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2012 Torsten Hueter, torstenhtr <at> gmx.de
- * Copyright (C) 2012-2023 Kicad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * Copyright (C) 2013-2017 CERN
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
@@ -33,14 +33,17 @@
 #endif
 
 #include <advanced_config.h>
+#include <build_version.h>
 #include <gal/opengl/opengl_gal.h>
 #include <gal/opengl/utils.h>
 #include <gal/definitions.h>
-#include <gl_context_mgr.h>
+#include <gal/opengl/gl_context_mgr.h>
 #include <geometry/shape_poly_set.h>
+#include <math/vector2wx.h>
 #include <bitmap_base.h>
 #include <bezier_curves.h>
 #include <math/util.h> // for KiROUND
+#include <pgm_base.h>
 #include <trace_helpers.h>
 
 #include <wx/frame.h>
@@ -49,10 +52,10 @@
 #include <geometry/geometry_utils.h>
 #include <thread_pool.h>
 
-#include <profile.h>
+#include <core/profile.h>
 #include <trace_helpers.h>
 
-#include <gl_utils.h>
+#include <gal/opengl/gl_utils.h>
 
 #include <functional>
 #include <limits>
@@ -70,8 +73,15 @@ using namespace KIGFX;
 #include <glsl_kicad_vert.h>
 using namespace KIGFX::BUILTIN_FONT;
 
-static void      InitTesselatorCallbacks( GLUtesselator* aTesselator );
-static const int glAttributes[] = { WX_GL_RGBA, WX_GL_DOUBLEBUFFER, WX_GL_DEPTH_SIZE, 8, 0 };
+static void InitTesselatorCallbacks( GLUtesselator* aTesselator );
+
+static wxGLAttributes getGLAttribs()
+{
+    wxGLAttributes attribs;
+    attribs.RGBA().DoubleBuffer().Depth( 8 ).EndList();
+
+    return attribs;
+}
 
 wxGLContext* OPENGL_GAL::m_glMainContext = nullptr;
 int          OPENGL_GAL::m_instanceCounter = 0;
@@ -94,9 +104,10 @@ public:
 private:
     struct CACHED_BITMAP
     {
-        GLuint id;
-        int    w, h;
-        size_t size;
+        GLuint        id;
+        int           w, h;
+        size_t        size;
+        long long int accessTime;
     };
 
     GLuint cacheBitmap( const BITMAP_BASE* aBitmap );
@@ -105,9 +116,9 @@ private:
     const size_t m_cacheMaxSize     = 256 * 1024 * 1024;
 
     std::map<const KIID, CACHED_BITMAP> m_bitmaps;
-    std::list<KIID> m_cacheLru;
-    size_t m_cacheSize;
-    std::list<GLuint> m_freedTextureIds;
+    std::list<KIID>                     m_cacheLru;
+    size_t                              m_cacheSize;
+    std::list<GLuint>                   m_freedTextureIds;
 };
 
 }; // namespace KIGFX
@@ -130,6 +141,7 @@ GLuint GL_BITMAP_CACHE::RequestBitmap( const BITMAP_BASE* aBitmap )
         // A bitmap is found in cache bitmap. Ensure the associated texture is still valid.
         if( glIsTexture( it->second.id ) )
         {
+            it->second.accessTime = wxGetUTCTimeMillis().GetValue();
             return it->second.id;
         }
         else
@@ -182,66 +194,129 @@ GLuint GL_BITMAP_CACHE::cacheBitmap( const BITMAP_BASE* aBitmap )
         m_freedTextureIds.pop_front();
     }
 
-    bmp.size = bmp.w * bmp.h * 4;
-    auto buf = std::make_unique<uint8_t[]>( bmp.size );
+    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
 
-    for( int y = 0; y < bmp.h; y++ )
+    if( imgData.HasAlpha() || imgData.HasMask() )
     {
-        for( int x = 0; x < bmp.w; x++ )
+        bmp.size = bmp.w * bmp.h * 4;
+        auto buf = std::make_unique<uint8_t[]>( bmp.size );
+
+        uint8_t* dstP = buf.get();
+        uint8_t* srcP = imgData.GetData();
+
+        long long pxCount = static_cast<long long>( bmp.w ) * bmp.h;
+
+        if( imgData.HasAlpha() )
         {
-            uint8_t* p = buf.get() + ( bmp.w * y + x ) * 4;
+            uint8_t* srcAlpha = imgData.GetAlpha();
 
-            p[0] = imgData.GetRed( x, y );
-            p[1] = imgData.GetGreen( x, y );
-            p[2] = imgData.GetBlue( x, y );
+            for( long long px = 0; px < pxCount; px++ )
+            {
+                memcpy( dstP, srcP, 3 );
+                dstP[3] = *srcAlpha;
 
-            if( imgData.HasAlpha() )
-                p[3] = imgData.GetAlpha( x, y );
-            else if( imgData.HasMask() && p[0] == imgData.GetMaskRed()
-                     && p[1] == imgData.GetMaskGreen() && p[2] == imgData.GetMaskBlue() )
-                p[3] = wxALPHA_TRANSPARENT;
-            else
-                p[3] = wxALPHA_OPAQUE;
+                srcAlpha += 1;
+                srcP += 3;
+                dstP += 4;
+            }
         }
-    }
+        else if( imgData.HasMask() )
+        {
+            uint8_t maskRed = imgData.GetMaskRed();
+            uint8_t maskGreen = imgData.GetMaskGreen();
+            uint8_t maskBlue = imgData.GetMaskBlue();
 
-    glBindTexture( GL_TEXTURE_2D, textureID );
-    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA8, bmp.w, bmp.h, 0, GL_RGBA, GL_UNSIGNED_BYTE,
-                  buf.get() );
+            for( long long px = 0; px < pxCount; px++ )
+            {
+                memcpy( dstP, srcP, 3 );
+
+                if( srcP[0] == maskRed && srcP[1] == maskGreen && srcP[2] == maskBlue )
+                    dstP[3] = wxALPHA_TRANSPARENT;
+                else
+                    dstP[3] = wxALPHA_OPAQUE;
+
+                srcP += 3;
+                dstP += 4;
+            }
+        }
+
+        glBindTexture( GL_TEXTURE_2D, textureID );
+        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA8, bmp.w, bmp.h, 0, GL_RGBA, GL_UNSIGNED_BYTE,
+                      buf.get() );
+    }
+    else
+    {
+        bmp.size = bmp.w * bmp.h * 3;
+
+        uint8_t* srcP = imgData.GetData();
+
+        glBindTexture( GL_TEXTURE_2D, textureID );
+        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB8, bmp.w, bmp.h, 0, GL_RGB, GL_UNSIGNED_BYTE, srcP );
+    }
 
     glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
     glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
 
+    long long currentTime = wxGetUTCTimeMillis().GetValue();
+
     bmp.id = textureID;
+    bmp.accessTime = currentTime;
 
 #ifndef DISABLE_BITMAP_CACHE
-    m_cacheLru.emplace_back( aBitmap->GetImageID() );
-    m_cacheSize += bmp.size;
-    m_bitmaps.emplace( aBitmap->GetImageID(), std::move( bmp ) );
-
-    if( m_cacheLru.size() > m_cacheMaxElements
-        || m_cacheSize > m_cacheMaxSize )
+    if( ( m_cacheLru.size() + 1 > m_cacheMaxElements || m_cacheSize + bmp.size > m_cacheMaxSize )
+        && !m_cacheLru.empty() )
     {
-        KIID last = m_cacheLru.front();
-        CACHED_BITMAP& cachedBitmap = m_bitmaps[last];
+        KIID toRemove( 0 );
+        auto toRemoveLru = m_cacheLru.end();
+
+        // Remove entries accessed > 1s ago first
+        for( const auto& [kiid, cachedBmp] : m_bitmaps )
+        {
+            const int cacheTimeoutMillis = 1000L;
+
+            if( currentTime - cachedBmp.accessTime > cacheTimeoutMillis )
+            {
+                toRemove = kiid;
+                toRemoveLru = std::find( m_cacheLru.begin(), m_cacheLru.end(), toRemove );
+                break;
+            }
+        }
+
+        // Otherwise, remove the latest entry (it's less likely to be needed soon)
+        if( toRemove == niluuid )
+        {
+            toRemoveLru = m_cacheLru.end();
+            toRemoveLru--;
+
+            toRemove = *toRemoveLru;
+        }
+
+        CACHED_BITMAP& cachedBitmap = m_bitmaps[toRemove];
 
         m_cacheSize -= cachedBitmap.size;
         glDeleteTextures( 1, &cachedBitmap.id );
         m_freedTextureIds.emplace_back( cachedBitmap.id );
 
-        m_bitmaps.erase( last );
-        m_cacheLru.pop_front();
+        m_bitmaps.erase( toRemove );
+        m_cacheLru.erase( toRemoveLru );
     }
+
+    m_cacheLru.emplace_back( aBitmap->GetImageID() );
+    m_cacheSize += bmp.size;
+    m_bitmaps.emplace( aBitmap->GetImageID(), std::move( bmp ) );
 #endif
 
     return textureID;
 }
 
-OPENGL_GAL::OPENGL_GAL( GAL_DISPLAY_OPTIONS& aDisplayOptions, wxWindow* aParent,
+
+OPENGL_GAL::OPENGL_GAL( const KIGFX::VC_SETTINGS& aVcSettings, GAL_DISPLAY_OPTIONS& aDisplayOptions,
+                        wxWindow* aParent,
                         wxEvtHandler* aMouseListener, wxEvtHandler* aPaintListener,
                         const wxString& aName ) :
         GAL( aDisplayOptions ),
-        HIDPI_GL_CANVAS( aParent, wxID_ANY, (int*) glAttributes, wxDefaultPosition, wxDefaultSize,
+        HIDPI_GL_CANVAS( aVcSettings, aParent, getGLAttribs(), wxID_ANY, wxDefaultPosition,
+                         wxDefaultSize,
                          wxEXPAND, aName ),
         m_mouseListener( aMouseListener ),
         m_paintListener( aPaintListener ),
@@ -258,7 +333,7 @@ OPENGL_GAL::OPENGL_GAL( GAL_DISPLAY_OPTIONS& aDisplayOptions, wxWindow* aParent,
 {
     if( m_glMainContext == nullptr )
     {
-        m_glMainContext = GL_CONTEXT_MANAGER::Get().CreateCtx( this );
+        m_glMainContext = Pgm().GetGLContextManager()->CreateCtx( this );
 
         if( !m_glMainContext )
             throw std::runtime_error( "Could not create the main OpenGL context" );
@@ -267,7 +342,7 @@ OPENGL_GAL::OPENGL_GAL( GAL_DISPLAY_OPTIONS& aDisplayOptions, wxWindow* aParent,
     }
     else
     {
-        m_glPrivContext = GL_CONTEXT_MANAGER::Get().CreateCtx( this, m_glMainContext );
+        m_glPrivContext = Pgm().GetGLContextManager()->CreateCtx( this, m_glMainContext );
 
         if( !m_glPrivContext )
             throw std::runtime_error( "Could not create a private OpenGL context" );
@@ -313,15 +388,17 @@ OPENGL_GAL::OPENGL_GAL( GAL_DISPLAY_OPTIONS& aDisplayOptions, wxWindow* aParent,
     Connect( wxEVT_AUX2_UP, wxMouseEventHandler( OPENGL_GAL::skipMouseEvent ) );
     Connect( wxEVT_AUX2_DCLICK, wxMouseEventHandler( OPENGL_GAL::skipMouseEvent ) );
     Connect( wxEVT_MOUSEWHEEL, wxMouseEventHandler( OPENGL_GAL::skipMouseEvent ) );
-#if wxCHECK_VERSION( 3, 1, 0 ) || defined( USE_OSX_MAGNIFY_EVENT )
     Connect( wxEVT_MAGNIFY, wxMouseEventHandler( OPENGL_GAL::skipMouseEvent ) );
-#endif
+
 #if defined _WIN32 || defined _WIN64
     Connect( wxEVT_ENTER_WINDOW, wxMouseEventHandler( OPENGL_GAL::skipMouseEvent ) );
 #endif
 
+    Bind( wxEVT_GESTURE_ZOOM, &OPENGL_GAL::skipGestureEvent, this );
+    Bind( wxEVT_GESTURE_PAN, &OPENGL_GAL::skipGestureEvent, this );
+
     SetSize( aParent->GetClientSize() );
-    m_screenSize = VECTOR2I( GetNativePixelSize() );
+    m_screenSize = ToVECTOR2I( GetNativePixelSize() );
 
     // Grid color settings are different in Cairo and OpenGL
     SetGridColor( COLOR4D( 0.8, 0.8, 0.8, 0.1 ) );
@@ -346,7 +423,9 @@ OPENGL_GAL::OPENGL_GAL( GAL_DISPLAY_OPTIONS& aDisplayOptions, wxWindow* aParent,
 
 OPENGL_GAL::~OPENGL_GAL()
 {
-    GL_CONTEXT_MANAGER::Get().LockCtx( m_glPrivContext, this );
+
+    GL_CONTEXT_MANAGER* gl_mgr = Pgm().GetGLContextManager();
+    gl_mgr->LockCtx( m_glPrivContext, this );
 
     --m_instanceCounter;
     glFlush();
@@ -363,19 +442,19 @@ OPENGL_GAL::~OPENGL_GAL()
         delete m_tempManager;
     }
 
-    GL_CONTEXT_MANAGER::Get().UnlockCtx( m_glPrivContext );
+    gl_mgr->UnlockCtx( m_glPrivContext );
 
     // If it was the main context, then it will be deleted
     // when the last OpenGL GAL instance is destroyed (a few lines below)
     if( m_glPrivContext != m_glMainContext )
-        GL_CONTEXT_MANAGER::Get().DestroyCtx( m_glPrivContext );
+        gl_mgr->DestroyCtx( m_glPrivContext );
 
     delete m_shader;
 
     // Are we destroying the last GAL instance?
     if( m_instanceCounter == 0 )
     {
-        GL_CONTEXT_MANAGER::Get().LockCtx( m_glMainContext, this );
+        gl_mgr->LockCtx( m_glMainContext, this );
 
         if( m_isBitmapFontLoaded )
         {
@@ -383,8 +462,8 @@ OPENGL_GAL::~OPENGL_GAL()
             m_isBitmapFontLoaded = false;
         }
 
-        GL_CONTEXT_MANAGER::Get().UnlockCtx( m_glMainContext );
-        GL_CONTEXT_MANAGER::Get().DestroyCtx( m_glMainContext );
+        gl_mgr->UnlockCtx( m_glMainContext );
+        gl_mgr->DestroyCtx( m_glMainContext );
         m_glMainContext = nullptr;
     }
 }
@@ -401,7 +480,8 @@ wxString OPENGL_GAL::CheckFeatures( GAL_DISPLAY_OPTIONS& aOptions )
 
     try
     {
-        opengl_gal = new KIGFX::OPENGL_GAL( aOptions, testFrame );
+        KIGFX::VC_SETTINGS dummy;
+        opengl_gal = new KIGFX::OPENGL_GAL( dummy, aOptions, testFrame );
 
         testFrame->Raise();
         testFrame->Show();
@@ -443,12 +523,6 @@ bool OPENGL_GAL::updatedGalDisplayOptions( const GAL_DISPLAY_OPTIONS& aOptions )
         refresh = true;
     }
 
-    if( m_options.m_scaleFactor != GetScaleFactor() )
-    {
-        SetScaleFactor( m_options.m_scaleFactor );
-        refresh = true;
-    }
-
     if( super::updatedGalDisplayOptions( aOptions ) || refresh )
     {
         Refresh();
@@ -503,7 +577,15 @@ void OPENGL_GAL::BeginDrawing()
         // Prepare rendering target buffers
         m_compositor->Initialize();
         m_mainBuffer = m_compositor->CreateBuffer();
-        m_tempBuffer = m_compositor->CreateBuffer();
+        try
+        {
+            m_tempBuffer = m_compositor->CreateBuffer();
+        }
+        catch( const std::runtime_error& )
+        {
+            wxLogVerbose( "Could not create a framebuffer for diff mode blending.\n" );
+            m_tempBuffer = 0;
+        }
         try
         {
             m_overlayBuffer = m_compositor->CreateBuffer();
@@ -620,7 +702,7 @@ void OPENGL_GAL::BeginDrawing()
     m_shader->SetParameter( ufm_antialiasingOffset, renderingOffset );
     m_shader->Deactivate();
 
-    // Something betreen BeginDrawing and EndDrawing seems to depend on
+    // Something between BeginDrawing and EndDrawing seems to depend on
     // this texture unit being active, but it does not assure it itself.
     glActiveTexture( GL_TEXTURE0 );
 
@@ -639,14 +721,15 @@ void OPENGL_GAL::EndDrawing()
 {
     wxASSERT_MSG( m_isContextLocked, "What happened to the context lock?" );
 
-    PROF_TIMER cntTotal("gl-end-total");
-    PROF_TIMER cntEndCached("gl-end-cached");
-    PROF_TIMER cntEndNoncached("gl-end-noncached");
-    PROF_TIMER cntEndOverlay("gl-end-overlay");
-    PROF_TIMER cntComposite("gl-composite");
-    PROF_TIMER cntSwap("gl-swap");
+    PROF_TIMER cntTotal( "gl-end-total" );
+    PROF_TIMER cntEndCached( "gl-end-cached" );
+    PROF_TIMER cntEndNoncached( "gl-end-noncached" );
+    PROF_TIMER cntEndOverlay( "gl-end-overlay" );
+    PROF_TIMER cntComposite( "gl-composite" );
+    PROF_TIMER cntSwap( "gl-swap" );
 
     cntTotal.Start();
+
     // Cached & non-cached containers are rendered to the same buffer
     m_compositor->SetBuffer( m_mainBuffer );
 
@@ -667,6 +750,7 @@ void OPENGL_GAL::EndDrawing()
     cntEndOverlay.Stop();
 
     cntComposite.Start();
+
     // Be sure that the framebuffer is not colorized (happens on specific GPU&drivers combinations)
     glColor4d( 1.0, 1.0, 1.0, 1.0 );
 
@@ -699,7 +783,7 @@ void OPENGL_GAL::LockContext( int aClientCookie )
     m_isContextLocked = true;
     m_lockClientCookie = aClientCookie;
 
-    GL_CONTEXT_MANAGER::Get().LockCtx( m_glPrivContext, this );
+    Pgm().GetGLContextManager()->LockCtx( m_glPrivContext, this );
 }
 
 
@@ -708,12 +792,13 @@ void OPENGL_GAL::UnlockContext( int aClientCookie )
     wxASSERT_MSG( m_isContextLocked, "Context not locked.  A GAL_CONTEXT_LOCKER RAII object must "
                                      "be stacked rather than making separate lock/unlock calls." );
 
-    wxASSERT_MSG( m_lockClientCookie == aClientCookie, "Context was locked by a different client. "
-                                                       "Should not be possible with RAII objects." );
+    wxASSERT_MSG( m_lockClientCookie == aClientCookie,
+                  "Context was locked by a different client. "
+                  "Should not be possible with RAII objects." );
 
     m_isContextLocked = false;
 
-    GL_CONTEXT_MANAGER::Get().UnlockCtx( m_glPrivContext );
+    Pgm().GetGLContextManager()->UnlockCtx( m_glPrivContext );
 }
 
 
@@ -763,15 +848,16 @@ void OPENGL_GAL::drawSegment( const VECTOR2D& aStartPoint, const VECTOR2D& aEndP
     VECTOR2D startEndVector = aEndPoint - aStartPoint;
     double   lineLength = startEndVector.EuclideanNorm();
 
-    float startx = aStartPoint.x;
-    float starty = aStartPoint.y;
-    float endx = aStartPoint.x + lineLength;
-    float endy = aStartPoint.y + lineLength;
-
     // Be careful about floating point rounding.  As we draw segments in larger and larger
     // coordinates, the shader (which uses floats) will lose precision and stop drawing small
     // segments.  In this case, we need to draw a circle for the minimal segment.
-    if( startx == endx || starty == endy )
+    // Check if the coordinate differences can be accurately represented as floats
+    float startX = static_cast<float>( aStartPoint.x );
+    float startY = static_cast<float>( aStartPoint.y );
+    float endX = static_cast<float>( aEndPoint.x );
+    float endY = static_cast<float>( aEndPoint.y );
+
+    if( startX == endX && startY == endY )
     {
         drawCircle( aStartPoint, aWidth / 2, aReserve );
         return;
@@ -787,8 +873,8 @@ void OPENGL_GAL::drawSegment( const VECTOR2D& aStartPoint, const VECTOR2D& aEndP
     else
     {
         EDA_ANGLE lineAngle( startEndVector );
-        // Outlined tracks
 
+        // Outlined tracks
         SetLineWidth( 1.0 );
         m_currentManager->Color( m_strokeColor.r, m_strokeColor.g, m_strokeColor.b,
                                  m_strokeColor.a );
@@ -849,6 +935,7 @@ void OPENGL_GAL::drawCircle( const VECTOR2D& aCenterPoint, double aRadius, bool
         m_currentManager->Shader( SHADER_FILLED_CIRCLE, 3.0, aRadius );
         m_currentManager->Vertex( aCenterPoint.x, aCenterPoint.y, m_layerDepth );
     }
+
     if( m_isStrokeEnabled )
     {
         if( aReserve )
@@ -883,16 +970,16 @@ void OPENGL_GAL::drawCircle( const VECTOR2D& aCenterPoint, double aRadius, bool
 
 
 void OPENGL_GAL::DrawArc( const VECTOR2D& aCenterPoint, double aRadius,
-                          const EDA_ANGLE& aStartAngle, const EDA_ANGLE& aEndAngle )
+                          const EDA_ANGLE& aStartAngle, const EDA_ANGLE& aAngle )
 {
     if( aRadius <= 0 )
         return;
 
     double startAngle = aStartAngle.AsRadians();
-    double endAngle = aEndAngle.AsRadians();
+    double endAngle = startAngle + aAngle.AsRadians();
 
-    // Swap the angles, if start angle is greater than end angle
-    SWAP( startAngle, >, endAngle );
+    // Normalize arc angles
+    normalize( startAngle, endAngle );
 
     const double alphaIncrement = calcAngleStep( aRadius );
 
@@ -933,11 +1020,20 @@ void OPENGL_GAL::DrawArc( const VECTOR2D& aCenterPoint, double aRadius,
 
         VECTOR2D p( cos( startAngle ) * aRadius, sin( startAngle ) * aRadius );
         double   alpha;
+        unsigned int lineCount = 0;
+
+        for( alpha = startAngle + alphaIncrement; alpha <= endAngle; alpha += alphaIncrement )
+            lineCount++;
+
+        if( alpha != endAngle )
+            lineCount++;
+
+        reserveLineQuads( lineCount );
 
         for( alpha = startAngle + alphaIncrement; alpha <= endAngle; alpha += alphaIncrement )
         {
             VECTOR2D p_next( cos( alpha ) * aRadius, sin( alpha ) * aRadius );
-            DrawLine( p, p_next );
+            drawLineQuad( p, p_next, false );
 
             p = p_next;
         }
@@ -946,7 +1042,7 @@ void OPENGL_GAL::DrawArc( const VECTOR2D& aCenterPoint, double aRadius,
         if( alpha != endAngle )
         {
             VECTOR2D p_last( cos( endAngle ) * aRadius, sin( endAngle ) * aRadius );
-            DrawLine( p, p_last );
+            drawLineQuad( p, p_last, false );
         }
     }
 
@@ -955,7 +1051,7 @@ void OPENGL_GAL::DrawArc( const VECTOR2D& aCenterPoint, double aRadius,
 
 
 void OPENGL_GAL::DrawArcSegment( const VECTOR2D& aCenterPoint, double aRadius,
-                                 const EDA_ANGLE& aStartAngle, const EDA_ANGLE& aEndAngle,
+                                 const EDA_ANGLE& aStartAngle, const EDA_ANGLE& aAngle,
                                  double aWidth, double aMaxError )
 {
     if( aRadius <= 0 )
@@ -968,10 +1064,10 @@ void OPENGL_GAL::DrawArcSegment( const VECTOR2D& aCenterPoint, double aRadius,
     }
 
     double startAngle = aStartAngle.AsRadians();
-    double endAngle = aEndAngle.AsRadians();
+    double endAngle = startAngle + aAngle.AsRadians();
 
     // Swap the angles, if start angle is greater than end angle
-    SWAP( startAngle, >, endAngle );
+    normalize( startAngle, endAngle );
 
     // Calculate the seg count to approximate the arc with aMaxError or less
     int segCount360 = GetArcToSegmentCount( aRadius, aMaxError, FULL_CIRCLE );
@@ -986,9 +1082,21 @@ void OPENGL_GAL::DrawArcSegment( const VECTOR2D& aCenterPoint, double aRadius,
     if( seg_count % 2 != 0 )
         seg_count += 1;
 
+    // Our shaders have trouble rendering null line quads, so delegate this task to DrawSegment.
+    if( seg_count == 0 )
+    {
+        VECTOR2D p_start( aCenterPoint.x + cos( startAngle ) * aRadius,
+                          aCenterPoint.y + sin( startAngle ) * aRadius );
+
+        VECTOR2D p_end( aCenterPoint.x + cos( endAngle ) * aRadius,
+                        aCenterPoint.y + sin( endAngle ) * aRadius );
+
+        DrawSegment( p_start, p_end, aWidth );
+        return;
+    }
+
     // Recalculate alphaIncrement with a even integer number of segment
-    if( seg_count )
-        alphaIncrement = ( endAngle - startAngle ) / seg_count;
+    alphaIncrement = ( endAngle - startAngle ) / seg_count;
 
     Save();
     m_currentManager->Translate( aCenterPoint.x, aCenterPoint.y, 0.0 );
@@ -1111,13 +1219,23 @@ void OPENGL_GAL::DrawRectangle( const VECTOR2D& aStartPoint, const VECTOR2D& aEn
         m_currentManager->Color( m_strokeColor.r, m_strokeColor.g, m_strokeColor.b,
                                  m_strokeColor.a );
 
-        std::deque<VECTOR2D> pointList;
-        pointList.push_back( aStartPoint );
-        pointList.push_back( diagonalPointA );
-        pointList.push_back( aEndPoint );
-        pointList.push_back( diagonalPointB );
-        pointList.push_back( aStartPoint );
-        DrawPolyline( pointList );
+        // DrawLine (and DrawPolyline )
+        // has problem with 0 length lines so enforce minimum
+        if( aStartPoint == aEndPoint )
+        {
+            DrawLine( aStartPoint + VECTOR2D( 1.0, 0.0 ), aEndPoint );
+        }
+        else
+        {
+            std::deque<VECTOR2D> pointList;
+
+            pointList.push_back( aStartPoint );
+            pointList.push_back( diagonalPointA );
+            pointList.push_back( aEndPoint );
+            pointList.push_back( diagonalPointB );
+            pointList.push_back( aStartPoint );
+            DrawPolyline( pointList );
+        }
     }
 }
 
@@ -1310,9 +1428,7 @@ void OPENGL_GAL::drawTriangulatedPolyset( const SHAPE_POLY_SET& aPolySet,
 
     if( aStrokeTriangulation )
     {
-        COLOR4D oldStrokeColor = m_strokeColor;
-        double  oldLayerDepth = m_layerDepth;
-
+        GAL_SCOPED_ATTRS( *this, GAL_SCOPED_ATTRS::STROKE_COLOR | GAL_SCOPED_ATTRS::LAYER_DEPTH );
         SetLayerDepth( m_layerDepth - 1 );
 
         for( unsigned int j = 0; j < aPolySet.TriangulatedPolyCount(); ++j )
@@ -1328,9 +1444,6 @@ void OPENGL_GAL::drawTriangulatedPolyset( const SHAPE_POLY_SET& aPolySet,
                 DrawLine( c, a );
             }
         }
-
-        SetStrokeColor( oldStrokeColor );
-        SetLayerDepth( oldLayerDepth );
     }
 }
 
@@ -1353,7 +1466,8 @@ void OPENGL_GAL::DrawPolygon( const SHAPE_POLY_SET& aPolySet, bool aStrokeTriang
 
 void OPENGL_GAL::DrawPolygon( const SHAPE_LINE_CHAIN& aPolygon )
 {
-    wxCHECK( aPolygon.PointCount() >= 2, /* void */ );
+    if( aPolygon.PointCount() < 2 )
+        return;
 
     const int                   pointCount = aPolygon.SegmentCount() + 1;
     std::unique_ptr<GLdouble[]> points( new GLdouble[3 * pointCount] );
@@ -1386,7 +1500,10 @@ void OPENGL_GAL::DrawCurve( const VECTOR2D& aStartPoint, const VECTOR2D& aContro
     BEZIER_POLY converter( pointCtrl );
     converter.GetPoly( output, aFilterValue );
 
-    DrawPolyline( &output[0], output.size() );
+    if( output.size() == 1 )
+        output.push_back( output.front() );
+
+    DrawPolygon( &output[0], output.size() );
 }
 
 
@@ -1411,6 +1528,11 @@ void OPENGL_GAL::DrawBitmap( const BITMAP_BASE& aBitmap, double alphaBlend )
     if( !glIsTexture( texture_id ) ) // ensure the bitmap texture is still valid
         return;
 
+    glDepthFunc( GL_ALWAYS );
+
+    glAlphaFunc( GL_GREATER, 0.01f );
+    glEnable( GL_ALPHA_TEST );
+
     glMatrixMode( GL_TEXTURE );
     glPushMatrix();
     glTranslated( 0.5, 0.5, 0.5 );
@@ -1456,6 +1578,10 @@ void OPENGL_GAL::DrawBitmap( const BITMAP_BASE& aBitmap, double alphaBlend )
     glMatrixMode( GL_TEXTURE );
     glPopMatrix();
     glMatrixMode( GL_MODELVIEW );
+
+    glDisable( GL_ALPHA_TEST );
+
+    glDepthFunc( GL_LESS );
 }
 
 
@@ -1506,6 +1632,10 @@ void OPENGL_GAL::BitmapText( const wxString& aText, const VECTOR2I& aPosition,
         //if( IsTextMirrored() )
         //Translate( VECTOR2D( -textSize.x, 0 ) );
         break;
+
+    case GR_TEXT_H_ALIGN_INDETERMINATE:
+        wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+        break;
     }
 
     switch( GetVerticalJustify() )
@@ -1522,6 +1652,10 @@ void OPENGL_GAL::BitmapText( const wxString& aText, const VECTOR2I& aPosition,
         Translate( VECTOR2D( 0, -textSize.y ) );
         overbarHeight = -textSize.y / 2.0;
         break;
+
+    case GR_TEXT_V_ALIGN_INDETERMINATE:
+        wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+        break;
     }
 
     int overbarLength = 0;
@@ -1529,51 +1663,51 @@ void OPENGL_GAL::BitmapText( const wxString& aText, const VECTOR2I& aPosition,
     int braceNesting = 0;
 
     auto iterateString =
-            [&]( std::function<void( int aOverbarLength, int aOverbarHeight )> overbarFn,
-                 std::function<int( unsigned long aChar )>                     bitmapCharFn )
-    {
-        for( UTF8::uni_iter chIt = text.ubegin(), end = text.uend(); chIt < end; ++chIt )
-        {
-            wxASSERT_MSG( *chIt != '\n' && *chIt != '\r',
-                          "No support for multiline bitmap text yet" );
-
-            if( *chIt == '~' && overbarDepth == -1 )
+            [&]( const std::function<void( int aOverbarLength, int aOverbarHeight )>& overbarFn,
+                 const std::function<int( unsigned long aChar )>& bitmapCharFn )
             {
-                UTF8::uni_iter lookahead = chIt;
-
-                if( ++lookahead != end && *lookahead == '{' )
+                for( UTF8::uni_iter chIt = text.ubegin(), end = text.uend(); chIt < end; ++chIt )
                 {
-                    chIt = lookahead;
-                    overbarDepth = braceNesting;
-                    braceNesting++;
-                    continue;
+                    wxASSERT_MSG( *chIt != '\n' && *chIt != '\r',
+                                  "No support for multiline bitmap text yet" );
+
+                    if( *chIt == '~' && overbarDepth == -1 )
+                    {
+                        UTF8::uni_iter lookahead = chIt;
+
+                        if( ++lookahead != end && *lookahead == '{' )
+                        {
+                            chIt = lookahead;
+                            overbarDepth = braceNesting;
+                            braceNesting++;
+                            continue;
+                        }
+                    }
+                    else if( *chIt == '{' )
+                    {
+                        braceNesting++;
+                    }
+                    else if( *chIt == '}' )
+                    {
+                        if( braceNesting > 0 )
+                            braceNesting--;
+
+                        if( braceNesting == overbarDepth )
+                        {
+                            overbarFn( overbarLength, overbarHeight );
+                            overbarLength = 0;
+
+                            overbarDepth = -1;
+                            continue;
+                        }
+                    }
+
+                    if( overbarDepth != -1 )
+                        overbarLength += bitmapCharFn( *chIt );
+                    else
+                        bitmapCharFn( *chIt );
                 }
-            }
-            else if( *chIt == '{' )
-            {
-                braceNesting++;
-            }
-            else if( *chIt == '}' )
-            {
-                if( braceNesting > 0 )
-                    braceNesting--;
-
-                if( braceNesting == overbarDepth )
-                {
-                    overbarFn( overbarLength, overbarHeight );
-                    overbarLength = 0;
-
-                    overbarDepth = -1;
-                    continue;
-                }
-            }
-
-            if( overbarDepth != -1 )
-                overbarLength += bitmapCharFn( *chIt );
-            else
-                bitmapCharFn( *chIt );
-        }
-    };
+            };
 
     // First, calculate the amount of characters and overbars to reserve
 
@@ -1664,9 +1798,9 @@ void OPENGL_GAL::DrawGrid()
     int gridStartY = KiROUND( ( worldStartPoint.y - m_gridOrigin.y ) / gridScreenSize.y );
     int gridEndY = KiROUND( ( worldEndPoint.y - m_gridOrigin.y ) / gridScreenSize.y );
 
-    // Ensure start coordinate > end coordinate
-    SWAP( gridStartX, >, gridEndX );
-    SWAP( gridStartY, >, gridEndY );
+    // Ensure start coordinate < end coordinate
+    normalize( gridStartX, gridEndX );
+    normalize( gridStartY, gridEndY );
 
     // Ensure the grid fills the screen
     --gridStartX;
@@ -1840,7 +1974,7 @@ void OPENGL_GAL::Translate( const VECTOR2D& aVector )
 
 void OPENGL_GAL::Scale( const VECTOR2D& aScale )
 {
-    m_currentManager->Scale( aScale.x, aScale.y, 0.0f );
+    m_currentManager->Scale( aScale.x, aScale.y, 1.0f );
 }
 
 
@@ -1956,7 +2090,8 @@ void OPENGL_GAL::ClearTarget( RENDER_TARGET aTarget )
         break;
 
     case TARGET_TEMP:
-        m_compositor->SetBuffer( m_tempBuffer );
+        if( m_tempBuffer )
+            m_compositor->SetBuffer( m_tempBuffer );
         break;
 
     case TARGET_OVERLAY:
@@ -1980,10 +2115,10 @@ bool OPENGL_GAL::HasTarget( RENDER_TARGET aTarget )
     switch( aTarget )
     {
     default:
-    case TARGET_TEMP:
     case TARGET_CACHED:
     case TARGET_NONCACHED: return true;
     case TARGET_OVERLAY:   return ( m_overlayBuffer != 0 );
+    case TARGET_TEMP:      return ( m_tempBuffer != 0 );
     }
 }
 
@@ -1991,28 +2126,45 @@ bool OPENGL_GAL::HasTarget( RENDER_TARGET aTarget )
 void OPENGL_GAL::StartDiffLayer()
 {
     m_currentManager->EndDrawing();
-    SetTarget( TARGET_TEMP );
-    ClearTarget( TARGET_TEMP );
+
+    if( m_tempBuffer )
+    {
+        SetTarget( TARGET_TEMP );
+        ClearTarget( TARGET_TEMP );
+    }
 }
 
 
 void OPENGL_GAL::EndDiffLayer()
 {
-    glBlendEquation( GL_MAX );
-    m_currentManager->EndDrawing();
-    glBlendEquation( GL_FUNC_ADD );
+    if( m_tempBuffer )
+    {
+        glBlendEquation( GL_MAX );
+        m_currentManager->EndDrawing();
+        glBlendEquation( GL_FUNC_ADD );
 
-    m_compositor->DrawBuffer( m_tempBuffer, m_mainBuffer );
+        m_compositor->DrawBuffer( m_tempBuffer, m_mainBuffer );
+    }
+    else
+    {
+        // Fall back to imperfect alpha blending on single buffer
+        glBlendFunc( GL_SRC_ALPHA, GL_ONE );
+        m_currentManager->EndDrawing();
+        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+    }
 }
 
 
-bool OPENGL_GAL::SetNativeCursorStyle( KICURSOR aCursor )
+bool OPENGL_GAL::SetNativeCursorStyle( KICURSOR aCursor, bool aHiDPI )
 {
     // Store the current cursor type and get the wxCursor for it
-    if( !GAL::SetNativeCursorStyle( aCursor ) )
+    if( !GAL::SetNativeCursorStyle( aCursor, aHiDPI ) )
         return false;
 
-    m_currentwxCursor = CURSOR_STORE::GetCursor( m_currentNativeCursor );
+    if( aHiDPI )
+        m_currentwxCursor = CURSOR_STORE::GetHiDPICursor( m_currentNativeCursor );
+    else
+        m_currentwxCursor = CURSOR_STORE::GetCursor( m_currentNativeCursor );
 
     // Update the cursor in the wx control
     HIDPI_GL_CANVAS::SetCursor( m_currentwxCursor );
@@ -2257,18 +2409,17 @@ void OPENGL_GAL::drawSegmentChain( const std::function<VECTOR2D( int )>& aPointG
         auto start = aPointGetter( i - 1 );
         auto end = aPointGetter( i );
 
-        VECTOR2D startEndVector = start - end;
-        double   lineLength = startEndVector.EuclideanNorm();
-
         float startx = start.x;
-        float starty = end.y;
-        float endx = start.x + lineLength;
-        float endy = end.y + lineLength;
+        float starty = start.y;
+        float endx = end.x;
+        float endy = end.y;
 
         // Be careful about floating point rounding.  As we draw segments in larger and larger
         // coordinates, the shader (which uses floats) will lose precision and stop drawing small
         // segments.  In this case, we need to draw a circle for the minimal segment.
-        if( startx == endx || starty == endy )
+        // Check if the coordinate differences can be accurately represented as floats
+
+        if( startx == endx && starty == endy )
         {
             vertices += 3; // One circle
             continue;
@@ -2480,6 +2631,14 @@ void OPENGL_GAL::skipMouseEvent( wxMouseEvent& aEvent )
 }
 
 
+void OPENGL_GAL::skipGestureEvent( wxGestureEvent& aEvent )
+{
+    // Post the gesture event to the event listener registered in constructor, if any
+    if( m_mouseListener )
+        wxPostEvent( m_mouseListener, aEvent );
+}
+
+
 void OPENGL_GAL::blitCursor()
 {
     if( !IsCursorEnabled() )
@@ -2493,11 +2652,13 @@ void OPENGL_GAL::blitCursor()
     VECTOR2D cursorEnd = m_cursorPosition + cursorSize / ( 2 * m_worldScale );
     VECTOR2D cursorCenter = ( cursorBegin + cursorEnd ) / 2;
 
-    const COLOR4D cColor = getCursorColor();
-    const COLOR4D color( cColor.r * cColor.a, cColor.g * cColor.a, cColor.b * cColor.a, 1.0 );
+    const COLOR4D color = getCursorColor();
 
     glActiveTexture( GL_TEXTURE0 );
     glDisable( GL_TEXTURE_2D );
+    glEnable( GL_BLEND );
+    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+
     glLineWidth( 1.0 );
     glColor4d( color.r, color.g, color.b, color.a );
 
@@ -2525,24 +2686,33 @@ unsigned int OPENGL_GAL::getNewGroupNumber()
 
 void OPENGL_GAL::init()
 {
+#ifndef KICAD_USE_EGL
     wxASSERT( IsShownOnScreen() );
+#endif // KICAD_USE_EGL
 
     wxASSERT_MSG( m_isContextLocked, "This should only be called from within a locked context." );
 
-// IsDisplayAttr() handles WX_GL_{MAJOR,MINOR}_VERSION correctly only in 3.0.4
-// starting with 3.1.0 one should use wxGLContext::IsOk() (done by GL_CONTEXT_MANAGER)
-#if wxCHECK_VERSION( 3, 0, 3 ) and !wxCHECK_VERSION( 3, 1, 0 )
-    const int attr[] = { WX_GL_MAJOR_VERSION, 2, WX_GL_MINOR_VERSION, 1, 0 };
-
-    if( !IsDisplaySupported( attr ) )
-        throw std::runtime_error( "OpenGL 2.1 or higher is required!" );
-#endif /* wxCHECK_VERSION( 3, 0, 3 ) */
-
     // Check correct initialization from the constructor
     if( m_tesselator == nullptr )
         throw std::runtime_error( "Could not create the tesselator" );
     GLenum err = glewInit();
 
+#ifdef KICAD_USE_EGL
+    // TODO: better way to check when EGL is ready (init fails at "getString(GL_VERSION)")
+    for( int i = 0; i < 10; i++ )
+    {
+        if( GLEW_OK == err )
+            break;
+
+        std::this_thread::sleep_for( std::chrono::milliseconds( 250 ) );
+        err = glewInit();
+    }
+
+#endif // KICAD_USE_EGL
+
+    SetOpenGLInfo( (const char*) glGetString( GL_VENDOR ), (const char*) glGetString( GL_RENDERER ),
+                   (const char*) glGetString( GL_VERSION ) );
+
     if( GLEW_OK != err )
         throw std::runtime_error( (const char*) glewGetErrorString( err ) );
 
@@ -2661,6 +2831,7 @@ static void InitTesselatorCallbacks( GLUtesselator* aTesselator )
     gluTessCallback( aTesselator, GLU_TESS_ERROR, (void( CALLBACK* )()) ErrorCallback );
 }
 
+
 void OPENGL_GAL::EnableDepthTest( bool aEnabled )
 {
     m_cachedManager->EnableDepthTest( aEnabled );
@@ -2777,30 +2948,9 @@ void OPENGL_GAL::DrawGlyphs( const std::vector<std::unique_ptr<KIFONT::GLYPH>>&
     }
     else if( allGlyphsAreOutline )
     {
-        // Optimized path for stroke fonts that pre-reserves glyph triangles.
+        // Optimized path for outline fonts that pre-reserves glyph triangles.
         int triangleCount = 0;
 
-        if( aGlyphs.size() > 0 )
-        {
-            thread_pool& tp = GetKiCadThreadPool();
-
-            tp.push_loop( aGlyphs.size(),
-                    [&]( const int a, const int b)
-                    {
-                        for( int ii = a; ii < b; ++ii )
-                        {
-                            auto glyph = static_cast<KIFONT::OUTLINE_GLYPH*>( aGlyphs.at( ii ).get() );
-
-                            // Only call CacheTriangulation() if it has never been done before.
-                            // Otherwise we'll hash the triangulation to see if it has been edited,
-                            // and all our glpyh editing ops update the triangulation anyway.
-                            if( glyph->TriangulatedPolyCount() == 0 )
-                                glyph->CacheTriangulation( false );
-                        }
-                    } );
-            tp.wait_for_tasks();
-        }
-
         for( const std::unique_ptr<KIFONT::GLYPH>& glyph : aGlyphs )
         {
             const auto& outlineGlyph = static_cast<const KIFONT::OUTLINE_GLYPH&>( *glyph );
