@@ -2,7 +2,7 @@
  * This file is part of libeval, a simple math expression evaluator
  *
  * Copyright (C) 2017 Michael Geselbracht, mgeselbracht3@gmail.com
- * Copyright (C) 2019-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ namespace LIBEVAL
 
 TREE_NODE* newNode( LIBEVAL::COMPILER* compiler, int op, const T_TOKEN_VALUE& value )
 {
-    auto t2          = new TREE_NODE();
+    TREE_NODE* t2    = new TREE_NODE();
 
     t2->valid        = true;
     t2->value.str    = value.str ? new wxString( *value.str ) : nullptr;
@@ -88,16 +88,25 @@ static const wxString formatOpName( int op )
 {
     static const struct
     {
-        int         op;
+        int      op;
         wxString mnemonic;
     }
     simpleOps[] =
     {
-        { TR_OP_MUL, "MUL" }, { TR_OP_DIV, "DIV" }, { TR_OP_ADD, "ADD" },
-        { TR_OP_SUB, "SUB" }, { TR_OP_LESS, "LESS" }, { TR_OP_GREATER, "GREATER" },
-        { TR_OP_LESS_EQUAL, "LESS_EQUAL" }, { TR_OP_GREATER_EQUAL, "GREATER_EQUAL" },
-        { TR_OP_EQUAL, "EQUAL" }, { TR_OP_NOT_EQUAL, "NEQUAL" }, { TR_OP_BOOL_AND, "AND" },
-        { TR_OP_BOOL_OR, "OR" }, { TR_OP_BOOL_NOT, "NOT" }, { -1, "" }
+        { TR_OP_MUL, "MUL" },
+        { TR_OP_DIV, "DIV" },
+        { TR_OP_ADD, "ADD" },
+        { TR_OP_SUB, "SUB" },
+        { TR_OP_LESS, "LESS" },
+        { TR_OP_GREATER, "GREATER" },
+        { TR_OP_LESS_EQUAL, "LESS_EQUAL" },
+        { TR_OP_GREATER_EQUAL, "GREATER_EQUAL" },
+        { TR_OP_EQUAL, "EQUAL" },
+        { TR_OP_NOT_EQUAL, "NEQUAL" },
+        { TR_OP_BOOL_AND, "AND" },
+        { TR_OP_BOOL_OR, "OR" },
+        { TR_OP_BOOL_NOT, "NOT" },
+        { -1, "" }
     };
 
     for( int i = 0; simpleOps[i].op >= 0; i++ )
@@ -115,6 +124,9 @@ bool VALUE::EqualTo( CONTEXT* aCtx, const VALUE* b ) const
     if( m_type == VT_UNDEFINED || b->m_type == VT_UNDEFINED )
         return false;
 
+    if( m_type == VT_NULL && b->m_type == VT_NULL )
+        return true;
+
     if( m_type == VT_NUMERIC && b->m_type == VT_NUMERIC )
     {
         return AsDouble() == b->AsDouble();
@@ -151,14 +163,12 @@ wxString UOP::Format() const
         break;
 
     case TR_UOP_PUSH_VALUE:
-    {
         if( !m_value )
             str = wxString::Format( "PUSH nullptr" );
         else if( m_value->GetType() == VT_NUMERIC )
             str = wxString::Format( "PUSH NUM [%.10f]", m_value->AsDouble() );
         else
             str = wxString::Format( "PUSH STR [%ls]", m_value->AsString() );
-    }
         break;
 
     case TR_OP_METHOD_CALL:
@@ -180,10 +190,8 @@ wxString UOP::Format() const
 
 UCODE::~UCODE()
 {
-    for ( auto op : m_ucode )
-    {
+    for( UOP* op : m_ucode )
         delete op;
-    }
 }
 
 
@@ -191,7 +199,7 @@ wxString UCODE::Dump() const
 {
     wxString rv;
 
-    for( auto op : m_ucode )
+    for( UOP* op : m_ucode )
     {
         rv += op->Format();
         rv += "\n";
@@ -201,6 +209,24 @@ wxString UCODE::Dump() const
 };
 
 
+wxString TOKENIZER::GetString()
+{
+    wxString rv;
+
+    while( m_pos < m_str.length() && m_str[ m_pos ] != '\'' )
+    {
+        if( m_str[ m_pos ] == '\\' && m_pos + 1 < m_str.length() && m_str[ m_pos + 1 ] == '\'' )
+            m_pos++;
+
+        rv.append( 1, m_str[ m_pos++ ] );
+    }
+
+    m_pos++;
+
+    return rv;
+}
+
+
 wxString TOKENIZER::GetChars( const std::function<bool( wxUniChar )>& cond ) const
 {
     wxString rv;
@@ -215,6 +241,7 @@ wxString TOKENIZER::GetChars( const std::function<bool( wxUniChar )>& cond ) con
     return rv;
 }
 
+
 bool TOKENIZER::MatchAhead( const wxString& match,
                             const std::function<bool( wxUniChar )>& stopCond ) const
 {
@@ -231,7 +258,7 @@ bool TOKENIZER::MatchAhead( const wxString& match,
 
 
 COMPILER::COMPILER() :
-    m_lexerState( COMPILER::LS_DEFAULT )
+        m_lexerState( COMPILER::LS_DEFAULT )
 {
     m_localeDecimalSeparator = '.';
     m_sourcePos = 0;
@@ -273,10 +300,10 @@ void COMPILER::Clear()
 
     m_tree = nullptr;
 
-    for( auto tok : m_gcItems )
+    for( TREE_NODE* tok : m_gcItems )
         delete tok;
 
-    for( auto tok: m_gcStrings )
+    for( wxString* tok: m_gcStrings )
         delete tok;
 
     m_gcItems.clear();
@@ -371,6 +398,7 @@ T_TOKEN COMPILER::getToken()
         case LS_DEFAULT:
             done = lexDefault( rv );
             break;
+
         case LS_STRING:
             done = lexString( rv );
             break;
@@ -383,12 +411,9 @@ T_TOKEN COMPILER::getToken()
 
 bool COMPILER::lexString( T_TOKEN& aToken )
 {
-    wxString str = m_tokenizer.GetChars( []( int c ) -> bool { return c != '\''; } );
-
     aToken.token = G_STRING;
-    aToken.value.str = new wxString( str );
+    aToken.value.str = new wxString( m_tokenizer.GetString() );
 
-    m_tokenizer.NextChar( str.length() + 1 );
     m_lexerState = LS_DEFAULT;
     return true;
 }
@@ -400,7 +425,11 @@ int COMPILER::resolveUnits()
 
     for( const wxString& unitName : m_unitResolver->GetSupportedUnits() )
     {
-        if( m_tokenizer.MatchAhead( unitName, []( int c ) -> bool { return !isalnum( c ); } ) )
+        if( m_tokenizer.MatchAhead( unitName,
+                                    []( int c ) -> bool
+                                    {
+                                        return !isalnum( c );
+                                    } ) )
         {
             libeval_dbg(10, "Match unit '%s'\n", unitName.c_str() );
             m_tokenizer.NextChar( unitName.length() );
@@ -467,7 +496,6 @@ bool COMPILER::lexDefault( T_TOKEN& aToken )
                 }
             };
 
-
     int ch;
 
     // Start processing of first/next token: Remove whitespace
@@ -567,8 +595,7 @@ bool COMPILER::lexDefault( T_TOKEN& aToken )
         case ',': retval.token = G_COMMA;        break;
 
         default:
-            reportError( CST_PARSE, wxString::Format( _( "Unrecognized character '%c'" ),
-                                                      (char) ch ) );
+            reportError( CST_PARSE, wxString::Format( _( "Unrecognized character '%c'" ), (char) ch ) );
             break;
         }
 
@@ -582,7 +609,7 @@ bool COMPILER::lexDefault( T_TOKEN& aToken )
 
 const wxString formatNode( TREE_NODE* node )
 {
-    return node->value.str ? *(node->value.str) : "";
+    return node->value.str ? *(node->value.str) : wxString( wxEmptyString );
 }
 
 
@@ -646,7 +673,7 @@ void dumpNode( wxString& buf, TREE_NODE* tok, int depth = 0 )
 
      case TR_OP_FUNC_CALL:
         buf += "CALL '";
-        buf += *tok->leaf[0]->value.str;
+        buf += formatNode( tok->leaf[0] );
         buf += "': ";
         dumpNode( buf, tok->leaf[1], depth + 1 );
         break;
@@ -753,14 +780,13 @@ static void prepareTree( LIBEVAL::TREE_NODE *node )
         prepareTree( node->leaf[1] );
 }
 
+
 static std::vector<TREE_NODE*> squashParamList( TREE_NODE* root )
 {
     std::vector<TREE_NODE*> args;
 
     if( !root )
-    {
         return args;
-    }
 
     if( root->op != TR_ARG_LIST && root->op != TR_NULL )
     {
@@ -778,15 +804,13 @@ static std::vector<TREE_NODE*> squashParamList( TREE_NODE* root )
         } while ( n && n->op == TR_ARG_LIST );
 
         if( n )
-        {
             args.push_back( n );
-        }
     }
 
     std::reverse( args.begin(), args.end() );
 
     for( size_t i = 0; i < args.size(); i++ )
-        libeval_dbg( 10, "squash arg%d: %s\n", int( i ), *args[i]->value.str );
+        libeval_dbg( 10, "squash arg%d: %s\n", int( i ), formatNode( args[i] ) );
 
     return args;
 }
@@ -796,6 +820,9 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
 {
     std::vector<TREE_NODE*> stack;
     wxString                msg;
+    int                     numericValueCount = 0;
+    wxString                missingUnitsMsg;
+    int                     missingUnitsSrcPos = 0;
 
     if( !m_tree )
     {
@@ -818,8 +845,7 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
     {
         TREE_NODE* node = stack.back();
 
-        libeval_dbg( 4, "process node %p [op %d] [stack %lu]\n",
-                     node, node->op, (unsigned long)stack.size() );
+        libeval_dbg( 4, "process node %p [op %d] [stack %lu]\n", node, node->op, (unsigned long)stack.size() );
 
         // process terminal nodes first
         switch( node->op )
@@ -827,10 +853,7 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
         case TR_OP_FUNC_CALL:
             // Function call's uop was generated inside TR_STRUCT_REF
             if( !node->uop )
-            {
-                reportError( CST_CODEGEN,  _( "Unknown parent of function parameters" ),
-                             node->srcPos );
-            }
+                reportError( CST_CODEGEN,  _( "Unknown parent of function parameters" ), node->srcPos );
 
             node->isTerminal = true;
             break;
@@ -845,7 +868,7 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
                 int pos = node->leaf[0]->srcPos;
 
                 if( node->leaf[0]->value.str )
-                    pos -= static_cast<int>( node->leaf[0]->value.str->length() );
+                    pos -= static_cast<int>( formatNode( node->leaf[0] ).length() );
 
                 reportError( CST_CODEGEN,  _( "Unknown parent of property" ), pos );
 
@@ -864,21 +887,19 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
                     // leaf[0]: object
                     // leaf[1]: field
 
-                    wxString itemName = *node->leaf[0]->value.str;
-                    wxString propName = *node->leaf[1]->value.str;
+                    wxString itemName = formatNode( node->leaf[0] );
+                    wxString propName = formatNode( node->leaf[1] );
                     std::unique_ptr<VAR_REF> vref = aCode->CreateVarRef( itemName, propName );
 
                     if( !vref )
                     {
                         msg.Printf( _( "Unrecognized item '%s'" ), itemName );
-                        reportError( CST_CODEGEN, msg,
-                                     node->leaf[0]->srcPos - (int) itemName.length() );
+                        reportError( CST_CODEGEN, msg, node->leaf[0]->srcPos - (int) itemName.length() );
                     }
                     else if( vref->GetType() == VT_PARSE_ERROR )
                     {
                         msg.Printf( _( "Unrecognized property '%s'" ), propName );
-                        reportError( CST_CODEGEN, msg,
-                                     node->leaf[1]->srcPos - (int) propName.length() );
+                        reportError( CST_CODEGEN, msg, node->leaf[1]->srcPos - (int) propName.length() );
                     }
 
                     node->leaf[0]->isVisited = true;
@@ -895,17 +916,16 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
                     //    leaf[0]: function name
                     //    leaf[1]: parameter
 
-                    wxString                 itemName = *node->leaf[0]->value.str;
+                    wxString                 itemName = formatNode( node->leaf[0] );
                     std::unique_ptr<VAR_REF> vref = aCode->CreateVarRef( itemName, "" );
 
                     if( !vref )
                     {
                         msg.Printf( _( "Unrecognized item '%s'" ), itemName );
-                        reportError( CST_CODEGEN, msg,
-                                     node->leaf[0]->srcPos - (int) itemName.length() );
+                        reportError( CST_CODEGEN, msg, node->leaf[0]->srcPos - (int) itemName.length() );
                     }
 
-                    wxString functionName = *node->leaf[1]->leaf[0]->value.str;
+                    wxString functionName = formatNode( node->leaf[1]->leaf[0] );
                     auto  func = aCode->CreateFuncCall( functionName );
                     std::vector<TREE_NODE*> params = squashParamList( node->leaf[1]->leaf[1] );
 
@@ -924,7 +944,7 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
                         for( TREE_NODE* pnode : params )
                         {
                             VALUE*   param = aPreflightContext->AllocValue();
-                            param->Set( *pnode->value.str );
+                            param->Set( formatNode( pnode ) );
                             aPreflightContext->Push( param );
                         }
 
@@ -968,15 +988,14 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
                     // leaf[0]: object
                     // leaf[1]: malformed syntax
 
-                    wxString itemName = *node->leaf[0]->value.str;
-                    wxString propName = *node->leaf[1]->value.str;
+                    wxString itemName = formatNode( node->leaf[0] );
+                    wxString propName = formatNode( node->leaf[1] );
                     std::unique_ptr<VAR_REF> vref = aCode->CreateVarRef( itemName, propName );
 
                     if( !vref )
                     {
                         msg.Printf( _( "Unrecognized item '%s'" ), itemName );
-                        reportError( CST_CODEGEN, msg,
-                                     node->leaf[0]->srcPos - (int) itemName.length() );
+                        reportError( CST_CODEGEN, msg, node->leaf[0]->srcPos - (int) itemName.length() );
                     }
 
                     msg.Printf( _( "Unrecognized property '%s'" ), propName );
@@ -1006,35 +1025,36 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
             {
                 if( m_unitResolver->GetSupportedUnits().empty() )
                 {
-                    msg.Printf( _( "Unexpected units for '%s'" ), *node->value.str );
+                    msg.Printf( _( "Unexpected units for '%s'" ), formatNode( node ) );
                     reportError( CST_CODEGEN, msg, node->srcPos );
                 }
 
                 int units = son->value.idx;
-                value =  m_unitResolver->Convert( *node->value.str, units );
+                value =  m_unitResolver->Convert( formatNode( node ), units );
                 son->isVisited = true;
             }
             else
             {
                 if( !m_unitResolver->GetSupportedUnitsMessage().empty() )
                 {
-                    msg.Printf( _( "Missing units for '%s'| (%s)" ),
-                                *node->value.str,
-                                m_unitResolver->GetSupportedUnitsMessage() );
-                    reportError( CST_CODEGEN, msg, node->srcPos );
+                    missingUnitsMsg.Printf( _( "Missing units for '%s'| (%s)" ),
+                                            formatNode( node ),
+                                            m_unitResolver->GetSupportedUnitsMessage() );
+                    missingUnitsSrcPos = node->srcPos;
                 }
 
-                value = EDA_UNIT_UTILS::UI::DoubleValueFromString( *node->value.str );
+                value = EDA_UNIT_UTILS::UI::DoubleValueFromString( formatNode( node ) );
             }
 
             node->SetUop( TR_UOP_PUSH_VALUE, value );
             node->isTerminal = true;
+            numericValueCount++;
             break;
         }
 
         case TR_STRING:
         {
-            wxString str = *node->value.str;
+            wxString str = formatNode( node );
             bool isWildcard = str.Contains("?") || str.Contains("*");
             node->SetUop( TR_UOP_PUSH_VALUE, str, isWildcard );
             node->isTerminal = true;
@@ -1043,12 +1063,12 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
 
         case TR_IDENTIFIER:
         {
-            std::unique_ptr<VAR_REF> vref = aCode->CreateVarRef( *node->value.str, "" );
+            std::unique_ptr<VAR_REF> vref = aCode->CreateVarRef( formatNode( node ), "" );
 
             if( !vref )
             {
-                msg.Printf( _( "Unrecognized item '%s'" ), *node->value.str );
-                reportError( CST_CODEGEN, msg, node->srcPos - (int) node->value.str->length() );
+                msg.Printf( _( "Unrecognized item '%s'" ), formatNode( node ) );
+                reportError( CST_CODEGEN, msg, node->srcPos - (int) formatNode( node ).length() );
             }
 
             node->SetUop( TR_UOP_PUSH_VAR, std::move( vref ) );
@@ -1091,6 +1111,17 @@ bool COMPILER::generateUCode( UCODE* aCode, CONTEXT* aPreflightContext )
         stack.pop_back();
     }
 
+    // Report the common error condition of a single numeric value with no units (which will result in
+    // nanometers, and rarely leads to anything useful).
+    // Reporting missing units with multiple numeric values is far more complicated as we have to
+    // separate comparison operators from multiplicative operators from additive operators.  Consider:
+    //   2 * 1.5mm
+    //   2mm + 105um
+    //   2mm > 20um
+    //   (2mm + 1.5mm) * 3
+    if( !missingUnitsMsg.IsEmpty() && numericValueCount == 1 )
+        reportError( CST_CODEGEN, missingUnitsMsg, missingUnitsSrcPos );
+
     libeval_dbg(2,"dump: \n%s\n", aCode->Dump().c_str() );
 
     return true;
@@ -1111,8 +1142,8 @@ void UOP::Exec( CONTEXT* ctx )
             value = ctx->AllocValue();
 
         ctx->Push( value );
-    }
         break;
+    }
 
     case TR_UOP_PUSH_VALUE:
         ctx->Push( m_value.get() );
@@ -1126,12 +1157,12 @@ void UOP::Exec( CONTEXT* ctx )
         break;
     }
 
+#define AS_DOUBLE( arg ) ( arg ? arg->AsDouble() : 0.0 )
+
     if( m_op & TR_OP_BINARY_MASK )
     {
         LIBEVAL::VALUE* arg2 = ctx->Pop();
         LIBEVAL::VALUE* arg1 = ctx->Pop();
-        double          arg2Value = arg2 ? arg2->AsDouble() : 0.0;
-        double          arg1Value = arg1 ? arg1->AsDouble() : 0.0;
         double          result;
 
         if( ctx->HasErrorCallback() )
@@ -1153,29 +1184,36 @@ void UOP::Exec( CONTEXT* ctx )
         switch( m_op )
         {
         case TR_OP_ADD:
-            result = arg1Value + arg2Value;
+            result = AS_DOUBLE( arg1 ) + AS_DOUBLE( arg2 );
             break;
+
         case TR_OP_SUB:
-            result = arg1Value - arg2Value;
+            result = AS_DOUBLE( arg1 ) - AS_DOUBLE( arg2 );
             break;
+
         case TR_OP_MUL:
-            result = arg1Value * arg2Value;
+            result = AS_DOUBLE( arg1 ) * AS_DOUBLE( arg2 );
             break;
+
         case TR_OP_DIV:
-            result = arg1Value / arg2Value;
+            result = AS_DOUBLE( arg1 ) / AS_DOUBLE( arg2 );
             break;
+
         case TR_OP_LESS_EQUAL:
-            result = arg1Value <= arg2Value ? 1 : 0;
+            result = AS_DOUBLE( arg1 ) <= AS_DOUBLE( arg2 ) ? 1 : 0;
             break;
+
         case TR_OP_GREATER_EQUAL:
-            result = arg1Value >= arg2Value ? 1 : 0;
+            result = AS_DOUBLE( arg1 ) >= AS_DOUBLE( arg2 ) ? 1 : 0;
             break;
+
         case TR_OP_LESS:
-            result = arg1Value < arg2Value ? 1 : 0;
+            result = AS_DOUBLE( arg1 ) < AS_DOUBLE( arg2 ) ? 1 : 0;
             break;
         case TR_OP_GREATER:
-            result = arg1Value > arg2Value ? 1 : 0;
+            result = AS_DOUBLE( arg1 ) > AS_DOUBLE( arg2 ) ? 1 : 0;
             break;
+
         case TR_OP_EQUAL:
             if( !arg1 || !arg2 )
                 result = arg1 == arg2 ? 1 : 0;
@@ -1184,6 +1222,7 @@ void UOP::Exec( CONTEXT* ctx )
             else
                 result = arg1->EqualTo( ctx, arg2 ) ? 1 : 0;
             break;
+
         case TR_OP_NOT_EQUAL:
             if( !arg1 || !arg2 )
                 result = arg1 != arg2 ? 1 : 0;
@@ -1192,18 +1231,21 @@ void UOP::Exec( CONTEXT* ctx )
             else
                 result = arg1->NotEqualTo( ctx, arg2 ) ? 1 : 0;
             break;
+
         case TR_OP_BOOL_AND:
-            result = arg1Value != 0.0 && arg2Value != 0.0 ? 1 : 0;
+            result = AS_DOUBLE( arg1 ) != 0.0 && AS_DOUBLE( arg2 ) != 0.0 ? 1 : 0;
             break;
+
         case TR_OP_BOOL_OR:
-            result = arg1Value != 0.0 || arg2Value != 0.0 ? 1 : 0;
+            result = AS_DOUBLE( arg1 ) != 0.0 || AS_DOUBLE( arg2 ) != 0.0 ? 1 : 0;
             break;
+
         default:
             result = 0.0;
             break;
         }
 
-        auto rp = ctx->AllocValue();
+        VALUE* rp = ctx->AllocValue();
         rp->Set( result );
         ctx->Push( rp );
         return;
@@ -1211,20 +1253,20 @@ void UOP::Exec( CONTEXT* ctx )
     else if( m_op & TR_OP_UNARY_MASK )
     {
         LIBEVAL::VALUE* arg1 = ctx->Pop();
-        double          arg1Value = arg1 ? arg1->AsDouble() : 0.0;
+        double          ARG1VALUE = arg1 ? arg1->AsDouble() : 0.0;
         double          result;
 
         switch( m_op )
         {
         case TR_OP_BOOL_NOT:
-            result = arg1Value != 0.0 ? 0 : 1;
+            result = ARG1VALUE != 0.0 ? 0 : 1;
             break;
         default:
-            result = arg1Value != 0.0 ? 1 : 0;
+            result = ARG1VALUE != 0.0 ? 1 : 0;
             break;
         }
 
-        auto rp = ctx->AllocValue();
+        VALUE* rp = ctx->AllocValue();
         rp->Set( result );
         ctx->Push( rp );
         return;
@@ -1234,8 +1276,6 @@ void UOP::Exec( CONTEXT* ctx )
 
 VALUE* UCODE::Run( CONTEXT* ctx )
 {
-    static VALUE g_false( 0 );
-
     try
     {
         for( UOP* op : m_ucode )
@@ -1244,7 +1284,8 @@ VALUE* UCODE::Run( CONTEXT* ctx )
     catch(...)
     {
         // rules which fail outright should not be fired
-        return &g_false;
+        std::unique_ptr<VALUE> temp_false = std::make_unique<VALUE>( 0 );
+        return ctx->StoreValue( temp_false.get() );
     }
 
     if( ctx->SP() == 1 )
@@ -1260,7 +1301,8 @@ VALUE* UCODE::Run( CONTEXT* ctx )
         wxASSERT( ctx->SP() == 1 );
 
         // non-well-formed rules should not be fired on a release build
-        return &g_false;
+        std::unique_ptr<VALUE> temp_false = std::make_unique<VALUE>( 0 );
+        return ctx->StoreValue( temp_false.get() );
     }
 }
 
