@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020-2021 CERN
- * Copyright (C) 2021-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Wayne Stambaugh <stambaughw@gmail.com>
  *
@@ -23,6 +23,7 @@
 #include <algorithm>
 
 #include <bitmaps.h>
+#include <connection_graph.h>
 #include <string_utils.h>   // WildCompareString
 #include <kiway.h>
 #include <refdes_utils.h>
@@ -34,6 +35,8 @@
 #include <schematic.h>
 #include <template_fieldnames.h>
 #include <widgets/wx_html_report_panel.h>
+#include <widgets/std_bitmap_button.h>
+#include <sch_commit.h>
 
 bool g_selectRefDes = false;
 bool g_selectValue  = false;
@@ -45,6 +48,9 @@ bool g_resetFieldVisibilities[2] = { true,   false  };
 bool g_resetFieldEffects[2]      = { true,   false  };
 bool g_resetFieldPositions[2]    = { true,   false  };
 bool g_resetAttributes[2]        = { true,   false  };
+bool g_resetPinVisibilities[2]   = { true,   false  };
+bool g_resetCustomPower[2]       = { false,  false  };
+bool g_resetAlternatePins[2]     = { true,   false  };
 
 
 DIALOG_CHANGE_SYMBOLS::DIALOG_CHANGE_SYMBOLS( SCH_EDIT_FRAME* aParent, SCH_SYMBOL* aSymbol,
@@ -75,7 +81,8 @@ DIALOG_CHANGE_SYMBOLS::DIALOG_CHANGE_SYMBOLS( SCH_EDIT_FRAME* aParent, SCH_SYMBO
 
         m_newId->ChangeValue( UnescapeString( m_symbol->GetLibId().Format() ) );
         m_specifiedReference->ChangeValue( m_symbol->GetRef( currentSheet ) );
-        m_specifiedValue->ChangeValue( UnescapeString( m_symbol->GetField( VALUE_FIELD )->GetText() ) );
+        m_specifiedValue->ChangeValue(
+                UnescapeString( m_symbol->GetField( VALUE_FIELD )->GetText() ) );
         m_specifiedId->ChangeValue( UnescapeString( m_symbol->GetLibId().Format() ) );
     }
     else
@@ -83,8 +90,8 @@ DIALOG_CHANGE_SYMBOLS::DIALOG_CHANGE_SYMBOLS( SCH_EDIT_FRAME* aParent, SCH_SYMBO
         m_matchSizer->FindItem( m_matchBySelection )->Show( false );
     }
 
-    m_matchIdBrowserButton->SetBitmap( KiBitmap( BITMAPS::small_library ) );
-    m_newIdBrowserButton->SetBitmap( KiBitmap( BITMAPS::small_library ) );
+    m_matchIdBrowserButton->SetBitmap( KiBitmapBundle( BITMAPS::small_library ) );
+    m_newIdBrowserButton->SetBitmap( KiBitmapBundle( BITMAPS::small_library ) );
 
     if( m_mode == MODE::CHANGE )
     {
@@ -96,9 +103,9 @@ DIALOG_CHANGE_SYMBOLS::DIALOG_CHANGE_SYMBOLS( SCH_EDIT_FRAME* aParent, SCH_SYMBO
     m_matchSizer->SetEmptyCellSize( wxSize( 0, 0 ) );
     m_matchSizer->Layout();
 
-    for( int i = 0; i < MANDATORY_FIELDS; ++i )
+    for( int i = 0; i < MANDATORY_FIELD_COUNT; ++i )
     {
-        m_fieldsBox->Append( TEMPLATE_FIELDNAME::GetDefaultFieldName( i, DO_TRANSLATE ) );
+        m_fieldsBox->Append( GetDefaultFieldName( i, DO_TRANSLATE ) );
 
         if( i == REFERENCE_FIELD )
             m_fieldsBox->Check( i, g_selectRefDes );
@@ -144,6 +151,9 @@ DIALOG_CHANGE_SYMBOLS::DIALOG_CHANGE_SYMBOLS( SCH_EDIT_FRAME* aParent, SCH_SYMBO
     m_resetFieldEffects->SetValue( g_resetFieldEffects[ (int) m_mode ] );
     m_resetFieldPositions->SetValue( g_resetFieldPositions[ (int) m_mode ] );
     m_resetAttributes->SetValue( g_resetAttributes[ (int) m_mode ] );
+    m_resetPinTextVisibility->SetValue( g_resetPinVisibilities[ (int) m_mode ] );
+    m_resetCustomPower->SetValue( g_resetCustomPower[ (int) m_mode ] );
+    m_resetAlternatePin->SetValue( g_resetAlternatePins[ (int) m_mode ] );
 
     // DIALOG_SHIM needs a unique hash_key because classname is not sufficient
     // because the update and change versions of this dialog have different controls.
@@ -225,6 +235,9 @@ DIALOG_CHANGE_SYMBOLS::~DIALOG_CHANGE_SYMBOLS()
     g_resetFieldEffects[ (int) m_mode ] = m_resetFieldEffects->GetValue();
     g_resetFieldPositions[ (int) m_mode ] = m_resetFieldPositions->GetValue();
     g_resetAttributes[ (int) m_mode ] = m_resetAttributes->GetValue();
+    g_resetPinVisibilities[ (int) m_mode ] = m_resetPinTextVisibility->GetValue();
+    g_resetCustomPower[ (int) m_mode ] = m_resetCustomPower->GetValue();
+    g_resetAlternatePins[ (int) m_mode ] = m_resetAlternatePin->GetValue();
 }
 
 
@@ -242,15 +255,16 @@ void DIALOG_CHANGE_SYMBOLS::launchMatchIdSymbolBrowser( wxCommandEvent& aEvent )
 {
     wxString newName = getLibIdValue( m_specifiedId );
 
-    KIWAY_PLAYER* frame = Kiway().Player( FRAME_SCH_VIEWER_MODAL, true );
-
-    if( frame->ShowModal( &newName, this ) )
+    if( KIWAY_PLAYER* frame = Kiway().Player( FRAME_SYMBOL_CHOOSER, true, this ) )
     {
-        m_specifiedId->SetValue( UnescapeString( newName ) );
-        updateFieldsList();
-    }
+        if( frame->ShowModal( &newName, this ) )
+        {
+            m_specifiedId->SetValue( UnescapeString( newName ) );
+            updateFieldsList();
+        }
 
-    frame->Destroy();
+        frame->Destroy();
+    }
 }
 
 
@@ -258,15 +272,16 @@ void DIALOG_CHANGE_SYMBOLS::launchNewIdSymbolBrowser( wxCommandEvent& aEvent )
 {
     wxString newName = getLibIdValue( m_newId );
 
-    KIWAY_PLAYER* frame = Kiway().Player( FRAME_SCH_VIEWER_MODAL, true );
-
-    if( frame->ShowModal( &newName, this ) )
+    if( KIWAY_PLAYER* frame = Kiway().Player( FRAME_SYMBOL_CHOOSER, true, this ) )
     {
-        m_newId->SetValue( UnescapeString( newName ) );
-        updateFieldsList();
-    }
+        if( frame->ShowModal( &newName, this ) )
+        {
+            m_newId->SetValue( UnescapeString( newName ) );
+            updateFieldsList();
+        }
 
-    frame->Destroy();
+        frame->Destroy();
+    }
 }
 
 
@@ -276,14 +291,12 @@ void DIALOG_CHANGE_SYMBOLS::updateFieldsList()
 
     wxCHECK( frame, /* void */ );
 
-    SCH_SHEET_LIST  hierarchy = frame->Schematic().GetSheets();
-
     // Load non-mandatory fields from all matching symbols and their library symbols
     std::vector<SCH_FIELD*> fields;
-    std::vector<LIB_FIELD*> libFields;
+    std::vector<SCH_FIELD*> libFields;
     std::set<wxString>      fieldNames;
 
-    for( SCH_SHEET_PATH& instance : hierarchy )
+    for( SCH_SHEET_PATH& instance : frame->Schematic().Hierarchy() )
     {
         SCH_SCREEN* screen = instance.LastScreen();
 
@@ -301,8 +314,11 @@ void DIALOG_CHANGE_SYMBOLS::updateFieldsList()
             fields.clear();
             symbol->GetFields( fields, false );
 
-            for( unsigned i = MANDATORY_FIELDS; i < fields.size(); ++i )
-                fieldNames.insert( fields[i]->GetName() );
+            for( SCH_FIELD* field : fields )
+            {
+                if( !field->IsMandatory() && !field->IsPrivate() )
+                    fieldNames.insert( field->GetName() );
+            }
 
             if( m_mode == MODE::UPDATE && symbol->GetLibId().IsValid() )
             {
@@ -314,8 +330,11 @@ void DIALOG_CHANGE_SYMBOLS::updateFieldsList()
 
                     flattenedSymbol->GetFields( libFields );
 
-                    for( unsigned i = MANDATORY_FIELDS; i < libFields.size(); ++i )
-                        fieldNames.insert( libFields[i]->GetName() );
+                    for( SCH_FIELD* libField : libFields )
+                    {
+                        if( !libField->IsMandatory() && !libField->IsPrivate() )
+                            fieldNames.insert( libField->GetName() );
+                    }
 
                     libFields.clear();  // flattenedSymbol is about to go out of scope...
                 }
@@ -340,8 +359,11 @@ void DIALOG_CHANGE_SYMBOLS::updateFieldsList()
 
                 flattenedSymbol->GetFields( libFields );
 
-                for( unsigned i = MANDATORY_FIELDS; i < libFields.size(); ++i )
-                    fieldNames.insert( libFields[i]->GetName() );
+                for( SCH_FIELD* libField : libFields )
+                {
+                    if( !libField->IsMandatory() && !libField->IsPrivate() )
+                        fieldNames.insert( libField->GetName() );
+                }
 
                 libFields.clear();  // flattenedSymbol is about to go out of scope...
             }
@@ -349,31 +371,69 @@ void DIALOG_CHANGE_SYMBOLS::updateFieldsList()
     }
 
     // Update the listbox widget
-    for( unsigned i = m_fieldsBox->GetCount() - 1; i >= MANDATORY_FIELDS; --i )
-        m_fieldsBox->Delete( i );
+    wxArrayInt    checkedItems;
+    wxArrayString checkedNames;
+
+    m_fieldsBox->GetCheckedItems( checkedItems );
+
+    for( int ii : checkedItems )
+        checkedNames.push_back( m_fieldsBox->GetString( ii ) );
+
+    bool allChecked = true;
+
+    for( unsigned ii = 0; ii < m_fieldsBox->GetCount(); ++ii )
+    {
+        if( ii == REFERENCE_FIELD || ii == VALUE_FIELD )
+            continue;
+
+        if( !m_fieldsBox->IsChecked( ii ) )
+            allChecked = false;
+    }
+
+    for( unsigned ii = m_fieldsBox->GetCount() - 1; ii >= MANDATORY_FIELD_COUNT; --ii )
+        m_fieldsBox->Delete( ii );
 
     for( const wxString& fieldName : fieldNames )
+    {
         m_fieldsBox->Append( fieldName );
 
-    for( unsigned i = MANDATORY_FIELDS; i < m_fieldsBox->GetCount(); ++i )
-        m_fieldsBox->Check( i, true );
+        if( allChecked || alg::contains( checkedNames, fieldName ) )
+            m_fieldsBox->Check( m_fieldsBox->GetCount() - 1, true );
+    }
+}
+
+
+void DIALOG_CHANGE_SYMBOLS::selectAll( bool aSelect )
+{
+    for( unsigned i = 0; i < m_fieldsBox->GetCount(); ++i )
+        m_fieldsBox->Check( i, aSelect );
 }
 
 
 void DIALOG_CHANGE_SYMBOLS::checkAll( bool aCheck )
 {
-    for( unsigned i = 0; i < m_fieldsBox->GetCount(); ++i )
-        m_fieldsBox->Check( i, aCheck );
+    m_removeExtraBox->SetValue( aCheck );
+    m_resetEmptyFields->SetValue( aCheck );
+    m_resetFieldText->SetValue( aCheck );
+    m_resetFieldVisibilities->SetValue( aCheck );
+    m_resetFieldEffects->SetValue( aCheck );
+    m_resetFieldPositions->SetValue( aCheck );
+    m_resetPinTextVisibility->SetValue( aCheck );
+    m_resetAlternatePin->SetValue( aCheck );
+    m_resetAttributes->SetValue( aCheck );
+    m_resetCustomPower->SetValue( aCheck );
 }
 
 
 void DIALOG_CHANGE_SYMBOLS::onOkButtonClicked( wxCommandEvent& aEvent )
 {
-    wxBusyCursor dummy;
     SCH_EDIT_FRAME* parent = dynamic_cast<SCH_EDIT_FRAME*>( GetParent() );
 
     wxCHECK( parent, /* void */ );
 
+    wxBusyCursor    dummy;
+    SCH_COMMIT      commit( parent );
+
     m_messagePanel->Clear();
     m_messagePanel->Flush( false );
 
@@ -385,21 +445,15 @@ void DIALOG_CHANGE_SYMBOLS::onOkButtonClicked( wxCommandEvent& aEvent )
     {
         if( m_fieldsBox->IsChecked( i ) )
         {
-            if( i < MANDATORY_FIELDS )
-            {
-                LIB_FIELD dummy_field( i );
-                m_updateFields.insert( dummy_field.GetCanonicalName() );
-            }
+            if( i < MANDATORY_FIELD_COUNT )
+                m_updateFields.insert( GetCanonicalFieldName( i ) );
             else
                 m_updateFields.insert( m_fieldsBox->GetString( i ) );
         }
     }
 
-    if( processMatchingSymbols() )
-    {
-        parent->TestDanglingEnds();   // This will also redraw the changed symbols.
-        parent->OnModify();
-    }
+    if( processMatchingSymbols( &commit) )
+        commit.Push( m_mode == MODE::CHANGE ? _( "Change Symbols" ) : _( "Update Symbols" ) );
 
     m_messagePanel->Flush( false );
 }
@@ -407,14 +461,13 @@ void DIALOG_CHANGE_SYMBOLS::onOkButtonClicked( wxCommandEvent& aEvent )
 
 bool DIALOG_CHANGE_SYMBOLS::isMatch( SCH_SYMBOL* aSymbol, SCH_SHEET_PATH* aInstance )
 {
-    LIB_ID id;
-
-    wxCHECK( aSymbol, false );
-
     SCH_EDIT_FRAME* frame = dynamic_cast<SCH_EDIT_FRAME*>( GetParent() );
 
     wxCHECK( frame, false );
 
+    if( !aSymbol )
+        return false;
+
     if( m_matchAll->GetValue() )
     {
         return true;
@@ -437,6 +490,8 @@ bool DIALOG_CHANGE_SYMBOLS::isMatch( SCH_SYMBOL* aSymbol, SCH_SHEET_PATH* aInsta
     }
     else if( m_matchById )
     {
+        LIB_ID id;
+
         id.Parse( getLibIdValue( m_specifiedId ) );
         return aSymbol->GetLibId() == id;
     }
@@ -445,19 +500,16 @@ bool DIALOG_CHANGE_SYMBOLS::isMatch( SCH_SYMBOL* aSymbol, SCH_SHEET_PATH* aInsta
 }
 
 
-
-
-int DIALOG_CHANGE_SYMBOLS::processMatchingSymbols()
+int DIALOG_CHANGE_SYMBOLS::processMatchingSymbols( SCH_COMMIT* aCommit )
 {
     SCH_EDIT_FRAME* frame = dynamic_cast<SCH_EDIT_FRAME*>( GetParent() );
 
     wxCHECK( frame, false );
 
-    LIB_ID newId;
-    wxString msg;
-    int matchesProcessed = 0;
+    LIB_ID      newId;
+    wxString    msg;
+    int         matchesProcessed = 0;
     SCH_SYMBOL* symbol = nullptr;
-    SCH_SHEET_LIST hierarchy = frame->Schematic().GetSheets();
 
     if( m_mode == MODE::CHANGE )
     {
@@ -469,7 +521,7 @@ int DIALOG_CHANGE_SYMBOLS::processMatchingSymbols()
 
     std::map<SCH_SYMBOL*, SYMBOL_CHANGE_INFO> symbols;
 
-    for( SCH_SHEET_PATH& instance : hierarchy )
+    for( SCH_SHEET_PATH& instance : frame->Schematic().Hierarchy() )
     {
         SCH_SCREEN* screen = instance.LastScreen();
 
@@ -505,7 +557,11 @@ int DIALOG_CHANGE_SYMBOLS::processMatchingSymbols()
         }
     }
 
-    matchesProcessed += processSymbols( symbols );
+    if( symbols.size() > 0 )
+        matchesProcessed += processSymbols( aCommit, symbols );
+    else
+        m_messagePanel->Report( _( "*** No symbols matching criteria found ***" ),
+                                RPT_SEVERITY_ERROR );
 
     frame->GetCurrentSheet().UpdateAllScreenReferences();
 
@@ -513,8 +569,9 @@ int DIALOG_CHANGE_SYMBOLS::processMatchingSymbols()
 }
 
 
-int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
-                                                          SYMBOL_CHANGE_INFO>& aSymbols )
+int DIALOG_CHANGE_SYMBOLS::processSymbols( SCH_COMMIT* aCommit,
+                                           const std::map<SCH_SYMBOL*,
+                                           SYMBOL_CHANGE_INFO>& aSymbols )
 {
     wxCHECK( !aSymbols.empty(), 0 );
 
@@ -528,7 +585,7 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
     std::map<SCH_SYMBOL*, SYMBOL_CHANGE_INFO>::iterator it = symbols.begin();
 
     // Remove all symbols that don't have a valid library symbol link or enough units to
-    // satify the library symbol update.
+    // satisfy the library symbol update.
     while( it != symbols.end() )
     {
         SCH_SYMBOL* symbol = it->first;
@@ -561,8 +618,6 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
         }
     }
 
-    bool appendUndo = false;
-
     // Removing the symbol needs to be done before the LIB_SYMBOL is changed to prevent stale
     // library symbols in the schematic file.
     for( const auto& [ symbol, symbol_change_info ] : symbols )
@@ -574,12 +629,24 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
         wxCHECK( screen, 0 );
 
         screen->Remove( symbol );
-        frame->SaveCopyInUndoList( screen, symbol, UNDO_REDO::CHANGED, appendUndo );
-        appendUndo = true;
+        SCH_SYMBOL* symbol_copy = static_cast<SCH_SYMBOL*>( symbol->Clone() );
+        aCommit->Modified( symbol, symbol_copy, screen );
+
+        CONNECTION_GRAPH*           connectionGraph = screen->Schematic()->ConnectionGraph();
+
+        // When we replace the lib symbol below, we free the associated pins if the new symbol has
+        // fewer than the original.  This will cause the connection graph to be out of date unless
+        // we replace references in the graph to the old symbol/pins with references to the ones
+        // stored in the undo stack.
+        if( connectionGraph )
+            connectionGraph->ExchangeItem( symbol, symbol_copy );
     }
 
     for( const auto& [ symbol, symbol_change_info ] : symbols )
     {
+        // Remember initial link before changing for diags purpose
+        wxString initialLibLinkName = UnescapeString( symbol->GetLibId().Format() );
+
         if( symbol_change_info.m_LibId != symbol->GetLibId() )
             symbol->SetLibId( symbol_change_info.m_LibId );
 
@@ -593,8 +660,15 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
         {
             // Fetch the attributes from the *flattened* library symbol.  They are not supported
             // in derived symbols.
-            symbol->SetIncludeInBom( symbol->GetLibSymbolRef()->GetIncludeInBom() );
-            symbol->SetIncludeOnBoard( symbol->GetLibSymbolRef()->GetIncludeOnBoard() );
+            symbol->SetExcludedFromSim( symbol->GetLibSymbolRef()->GetExcludedFromSim() );
+            symbol->SetExcludedFromBOM( symbol->GetLibSymbolRef()->GetExcludedFromBOM() );
+            symbol->SetExcludedFromBoard( symbol->GetLibSymbolRef()->GetExcludedFromBoard() );
+        }
+
+        if( m_resetPinTextVisibility->GetValue() )
+        {
+            symbol->SetShowPinNames( symbol->GetLibSymbolRef()->GetShowPinNames() );
+            symbol->SetShowPinNumbers( symbol->GetLibSymbolRef()->GetShowPinNumbers() );
         }
 
         bool removeExtras = m_removeExtraBox->GetValue();
@@ -605,20 +679,25 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
         for( unsigned i = 0; i < symbol->GetFields().size(); ++i )
         {
             SCH_FIELD& field = symbol->GetFields()[i];
-            LIB_FIELD* libField = nullptr;
+            SCH_FIELD* libField = nullptr;
+            bool       doUpdate = field.IsPrivate();
 
             // Mandatory fields always exist in m_updateFields, but these names can be translated.
             // so use GetCanonicalName().
-            if( !alg::contains( m_updateFields, field.GetCanonicalName() ) )
+            doUpdate |= alg::contains( m_updateFields, field.GetCanonicalName() );
+
+            if( !doUpdate )
                 continue;
 
-            if( i < MANDATORY_FIELDS )
+            if( i < MANDATORY_FIELD_COUNT )
                 libField = symbol->GetLibSymbolRef()->GetFieldById( (int) i );
             else
                 libField = symbol->GetLibSymbolRef()->FindField( field.GetName() );
 
             if( libField )
             {
+                field.SetPrivate( libField->IsPrivate() );
+
                 bool resetText = libField->GetText().IsEmpty() ? m_resetEmptyFields->GetValue()
                                                                : m_resetFieldText->GetValue();
 
@@ -626,15 +705,26 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
                 {
                     if( i == REFERENCE_FIELD )
                     {
+                        wxString prefix = UTIL::GetRefDesPrefix( libField->GetText() );
+
                         for( const SCH_SHEET_PATH& instance : symbol_change_info.m_Instances )
                         {
-                            symbol->SetRef( &instance,
-                                            UTIL::GetRefDesUnannotated( libField->GetText() ) );
+                            wxString ref = symbol->GetRef( &instance );
+                            int      number = UTIL::GetRefDesNumber( ref );
+
+                            if( number >= 0 )
+                                ref.Printf( wxS( "%s%d" ), prefix, number );
+                            else
+                                ref = UTIL::GetRefDesUnannotated( prefix );
+
+                            symbol->SetRef( &instance, ref );
                         }
                     }
                     else if( i == VALUE_FIELD )
                     {
-                        symbol->SetValueFieldText( UnescapeString( libField->GetText() ) );
+                        if( ( symbol->IsPower() && m_resetCustomPower->IsChecked() )
+                            || !symbol->IsPower() )
+                            symbol->SetValueFieldText( UnescapeString( libField->GetText() ) );
                     }
                     else if( i == FOOTPRINT_FIELD )
                     {
@@ -666,45 +756,79 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
                 if( resetPositions )
                     field.SetTextPos( symbol->GetPosition() + libField->GetTextPos() );
             }
-            else if( i >= MANDATORY_FIELDS && removeExtras )
+            else if( i >= MANDATORY_FIELD_COUNT && removeExtras )
             {
                 symbol->RemoveField( field.GetName() );
                 i--;
             }
         }
 
-        std::vector<LIB_FIELD*> libFields;
+        std::vector<SCH_FIELD*> libFields;
         symbol->GetLibSymbolRef()->GetFields( libFields );
 
-        for( unsigned i = MANDATORY_FIELDS; i < libFields.size(); ++i )
+        for( SCH_FIELD* libField : libFields )
         {
-            const LIB_FIELD& libField = *libFields[i];
-
-            if( !alg::contains( m_updateFields, libField.GetCanonicalName() ) )
+            if( libField->IsMandatory() )
                 continue;
 
-            if( !symbol->FindField( libField.GetName(), false ) )
+            if( !alg::contains( m_updateFields, libField->GetCanonicalName() ) )
+                continue;
+
+            if( !symbol->FindField( libField->GetName(), false ) )
             {
-                wxString   fieldName = libField.GetCanonicalName();
-                SCH_FIELD  newField( VECTOR2I( 0, 0 ), symbol->GetFieldCount(), symbol,
-                                     fieldName );
+                SCH_FIELD  newField( VECTOR2I( 0, 0 ), symbol->GetNextFieldId(), symbol,
+                                     libField->GetCanonicalName() );
                 SCH_FIELD* schField = symbol->AddField( newField );
 
                 // Careful: the visible bit and position are also set by SetAttributes()
-                schField->SetAttributes( libField );
-                schField->SetText( libField.GetText() );
-                schField->SetTextPos( symbol->GetPosition() + libField.GetTextPos() );
+                schField->SetAttributes( *libField );
+                schField->SetVisible( libField->IsVisible() );
+                schField->SetText( libField->GetText() );
+                schField->SetTextPos( symbol->GetPosition() + libField->GetTextPos() );
+                schField->SetPrivate( libField->IsPrivate() );
             }
 
             if( resetPositions && frame->eeconfig()->m_AutoplaceFields.enable )
-                symbol->AutoAutoplaceFields( screen );
+            {
+                AUTOPLACE_ALGO fieldsAutoplaced = symbol->GetFieldsAutoplaced();
+
+                if( fieldsAutoplaced == AUTOPLACE_AUTO || fieldsAutoplaced == AUTOPLACE_MANUAL )
+                    symbol->AutoplaceFields( screen, fieldsAutoplaced );
+            }
         }
 
         symbol->SetSchSymbolLibraryName( wxEmptyString );
         screen->Append( symbol );
+
+        if( resetPositions )
+        {
+            AUTOPLACE_ALGO fieldsAutoplaced = symbol->GetFieldsAutoplaced();
+
+            if( fieldsAutoplaced == AUTOPLACE_AUTO || fieldsAutoplaced == AUTOPLACE_MANUAL )
+                symbol->AutoplaceFields( screen, fieldsAutoplaced );
+        }
+
+        // Clear alternate pins as required.
+        for( const SCH_SHEET_PATH& instance : symbol_change_info.m_Instances )
+        {
+            for( SCH_PIN* pin : symbol->GetPins( &instance ) )
+            {
+                if( !pin->GetAlt().IsEmpty() )
+                {
+                    // There is a bug that allows the alternate pin name to be set to the default pin
+                    // name which causes library symbol comparison errors.  Clear the alternate pin
+                    // name in this case even if the reset option is not checked.  Also clear the
+                    // alternate pin name if it no longer exists in the alternate pin map.
+                    if( m_resetAlternatePin->IsChecked() || ( pin->GetAlt() == pin->GetBaseName() )
+                        || ( pin->GetAlternates().count( pin->GetAlt() ) == 0 ) )
+                        pin->SetAlt( wxEmptyString );
+                }
+            }
+        }
+
         frame->GetCanvas()->GetView()->Update( symbol );
 
-        msg = getSymbolReferences( *symbol, symbol_change_info.m_LibId );
+        msg = getSymbolReferences( *symbol, symbol_change_info.m_LibId, &initialLibLinkName );
         msg += wxS( ": OK" );
         m_messagePanel->Report( msg, RPT_SEVERITY_ACTION );
         matchesProcessed +=1;
@@ -714,14 +838,33 @@ int DIALOG_CHANGE_SYMBOLS::processSymbols( const std::map<SCH_SYMBOL*,
 }
 
 
-wxString DIALOG_CHANGE_SYMBOLS::getSymbolReferences( SCH_SYMBOL& aSymbol, const LIB_ID& aNewId )
+wxString DIALOG_CHANGE_SYMBOLS::getSymbolReferences( SCH_SYMBOL& aSymbol,
+                                                     const LIB_ID& aNewId,
+                                                     const wxString* aOldLibLinkName )
 {
     wxString msg;
     wxString references;
-    LIB_ID oldId = aSymbol.GetLibId();
+    const LIB_ID& oldId = aSymbol.GetLibId();
 
-    for( const SCH_SYMBOL_INSTANCE& instance : aSymbol.GetInstanceReferences() )
+    wxString oldLibLinkName;    // For report
+
+    if( aOldLibLinkName )
+        oldLibLinkName = *aOldLibLinkName;
+    else
+        oldLibLinkName = UnescapeString( oldId.Format() );
+
+    SCH_EDIT_FRAME* parent = dynamic_cast< SCH_EDIT_FRAME* >( GetParent() );
+
+    wxCHECK( parent, msg );
+
+    SCH_SHEET_LIST sheets = parent->Schematic().Hierarchy();
+
+    for( const SCH_SYMBOL_INSTANCE& instance : aSymbol.GetInstances() )
     {
+        // Only include the symbol instances for the current project.
+        if( !sheets.HasPath( instance.m_Path ) )
+            continue;
+
         if( references.IsEmpty() )
             references = instance.m_Reference;
         else
@@ -730,35 +873,35 @@ wxString DIALOG_CHANGE_SYMBOLS::getSymbolReferences( SCH_SYMBOL& aSymbol, const
 
     if( m_mode == MODE::UPDATE )
     {
-        if( aSymbol.GetInstanceReferences().size() == 1 )
+        if( aSymbol.GetInstances().size() == 1 )
         {
             msg.Printf( _( "Update symbol %s from '%s' to '%s'" ),
                         references,
-                        UnescapeString( oldId.Format() ),
+                        oldLibLinkName,
                         UnescapeString( aNewId.Format() ) );
         }
         else
         {
             msg.Printf( _( "Update symbols %s from '%s' to '%s'" ),
                         references,
-                        UnescapeString( oldId.Format() ),
+                        oldLibLinkName,
                         UnescapeString( aNewId.Format() ) );
         }
     }
-    else
+    else    // mode is MODE::CHANGE
     {
-        if( aSymbol.GetInstanceReferences().size() == 1 )
+        if( aSymbol.GetInstances().size() == 1 )
         {
             msg.Printf( _( "Change symbol %s from '%s' to '%s'" ),
                         references,
-                        UnescapeString( oldId.Format() ),
+                        oldLibLinkName,
                         UnescapeString( aNewId.Format() ) );
         }
         else
         {
             msg.Printf( _( "Change symbols %s from '%s' to '%s'" ),
                         references,
-                        UnescapeString( oldId.Format() ),
+                        oldLibLinkName,
                         UnescapeString( aNewId.Format() ) );
         }
     }
