@@ -27,25 +27,21 @@
 #include <gestfich.h>
 #include <sch_screen.h>
 #include <sch_edit_frame.h>
-#include <schematic.h>
 #include <project.h>
 #include <kiface_base.h>
 #include <reporter.h>
 #include <wildcards_and_files_ext.h>
 #include <sch_marker.h>
 #include <connection_graph.h>
-#include <tools/ee_actions.h>
-#include <tools/ee_inspection_tool.h>
+#include <tools/sch_actions.h>
+#include <tools/sch_inspection_tool.h>
 #include <dialog_erc.h>
 #include <erc/erc.h>
 #include <erc/erc_report.h>
 #include <id.h>
 #include <confirm.h>
-#include <common.h>
 #include <widgets/wx_html_report_box.h>
 #include <dialogs/dialog_text_entry.h>
-#include <erc/erc_item.h>
-#include <eeschema_settings.h>
 #include <string_utils.h>
 #include <kiplatform/ui.h>
 
@@ -53,6 +49,7 @@
 #include <wx/filedlg.h>
 #include <wx/hyperlink.h>
 #include <wx/msgdlg.h>
+#include <wx/wupdlock.h>
 
 
 wxDEFINE_EVENT( EDA_EVT_CLOSE_ERC_DIALOG, wxCommandEvent );
@@ -292,7 +289,7 @@ void DIALOG_ERC::OnDeleteOneClick( wxCommandEvent& aEvent )
     if( m_notebook->GetSelection() == 0 )
     {
         // Clear the selection.  It may be the selected ERC marker.
-        m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
+        m_parent->GetToolManager()->RunAction( SCH_ACTIONS::clearSelection );
 
         m_markerTreeModel->DeleteCurrentItem( true );
 
@@ -420,7 +417,7 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
 
     m_runningResultsBook->ChangeSelection( 0 );   // Display the "Tests Running..." tab
     m_messages->Clear();
-    wxYield();                                    // Allow time slice to refresh Messages
+    wxSafeYield();                                // Allow time slice to refresh Messages
 
     m_running = true;
     m_sdbSizer1Cancel->SetLabel( _( "Cancel" ) );
@@ -447,8 +444,11 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
     testErc();
 
     if( itemsNotAnnotated )
+    {
         m_messages->ReportHead( wxString::Format( _( "%d symbol(s) require annotation.<br><br>" ),
-                                                  itemsNotAnnotated ), RPT_SEVERITY_INFO );
+                                                  itemsNotAnnotated ),
+                                RPT_SEVERITY_INFO );
+    }
 
     if( m_cancelled )
         // @spellingerror
@@ -457,7 +457,7 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
         m_messages->Report( _( "Done.<br><br>" ), RPT_SEVERITY_INFO );
 
     Raise();
-    wxYield();                                    // Allow time slice to refresh Messages
+    wxSafeYield();                                // Allow time slice to refresh Messages
 
     m_running = false;
     m_sdbSizer1Cancel->SetLabel( _( "Close" ) );
@@ -570,7 +570,7 @@ void DIALOG_ERC::OnERCItemSelected( wxDataViewEvent& aEvent )
         if( !sheet.empty() && sheet != m_parent->GetCurrentSheet() )
         {
             m_parent->GetToolManager()->RunAction( ACTIONS::cancelInteractive );
-            m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
+            m_parent->GetToolManager()->RunAction( SCH_ACTIONS::clearSelection );
 
             // Store the current zoom level into the current screen before switching
             m_parent->GetScreen()->m_LastZoomLevel = m_parent->GetCanvas()->GetView()->GetScale();
@@ -604,9 +604,9 @@ void DIALOG_ERC::OnERCItemDClick( wxDataViewEvent& aEvent )
 
 void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
 {
-    TOOL_MANAGER*       toolMgr = m_parent->GetToolManager();
-    EE_INSPECTION_TOOL* inspectionTool = toolMgr->GetTool<EE_INSPECTION_TOOL>();
-    RC_TREE_NODE*       node = RC_TREE_MODEL::ToNode( aEvent.GetItem() );
+    TOOL_MANAGER*        toolMgr = m_parent->GetToolManager();
+    SCH_INSPECTION_TOOL* inspectionTool = toolMgr->GetTool<SCH_INSPECTION_TOOL>();
+    RC_TREE_NODE*        node = RC_TREE_MODEL::ToNode( aEvent.GetItem() );
 
     if( !node )
         return;
@@ -836,7 +836,7 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
 
         // Clear the selection before deleting markers. It may be some selected ERC markers.
         // Deleting a selected marker without deselecting it first generates a crash
-        m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
+        m_parent->GetToolManager()->RunAction( SCH_ACTIONS::clearSelection );
 
         SCH_SCREENS ScreenList( m_parent->Schematic().Root() );
         ScreenList.DeleteMarkers( MARKER_BASE::MARKER_ERC, rcItem->GetErrorCode() );
@@ -938,6 +938,9 @@ void DIALOG_ERC::SelectMarker( const SCH_MARKER* aMarker )
 
 void DIALOG_ERC::centerMarkerIdleHandler( wxIdleEvent& aEvent )
 {
+    if( m_markerTreeModel->GetView()->IsFrozen() )
+        return;
+
     m_markerTreeModel->CenterMarker( m_centerMarkerOnIdle );
     m_centerMarkerOnIdle = nullptr;
     Unbind( wxEVT_IDLE, &DIALOG_ERC::centerMarkerIdleHandler, this );
@@ -1015,14 +1018,14 @@ void DIALOG_ERC::deleteAllMarkers( bool aIncludeExclusions )
     // Clear current selection list to avoid selection of deleted items
     // Freeze to avoid repainting the dialog, which can cause a RePaint()
     // of the screen as well
-    Freeze();
-    m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection );
+    wxWindowUpdateLocker updateLock( this );
+
+    m_parent->GetToolManager()->RunAction( SCH_ACTIONS::clearSelection );
 
     m_markerTreeModel->DeleteItems( false, aIncludeExclusions, false );
 
     SCH_SCREENS screens( m_parent->Schematic().Root() );
     screens.DeleteAllMarkers( MARKER_BASE::MARKER_ERC, aIncludeExclusions );
-    Thaw();
 }
 
 
