@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2012 Jean-Pierre Charras, jean-pierre.charras@gipsa-lab.inpg.com
  * Copyright (C) 2016 Wayne Stambaugh, stambaughw@gmail.com
- * Copyright (C) 2004-2022 KiCad Developers, see AITHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AITHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,22 +24,24 @@
  */
 
 #include <widgets/bitmap_button.h>
+#include <widgets/std_bitmap_button.h>
 #include <widgets/font_choice.h>
 #include <widgets/color_swatch.h>
 #include <settings/color_settings.h>
 #include <bitmaps.h>
 #include <kiway.h>
+#include <kiway_express.h>
 #include <confirm.h>
+#include <common.h>
 #include <string_utils.h>
-#include <sch_base_frame.h>
 #include <sch_edit_frame.h>
-#include <ee_collectors.h>
+#include <sch_collectors.h>
 #include <sch_symbol.h>
-#include <lib_field.h>
 #include <template_fieldnames.h>
 #include <symbol_library.h>
 #include <sch_validators.h>
 #include <schematic.h>
+#include <sch_commit.h>
 #include <dialog_field_properties.h>
 #include <sch_text.h>
 #include <scintilla_tricks.h>
@@ -47,68 +49,86 @@
 
 
 DIALOG_FIELD_PROPERTIES::DIALOG_FIELD_PROPERTIES( SCH_BASE_FRAME* aParent, const wxString& aTitle,
-                                                  const EDA_TEXT* aTextItem ) :
+                                                  const SCH_FIELD* aField ) :
     DIALOG_FIELD_PROPERTIES_BASE( aParent, wxID_ANY, aTitle ),
     m_posX( aParent, m_xPosLabel, m_xPosCtrl, m_xPosUnits, true ),
     m_posY( aParent, m_yPosLabel, m_yPosCtrl, m_yPosUnits, true ),
     m_textSize( aParent, m_textSizeLabel, m_textSizeCtrl, m_textSizeUnits, true ),
     m_font( nullptr ),
     m_firstFocus( true ),
-    m_scintillaTricks( nullptr )
+    m_scintillaTricks( nullptr ),
+    m_field( aField )
 {
     COLOR_SETTINGS* colorSettings = aParent->GetColorSettings();
     COLOR4D         schematicBackground = colorSettings->GetColor( LAYER_SCHEMATIC_BACKGROUND );
 
-    wxASSERT( aTextItem );
+    wxASSERT( m_field );
 
     m_note->SetFont( KIUI::GetInfoFont( this ).Italic() );
     m_note->Show( false );
 
-    // The field ID and power status are Initialized in the derived object's ctor.
-    m_fieldId = VALUE_FIELD;
-    m_isPower = false;
-
     m_scintillaTricks = new SCINTILLA_TRICKS( m_StyledTextCtrl, wxT( "{}" ), true,
-            [this]()
+            // onAcceptFn
+            [this]( wxKeyEvent& aEvent )
             {
                 wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
             } );
+
     m_StyledTextCtrl->SetEOLMode( wxSTC_EOL_LF );   // Normalize EOL across platforms
 
+#ifdef __WXGTK__
+    m_StyledTextCtrl->SetExtraAscent( 6 );
+    m_StyledTextCtrl->SetExtraDescent( 6 );
+#elif defined ( __WXMSW__ )
+    // Do nothing: SetExtraAscent() + SetExtraDescent(), when set to a value not 0
+    // Generate a strange bug: the text is not always shown (Perhaps a wxMSW bug)
+    // and this call is not needed on WXMSW
+#else
+    m_StyledTextCtrl->SetExtraAscent( 1 );
+    m_StyledTextCtrl->SetExtraDescent( 2 );
+#endif
+
+#ifdef _WIN32
+    // Without this setting, on Windows, some esoteric unicode chars create display issue
+    // in a wxStyledTextCtrl.
+    // for SetTechnology() info, see https://www.scintilla.org/ScintillaDoc.html#SCI_SETTECHNOLOGY
+    m_StyledTextCtrl->SetTechnology(wxSTC_TECHNOLOGY_DIRECTWRITE);
+#endif
+
     m_textColorSwatch->SetDefaultColor( COLOR4D::UNSPECIFIED );
     m_textColorSwatch->SetSwatchBackground( schematicBackground );
 
     m_separator1->SetIsSeparator();
 
     m_horizontal->SetIsRadioButton();
-    m_horizontal->SetBitmap( KiBitmap( BITMAPS::text_horizontal ) );
+    m_horizontal->SetBitmap( KiBitmapBundle( BITMAPS::text_horizontal ) );
     m_vertical->SetIsRadioButton();
-    m_vertical->SetBitmap( KiBitmap( BITMAPS::text_vertical ) );
+    m_vertical->SetBitmap( KiBitmapBundle( BITMAPS::text_vertical ) );
 
     m_separator2->SetIsSeparator();
 
     m_bold->SetIsCheckButton();
-    m_bold->SetBitmap( KiBitmap( BITMAPS::text_bold ) );
+    m_bold->SetBitmap( KiBitmapBundle( BITMAPS::text_bold ) );
     m_italic->SetIsCheckButton();
-    m_italic->SetBitmap( KiBitmap( BITMAPS::text_italic ) );
+    m_italic->SetBitmap( KiBitmapBundle( BITMAPS::text_italic ) );
 
     m_separator3->SetIsSeparator();
 
     m_hAlignLeft->SetIsRadioButton();
-    m_hAlignLeft->SetBitmap( KiBitmap( BITMAPS::text_align_left ) );
+    m_hAlignLeft->SetBitmap( KiBitmapBundle( BITMAPS::text_align_left ) );
     m_hAlignCenter->SetIsRadioButton();
-    m_hAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_align_center ) );
+    m_hAlignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_align_center ) );
     m_hAlignRight->SetIsRadioButton();
-    m_hAlignRight->SetBitmap( KiBitmap( BITMAPS::text_align_right ) );
+    m_hAlignRight->SetBitmap( KiBitmapBundle( BITMAPS::text_align_right ) );
 
     m_separator4->SetIsSeparator();
 
     m_vAlignTop->SetIsRadioButton();
-    m_vAlignTop->SetBitmap( KiBitmap( BITMAPS::text_valign_top ) );
+    m_vAlignTop->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_top ) );
     m_vAlignCenter->SetIsRadioButton();
-    m_vAlignCenter->SetBitmap( KiBitmap( BITMAPS::text_valign_center ) );
+    m_vAlignCenter->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_center ) );
     m_vAlignBottom->SetIsRadioButton();
-    m_vAlignBottom->SetBitmap( KiBitmap( BITMAPS::text_valign_bottom ) );
+    m_vAlignBottom->SetBitmap( KiBitmapBundle( BITMAPS::text_valign_bottom ) );
 
     m_separator5->SetIsSeparator();
 
@@ -123,20 +143,152 @@ DIALOG_FIELD_PROPERTIES::DIALOG_FIELD_PROPERTIES( SCH_BASE_FRAME* aParent, const
     m_vAlignCenter->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onVAlignButton, this );
     m_vAlignBottom->Bind( wxEVT_BUTTON, &DIALOG_FIELD_PROPERTIES::onVAlignButton, this );
 
-    m_text = aTextItem->GetText();
-    m_isItalic = aTextItem->IsItalic();
-    m_isBold = aTextItem->IsBold();
-    m_color = aTextItem->GetTextColor();
-    m_position = aTextItem->GetTextPos();
-    m_size = aTextItem->GetTextWidth();
-    m_isVertical = aTextItem->GetTextAngle().IsVertical();
-    m_verticalJustification = aTextItem->GetVertJustify();
-    m_horizontalJustification = aTextItem->GetHorizJustify();
-    m_isVisible = aTextItem->IsVisible();
+    // show text variable cross-references in a human-readable format
+    if( aField->Schematic() )
+        m_text = aField->Schematic()->ConvertKIIDsToRefs( aField->GetText() );
+    else
+        m_text = aField->GetText();
 
-    // These should be initialized in the child classes implementing dialogs for lib and sch items.
-    m_isNameVisible  = false;
-    m_allowAutoplace = true;
+    m_font = m_field->GetFont();
+    m_isItalic = aField->IsItalic();
+    m_isBold = aField->IsBold();
+    m_color = aField->GetTextColor();
+    m_position = aField->GetTextPos();
+    m_size = aField->GetTextWidth();
+    m_isVertical = aField->GetTextAngle().IsVertical();
+    m_verticalJustification = aField->GetVertJustify();
+    m_horizontalJustification = aField->GetHorizJustify();
+    m_isVisible = aField->IsVisible();
+
+    m_isSheetFilename = false;
+
+    if( aField->GetParent() && aField->GetParent()->Type() == LIB_SYMBOL_T )
+    {
+        const LIB_SYMBOL* symbol = static_cast<const LIB_SYMBOL*>( aField->GetParentSymbol() );
+
+        m_fieldId = aField->GetId();
+
+        /*
+         * Symbol netlist format:
+         *   pinNumber pinName <tab> pinNumber pinName...
+         *   fpFilter fpFilter...
+         */
+        wxString      netlist;
+        wxArrayString pins;
+
+        for( SCH_PIN* pin : symbol->GetPins( 0 /* all units */, 1 /* single bodyStyle */ ) )
+            pins.push_back( pin->GetNumber() + ' ' + pin->GetShownName() );
+
+        if( !pins.IsEmpty() )
+            netlist << EscapeString( wxJoin( pins, '\t' ), CTX_LINE );
+
+        netlist << wxS( "\r" );
+
+        wxArrayString fpFilters = symbol->GetFPFilters();
+
+        if( !fpFilters.IsEmpty() )
+            netlist << EscapeString( wxJoin( fpFilters, ' ' ), CTX_LINE );
+
+        netlist << wxS( "\r" );
+
+        m_netlist = netlist;
+    }
+    else if( aField->GetParent() && aField->GetParent()->Type() == SCH_SYMBOL_T )
+    {
+        const SCH_SYMBOL* symbol = static_cast<const SCH_SYMBOL*>( aField->GetParentSymbol() );
+        SCH_SHEET_PATH    sheetPath = static_cast<SCH_EDIT_FRAME*>( aParent )->GetCurrentSheet();
+
+        m_fieldId = aField->GetId();
+
+        /*
+         * Symbol netlist format:
+         *   pinNumber pinName <tab> pinNumber pinName...
+         *   fpFilter fpFilter...
+         */
+        wxString netlist;
+
+        // We need the list of pins of the lib symbol, not just the pins of the current
+        // sch symbol, that can be just an unit of a multi-unit symbol, to be able to
+        // select/filter right footprints
+        wxArrayString pins;
+
+        const std::unique_ptr< LIB_SYMBOL >& lib_symbol = symbol->GetLibSymbolRef();
+
+        if( lib_symbol )
+        {
+            for( SCH_PIN* pin : lib_symbol->GetPins( 0 /* all units */, 1 /* single bodyStyle */ ) )
+                pins.push_back( pin->GetNumber() + ' ' + pin->GetShownName() );
+        }
+
+        if( !pins.IsEmpty() )
+            netlist << EscapeString( wxJoin( pins, '\t' ), CTX_LINE );
+
+        netlist << wxS( "\r" );
+
+        wxArrayString fpFilters;
+
+        if( symbol->GetLibSymbolRef() )     // can be null with very old schematic
+            fpFilters = symbol->GetLibSymbolRef()->GetFPFilters();
+
+        if( !fpFilters.IsEmpty() )
+            netlist << EscapeString( wxJoin( fpFilters, ' ' ), CTX_LINE );
+
+        netlist << wxS( "\r" );
+
+        m_netlist = netlist;
+    }
+    else if( aField->GetParent() && aField->GetParent()->Type() == SCH_SHEET_T )
+    {
+        switch( aField->GetId() )
+        {
+        case SHEETNAME:
+            m_fieldId = SHEETNAME_V;
+            break;
+
+        case SHEETFILENAME:
+            m_isSheetFilename = true;
+            m_fieldId = SHEETFILENAME_V;
+            m_note->SetLabel( wxString::Format( m_note->GetLabel(),
+                              _( "Sheet filename can only be modified in Sheet Properties "
+                                 "dialog." ) ) );
+            m_note->Show( true );
+            break;
+
+        default:
+            m_fieldId = SHEETUSERFIELD_V;
+            break;
+        }
+    }
+    else if( aField->GetParent() && aField->GetParent()->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
+    {
+        m_fieldId = LABELUSERFIELD_V;
+    }
+
+    m_textLabel->SetLabel( aField->GetName() + wxS( ":" ) );
+
+    m_position = m_field->GetPosition();
+
+    m_isNameVisible = m_field->IsNameShown();
+    m_allowAutoplace = m_field->CanAutoplace();
+
+    m_horizontalJustification = m_field->GetEffectiveHorizJustify();
+    m_verticalJustification = m_field->GetEffectiveVertJustify();
+
+    m_StyledTextCtrl->Bind( wxEVT_STC_CHARADDED,
+                            &DIALOG_FIELD_PROPERTIES::onScintillaCharAdded, this );
+    m_StyledTextCtrl->Bind( wxEVT_STC_AUTOCOMP_CHAR_DELETED,
+                            &DIALOG_FIELD_PROPERTIES::onScintillaCharAdded, this );
+
+    m_nameVisible->Show();
+    m_cbAllowAutoPlace->Show();
+
+    init();
+
+    if( m_isSheetFilename || m_field->IsGeneratedField() )
+    {
+        m_StyledTextCtrl->Enable( false );
+        m_TextCtrl->Enable( false );
+    }
 }
 
 
@@ -148,29 +300,23 @@ DIALOG_FIELD_PROPERTIES::~DIALOG_FIELD_PROPERTIES()
 
 void DIALOG_FIELD_PROPERTIES::init()
 {
-    SCH_BASE_FRAME* parent = GetParent();
-    bool isSymbolEditor = parent && parent->IsType( FRAME_SCH_SYMBOL_EDITOR );
-
     // Disable options for graphic text editing which are not needed for fields.
-    m_CommonConvert->Show( false );
-    m_CommonUnit->Show( false );
+    m_commonToAllBodyStyles->Show( false );
+    m_commonToAllUnits->Show( false );
 
-    // Predefined fields cannot contain some chars, or cannot be empty,
-    // and need a SCH_FIELD_VALIDATOR (m_StyledTextCtrl cannot use a SCH_FIELD_VALIDATOR).
-    // VALUE_FIELD does not use the validator so that it uses the Scintilla text control, since we
-    // allow text replacement for this field (but not the other predefined fields)
-    bool use_validator = m_fieldId == REFERENCE_FIELD
-                         || m_fieldId == FOOTPRINT_FIELD
-                         || m_fieldId == DATASHEET_FIELD
-                         || m_fieldId == SHEETNAME_V
-                         || m_fieldId == SHEETFILENAME_V;
-
-    if( use_validator )
+    // Predefined fields cannot contain some chars and cannot be empty, so they need a
+    // SCH_FIELD_VALIDATOR (m_StyledTextCtrl cannot use a SCH_FIELD_VALIDATOR).
+    if( m_fieldId == REFERENCE_FIELD
+            || m_fieldId == FOOTPRINT_FIELD
+            || m_fieldId == DATASHEET_FIELD
+            || m_fieldId == SHEETNAME_V
+            || m_fieldId == SHEETFILENAME_V )
     {
-        m_TextCtrl->SetValidator( SCH_FIELD_VALIDATOR( isSymbolEditor, m_fieldId, &m_text ) );
+        m_TextCtrl->SetValidator( FIELD_VALIDATOR( m_fieldId, &m_text ) );
         SetInitialFocus( m_TextCtrl );
 
         m_StyledTextCtrl->Show( false );
+        m_StyledTextCtrlBorder->Show( false );
     }
     else
     {
@@ -179,36 +325,30 @@ void DIALOG_FIELD_PROPERTIES::init()
         m_TextCtrl->Show( false );
     }
 
+    // Show the unit selector for reference fields on multi-unit symbols
+    bool showUnitSelector = m_fieldId == REFERENCE_FIELD
+                            && m_field->GetParentSymbol()
+                            && m_field->GetParentSymbol()->Type() == SCH_SYMBOL_T
+                            && m_field->GetParentSymbol()->IsMulti();
+
+    m_unitLabel->Show( showUnitSelector );
+    m_unitChoice->Show( showUnitSelector );
+
     // Show the footprint selection dialog if this is the footprint field.
-    m_TextValueSelectButton->SetBitmap( KiBitmap( BITMAPS::small_library ) );
+    m_TextValueSelectButton->SetBitmap( KiBitmapBundle( BITMAPS::small_library ) );
     m_TextValueSelectButton->Show( m_fieldId == FOOTPRINT_FIELD );
 
-    // Value fields of power symbols cannot be modified. This will grey out
-    // the text box and display an explanation.
-    if( m_fieldId == VALUE_FIELD && m_isPower )
-    {
-        m_note->SetLabel( wxString::Format( m_note->GetLabel(), _( "Power symbol value field text "
-                                                                   "cannot be changed." ) ) );
-        m_note->Show( true );
-        m_TextCtrl->Enable( false );
-        m_StyledTextCtrl->Enable( false );
-    }
-    else
-    {
-        m_TextCtrl->Enable( true );
-        m_StyledTextCtrl->Enable( true );
-    }
+    m_TextCtrl->Enable( true );
 
     GetSizer()->SetSizeHints( this );
 
-    // Adjust the height of the scintilla text editor after the first layout
-    // To show only one line
-    // (multiline text are is supported in fields and will be removed)
+    // Adjust the height of the scintilla editor after the first layout to show a single line
+    // (multiline text is not supported in fields and will be removed)
     if( m_StyledTextCtrl->IsShown() )
     {
-        wxSize maxSize = m_StyledTextCtrl->GetSize();
+        wxSize maxSize;
         maxSize.x = -1;     // Do not fix the max width
-        maxSize.y = m_xPosCtrl->GetSize().y;
+        maxSize.y = m_StyledTextCtrl->TextHeight( 0 );
         m_StyledTextCtrl->SetMaxSize( maxSize );
         m_StyledTextCtrl->SetUseVerticalScrollBar( false );
         m_StyledTextCtrl->SetUseHorizontalScrollBar( false );
@@ -231,8 +371,11 @@ void DIALOG_FIELD_PROPERTIES::OnTextValueSelectButtonClick( wxCommandEvent& aEve
     else
         fpid = m_TextCtrl->GetValue();
 
-    if( KIWAY_PLAYER* frame = Kiway().Player( FRAME_FOOTPRINT_VIEWER_MODAL, true ) )
+    if( KIWAY_PLAYER* frame = Kiway().Player( FRAME_FOOTPRINT_CHOOSER, true, this ) )
     {
+        KIWAY_EXPRESS event( FRAME_FOOTPRINT_CHOOSER, MAIL_SYMBOL_NETLIST, m_netlist );
+        frame->KiwayMailIn( event );
+
         if( frame->ShowModal( &fpid, this ) )
         {
             if( m_StyledTextCtrl->IsShown() )
@@ -307,14 +450,30 @@ bool DIALOG_FIELD_PROPERTIES::TransferDataToWindow()
 {
     if( m_TextCtrl->IsShown() )
     {
-        m_TextCtrl->SetValue( m_text );
+        m_TextCtrl->SetValue( EscapeString( m_text, CTX_LINE ) );
     }
     else if( m_StyledTextCtrl->IsShown() )
     {
-        m_StyledTextCtrl->SetValue( m_text );
+        m_StyledTextCtrl->SetValue( EscapeString( m_text, CTX_LINE ) );
         m_StyledTextCtrl->EmptyUndoBuffer();
     }
 
+    if( m_unitChoice->IsShown() )
+    {
+        const SCH_SYMBOL* symbol = static_cast<const SCH_SYMBOL*>( m_field->GetParentSymbol() );
+
+        for( int ii = 1; ii <= symbol->GetUnitCount(); ii++ )
+        {
+            if( symbol->HasUnitDisplayName( ii ) )
+                m_unitChoice->Append( symbol->GetUnitDisplayName( ii ) );
+            else
+                m_unitChoice->Append( symbol->SubReference( ii, false ) );
+        }
+
+        if( symbol->GetUnit() <= ( int )m_unitChoice->GetCount() )
+            m_unitChoice->SetSelection( symbol->GetUnit() - 1 );
+    }
+
     m_fontCtrl->SetFontSelection( m_font );
 
     m_posX.SetValue( m_position.x );
@@ -331,16 +490,18 @@ bool DIALOG_FIELD_PROPERTIES::TransferDataToWindow()
 
     switch ( m_horizontalJustification )
     {
-    case GR_TEXT_H_ALIGN_LEFT:   m_hAlignLeft->Check( true );   break;
-    case GR_TEXT_H_ALIGN_CENTER: m_hAlignCenter->Check( true ); break;
-    case GR_TEXT_H_ALIGN_RIGHT:  m_hAlignRight->Check( true );  break;
+    case GR_TEXT_H_ALIGN_LEFT:          m_hAlignLeft->Check( true );   break;
+    case GR_TEXT_H_ALIGN_CENTER:        m_hAlignCenter->Check( true ); break;
+    case GR_TEXT_H_ALIGN_RIGHT:         m_hAlignRight->Check( true );  break;
+    case GR_TEXT_H_ALIGN_INDETERMINATE:                                break;
     }
 
     switch ( m_verticalJustification )
     {
-    case GR_TEXT_V_ALIGN_TOP:    m_vAlignTop->Check( true );    break;
-    case GR_TEXT_V_ALIGN_CENTER: m_vAlignCenter->Check( true ); break;
-    case GR_TEXT_V_ALIGN_BOTTOM: m_vAlignBottom->Check( true ); break;
+    case GR_TEXT_V_ALIGN_TOP:           m_vAlignTop->Check( true );    break;
+    case GR_TEXT_V_ALIGN_CENTER:        m_vAlignCenter->Check( true ); break;
+    case GR_TEXT_V_ALIGN_BOTTOM:        m_vAlignBottom->Check( true ); break;
+    case GR_TEXT_V_ALIGN_INDETERMINATE:                                break;
     }
 
     m_visible->SetValue( m_isVisible );
@@ -354,34 +515,15 @@ bool DIALOG_FIELD_PROPERTIES::TransferDataToWindow()
 bool DIALOG_FIELD_PROPERTIES::TransferDataFromWindow()
 {
     if( m_TextCtrl->IsShown() )
-        m_text = m_TextCtrl->GetValue();
+        m_text = UnescapeString( m_TextCtrl->GetValue() );
     else if( m_StyledTextCtrl->IsShown() )
-        m_text = m_StyledTextCtrl->GetValue();
+        m_text = UnescapeString( m_StyledTextCtrl->GetValue() );
 
-    if( m_fieldId == REFERENCE_FIELD )
-    {
-        // Test if the reference string is valid:
-        if( !SCH_SYMBOL::IsReferenceStringValid( m_text ) )
-        {
-            DisplayError( this, _( "Illegal reference designator value!" ) );
-            return false;
-        }
-    }
-    else if( m_fieldId == SHEETFILENAME_V )
-    {
-        wxFileName fn( m_text );
+    if( m_fieldId == SHEETFILENAME_V )
+        m_text = EnsureFileExtension( m_text, FILEEXT::KiCadSchematicFileExtension );
 
-        // It's annoying to throw up nag dialogs when the extension isn't right.  Just
-        // fix it.
-        if( fn.GetExt().CmpNoCase( KiCadSchematicFileExtension ) != 0 )
-        {
-            fn.SetExt( KiCadSchematicFileExtension );
-            m_text = fn.GetFullPath();
-        }
-    }
-
-    m_position = wxPoint( m_posX.GetValue(), m_posY.GetValue() );
-    m_size = m_textSize.GetValue();
+    m_position = VECTOR2I( m_posX.GetIntValue(), m_posY.GetIntValue() );
+    m_size = m_textSize.GetIntValue();
 
     if( m_fontCtrl->HaveFontSelection() )
         m_font = m_fontCtrl->GetFontSelection( m_bold->IsChecked(), m_italic->IsChecked() );
@@ -414,41 +556,21 @@ bool DIALOG_FIELD_PROPERTIES::TransferDataFromWindow()
 }
 
 
-void DIALOG_FIELD_PROPERTIES::updateText( EDA_TEXT* aText )
+void DIALOG_FIELD_PROPERTIES::updateText( SCH_FIELD* aField )
 {
-    if( aText->GetTextWidth() != m_size )
-        aText->SetTextSize( wxSize( m_size, m_size ) );
+    if( aField->GetTextWidth() != m_size )
+        aField->SetTextSize( VECTOR2I( m_size, m_size ) );
 
-    aText->SetVisible( m_isVisible );
-    aText->SetTextAngle( m_isVertical ? ANGLE_VERTICAL : ANGLE_HORIZONTAL );
-    aText->SetItalic( m_isItalic );
-    aText->SetBold( m_isBold );
-    aText->SetTextColor( m_color );
+    aField->SetFont( m_font );
+    aField->SetVisible( m_isVisible );
+    aField->SetTextAngle( m_isVertical ? ANGLE_VERTICAL : ANGLE_HORIZONTAL );
+    aField->SetItalic( m_isItalic );
+    aField->SetBold( m_isBold );
+    aField->SetTextColor( m_color );
 }
 
 
-DIALOG_LIB_FIELD_PROPERTIES::DIALOG_LIB_FIELD_PROPERTIES( SCH_BASE_FRAME* aParent,
-                                                          const wxString& aTitle,
-                                                          const LIB_FIELD* aField ) :
-        DIALOG_FIELD_PROPERTIES( aParent, aTitle, aField )
-{
-    m_fieldId        = aField->GetId();
-    m_isNameVisible  = aField->IsNameShown();
-    m_allowAutoplace = aField->CanAutoplace();
-
-    if( m_fieldId == VALUE_FIELD )
-        m_text = UnescapeString( aField->GetText() );
-
-    m_nameVisible->Show();
-    m_cbAllowAutoPlace->Show();
-
-    // When in the library editor, power symbols can be renamed.
-    m_isPower = false;
-    init();
-}
-
-
-void DIALOG_LIB_FIELD_PROPERTIES::UpdateField( LIB_FIELD* aField )
+void DIALOG_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField )
 {
     aField->SetText( m_text );
 
@@ -463,181 +585,14 @@ void DIALOG_LIB_FIELD_PROPERTIES::UpdateField( LIB_FIELD* aField )
 }
 
 
-DIALOG_SCH_FIELD_PROPERTIES::DIALOG_SCH_FIELD_PROPERTIES( SCH_BASE_FRAME* aParent,
-                                                          const wxString& aTitle,
-                                                          const SCH_FIELD* aField ) :
-        DIALOG_FIELD_PROPERTIES( aParent, aTitle, aField ),
-        m_field( aField )
+void DIALOG_FIELD_PROPERTIES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
 {
-    m_isSheetFilename = false;
-
-    if( aField->GetParent() && aField->GetParent()->Type() == SCH_SYMBOL_T )
-    {
-        m_fieldId = aField->GetId();
-    }
-    else if( aField->GetParent() && aField->GetParent()->Type() == SCH_SHEET_T )
-    {
-        switch( aField->GetId() )
-        {
-        case SHEETNAME:
-            m_fieldId = SHEETNAME_V;
-            break;
-
-        case SHEETFILENAME:
-            m_isSheetFilename = true;
-            m_fieldId = SHEETFILENAME_V;
-            m_note->SetLabel( wxString::Format( m_note->GetLabel(),
-                              _( "Sheet filename can only be modified in Sheet Properties dialog." ) ) );
-            m_note->Show( true );
-            break;
-
-        default:
-            m_fieldId = SHEETUSERFIELD_V;
-            break;
-        }
-    }
-    else if( aField->GetParent() && aField->GetParent()->IsType( { SCH_LABEL_LOCATE_ANY_T } ) )
-    {
-        m_fieldId = LABELUSERFIELD_V;
-    }
-
-    // show text variable cross-references in a human-readable format
-    m_text = aField->Schematic()->ConvertKIIDsToRefs( aField->GetText() );
-
-    m_font = m_field->GetFont();
-
-    m_isPower = false;
-
-    m_textLabel->SetLabel( aField->GetName() + wxS( ":" ) );
-
-    m_position = m_field->GetPosition();
-
-    m_isNameVisible = m_field->IsNameShown();
-    m_allowAutoplace = m_field->CanAutoplace();
-
-    m_horizontalJustification = m_field->GetEffectiveHorizJustify();
-    m_verticalJustification = m_field->GetEffectiveVertJustify();
-
-    // The library symbol may have been removed so using SCH_SYMBOL::GetLibSymbolRef() here
-    // could result in a segfault.  If the library symbol is no longer available, the
-    // schematic fields can still edit so set the power symbol flag to false.  This may not
-    // be entirely accurate if the power library is missing but it's better then a segfault.
-    if( aField->GetParent() && aField->GetParent()->Type() == SCH_SYMBOL_T )
-    {
-        const SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( aField->GetParent() );
-        const LIB_SYMBOL* libSymbol = GetParent()->GetLibSymbol( symbol->GetLibId(), true );
-
-        if( libSymbol && libSymbol->IsPower() )
-            m_isPower = true;
-    }
-
-    m_StyledTextCtrl->Bind( wxEVT_STC_CHARADDED, &DIALOG_SCH_FIELD_PROPERTIES::onScintillaCharAdded,
-                            this );
-
-    m_nameVisible->Show();
-    m_cbAllowAutoPlace->Show();
-
-    init();
-
-    if( m_isSheetFilename )
-    {
-        m_StyledTextCtrl->Enable( false );
-        m_TextCtrl->Enable( false );
-    }
+    m_field->OnScintillaCharAdded( m_scintillaTricks, aEvent );
 }
 
 
-void DIALOG_SCH_FIELD_PROPERTIES::onScintillaCharAdded( wxStyledTextEvent &aEvent )
-{
-    int key = aEvent.GetKey();
-
-    SCH_EDIT_FRAME* editFrame = static_cast<SCH_EDIT_FRAME*>( GetParent() );
-    wxArrayString   autocompleteTokens;
-    int             pos = m_StyledTextCtrl->GetCurrentPos();
-    int             start = m_StyledTextCtrl->WordStartPosition( pos, true );
-    wxString        partial;
-
-    // Currently, '\n' is not allowed in fields. So remove it when entered
-    // TODO: see if we must close the dialog. However this is not obvious, as
-    // if a \n is typed (and removed) when a text is selected, this text is deleted
-    // (in fact replaced by \n, that is removed by the filter)
-    if( key == '\n' )
-    {
-        wxString text = m_StyledTextCtrl->GetText();
-        int currpos = m_StyledTextCtrl->GetCurrentPos();
-        text.Replace( wxS( "\n" ), wxS( "" ) );
-        m_StyledTextCtrl->SetText( text );
-        m_StyledTextCtrl->GotoPos( currpos-1 );
-        return;
-    }
-
-    auto textVarRef =
-            [&]( int pt )
-            {
-                return pt >= 2
-                        && m_StyledTextCtrl->GetCharAt( pt - 2 ) == '$'
-                        && m_StyledTextCtrl->GetCharAt( pt - 1 ) == '{';
-            };
-
-    // Check for cross-reference
-    if( start > 1 && m_StyledTextCtrl->GetCharAt( start - 1 ) == ':' )
-    {
-        int refStart = m_StyledTextCtrl->WordStartPosition( start - 1, true );
-
-        if( textVarRef( refStart ) )
-        {
-            partial = m_StyledTextCtrl->GetRange( start, pos );
-
-            wxString           ref = m_StyledTextCtrl->GetRange( refStart, start - 1 );
-            SCH_SHEET_LIST     sheets = editFrame->Schematic().GetSheets();
-            SCH_REFERENCE_LIST refs;
-            SCH_SYMBOL*        refSymbol = nullptr;
-
-            sheets.GetSymbols( refs );
-
-            for( size_t jj = 0; jj < refs.GetCount(); jj++ )
-            {
-                if( refs[ jj ].GetSymbol()->GetRef( &refs[ jj ].GetSheetPath(), true ) == ref )
-                {
-                    refSymbol = refs[ jj ].GetSymbol();
-                    break;
-                }
-            }
-
-            if( refSymbol )
-                refSymbol->GetContextualTextVars( &autocompleteTokens );
-        }
-    }
-    else if( textVarRef( start ) )
-    {
-        partial = m_StyledTextCtrl->GetTextRange( start, pos );
-
-        SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( m_field->GetParent() );
-        SCH_SHEET*  sheet = dynamic_cast<SCH_SHEET*>( m_field->GetParent() );
-
-        if( symbol )
-        {
-            symbol->GetContextualTextVars( &autocompleteTokens );
-
-            SCHEMATIC* schematic = symbol->Schematic();
-
-            if( schematic && schematic->CurrentSheet().Last() )
-                schematic->CurrentSheet().Last()->GetContextualTextVars( &autocompleteTokens );
-        }
-
-        if( sheet )
-            sheet->GetContextualTextVars( &autocompleteTokens );
-
-        for( std::pair<wxString, wxString> entry : Prj().GetTextVars() )
-            autocompleteTokens.push_back( entry.first );
-    }
-
-    m_scintillaTricks->DoAutocomplete( partial, autocompleteTokens );
-    m_StyledTextCtrl->SetFocus();
-}
-
-
-void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH* aSheetPath )
+void DIALOG_FIELD_PROPERTIES::UpdateField( SCH_COMMIT* aCommit, SCH_FIELD* aField,
+                                           SCH_SHEET_PATH* aSheetPath )
 {
     SCH_EDIT_FRAME* editFrame = dynamic_cast<SCH_EDIT_FRAME*>( GetParent() );
     SCH_ITEM*       parent = dynamic_cast<SCH_ITEM*>( aField->GetParent() );
@@ -653,10 +608,18 @@ void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH
             symbol->SetValueFieldText( m_text );
         else if( fieldType == FOOTPRINT_FIELD )
             symbol->SetFootprintFieldText( m_text );
+
+        // Set the unit selection in multiple units per package
+        if( m_unitChoice->IsShown() )
+        {
+            int unit_selection = m_unitChoice->IsEnabled() ? m_unitChoice->GetSelection() + 1 : 1;
+            symbol->SetUnitSelection( aSheetPath, unit_selection );
+            symbol->SetUnit( unit_selection );
+        }
     }
     else if( parent && parent->Type() == SCH_GLOBAL_LABEL_T )
     {
-        if( fieldType == 0 )
+        if( aField->GetCanonicalName() == wxT( "Intersheetrefs" ) )
         {
             if( m_visible->GetValue() != parent->Schematic()->Settings().m_IntersheetRefsShow )
             {
@@ -684,12 +647,16 @@ void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH
     // convert any text variable cross-references to their UUIDs
     m_text = aField->Schematic()->ConvertRefsToKIIDs( m_text );
 
+    // Changing a sheetname need to update the hierarchy navigator
+    bool needUpdateHierNav = false;
+
+    if( parent && parent->Type() == SCH_SHEET_T && fieldType == SHEETNAME )
+        needUpdateHierNav = m_text != aField->GetText();
+
     aField->SetText( m_text );
     updateText( aField );
     aField->SetPosition( m_position );
 
-    aField->SetFont( m_font );
-
     aField->SetNameShown( m_isNameVisible );
     aField->SetCanAutoplace( m_allowAutoplace );
 
@@ -719,25 +686,23 @@ void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH
             int      unit = symbol->GetUnit();
             LIB_ID   libId = symbol->GetLibId();
 
-            for( SCH_SHEET_PATH& sheet : editFrame->Schematic().GetSheets() )
+            for( SCH_SHEET_PATH& sheet : editFrame->Schematic().Hierarchy() )
             {
                 SCH_SCREEN*              screen = sheet.LastScreen();
                 std::vector<SCH_SYMBOL*> otherUnits;
-                constexpr bool           appendUndo = true;
 
                 CollectOtherUnits( ref, unit, libId, sheet, &otherUnits );
 
                 for( SCH_SYMBOL* otherUnit : otherUnits )
                 {
-                    editFrame->SaveCopyInUndoList( screen, otherUnit, UNDO_REDO::CHANGED,
-                                                   appendUndo );
+                    aCommit->Modify( otherUnit, screen );
 
                     if( fieldType == VALUE_FIELD )
                         otherUnit->SetValueFieldText( m_text );
                     else if( fieldType == FOOTPRINT_FIELD )
                         otherUnit->SetFootprintFieldText( m_text );
                     else
-                        otherUnit->GetField( DATASHEET_FIELD )->SetText( m_text );
+                        otherUnit->GetField( (MANDATORY_FIELD_T) fieldType )->SetText( m_text );
 
                     editFrame->UpdateItem( otherUnit, false, true );
                 }
@@ -746,5 +711,9 @@ void DIALOG_SCH_FIELD_PROPERTIES::UpdateField( SCH_FIELD* aField, SCH_SHEET_PATH
     }
 
     if( positioningModified && parent )
-        parent->ClearFieldsAutoplaced();
+        parent->SetFieldsAutoplaced( AUTOPLACE_NONE );
+
+    // Update the hierarchy navigator labels if needed.
+    if( editFrame && needUpdateHierNav )
+        editFrame->UpdateLabelsHierarchyNavigator();
 }
