@@ -310,7 +310,7 @@ void ERC_TESTER::TestTextVars( DS_PROXY_VIEW_ITEM* aDrawingSheet )
                                 {
                                     SCH_TEXTBOX* textboxItem = static_cast<SCH_TEXTBOX*>( child );
 
-                                    if( unresolved( textboxItem->GetShownText( &sheet, true ) ) )
+                                    if( unresolved( textboxItem->GetShownText( nullptr, &sheet, true ) ) )
                                     {
                                         auto ercItem = ERC_ITEM::Create( ERCE_UNRESOLVED_VARIABLE );
                                         ercItem->SetItems( symbol );
@@ -398,7 +398,7 @@ void ERC_TESTER::TestTextVars( DS_PROXY_VIEW_ITEM* aDrawingSheet )
             }
             else if( SCH_TEXTBOX* textBox = dynamic_cast<SCH_TEXTBOX*>( item ) )
             {
-                if( textBox->GetShownText( &sheet, true ).Matches( wxS( "*${*}*" ) ) )
+                if( textBox->GetShownText( nullptr, &sheet, true ).Matches( wxS( "*${*}*" ) ) )
                 {
                     auto ercItem = ERC_ITEM::Create( ERCE_UNRESOLVED_VARIABLE );
                     ercItem->SetItems( textBox );
@@ -714,7 +714,7 @@ int ERC_TESTER::TestMissingNetclasses()
                             {
                                 wxString netclass = field->GetShownText( &sheet, false );
 
-                                if( !netclass.IsSameAs( defaultNetclass )
+                                if( !netclass.empty() && !netclass.IsSameAs( defaultNetclass )
                                     && !settings->HasNetclass( netclass ) )
                                 {
                                     logError( sheet, item, netclass );
@@ -790,6 +790,23 @@ int ERC_TESTER::TestFourWayJunction()
 {
     int err_count = 0;
 
+    auto pinStackAlreadyRepresented =
+            []( SCH_PIN* pin, std::vector<SCH_ITEM*>& collection ) -> bool
+            {
+                for( SCH_ITEM*& item : collection )
+                {
+                    if( item->Type() == SCH_PIN_T && item->GetParentSymbol() == pin->GetParentSymbol() )
+                    {
+                        if( pin->IsVisible() && !static_cast<SCH_PIN*>( item )->IsVisible() )
+                            item = pin;
+
+                        return true;
+                    }
+                }
+
+                return false;
+            };
+
     for( const SCH_SHEET_PATH& sheet : m_sheetList )
     {
         std::map<VECTOR2I, std::vector<SCH_ITEM*>> connMap;
@@ -800,7 +817,15 @@ int ERC_TESTER::TestFourWayJunction()
             SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
 
             for( SCH_PIN* pin : symbol->GetPins( &sheet ) )
-                connMap[pin->GetPosition()].emplace_back( pin );
+            {
+                std::vector<SCH_ITEM*>& entry = connMap[pin->GetPosition()];
+
+                // Only one pin per pin-stack.
+                if( pinStackAlreadyRepresented( pin, entry ) )
+                    continue;
+
+                entry.emplace_back( pin );
+            }
         }
 
         for( SCH_ITEM* item : screen->Items().OfType( SCH_LINE_T ) )
@@ -878,7 +903,7 @@ int ERC_TESTER::TestNoConnectPins()
                         addOther( pin->GetPosition(), pin );
                 }
             }
-            else if( item->IsConnectable() )
+            else if( item->IsConnectable() && item->Type() != SCH_NO_CONNECT_T )
             {
                 for( const VECTOR2I& pt : item->GetConnectionPoints() )
                     addOther( pt, item );
@@ -1322,6 +1347,13 @@ int ERC_TESTER::TestSimilarLabels()
 
                         if( unnormalized != otherText )
                         {
+                            // Similar local labels on different sheets are fine
+                            if( item->Type() == SCH_LABEL_T && otherItem->Type() == SCH_LABEL_T
+                                    && sheet != otherSheet )
+                            {
+                                continue;
+                            }
+
                             logError( normalized, label, sheet, otherTuple );
                             errors += 1;
                         }
@@ -1385,10 +1417,11 @@ int ERC_TESTER::TestLibSymbolIssues()
             SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
             LIB_SYMBOL* libSymbolInSchematic = symbol->GetLibSymbolRef().get();
 
-            wxCHECK2( libSymbolInSchematic, continue );
+            if( !libSymbolInSchematic )
+                continue;
 
-            wxString       libName = symbol->GetLibId().GetLibNickname();
-            LIB_TABLE_ROW* libTableRow = libTable->FindRow( libName, true );
+            wxString             libName = symbol->GetLibId().GetLibNickname();
+            const LIB_TABLE_ROW* libTableRow = libTable->FindRow( libName, true );
 
             if( !libTableRow )
             {
@@ -1411,7 +1444,7 @@ int ERC_TESTER::TestLibSymbolIssues()
                 {
                     std::shared_ptr<ERC_ITEM> ercItem = ERC_ITEM::Create( ERCE_LIB_SYMBOL_ISSUES );
                     ercItem->SetItems( symbol );
-                    msg.Printf( _( "The library '%s' is not enabled in the current configuration" ),
+                    msg.Printf( _( "The symbol library '%s' is not enabled in the current configuration" ),
                                 UnescapeString( libName ) );
                     ercItem->SetErrorMessage( msg );
 
@@ -1420,6 +1453,22 @@ int ERC_TESTER::TestLibSymbolIssues()
 
                 continue;
             }
+            else if( !libTableRow->LibraryExists() )
+            {
+                if( m_settings.IsTestEnabled( ERCE_LIB_SYMBOL_ISSUES ) )
+                {
+                    std::shared_ptr<ERC_ITEM> ercItem = ERC_ITEM::Create( ERCE_LIB_SYMBOL_ISSUES );
+                    ercItem->SetItems( symbol );
+                    msg.Printf( _( "The symbol library '%s' was not found at '%s'." ),
+                                UnescapeString( libName ),
+                                libTableRow->GetFullURI( true ) );
+                    ercItem->SetErrorMessage( msg );
+
+                    markers.emplace_back( new SCH_MARKER( ercItem, symbol->GetPosition() ) );
+                }
+
+                continue;
+            }
 
             wxString    symbolName = symbol->GetLibId().GetLibItemName();
             LIB_SYMBOL* libSymbol = SchGetLibSymbol( symbol->GetLibId(), libTable );
@@ -1617,7 +1666,7 @@ int ERC_TESTER::TestFootprintFilters()
             {
                 std::shared_ptr<ERC_ITEM> ercItem = ERC_ITEM::Create( ERCE_FOOTPRINT_LINK_ISSUES );
                 msg.Printf( _( "Assigned footprint (%s) doesn't match footprint filters (%s)." ),
-                            lowerItemName,
+                            footprint.GetUniStringLibItemName(),
                             wxJoin( filters, ' ' ) );
                 ercItem->SetErrorMessage( msg );
                 ercItem->SetItems( sch_symbol );
@@ -1740,7 +1789,7 @@ int ERC_TESTER::TestSimModelIssues()
             // Reset for each symbol
             reporter.Clear();
 
-            SIM_LIBRARY::MODEL model = libMgr.CreateModel( &sheet, *symbol, reporter );
+            SIM_LIBRARY::MODEL model = libMgr.CreateModel( &sheet, *symbol, true, 0, reporter );
 
             if( reporter.HasMessage() )
             {
