@@ -71,9 +71,8 @@
 #include <wx/log.h>
 #include <wx/richmsgdlg.h>
 #include <wx/stdpaths.h>
-#include <tools/ee_actions.h>
-#include <tools/ee_inspection_tool.h>
-#include <tools/ee_selection_tool.h>
+#include <tools/sch_inspection_tool.h>
+#include <tools/sch_selection_tool.h>
 #include <paths.h>
 #include <wx_filename.h>  // For ::ResolvePossibleSymlinks
 #include <widgets/wx_progress_reporters.h>
@@ -149,7 +148,7 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
         ClearUndoRedoList();
         ClearRepeatItemsList();
         SetScreen( nullptr );
-        m_toolManager->GetTool<EE_INSPECTION_TOOL>()->Reset( TOOL_BASE::SUPERMODEL_RELOAD );
+        m_toolManager->GetTool<SCH_INSPECTION_TOOL>()->Reset( TOOL_BASE::SUPERMODEL_RELOAD );
         CreateScreens();
     }
 
@@ -511,12 +510,6 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
 
             for( SCH_SCREEN* screen = schematic.GetFirst(); screen; screen = schematic.GetNext() )
                 screen->MigrateSimModels();
-
-            if( schematic.GetFirst()->GetFileFormatVersionAtLoad() < SEXPR_SCHEMATIC_FILE_VERSION )
-            {
-                // Allow the schematic to be saved to new file format without making any edits.
-                OnModify();
-            }
         }
 
         // After the schematic is successfully loaded, we load the drawing sheet.
@@ -578,7 +571,7 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
 
     TestDanglingEnds();
 
-    UpdateHierarchyNavigator( false );
+    UpdateHierarchyNavigator( false, true );
 
     wxCommandEvent changedEvt( EDA_EVT_SCHEMATIC_CHANGED );
     ProcessEventLocally( changedEvt );
@@ -679,7 +672,7 @@ void SCH_EDIT_FRAME::OnImportProject( wxCommandEvent& aEvent )
     eeconfig()->m_System.show_import_issues = importOptions.GetShowIssues();
 
     // Don't leave dangling pointers to previously-opened document.
-    m_toolManager->GetTool<EE_SELECTION_TOOL>()->ClearSelection();
+    m_toolManager->GetTool<SCH_SELECTION_TOOL>()->ClearSelection();
     ClearUndoRedoList();
     ClearRepeatItemsList();
 
@@ -787,8 +780,6 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
         saveProjectSettings();
     }
 
-    wxString tempFile = wxFileName::CreateTempFileName( wxS( "eeschema" ) );
-
     // Save
     wxLogTrace( traceAutoSave, wxS( "Saving file " ) + schematicFileName.GetFullPath() );
 
@@ -805,7 +796,7 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
 
     try
     {
-        pi->SaveSchematicFile( tempFile, aSheet, &Schematic() );
+        pi->SaveSchematicFile( schematicFileName.GetFullPath(), aSheet, &Schematic() );
         success = true;
     }
     catch( const IO_ERROR& ioe )
@@ -815,38 +806,9 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
                     ioe.What() );
         DisplayError( this, msg );
 
-        msg.Printf( _( "Failed to create temporary file '%s'." ),
-                    tempFile );
-        SetMsgPanel( wxEmptyString, msg );
-
-        // In case we started a file but didn't fully write it, clean up
-        wxRemoveFile( tempFile );
-
         success = false;
     }
 
-    if( success )
-    {
-        // Preserve the permissions of the current file
-        KIPLATFORM::IO::DuplicatePermissions( schematicFileName.GetFullPath(), tempFile );
-
-        // Replace the original with the temporary file we just wrote
-        success = wxRenameFile( tempFile, schematicFileName.GetFullPath() );
-
-        if( !success )
-        {
-            msg.Printf( _( "Error saving schematic file '%s'.\n"
-                           "Failed to rename temporary file '%s'." ),
-                        schematicFileName.GetFullPath(),
-                        tempFile );
-            DisplayError( this, msg );
-
-            msg.Printf( _( "Failed to rename temporary file '%s'." ),
-                        tempFile );
-            SetMsgPanel( wxEmptyString, msg );
-        }
-    }
-
     if( success )
     {
         // Delete auto save file.
@@ -867,10 +829,6 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
         msg.Printf( _( "File '%s' saved." ),  screen->GetFileName() );
         SetStatusText( msg, 0 );
     }
-    else
-    {
-        DisplayError( this, _( "File write operation failed." ) );
-    }
 
     return success;
 }
@@ -1033,6 +991,10 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
         // File doesn't exist yet; true if we just imported something
         updateFileHistory = true;
     }
+    else if( screens.GetFirst()->GetFileFormatVersionAtLoad() < SEXPR_SCHEMATIC_FILE_VERSION )
+    {
+        // Allow the user to save un-edited files in new format
+    }
     else if( !IsContentModified() )
     {
         return true;
@@ -1246,6 +1208,9 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
 
 bool SCH_EDIT_FRAME::doAutoSave()
 {
+    if( !Schematic().IsValid() )
+        return true;
+
     wxFileName  tmpFileName = Schematic().Root().GetFileName();
     wxFileName  fn = tmpFileName;
     wxFileName  tmp;
@@ -1330,7 +1295,8 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType,
     case SCH_IO_MGR::SCH_EASYEDAPRO:
     {
         // We insist on caller sending us an absolute path, if it does not, we say it's a bug.
-        wxCHECK_MSG( filename.IsAbsolute(), false,
+        // Unless we are passing the files in aproperties, in which case aFileName can be empty.
+        wxCHECK_MSG( aFileName.IsEmpty() || filename.IsAbsolute(), false,
                      wxS( "Import schematic: path is not absolute!" ) );
 
         try
@@ -1424,7 +1390,7 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType,
         SetSheetNumberAndCount();
         SyncView();
 
-        UpdateHierarchyNavigator();
+        UpdateHierarchyNavigator( false, true );
 
         wxCommandEvent e( EDA_EVT_SCHEMATIC_CHANGED );
         ProcessEventLocally( e );
@@ -1492,9 +1458,14 @@ bool SCH_EDIT_FRAME::updateAutoSaveFile()
     wxFileName tmpFn = Prj().GetProjectFullName();
     wxFileName autoSaveFileName( tmpFn.GetPath(), getAutoSaveFileName() );
 
-    wxLogTrace( traceAutoSave, "Creating auto save file %s", autoSaveFileName.GetFullPath() );
+    if( !autoSaveFileName.IsDirWritable() )
+    {
+        wxLogTrace( traceAutoSave, "Insufficient permissions to auto save file '%s'",
+                    autoSaveFileName.GetFullPath() );
+        return false;
+    }
 
-    wxCHECK( autoSaveFileName.IsDirWritable(), false );
+    wxLogTrace( traceAutoSave, "Creating auto save file '%s'", autoSaveFileName.GetFullPath() );
 
     wxFileName fn;
     SCH_SCREENS screens( Schematic().Root() );
@@ -1517,7 +1488,7 @@ bool SCH_EDIT_FRAME::updateAutoSaveFile()
 
     if( autoSaveFileName.FileExists() && !wxRemoveFile( autoSaveFileName.GetFullPath() ) )
     {
-        wxLogTrace( traceAutoSave, "Error removing auto save file %s",
+        wxLogTrace( traceAutoSave, "Error removing auto save file '%s'",
                     autoSaveFileName.GetFullPath() );
 
         return false;
@@ -1532,7 +1503,7 @@ bool SCH_EDIT_FRAME::updateAutoSaveFile()
 
     for( const wxString& fileName : autoSavedFiles )
     {
-        wxLogTrace( traceAutoSave, "Adding auto save file %s to %s",
+        wxLogTrace( traceAutoSave, "Adding auto save file '%s' to '%s'",
                     fileName, autoSaveFileName.GetName() );
         autoSaveFile.AddLine( fileName );
     }
@@ -1548,7 +1519,7 @@ bool SCH_EDIT_FRAME::updateAutoSaveFile()
 
 void removeFile( const wxString& aFilename, wxArrayString& aUnremoved )
 {
-    wxLogTrace( traceAutoSave, wxS( "Removing auto save file " ) + aFilename );
+    wxLogTrace( traceAutoSave, wxS( "Removing auto save file '%s'" ), aFilename );
 
     if( wxFileExists( aFilename ) && !wxRemoveFile( aFilename ) )
         aUnremoved.Add( aFilename );
@@ -1562,19 +1533,16 @@ void SCH_EDIT_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
 
     wxCHECK_RET( aFileName.IsOk(), wxS( "Invalid file name!" ) );
 
-    wxLogTrace( traceAutoSave,
-                wxS( "Checking for auto save file " ) + aFileName.GetFullPath() );
+    wxLogTrace( traceAutoSave, wxS( "Checking for auto save file '%s'" ), aFileName.GetFullPath() );
 
     if( !aFileName.FileExists() )
         return;
 
-    wxString msg = _(
-            "Well this is potentially embarrassing!\n"
-            "It appears that the last time you were editing one or more of the schematic files\n"
-            "were not saved properly.  Do you wish to restore the last saved edits you made?" );
+    wxString msg = _( "Well this is potentially embarrassing!\n"
+                      "It appears that the last time you were editing one or more of the schematic files\n"
+                      "were not saved properly.  Do you wish to restore the last saved edits you made?" );
 
-    int response = wxMessageBox( msg, Pgm().App().GetAppDisplayName(), wxYES_NO | wxICON_QUESTION,
-                                 this );
+    int response = wxMessageBox( msg, Pgm().App().GetAppDisplayName(), wxYES_NO | wxICON_QUESTION, this );
 
     wxTextFile fileList( aFileName.GetFullPath() );
 
@@ -1637,8 +1605,7 @@ void SCH_EDIT_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
                 msg += unrecoveredFiles[i] + wxS( "\n" );
 
             msg += _( "Manual recovery will be required to restore the file(s) above." );
-            wxMessageBox( msg, Pgm().App().GetAppDisplayName(), wxOK | wxICON_EXCLAMATION,
-                          this );
+            wxMessageBox( msg, Pgm().App().GetAppDisplayName(), wxOK | wxICON_EXCLAMATION, this );
         }
 
         wxArrayString unremovedFiles;
