@@ -102,13 +102,7 @@ public:
         return _( "Field" );
     }
 
-    bool IsHypertext() const override
-    {
-        if( GetCanonicalName() == wxT( "Intersheetrefs" ) )
-            return true;
-
-        return IsURL( GetShownText( false ) );
-    }
+    bool IsHypertext() const override;
 
     void DoHypertextAction( EDA_DRAW_FRAME* aFrame ) const override;
 
@@ -201,6 +195,8 @@ public:
      */
     EDA_ANGLE GetDrawRotation() const override;
 
+    KIFONT::FONT* GetDrawFont( const RENDER_SETTINGS* aSettings ) const override;
+
     const BOX2I GetBoundingBox() const override;
 
     /**
@@ -335,8 +331,6 @@ public:
 #endif
 
 protected:
-    KIFONT::FONT* getDrawFont() const override;
-
     const KIFONT::METRICS& getFontMetrics() const override { return GetFontMetrics(); }
 
     /**
