@@ -28,7 +28,7 @@
 #include <base_units.h>
 #include <bitmap_base.h>
 #include <build_version.h>
-#include <ee_selection.h>
+#include <sch_selection.h>
 #include <font/fontconfig.h>
 #include <io/kicad/kicad_io_utils.h>
 #include <locale_io.h>
@@ -401,6 +401,15 @@ void SCH_IO_KICAD_SEXPR::Format( SCH_SHEET* aSheet )
                 if( a->Type() != b->Type() )
                     return a->Type() < b->Type();
 
+                // sorting fix specific for 9.0 to avoid format change
+                if( a->Type() == SCH_TABLE_T && b->Type() == SCH_TABLE_T )
+                {
+                    const SCH_TABLE* aTable = static_cast<const SCH_TABLE*>( a );
+                    const SCH_TABLE* bTable = static_cast<const SCH_TABLE*>( b );
+
+                    return aTable->GetHash() < bTable->GetHash();
+                }
+
                 return a->m_Uuid < b->m_Uuid;
             };
 
@@ -493,7 +502,7 @@ void SCH_IO_KICAD_SEXPR::Format( SCH_SHEET* aSheet )
 }
 
 
-void SCH_IO_KICAD_SEXPR::Format( EE_SELECTION* aSelection, SCH_SHEET_PATH* aSelectionPath,
+void SCH_IO_KICAD_SEXPR::Format( SCH_SELECTION* aSelection, SCH_SHEET_PATH* aSelectionPath,
                                  SCHEMATIC& aSchematic, OUTPUTFORMATTER* aFormatter,
                                  bool aForClipboard )
 {
@@ -506,23 +515,16 @@ void SCH_IO_KICAD_SEXPR::Format( EE_SELECTION* aSelection, SCH_SHEET_PATH* aSele
     m_out = aFormatter;
 
     size_t                          i;
-    SCH_ITEM*                       item;
     std::map<wxString, LIB_SYMBOL*> libSymbols;
     SCH_SCREEN*                     screen = aSelection->GetScreen();
     std::set<SCH_TABLE*>            promotedTables;
 
-    for( i = 0; i < aSelection->GetSize(); ++i )
+    for( EDA_ITEM* item : *aSelection )
     {
-        item = dynamic_cast<SCH_ITEM*>( aSelection->GetItem( i ) );
-
-        wxCHECK2( item, continue );
-
         if( item->Type() != SCH_SYMBOL_T )
             continue;
 
-        SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( item );
-
-        wxCHECK2( symbol, continue );
+        SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
 
         wxString libSymbolLookup = symbol->GetLibId().Format().wx_str();
 
@@ -539,49 +541,48 @@ void SCH_IO_KICAD_SEXPR::Format( EE_SELECTION* aSelection, SCH_SHEET_PATH* aSele
     {
         m_out->Print( "(lib_symbols" );
 
-        for( const std::pair<const wxString, LIB_SYMBOL*>& libSymbol : libSymbols )
-        {
-            SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( libSymbol.second, *m_out, libSymbol.first,
-                                                      false );
-        }
+        for( const auto& [name, libSymbol] : libSymbols )
+            SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( libSymbol, *m_out, name, false );
 
         m_out->Print( ")" );
     }
 
-    for( i = 0; i < aSelection->GetSize(); ++i )
+    for( EDA_ITEM* edaItem : *aSelection )
     {
-        item = (SCH_ITEM*) aSelection->GetItem( i );
+        if( !edaItem->IsSCH_ITEM() )
+            continue;
+
+        SCH_ITEM* item = static_cast<SCH_ITEM*>( edaItem );
 
         switch( item->Type() )
         {
         case SCH_SYMBOL_T:
-            saveSymbol( static_cast<SCH_SYMBOL*>( item ), aSchematic, sheets, aForClipboard,
-                        aSelectionPath );
+            saveSymbol( static_cast<SCH_SYMBOL*>( item ), aSchematic, sheets, aForClipboard, aSelectionPath );
             break;
 
         case SCH_BITMAP_T:
-            saveBitmap( static_cast< SCH_BITMAP& >( *item ) );
+            saveBitmap( static_cast<SCH_BITMAP&>( *item ) );
             break;
 
         case SCH_SHEET_T:
-            saveSheet( static_cast< SCH_SHEET* >( item ), sheets );
+            saveSheet( static_cast<SCH_SHEET*>( item ), sheets );
             break;
 
         case SCH_JUNCTION_T:
-            saveJunction( static_cast< SCH_JUNCTION* >( item ) );
+            saveJunction( static_cast<SCH_JUNCTION*>( item ) );
             break;
 
         case SCH_NO_CONNECT_T:
-            saveNoConnect( static_cast< SCH_NO_CONNECT* >( item ) );
+            saveNoConnect( static_cast<SCH_NO_CONNECT*>( item ) );
             break;
 
         case SCH_BUS_WIRE_ENTRY_T:
         case SCH_BUS_BUS_ENTRY_T:
-            saveBusEntry( static_cast< SCH_BUS_ENTRY_BASE* >( item ) );
+            saveBusEntry( static_cast<SCH_BUS_ENTRY_BASE*>( item ) );
             break;
 
         case SCH_LINE_T:
-            saveLine( static_cast< SCH_LINE* >( item ) );
+            saveLine( static_cast<SCH_LINE*>( item ) );
             break;
 
         case SCH_SHAPE_T:
@@ -791,7 +792,10 @@ void SCH_IO_KICAD_SEXPR::saveSymbol( SCH_SYMBOL* aSymbol, const SCHEMATIC& aSche
 
     for( const std::unique_ptr<SCH_PIN>& pin : aSymbol->GetRawPins() )
     {
-        if( pin->GetAlt().IsEmpty() )
+        // There was a bug introduced somewhere in the original alternated pin code that would
+        // set the alternate pin to the default pin name which caused a number of library symbol
+        // comparison issues.  Clearing the alternate pin resolves this issue.
+        if( pin->GetAlt().IsEmpty() || ( pin->GetAlt() == pin->GetBaseName() ) )
         {
             m_out->Print( "(pin %s", m_out->Quotew( pin->GetNumber() ).c_str() );
             KICAD_FORMAT::FormatUuid( m_out, pin->m_Uuid );
@@ -1417,9 +1421,9 @@ void SCH_IO_KICAD_SEXPR::saveTable( SCH_TABLE* aTable )
 
     m_out->Print( "(border" );
     KICAD_FORMAT::FormatBool( m_out, "external", aTable->StrokeExternal() );
-    KICAD_FORMAT::FormatBool( m_out, "header", aTable->StrokeHeader() );
+    KICAD_FORMAT::FormatBool( m_out, "header", aTable->StrokeHeaderSeparator() );
 
-    if( aTable->StrokeExternal() || aTable->StrokeHeader() )
+    if( aTable->StrokeExternal() || aTable->StrokeHeaderSeparator() )
         aTable->GetBorderStroke().Format( m_out, schIUScale );
 
     m_out->Print( ")" );               // Close `border` token.
@@ -1658,7 +1662,7 @@ void SCH_IO_KICAD_SEXPR::CreateLibrary( const wxString& aLibraryPath,
     m_cache = new SCH_IO_KICAD_SEXPR_LIB_CACHE( aLibraryPath );
     m_cache->SetModified();
     m_cache->Save();
-    m_cache->Load();    // update m_writable and m_mod_time
+    m_cache->Load();    // update m_writable and m_timestamp
 }
 
 
@@ -1697,9 +1701,7 @@ void SCH_IO_KICAD_SEXPR::SaveLibrary( const wxString& aLibraryPath,
     wxString oldFileName = m_cache->GetFileName();
 
     if( !m_cache->IsFile( aLibraryPath ) )
-    {
         m_cache->SetFileName( aLibraryPath );
-    }
 
     // This is a forced save.
     m_cache->SetModified();
@@ -1708,6 +1710,18 @@ void SCH_IO_KICAD_SEXPR::SaveLibrary( const wxString& aLibraryPath,
 }
 
 
+bool SCH_IO_KICAD_SEXPR::CanReadLibrary( const wxString& aLibraryPath ) const
+{
+    if( !SCH_IO::CanReadLibrary( aLibraryPath ) )
+        return false;
+
+    // Above just checks for proper extension; now check that it actually exists
+
+    wxFileName fn( aLibraryPath );
+    return fn.IsOk() && fn.FileExists();
+}
+
+
 bool SCH_IO_KICAD_SEXPR::IsLibraryWritable( const wxString& aLibraryPath )
 {
     wxFileName fn( aLibraryPath );
@@ -1785,7 +1799,6 @@ std::vector<LIB_SYMBOL*> SCH_IO_KICAD_SEXPR::ParseLibSymbols( std::string& aSymb
 
 void SCH_IO_KICAD_SEXPR::FormatLibSymbol( LIB_SYMBOL* symbol, OUTPUTFORMATTER & formatter )
 {
-
     LOCALE_IO toggle;     // toggles on, then off, the C locale.
     SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( symbol, formatter );
 }
