@@ -22,6 +22,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include "font/kicad_font_name.h"
 #include <pgm_base.h>
 #include <settings/settings_manager.h>
 #include <eeschema_settings.h>
@@ -147,6 +148,28 @@ SCH_ITEM* SCH_ITEM::Duplicate( bool doClone ) const
 }
 
 
+void SCH_ITEM::SetUnitProp( int aUnit )
+{
+    if( GetParentSymbol() )
+        aUnit = std::min( aUnit, GetParentSymbol()->GetUnitCount() );
+
+    aUnit = std::max( aUnit, 0 );
+
+    m_unit = aUnit;
+}
+
+
+void SCH_ITEM::SetBodyStyleProp( int aBodyStyle )
+{
+    if( GetParentSymbol() && GetParentSymbol()->HasAlternateBodyStyle() )
+        aBodyStyle = std::min( aBodyStyle, (int) BODY_STYLE::DEMORGAN );
+
+    aBodyStyle = std::max( aBodyStyle, 0 );
+
+    m_bodyStyle = aBodyStyle;
+}
+
+
 SCHEMATIC* SCH_ITEM::Schematic() const
 {
     EDA_ITEM* parent = GetParent();
@@ -453,12 +476,18 @@ int SCH_ITEM::compare( const SCH_ITEM& aOther, int aCompareFlags ) const
 }
 
 
-const wxString& SCH_ITEM::GetDefaultFont() const
+const wxString& SCH_ITEM::GetDefaultFont( const RENDER_SETTINGS* aSettings ) const
 {
-    SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
-    EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
+    static wxString defaultName = KICAD_FONT_NAME;
 
-    return cfg->m_Appearance.default_font;
+    SETTINGS_MANAGER&  mgr = Pgm().GetSettingsManager();
+
+    if( aSettings )
+        return aSettings->GetDefaultFont();
+    else if( EESCHEMA_SETTINGS* cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" ) )
+        return cfg->m_Appearance.default_font;
+    else
+        return defaultName;
 }
 
 
@@ -477,11 +506,20 @@ int SCH_ITEM::GetEffectivePenWidth( const SCH_RENDER_SETTINGS* aSettings ) const
     // numbers meant "don't stroke".
 
     if( GetPenWidth() < 0 )
+    {
         return 0;
+    }
     else if( GetPenWidth() == 0 )
-        return std::max( aSettings->GetDefaultPenWidth(), aSettings->GetMinPenWidth() );
+    {
+        if( GetParent() && GetParent()->Type() == LIB_SYMBOL_T )
+            return std::max( aSettings->m_SymbolLineWidth, aSettings->GetMinPenWidth() );
+        else
+            return std::max( aSettings->GetDefaultPenWidth(), aSettings->GetMinPenWidth() );
+    }
     else
+    {
         return std::max( GetPenWidth(), aSettings->GetMinPenWidth() );
+    }
 }
 
 
@@ -557,7 +595,7 @@ static struct SCH_ITEM_DESC
                 };
 
         propMgr.AddProperty( new PROPERTY<SCH_ITEM, int>( _HKI( "Unit" ),
-                    &SCH_ITEM::SetUnit, &SCH_ITEM::GetUnit ) )
+                    &SCH_ITEM::SetUnitProp, &SCH_ITEM::GetUnit ) )
                 .SetAvailableFunc( multiUnit )
                 .SetIsHiddenFromDesignEditors();
 
