@@ -163,6 +163,7 @@ SCH_LABEL_BASE::SCH_LABEL_BASE( const VECTOR2I& aPos, const wxString& aText, KIC
         m_shape( L_UNSPECIFIED ),
         m_connectionType( CONNECTION_TYPE::NONE ),
         m_isDangling( true ),
+        m_autoRotateOnPlacement( false ),
         m_lastResolvedColor( COLOR4D::UNSPECIFIED )
 {
     SetMultilineAllowed( false );
@@ -177,6 +178,7 @@ SCH_LABEL_BASE::SCH_LABEL_BASE( const SCH_LABEL_BASE& aLabel ) :
         m_shape( aLabel.m_shape ),
         m_connectionType( aLabel.m_connectionType ),
         m_isDangling( aLabel.m_isDangling ),
+        m_autoRotateOnPlacement( aLabel.m_autoRotateOnPlacement ),
         m_lastResolvedColor( aLabel.m_lastResolvedColor ),
         m_cached_driver_name( aLabel.m_cached_driver_name )
 {
@@ -193,6 +195,16 @@ SCH_LABEL_BASE& SCH_LABEL_BASE::operator=( const SCH_LABEL_BASE& aLabel )
 {
     SCH_TEXT::operator=( aLabel );
 
+    m_fields = aLabel.m_fields;
+
+    for( SCH_FIELD& field : m_fields )
+        field.SetParent( this );
+
+    m_shape = aLabel.m_shape;
+    m_connectionType = aLabel.m_connectionType;
+    m_isDangling = aLabel.m_isDangling;
+    m_autoRotateOnPlacement = aLabel.m_autoRotateOnPlacement;
+    m_lastResolvedColor = aLabel.m_lastResolvedColor;
     m_cached_driver_name = aLabel.m_cached_driver_name;
 
     return *this;
@@ -560,7 +572,7 @@ double SCH_LABEL_BASE::Similarity( const SCH_ITEM& aOther ) const
 void SCH_LABEL_BASE::AutoplaceFields( SCH_SCREEN* aScreen, AUTOPLACE_ALGO aAlgo )
 {
     int margin = GetTextOffset() * 2;
-    int labelLen = GetBodyBoundingBox().GetSizeMax();
+    int labelLen = GetBodyBoundingBox( nullptr ).GetSizeMax();
     int accumulated = GetTextHeight() / 2;
 
     if( Type() == SCH_GLOBAL_LABEL_T )
@@ -850,7 +862,43 @@ void SCH_LABEL_BASE::RunOnChildren( const std::function<void( SCH_ITEM* )>& aFun
 
 bool SCH_LABEL_BASE::Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) const
 {
-    return SCH_ITEM::Matches( UnescapeString( GetText() ), aSearchData );
+    if( SCH_ITEM::Matches( UnescapeString( GetText() ), aSearchData ) )
+    {
+        return true;
+    }
+
+    const SCH_SEARCH_DATA* searchData = dynamic_cast<const SCH_SEARCH_DATA*>( &aSearchData );
+    SCH_CONNECTION* connection = nullptr;
+    SCH_SHEET_PATH* sheetPath = reinterpret_cast<SCH_SHEET_PATH*>( aAuxData );
+
+    if( searchData && searchData->searchNetNames && sheetPath
+        && ( connection = Connection( sheetPath ) ) )
+    {
+        if( connection->IsBus() )
+        {
+            auto allMembers = connection->AllMembers();
+
+            std::set<wxString> netNames;
+
+            for( std::shared_ptr<SCH_CONNECTION> member : allMembers )
+                netNames.insert( member->GetNetName() );
+
+            for( const wxString& netName : netNames )
+            {
+                if( EDA_ITEM::Matches( netName, aSearchData ) )
+                    return true;
+            }
+
+            return false;
+        }
+
+        wxString netName = connection->GetNetName();
+
+        if( EDA_ITEM::Matches( netName, aSearchData ) )
+            return true;
+    }
+
+    return false;
 }
 
 
@@ -924,14 +972,14 @@ int SCH_LABEL_BASE::GetLabelBoxExpansion( const RENDER_SETTINGS* aSettings ) con
 }
 
 
-const BOX2I SCH_LABEL_BASE::GetBodyBoundingBox() const
+const BOX2I SCH_LABEL_BASE::GetBodyBoundingBox( const RENDER_SETTINGS* aSettings ) const
 {
     // build the bounding box of the label only, without taking into account its fields
 
     BOX2I                 box;
     std::vector<VECTOR2I> pts;
 
-    CreateGraphicShape( nullptr, pts, GetTextPos() );
+    CreateGraphicShape( aSettings, pts, GetTextPos() );
 
     for( const VECTOR2I& pt : pts )
         box.Merge( pt );
@@ -946,7 +994,7 @@ const BOX2I SCH_LABEL_BASE::GetBoundingBox() const
 {
     // build the bounding box of the entire label, including its fields
 
-    BOX2I box = GetBodyBoundingBox();
+    BOX2I box = GetBodyBoundingBox( nullptr );
 
     for( const SCH_FIELD& field : m_fields )
     {
@@ -969,7 +1017,7 @@ const BOX2I SCH_LABEL_BASE::GetBoundingBox() const
 
 bool SCH_LABEL_BASE::HitTest( const VECTOR2I& aPosition, int aAccuracy ) const
 {
-    BOX2I bbox = GetBodyBoundingBox();
+    BOX2I bbox = GetBodyBoundingBox( nullptr );
     bbox.Inflate( aAccuracy );
 
     if( bbox.Contains( aPosition ) )
@@ -1006,7 +1054,7 @@ bool SCH_LABEL_BASE::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy
     }
     else
     {
-        if( rect.Intersects( GetBodyBoundingBox() ) )
+        if( rect.Intersects( GetBodyBoundingBox( nullptr ) ) )
             return true;
 
         for( const SCH_FIELD& field : m_fields )
@@ -1242,21 +1290,17 @@ void SCH_LABEL_BASE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_O
     int              layer = ( connection && connection->IsBus() ) ? LAYER_BUS : m_layer;
     COLOR4D          color = settings->GetLayerColor( layer );
     int              penWidth = GetEffectiveTextPenWidth( settings->GetDefaultPenWidth() );
-    COLOR4D          labelColor = GetLabelColor();
 
-    if( aPlotter->GetColorMode() && labelColor != COLOR4D::UNSPECIFIED )
-        color = labelColor;
+    if( aPlotter->GetColorMode() && GetLabelColor() != COLOR4D::UNSPECIFIED )
+        color = GetLabelColor();
 
     penWidth = std::max( penWidth, settings->GetMinPenWidth() );
     aPlotter->SetCurrentLineWidth( penWidth );
 
-    KIFONT::FONT* font = GetFont();
+    KIFONT::FONT* font = GetDrawFont( settings );
 
-    if( !font )
-        font = KIFONT::FONT::GetFont( settings->GetDefaultFont(), IsBold(), IsItalic() );
-
-    VECTOR2I textpos = GetTextPos() + GetSchematicTextOffset( aPlotter->RenderSettings() );
-    CreateGraphicShape( aPlotter->RenderSettings(), s_poly, GetTextPos() );
+    VECTOR2I textpos = GetTextPos() + GetSchematicTextOffset( settings );
+    CreateGraphicShape( settings, s_poly, GetTextPos() );
 
     TEXT_ATTRIBUTES attrs = GetAttributes();
     attrs.m_StrokeWidth = penWidth;
@@ -1268,8 +1312,17 @@ void SCH_LABEL_BASE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_O
     }
     else
     {
-        aPlotter->PlotText( textpos, color, GetShownText( sheet, true ), attrs, font,
-                            GetFontMetrics() );
+        aPlotter->PlotText( textpos, color, GetShownText( sheet, true ), attrs, font, GetFontMetrics() );
+
+        if( aPlotter->GetColorMode() )
+        {
+            // For the graphic shape use the override color or the layer color, but not the
+            // net/netclass color.
+            if( GetTextColor() != COLOR4D::UNSPECIFIED )
+                aPlotter->SetColor( GetTextColor() );
+            else
+                aPlotter->SetColor( settings->GetLayerColor( m_layer ) );
+        }
 
         if( GetShape() == LABEL_FLAG_SHAPE::F_DOT )
         {
@@ -1296,7 +1349,8 @@ void SCH_LABEL_BASE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_O
         }
 
         // Make sheet pins and hierarchical labels clickable hyperlinks
-        bool linkAlreadyPlotted = false;
+        bool  linkAlreadyPlotted = false;
+        BOX2I bodyBBox = GetBodyBoundingBox( settings );
 
         if( aPlotOpts.m_PDFHierarchicalLinks )
         {
@@ -1306,8 +1360,7 @@ void SCH_LABEL_BASE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_O
                 {
                     SCH_SHEET_PATH path = *sheet;
                     path.pop_back();
-                    aPlotter->HyperlinkBox( GetBodyBoundingBox(),
-                                            EDA_TEXT::GotoPageHref( path.GetPageNumber() ) );
+                    aPlotter->HyperlinkBox( bodyBBox, EDA_TEXT::GotoPageHref( path.GetPageNumber() ) );
                     linkAlreadyPlotted = true;
                 }
             }
@@ -1316,8 +1369,7 @@ void SCH_LABEL_BASE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_O
                 SCH_SHEET_PATH path = *sheet;
                 SCH_SHEET*     parent = static_cast<SCH_SHEET*>( m_parent );
                 path.push_back( parent );
-                aPlotter->HyperlinkBox( GetBodyBoundingBox(),
-                                        EDA_TEXT::GotoPageHref( path.GetPageNumber() ) );
+                aPlotter->HyperlinkBox( bodyBBox, EDA_TEXT::GotoPageHref( path.GetPageNumber() ) );
                 linkAlreadyPlotted = true;
             }
         }
@@ -1333,9 +1385,9 @@ void SCH_LABEL_BASE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_O
                                                            _( "Net" ),
                                                            connection->Name() ) );
 
-                properties.emplace_back(
-                        wxString::Format( wxT( "!%s = %s" ), _( "Resolved netclass" ),
-                                          GetEffectiveNetClass()->GetHumanReadableName() ) );
+                properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
+                                                           _( "Resolved netclass" ),
+                                                           GetEffectiveNetClass()->GetHumanReadableName() ) );
             }
 
             for( const SCH_FIELD& field : GetFields() )
@@ -1346,14 +1398,11 @@ void SCH_LABEL_BASE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_O
             }
 
             if( !properties.empty() )
-                aPlotter->HyperlinkMenu( GetBodyBoundingBox(), properties );
+                aPlotter->HyperlinkMenu( bodyBBox, properties );
         }
 
         if( Type() == SCH_HIER_LABEL_T )
-        {
-            aPlotter->Bookmark( GetBodyBoundingBox(), GetShownText( false ),
-                                _( "Hierarchical Labels" ) );
-        }
+            aPlotter->Bookmark( bodyBBox, GetShownText( false ), _( "Hierarchical Labels" ) );
     }
 
     for( SCH_FIELD& field : m_fields )
@@ -1453,9 +1502,9 @@ bool SCH_LABEL::Deserialize( const google::protobuf::Any &aContainer )
 }
 
 
-const BOX2I SCH_LABEL::GetBodyBoundingBox() const
+const BOX2I SCH_LABEL::GetBodyBoundingBox( const RENDER_SETTINGS* aSettings ) const
 {
-    BOX2I rect = GetTextBox();
+    BOX2I rect = GetTextBox( aSettings );
 
     rect.Offset( 0, -GetTextOffset() );
     rect.Inflate( GetEffectiveTextPenWidth() );
@@ -1942,7 +1991,7 @@ void SCH_GLOBALLABEL::CreateGraphicShape( const RENDER_SETTINGS* aRenderSettings
     int margin    = GetLabelBoxExpansion( aRenderSettings );
     int halfSize  = ( GetTextHeight() / 2 ) + margin;
     int linewidth = GetPenWidth();
-    int symb_len  = GetTextBox().GetWidth() + 2 * margin;
+    int symb_len  = GetTextBox( aRenderSettings ).GetWidth() + 2 * margin;
 
     int x = symb_len + linewidth + 3;
     int y = halfSize + linewidth + 3;
@@ -2077,7 +2126,7 @@ void SCH_HIERLABEL::CreateGraphicShape( const RENDER_SETTINGS* aSettings,
 }
 
 
-const BOX2I SCH_HIERLABEL::GetBodyBoundingBox() const
+const BOX2I SCH_HIERLABEL::GetBodyBoundingBox( const RENDER_SETTINGS* aSettings ) const
 {
     int penWidth = GetEffectiveTextPenWidth();
     int margin = GetTextOffset();
@@ -2086,7 +2135,7 @@ const BOX2I SCH_HIERLABEL::GetBodyBoundingBox() const
     int y  = GetTextPos().y;
 
     int height = GetTextHeight() + penWidth + margin;
-    int length = GetTextBox().GetWidth();
+    int length = GetTextBox( aSettings ).GetWidth();
 
     length += height;       // add height for triangular shapes
 
