@@ -52,6 +52,7 @@
 #include <dialogs/dialog_sim_format_value.h>
 #include <eeschema_settings.h>
 #include "kiplatform/app.h"
+#include <magic_enum.hpp>
 
 
 SIM_TRACE_TYPE operator|( SIM_TRACE_TYPE aFirst, SIM_TRACE_TYPE aSecond )
@@ -1478,8 +1479,15 @@ void SIMULATOR_FRAME_UI::UpdateTunerValue( const SCH_SHEET_PATH& aSheetPath, con
     }
 
     NULL_REPORTER devnull;
-    SIM_LIB_MGR   mgr( &m_schematicFrame->Prj(), &m_schematicFrame->Schematic() );
-    SIM_MODEL&    model = mgr.CreateModel( &aSheetPath, *symbol, devnull ).model;
+    SIM_LIB_MGR   mgr( &m_schematicFrame->Prj() );
+
+    std::vector<EMBEDDED_FILES*> embeddedFilesStack;
+    embeddedFilesStack.push_back( m_schematicFrame->Schematic().GetEmbeddedFiles() );
+    embeddedFilesStack.push_back( symbol->GetEmbeddedFiles() );
+
+    mgr.SetFilesStack( embeddedFilesStack );
+
+    SIM_MODEL& model = mgr.CreateModel( &aSheetPath, *symbol, true, 0, devnull ).model;
 
     const SIM_MODEL::PARAM* tunerParam = model.GetTunerParam();
 
@@ -2400,7 +2408,10 @@ SIM_TRACE_TYPE SIMULATOR_FRAME_UI::getXAxisType( SIM_TYPE aType ) const
     case ST_DC:    return SPT_SWEEP;
     case ST_TRAN:  return SPT_TIME;
     case ST_NOISE: return SPT_LIN_FREQUENCY;
-    default:       wxFAIL_MSG( wxS( "Unhandled simulation type" ) ); return SPT_UNKNOWN;
+
+    default:
+        wxFAIL_MSG( wxString::Format( wxS( "Unhandled simulation type: %d" ), (int) aType ) );
+        return SPT_UNKNOWN;
     }
 }
 
@@ -2828,7 +2839,8 @@ void SIMULATOR_FRAME_UI::OnSimRefresh( bool aFinal )
         }
 
         SIM_PLOT_TAB* plotTab = dynamic_cast<SIM_PLOT_TAB*>( simTab );
-        wxCHECK_RET( plotTab, wxT( "not a SIM_PLOT_TAB" ) );
+        wxCHECK_RET( plotTab, wxString::Format( wxT( "No SIM_PLOT_TAB for: %s" ),
+                                                magic_enum::enum_name( simType ) ) );
 
         struct TRACE_INFO
         {
