@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2016 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -51,24 +51,10 @@ public:
      * @param aFrameType must be either #FRAME_SCH_LIB_VIEWER or #FRAME_SCH_LIB_VIEWER_MODAL.
      * @param aLibrary is the library to open when starting (default = NULL).
      */
-    SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrameType,
-                         const wxString& aLibraryName = wxEmptyString );
+    SYMBOL_VIEWER_FRAME( KIWAY* aKiway, wxWindow* aParent );
 
     ~SYMBOL_VIEWER_FRAME();
 
-    /**
-     * Runs the symbol viewer as a modal dialog.
-     *
-     * @param aSymbol an optional FPID string to initialize the viewer with and to
-     *                return a selected footprint through.
-     */
-    bool ShowModal( wxString* aSymbol, wxWindow* aParent ) override;
-
-    /**
-     * Send the selected symbol back to the caller.
-     */
-    void FinishModal();
-
     void OnSize( wxSizeEvent& event ) override;
 
     /**
@@ -92,27 +78,16 @@ public:
     void CloseLibraryViewer( wxCommandEvent& event );
     void ReCreateHToolbar() override;
     void ReCreateVToolbar() override;
-    void ReCreateOptToolbar() override {}
 
     void ClickOnLibList( wxCommandEvent& event );
     void ClickOnSymbolList( wxCommandEvent& event );
-    void OnSelectSymbol( wxCommandEvent& aEvent );
 
     void LoadSettings( APP_SETTINGS_BASE* aCfg ) override;
     void SaveSettings( APP_SETTINGS_BASE* aCfg ) override;
 
     WINDOW_SETTINGS* GetWindowSettings( APP_SETTINGS_BASE* aCfg ) override;
 
-    void CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged ) override;
-
-    /**
-     * Set a filter to display only libraries and/or symbols which match the filter.
-     *
-     * @param aFilter is a filter to pass the allowed library name list and/or some other filter
-     *                see SCH_BASE_FRAME::SelectSymbolFromLibrary() for details.
-     *                if aFilter == NULL, remove all filtering.
-     */
-    void SetFilter( const SYMBOL_LIBRARY_FILTER* aFilter );
+    void CommonSettingsChanged( int aFlags ) override;
 
     /**
      * Set the selected library in the library window.
@@ -130,11 +105,11 @@ public:
      * Set unit and convert, and set flag preventing them from automatically resetting to 1.
      *
      * @param aUnit is the unit, if invalid will be set to 1.
-     * @param aConvert is the alternate body style, if invalid will be set to 1.
+     * @param aBodyStyle is the alternate body style, if invalid will be set to 1.
      */
-    void SetUnitAndConvert( int aUnit, int aConvert );
+    void SetUnitAndBodyStyle( int aUnit, int aBodyStyle );
     int GetUnit() const { return m_unit; }
-    int GetConvert() const { return m_convert; }
+    int GetBodyStyle() const { return m_bodyStyle; }
 
     LIB_SYMBOL* GetSelectedSymbol() const;
 
@@ -193,16 +168,16 @@ private:
     static LIB_ID       m_currentSymbol;
 
     static int          m_unit;
-    static int          m_convert;
+    static int          m_bodyStyle;
     static bool         m_show_progress;
 
     /**
      * Updated to `true` if a list rewrite on GUI activation resulted in the symbol
      * selection changing, or if the user has changed the selection manually.
      */
-    bool m_selection_changed;
+    bool                m_selection_changed;
 
-    LIB_SYMBOL* m_previewItem;
+    std::unique_ptr<LIB_SYMBOL> m_previewItem;
 
     DECLARE_EVENT_TABLE()
 };
