@@ -811,7 +811,9 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         //------------------------------------------------------------------------
         // Handle cancel
         //
-        else if( evt->IsCancelInteractive() || evt->IsActivate() )
+        else if( evt->IsCancelInteractive()
+                 || evt->IsActivate()
+                 || evt->IsAction( &ACTIONS::undo ) )
         {
             if( evt->IsCancelInteractive() )
                 m_frame->GetInfoBar()->Dismiss();
@@ -843,18 +845,15 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         //------------------------------------------------------------------------
         // Handle TOOL_ACTION special cases
         //
-        else if( evt->Action() == TA_UNDO_REDO_PRE )
-        {
-            unselect = true;
-            break;
-        }
         else if( evt->IsAction( &ACTIONS::doDelete ) )
         {
             evt->SetPassEvent();
             // Exit on a delete; there will no longer be anything to drag.
             break;
         }
-        else if( evt->IsAction( &ACTIONS::duplicate ) )
+        else if( evt->IsAction( &ACTIONS::duplicate )
+                 || evt->IsAction( &SCH_ACTIONS::repeatDrawItem )
+                 || evt->IsAction( &ACTIONS::redo ) )
         {
             wxBell();
         }
@@ -868,8 +867,31 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         }
         else if( evt->IsAction( &ACTIONS::increment ) )
         {
-            m_toolMgr->RunSynchronousAction( ACTIONS::increment, aCommit,
-                                             evt->Parameter<ACTIONS::INCREMENT>() );
+            m_toolMgr->RunSynchronousAction( ACTIONS::increment, aCommit, evt->Parameter<ACTIONS::INCREMENT>() );
+        }
+        else if( evt->IsAction( &SCH_ACTIONS::toCLabel ) )
+        {
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::toCLabel, aCommit );
+        }
+        else if( evt->IsAction( &SCH_ACTIONS::toGLabel ) )
+        {
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::toGLabel, aCommit );
+        }
+        else if( evt->IsAction( &SCH_ACTIONS::toHLabel ) )
+        {
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::toHLabel, aCommit );
+        }
+        else if( evt->IsAction( &SCH_ACTIONS::toLabel ) )
+        {
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::toLabel, aCommit );
+        }
+        else if( evt->IsAction( &SCH_ACTIONS::toText ) )
+        {
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::toText, aCommit );
+        }
+        else if( evt->IsAction( &SCH_ACTIONS::toTextBox ) )
+        {
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::toTextBox, aCommit );
         }
         else if( evt->Action() == TA_CHOICE_MENU_CHOICE )
         {
@@ -1002,7 +1024,7 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         for( EDA_ITEM* item : selection )
             m_frame->AutoRotateItem( m_frame->GetScreen(), static_cast<SCH_ITEM*>( item ) );
 
-        m_frame->SchematicCleanUp( aCommit );
+        m_frame->Schematic().CleanUp( aCommit );
     }
 
     for( EDA_ITEM* item : m_frame->GetScreen()->Items() )
@@ -1030,7 +1052,7 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
 void SCH_MOVE_TOOL::trimDanglingLines( SCH_COMMIT* aCommit )
 {
     // Need a local cleanup first to ensure we remove unneeded junctions
-    m_frame->SchematicCleanUp( aCommit, m_frame->GetScreen() );
+    m_frame->Schematic().CleanUp( aCommit, m_frame->GetScreen() );
 
     std::set<SCH_ITEM*> danglers;
 
@@ -1143,8 +1165,7 @@ void SCH_MOVE_TOOL::getConnectedItems( SCH_ITEM* aOriginalItem, const VECTOR2I&
                     if( pin->IsConnected( aPoint ) )
                     {
                         if( pin->IsSelected() )
-                            m_specialCaseSheetPins[pin] = { line,
-                                                            line->GetStartPoint() == aPoint };
+                            m_specialCaseSheetPins[pin] = { line, line->GetStartPoint() == aPoint };
 
                         aList.push_back( pin );
                     }
@@ -1263,7 +1284,8 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_COMMIT* aCommit, SCH_ITEM* aSelec
 
             for( SCH_SHEET_PIN* pin : sheet->GetPins() )
             {
-                if( !pin->IsSelected() && pin->GetPosition() == aSelectedItem->GetPosition()
+                if( !pin->IsSelected()
+                    && pin->GetPosition() == aSelectedItem->GetPosition()
                     && pin->CanConnect( aSelectedItem ) )
                 {
                     itemsConnectable.push_back( pin );
@@ -1275,7 +1297,8 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_COMMIT* aCommit, SCH_ITEM* aSelec
 
         // Skip ourselves, skip already selected items (but not lines, they need both ends tested)
         // and skip unconnectable items
-        if( item == aSelectedItem || ( item->Type() != SCH_LINE_T && item->IsSelected() )
+        if( item == aSelectedItem
+            || ( item->Type() != SCH_LINE_T && item->IsSelected() )
             || !item->CanConnect( aSelectedItem ) )
         {
             continue;
@@ -1482,10 +1505,8 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_COMMIT* aCommit, SCH_ITEM* aSelec
 
                 if( line->HitTest( label->GetTextPos(), 1 ) )
                 {
-                    if( ( !line->HasFlag( STARTPOINT )
-                          && label->GetPosition() == line->GetStartPoint() )
-                        || ( !line->HasFlag( ENDPOINT )
-                             && label->GetPosition() == line->GetEndPoint() ) )
+                    if(    ( !line->HasFlag( STARTPOINT ) && label->GetPosition() == line->GetStartPoint() )
+                        || ( !line->HasFlag( ENDPOINT ) && label->GetPosition() == line->GetEndPoint() ) )
                     {
                         //If we have a line selected at only one end, don't grab labels
                         //connected directly to the unselected endpoint
@@ -1528,7 +1549,7 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_COMMIT* aCommit, SCH_ITEM* aSelec
             {
                 SCH_LINE* line = static_cast<SCH_LINE*>( aSelectedItem );
 
-                if( ( !line->HasFlag( STARTPOINT ) && test->IsConnected( line->GetStartPoint() ) )
+                if(    ( !line->HasFlag( STARTPOINT ) && test->IsConnected( line->GetStartPoint() ) )
                     || ( !line->HasFlag( ENDPOINT ) && test->IsConnected( line->GetEndPoint() ) ) )
                 {
                     // If we have a line selected at only one end, don't grab bus entries
@@ -1819,7 +1840,7 @@ int SCH_MOVE_TOOL::AlignToGrid( const TOOL_EVENT& aEvent )
 
     m_toolMgr->PostEvent( EVENTS::SelectedItemsMoved );
 
-    m_frame->SchematicCleanUp( &commit );
+    m_frame->Schematic().CleanUp( &commit );
     commit.Push( _( "Align Items to Grid" ) );
     return 0;
 }
