@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2018 jean-pierre.charras jp.charras at wanadoo.fr
- * Copyright (C) 2013-2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,18 +22,15 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#ifndef BITMAP_BASE_H
-#define BITMAP_BASE_H
+#pragma once
 
 #include <wx/bitmap.h>
 #include <wx/image.h>
+
+#include <core/mirror.h>
 #include <kiid.h>
 #include <math/box2.h>
-
-namespace KIGFX
-{
-class COLOR4D;
-}
+#include <gal/color4d.h>
 
 class LINE_READER;
 class PLOTTER;
@@ -73,8 +70,6 @@ public:
 
     const wxImage* GetOriginalImageData() const { return m_originalImage; }
 
-    void SetImage( wxImage* aImage );
-
     double GetScale() const { return m_scale; }
     void SetScale( double aScale ) { m_scale = aScale; }
 
@@ -83,7 +78,7 @@ public:
     /**
      * Copy aItem image to this object and update #m_bitmap.
      */
-    void ImportData( BITMAP_BASE* aItem );
+    void ImportData( BITMAP_BASE& aItem );
 
     /**
      * This scaling factor depends on #m_pixelSizeIu and #m_scale.
@@ -109,12 +104,12 @@ public:
     /**
      * @return the size in pixels of the image
      */
-    wxSize GetSizePixels() const
+    VECTOR2I GetSizePixels() const
     {
         if( m_image )
-            return wxSize( m_image->GetWidth(), m_image->GetHeight() );
+            return VECTOR2I( m_image->GetWidth(), m_image->GetHeight() );
         else
-            return wxSize( 0, 0 );
+            return VECTOR2I( 0, 0 );
     }
 
     /**
@@ -134,7 +129,8 @@ public:
      */
     const BOX2I GetBoundingBox() const;
 
-    void DrawBitmap( wxDC* aDC, const VECTOR2I& aPos );
+    void DrawBitmap( wxDC* aDC, const VECTOR2I& aPos,
+                     const KIGFX::COLOR4D& aBackgroundColor = KIGFX::COLOR4D::UNSPECIFIED ) const;
 
     /**
      * Reads and stores in memory an image file.
@@ -162,25 +158,32 @@ public:
     bool ReadImageFile( wxInputStream& aInStream );
 
     /**
-     * Write the bitmap data to \a aFile.
+     * Reads and stores in memory an image file.
      *
-     * The file format is png, in hexadecimal form.  If the hexadecimal data is converted to
-     * binary it gives exactly a .png image data.
+     * Initialize the bitmap format used to draw this item.
      *
-     * @param aFile The FILE to write to.
-     * @return true if success writing else false.
+     * Supported images formats are format supported by wxImage if all handlers are loaded.
+     * By default, .png, .jpeg are always loaded.
+     *
+     * @param aBuf a memory buffer containing the file data.
+     * @return true if success reading else false.
      */
-    bool SaveData( FILE* aFile ) const;
+    bool ReadImageFile( wxMemoryBuffer& aBuf );
 
     /**
-     * Write the bitmap data to an array string.
-     *
-     * The format is png, in Hexadecimal form.  If the hexadecimal data is converted to binary
-     * it gives exactly a .png image data.
-     *
-     * @param aPngStrings The wxArrayString to write to.
+     * Set the image from an existing wxImage.
      */
-    void SaveData( wxArrayString& aPngStrings ) const;
+    bool SetImage( const wxImage& aImage );
+
+    /**
+    * Write the bitmap data to \a aOutStream.
+    *
+    * This writes binary data, not hexadecimal strings
+    *
+    * @param aOutStream The output stream to write to.
+    * @return true if success writing else false.
+    */
+    bool SaveImageData( wxOutputStream& aOutStream ) const;
 
     /**
      * Load an image data saved by #SaveData.
@@ -192,14 +195,14 @@ public:
      *                  png bitmap data.
      * @return true if the bitmap loaded successfully.
      */
-    bool LoadData( LINE_READER& aLine, wxString& aErrorMsg );
+    bool LoadLegacyData( LINE_READER& aLine, wxString& aErrorMsg );
 
     /**
      * Mirror image vertically (i.e. relative to its horizontal X axis ) or horizontally (i.e
      * relative to its vertical Y axis).
-     * @param aVertically false to mirror horizontally or true to mirror vertically.
+     * @param aFlipDirection the direction to flip the image.
      */
-    void Mirror( bool aVertically );
+    void Mirror( FLIP_DIRECTION aFlipDirection );
 
     /**
      * Rotate image CW or CCW.
@@ -210,7 +213,8 @@ public:
 
     void ConvertToGreyscale();
 
-    bool IsMirrored() const { return m_isMirrored; }
+    bool IsMirroredX() const { return m_isMirroredX; }
+    bool IsMirroredY() const { return m_isMirroredY; }
     EDA_ANGLE Rotation() const { return m_rotation; }
 
     /**
@@ -226,8 +230,23 @@ public:
     void PlotImage( PLOTTER* aPlotter, const VECTOR2I& aPos,
                     const KIGFX::COLOR4D& aDefaultColor, int aDefaultPensize ) const;
 
+    /**
+     * Return the bitmap type (png, jpeg, etc.)
+     */
+    wxBitmapType GetImageType() const { return m_imageType; }
+
+    /**
+     * Set the bitmap type (png, jpeg, etc.)
+     */
+    void SetImageType( wxBitmapType aType ) { m_imageType = aType; }
+
 private:
-    /*
+    /**
+     * Resets the image data buffer using the current image data.
+     */
+    void updateImageDataBuffer();
+
+    /**
      * Rebuild the internal bitmap used to draw/plot image.
      *
      * This must be called after a #m_image change.
@@ -238,20 +257,20 @@ private:
 
     void updatePPI();
 
-    double    m_scale;              // The scaling factor of the bitmap
-                                    // With m_pixelSizeIu, controls the actual draw size
-    wxImage*  m_image;              // the raw image data (png format)
-    wxImage*  m_originalImage;      // Raw image data, not transformed by rotate/mirror
-    wxBitmap* m_bitmap;             // the bitmap used to draw/plot image
-    double    m_pixelSizeIu;        // The scaling factor of the bitmap
-                                    // to convert the bitmap size (in pixels)
-                                    // to internal KiCad units
-                                    // Usually does not change
-    int       m_ppi;                // the bitmap definition. the default is 300PPI
+    double    m_scale;              ///< The scaling factor of the bitmap
+                                    ///< with #m_pixelSizeIu, controls the actual draw size.
+    wxMemoryBuffer m_imageData;     ///< The original image data in its original format.
+    wxBitmapType   m_imageType;     ///< The image type (png, jpeg, etc.).
+
+    wxImage*  m_image;              ///< The raw, uncompressed image data.
+    wxImage*  m_originalImage;      ///< Raw image data, not transformed by rotate/mirror.
+    wxBitmap* m_bitmap;             ///< The bitmap used to draw/plot image.
+    double    m_pixelSizeIu;        ///< The scaling factor of the bitmap to convert the bitmap
+                                    ///< size (in pixels) to internal KiCad units.  This usually
+                                    ///< does not change.
+    int       m_ppi;                ///< The bitmap definition. The default is 300PPI.
     KIID      m_imageId;
-    bool      m_isMirrored;         // Used for OpenGL rendering only
+    bool      m_isMirroredX;        // Used for OpenGL rendering only
+    bool      m_isMirroredY;        // Used for OpenGL rendering only
     EDA_ANGLE m_rotation;           // Used for OpenGL rendering only
 };
-
-
-#endif    // BITMAP_BASE_H
