@@ -48,12 +48,17 @@
 #include <wx/filedlg.h>
 #include <design_block_lib_table.h>
 #include "dialog_pcm.h"
+#include <project/project_archiver.h>
+#include <project_tree_pane.h>
+#include <project_tree.h>
+#include <launch_ext.h>
 
 #include "widgets/filedlg_new_project.h"
 
 KICAD_MANAGER_CONTROL::KICAD_MANAGER_CONTROL() :
         TOOL_INTERACTIVE( "kicad.Control" ),
-        m_frame( nullptr )
+        m_frame( nullptr ),
+        m_inShowPlayer( false )
 {
 }
 
@@ -161,11 +166,13 @@ int KICAD_MANAGER_CONTROL::NewFromRepository( const TOOL_EVENT& aEvent )
     if( !pro.IsOk() )
         return -1;
 
-    GIT_CLONE_HANDLER cloneHandler;
+    PROJECT_TREE_PANE *pane = static_cast<PROJECT_TREE_PANE*>( m_frame->GetToolCanvas() );
 
-    cloneHandler.SetURL( dlg.GetRepoURL() );
+
+    GIT_CLONE_HANDLER cloneHandler( pane->m_TreeProject->GitCommon() );
+
+    cloneHandler.SetRemote( dlg.GetFullURL() );
     cloneHandler.SetClonePath( pro.GetPath() );
-    cloneHandler.SetConnType( dlg.GetRepoType() );
     cloneHandler.SetUsername( dlg.GetUsername() );
     cloneHandler.SetPassword( dlg.GetPassword() );
     cloneHandler.SetSSHKey( dlg.GetRepoSSHPath() );
@@ -209,8 +216,7 @@ int KICAD_MANAGER_CONTROL::NewFromRepository( const TOOL_EVENT& aEvent )
 int KICAD_MANAGER_CONTROL::NewJobsetFile( const TOOL_EVENT& aEvent )
 {
     wxString     default_dir = wxFileName( Prj().GetProjectFullName() ).GetPathWithSep();
-    wxFileDialog dlg( m_frame, _( "Create New Jobset" ), default_dir, wxEmptyString,
-                      FILEEXT::JobsetFileWildcard(),
+    wxFileDialog dlg( m_frame, _( "Create New Jobset" ), default_dir, wxEmptyString, FILEEXT::JobsetFileWildcard(),
                       wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
     if( dlg.ShowModal() == wxID_CANCEL )
@@ -251,8 +257,8 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
         titleDirMap.emplace( _( "User Templates" ), templatePath );
     }
 
-    DIALOG_TEMPLATE_SELECTOR ps( m_frame, settings->m_TemplateWindowPos,
-                                 settings->m_TemplateWindowSize, titleDirMap );
+    DIALOG_TEMPLATE_SELECTOR ps( m_frame, settings->m_TemplateWindowPos, settings->m_TemplateWindowSize,
+                                 titleDirMap );
 
     // Show the project template selector dialog
     int result = ps.ShowModal();
@@ -265,8 +271,8 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
 
     if( !ps.GetSelectedTemplate() )
     {
-        wxMessageBox( _( "No project template was selected.  Cannot generate new project." ),
-                      _( "Error" ), wxOK | wxICON_ERROR, m_frame );
+        wxMessageBox( _( "No project template was selected.  Cannot generate new project." ), _( "Error" ),
+                      wxOK | wxICON_ERROR, m_frame );
 
         return -1;
     }
@@ -348,10 +354,8 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
             for( const wxFileName& file : overwrittenFiles )
                 extendedMsg += "\n" + file.GetFullName();
 
-            KIDIALOG msgDlg( m_frame,
-                             _( "Similar files already exist in the destination folder." ),
-                             _( "Confirmation" ),
-                             wxOK | wxCANCEL | wxICON_WARNING );
+            KIDIALOG msgDlg( m_frame, _( "Similar files already exist in the destination folder." ),
+                             _( "Confirmation" ), wxOK | wxCANCEL | wxICON_WARNING );
             msgDlg.SetExtendedMessage( extendedMsg );
             msgDlg.SetOKLabel( _( "Overwrite" ) );
             msgDlg.DoNotShowCheckbox( __FILE__, __LINE__ );
@@ -367,10 +371,8 @@ int KICAD_MANAGER_CONTROL::NewFromTemplate( const TOOL_EVENT& aEvent )
     // create a project
     if( !ps.GetSelectedTemplate()->CreateProject( fn, &errorMsg ) )
     {
-        wxMessageDialog createDlg( m_frame,
-                                   _( "A problem occurred creating new project from template." ),
-                                   _( "Error" ),
-                                   wxOK | wxICON_ERROR );
+        wxMessageDialog createDlg( m_frame, _( "A problem occurred creating new project from template." ),
+                                   _( "Error" ), wxOK | wxICON_ERROR );
 
         if( !errorMsg.empty() )
             createDlg.SetExtendedMessage( errorMsg );
@@ -402,8 +404,13 @@ int KICAD_MANAGER_CONTROL::openProject( const wxString& aDefaultDir )
     if( !pro.IsAbsolute() )
         pro.MakeAbsolute();
 
-    if( !pro.FileExists() )
+    // You'd think wxFD_FILE_MUST_EXIST and the wild-cards would enforce these.  Sentry
+    // indicates otherwise (at least on MSW).
+    if( !pro.Exists() || (   pro.GetExt() != FILEEXT::ProjectFileExtension
+                          && pro.GetExt() != FILEEXT::LegacyProjectFileExtension ) )
+    {
         return -1;
+    }
 
     m_frame->LoadProject( pro );
 
@@ -426,8 +433,8 @@ int KICAD_MANAGER_CONTROL::OpenProject( const TOOL_EVENT& aEvent )
 int KICAD_MANAGER_CONTROL::OpenJobsetFile( const TOOL_EVENT& aEvent )
 {
     wxString     default_dir = wxFileName( Prj().GetProjectFullName() ).GetPathWithSep();
-    wxFileDialog dlg( m_frame, _( "Open Jobset" ), default_dir, wxEmptyString,
-                      FILEEXT::JobsetFileWildcard(), wxFD_OPEN | wxFD_FILE_MUST_EXIST );
+    wxFileDialog dlg( m_frame, _( "Open Jobset" ), default_dir, wxEmptyString, FILEEXT::JobsetFileWildcard(),
+                      wxFD_OPEN | wxFD_FILE_MUST_EXIST );
 
     if( dlg.ShowModal() == wxID_CANCEL )
         return -1;
@@ -534,7 +541,7 @@ public:
                  || ext == FILEEXT::LegacySymbolDocumentFileExtension
                  || ext == FILEEXT::KiCadSymbolLibFileExtension
                  || ext == FILEEXT::NetlistFileExtension
-               || destFile.GetName() == FILEEXT::SymbolLibraryTableFileName )
+                 || destFile.GetName() == FILEEXT::SymbolLibraryTableFileName )
         {
             KIFACE* eeschema = m_frame->Kiway().KiFACE( KIWAY::FACE_SCH );
             eeschema->SaveFileAs( m_projectDirPath, m_projectName, m_newProjectDirPath,
@@ -546,7 +553,7 @@ public:
                  || ext == FILEEXT::KiCadFootprintFileExtension
                  || ext == FILEEXT::LegacyFootprintLibPathExtension
                  || ext == FILEEXT::FootprintAssignmentFileExtension
-               || destFile.GetName() == FILEEXT::FootprintLibraryTableFileName )
+                 || destFile.GetName() == FILEEXT::FootprintLibraryTableFileName )
         {
             KIFACE* pcbnew = m_frame->Kiway().KiFACE( KIWAY::FACE_PCB );
             pcbnew->SaveFileAs( m_projectDirPath, m_projectName, m_newProjectDirPath,
@@ -679,8 +686,7 @@ int KICAD_MANAGER_CONTROL::SaveProjectAs( const TOOL_EVENT& aEvent )
         default_dir = default_dir_fn.GetPath();
     }
 
-    wxFileDialog dlg( m_frame, _( "Save Project To" ), default_dir, wxEmptyString, wxEmptyString,
-                      wxFD_SAVE );
+    wxFileDialog dlg( m_frame, _( "Save Project To" ), default_dir, wxEmptyString, wxEmptyString, wxFD_SAVE );
 
     if( dlg.ShowModal() == wxID_CANCEL )
         return -1;
@@ -719,8 +725,8 @@ int KICAD_MANAGER_CONTROL::SaveProjectAs( const TOOL_EVENT& aEvent )
     const wxString&   newProjectName = newProjectDir.GetDirs().Last();
     wxDir             currentProjectDir( currentProjectDirPath );
 
-    SAVE_AS_TRAVERSER traverser( m_frame, currentProjectDirPath, currentProjectName,
-                                 newProjectDirPath, newProjectName );
+    SAVE_AS_TRAVERSER traverser( m_frame, currentProjectDirPath, currentProjectName, newProjectDirPath,
+                                 newProjectName );
 
     currentProjectDir.Traverse( traverser );
 
@@ -766,22 +772,19 @@ int KICAD_MANAGER_CONTROL::ShowPlayer( const TOOL_EVENT& aEvent )
 
     if( playerType == FRAME_SCH && !m_frame->IsProjectActive() )
     {
-        DisplayInfoMessage( m_frame, _( "Create (or open) a project to edit a schematic." ),
-                            wxEmptyString );
+        DisplayInfoMessage( m_frame, _( "Create (or open) a project to edit a schematic." ), wxEmptyString );
         return -1;
     }
     else if( playerType == FRAME_PCB_EDITOR && !m_frame->IsProjectActive() )
     {
-        DisplayInfoMessage( m_frame, _( "Create (or open) a project to edit a pcb." ),
-                            wxEmptyString );
+        DisplayInfoMessage( m_frame, _( "Create (or open) a project to edit a pcb." ), wxEmptyString );
         return -1;
     }
 
-    // Prevent multiple KIWAY_PLAYER loading at one time
-    if( !m_loading.try_lock() )
+    if( m_inShowPlayer )
         return -1;
 
-    const std::lock_guard<std::mutex> lock( m_loading, std::adopt_lock );
+    REENTRANCY_GUARD guard( &m_inShowPlayer );
 
     try
     {
