@@ -469,6 +469,11 @@ public:
      */
     void DeleteAllFootprints();
 
+    /**
+     * Remove all footprints without deleting.  Footprint lifecycle MUST be managed elsewhere.
+     */
+    void DetachAllFootprints();
+
     /**
      * @return null if aID is null. Returns an object of Type() == NOT_USED if the aID is not found.
      */
@@ -602,7 +607,7 @@ public:
      *
      * @return the enabled layers in bit-mapped form.
      */
-    LSET GetEnabledLayers() const;
+    const LSET& GetEnabledLayers() const;
     LSET GetLayerSet() const override { return GetEnabledLayers(); }
 
     /**
@@ -610,7 +615,7 @@ public:
      *
      * @param aLayerMask the new bit-mask of enabled layers.
      */
-    void SetEnabledLayers( LSET aLayerMask );
+    void SetEnabledLayers( const LSET& aLayerMask );
     void SetLayerSet( const LSET& aLayerMask ) override { SetEnabledLayers( aLayerMask ); }
 
     /**
@@ -635,7 +640,7 @@ public:
      *
      * @return the visible layers in bit-mapped form.
      */
-    LSET  GetVisibleLayers() const;
+    const LSET& GetVisibleLayers() const;
 
     /**
      * A proxy function that calls the correspondent function in m_BoardSettings
@@ -643,7 +648,7 @@ public:
      *
      * @param aLayerMask is the new bit-mask of visible layers.
      */
-    void SetVisibleLayers( LSET aLayerMask );
+    void SetVisibleLayers( const LSET& aLayerMask );
 
     // these 2 functions are not tidy at this time, since there are PCB_LAYER_IDs that
     // are not stored in the bitmap.
@@ -817,6 +822,16 @@ public:
      */
     bool SetLayerDescr( PCB_LAYER_ID aIndex, const LAYER& aLayer );
 
+    /**
+     * @return true if the layer is a front layer, or a user layer designated "Off-board, front"
+     */
+    bool IsFrontLayer( PCB_LAYER_ID aLayer ) const;
+
+    /**
+     * @return true if the layer is a back layer, or a user layer designated "Off-board, back"
+     */
+    bool IsBackLayer( PCB_LAYER_ID aLayer ) const;
+
     /**
      * Return the type of the copper layer given by aLayer.
      *
@@ -1067,21 +1082,6 @@ public:
 
     /* Functions used in test, merge and cut outlines */
 
-    /**
-     * Add an empty copper area to board areas list.
-     *
-     * @param aNewZonesList is a PICKED_ITEMS_LIST * where to store new areas  pickers (useful
-     *                      in undo commands) can be NULL.
-     * @param aNetcode is the netcode of the copper area (0 = no net).
-     * @param aLayer is the layer of area.
-     * @param aStartPointPosition is position of the first point of the polygon outline of this
-     *        area.
-     * @param aHatch is the hatch option.
-     * @return a reference to the new area.
-     */
-    ZONE* AddArea( PICKED_ITEMS_LIST* aNewZonesList, int aNetcode, PCB_LAYER_ID aLayer,
-                   VECTOR2I aStartPointPosition, ZONE_BORDER_DISPLAY_STYLE aHatch );
-
     /**
      * Test for intersection of 2 copper areas.
      *
@@ -1113,17 +1113,6 @@ public:
      */
     PAD* GetPad( const PCB_TRACK* aTrace, ENDPOINT_T aEndPoint ) const;
 
-    /**
-     * Return pad found at \a aPosition on \a aLayerMask using the fast search method.
-     * <p>
-     * The fast search method only works if the pad list has already been built.
-     * </p>
-     * @param aPosition A VECTOR2I object containing the position to hit test.
-     * @param aLayerMask A layer or layers to mask the hit test.
-     * @return A pointer to a PAD object if found or NULL if not found.
-     */
-    PAD* GetPadFast( const VECTOR2I& aPosition, LSET aLayerMask ) const;
-
     /**
      * Locate the pad connected at \a aPosition on \a aLayer starting at list position
      * \a aPad
@@ -1283,6 +1272,8 @@ public:
     EMBEDDED_FILES* GetEmbeddedFiles() override;
     const EMBEDDED_FILES* GetEmbeddedFiles() const;
 
+    void SetEmbeddedFilesDelegate( EMBEDDED_FILES* aDelegate ) { m_embeddedFilesDelegate = aDelegate; }
+
     /**
      * Get the list of all outline fonts used in the board
      */
@@ -1414,6 +1405,10 @@ private:
 
     bool                         m_embedFonts;
 
+    // Used for dummy boards, such as a footprint holder, where we don't want to make a copy
+    // of all the parent's embedded data.
+    EMBEDDED_FILES*              m_embeddedFilesDelegate;
+
     COMPONENT_CLASS_MANAGER m_componentClassManager;
 };
 
