@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2016 CERN
- * Copyright (C) 2020-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -24,8 +24,11 @@
  */
 
 #include <macros.h>
+#include <pgm_base.h>
+#include <settings/settings_manager.h>
 #include <board.h>
 #include <footprint.h>
+#include <lset.h>
 #include <pcb_group.h>
 #include <pcb_track.h>
 #include <tool/tool_manager.h>
@@ -64,7 +67,7 @@ BOARD_COMMIT::BOARD_COMMIT( EDA_DRAW_FRAME* aFrame ) :
 
 
 BOARD_COMMIT::BOARD_COMMIT( TOOL_MANAGER* aMgr ) :
-        m_toolMgr( aMgr ),
+    m_toolMgr( aMgr ),
         m_isBoardEditor( false ),
         m_isFootprintEditor( false )
 {
@@ -76,6 +79,13 @@ BOARD_COMMIT::BOARD_COMMIT( TOOL_MANAGER* aMgr ) :
         m_isFootprintEditor = true;
 }
 
+BOARD_COMMIT::BOARD_COMMIT( TOOL_MANAGER* aMgr, bool aIsBoardEditor ) :
+    m_toolMgr( aMgr ),
+    m_isBoardEditor( aIsBoardEditor ),
+    m_isFootprintEditor( false )
+{
+}
+
 
 BOARD* BOARD_COMMIT::GetBoard() const
 {
@@ -85,8 +95,6 @@ BOARD* BOARD_COMMIT::GetBoard() const
 
 COMMIT& BOARD_COMMIT::Stage( EDA_ITEM* aItem, CHANGE_TYPE aChangeType, BASE_SCREEN* aScreen )
 {
-    wxCHECK( aItem, *this );
-
     // Many operations (move, rotate, etc.) are applied directly to a group's children, so they
     // must be staged as well.
     if( aChangeType == CHT_MODIFY )
@@ -119,37 +127,46 @@ COMMIT& BOARD_COMMIT::Stage( const PICKED_ITEMS_LIST& aItems, UNDO_REDO aModFlag
 }
 
 
-void BOARD_COMMIT::dirtyIntersectingZones( BOARD_ITEM* item, int aChangeType )
+void BOARD_COMMIT::propagateDamage( BOARD_ITEM* aChangedItem, std::vector<ZONE*>* aStaleZones,
+                                    std::vector<BOX2I>& aStaleRuleAreas )
 {
-    wxCHECK( item, /* void */ );
+    wxCHECK( aChangedItem, /* void */ );
 
-    ZONE_FILLER_TOOL* zoneFillerTool = m_toolMgr->GetTool<ZONE_FILLER_TOOL>();
+    if( aStaleZones && aChangedItem->Type() == PCB_ZONE_T )
+        aStaleZones->push_back( static_cast<ZONE*>( aChangedItem ) );
 
-    if( item->Type() == PCB_ZONE_T )
-        zoneFillerTool->DirtyZone( static_cast<ZONE*>( item ) );
-
-    item->RunOnChildren( std::bind( &BOARD_COMMIT::dirtyIntersectingZones, this, _1, aChangeType ) );
+    aChangedItem->RunOnChildren( std::bind( &BOARD_COMMIT::propagateDamage, this, _1, aStaleZones, aStaleRuleAreas ) );
 
     BOARD* board = static_cast<BOARD*>( m_toolMgr->GetModel() );
-    BOX2I  bbox = item->GetBoundingBox();
-    LSET   layers = item->GetLayerSet();
+    BOX2I  damageBBox = aChangedItem->GetBoundingBox();
+    LSET   damageLayers = aChangedItem->GetLayerSet();
 
-    if( layers.test( Edge_Cuts ) || layers.test( Margin ) )
-        layers = LSET::PhysicalLayersMask();
-    else
-        layers &= LSET::AllCuMask();
-
-    if( layers.any() )
+    if( m_isBoardEditor && aChangedItem->Type() == PCB_ZONE_T )
     {
-        for( ZONE* zone : board->Zones() )
-        {
-            if( zone->GetIsRuleArea() )
-                continue;
+        // A named zone can have custom DRC rules targetting it.
+        if( !static_cast<ZONE*>( aChangedItem )->GetZoneName().IsEmpty() )
+            aStaleRuleAreas.push_back( damageBBox );
+    }
 
-            if( ( zone->GetLayerSet() & layers ).any()
-                    && zone->GetBoundingBox().Intersects( bbox ) )
+    if( aStaleZones )
+    {
+        if( damageLayers.test( Edge_Cuts ) || damageLayers.test( Margin ) )
+            damageLayers = LSET::PhysicalLayersMask();
+        else
+            damageLayers &= LSET::AllCuMask();
+
+        if( damageLayers.any() )
+        {
+            for( ZONE* zone : board->Zones() )
             {
-                zoneFillerTool->DirtyZone( zone );
+                if( zone->GetIsRuleArea() )
+                    continue;
+
+                if( ( zone->GetLayerSet() & damageLayers ).any()
+                        && zone->GetBoundingBox().Intersects( damageBBox ) )
+                {
+                    aStaleZones->push_back( zone );
+                }
             }
         }
     }
@@ -158,7 +175,7 @@ void BOARD_COMMIT::dirtyIntersectingZones( BOARD_ITEM* item, int aChangeType )
 
 void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
 {
-    KIGFX::VIEW*        view = m_toolMgr->GetView();
+    KIGFX::PCB_VIEW*    view = static_cast<KIGFX::PCB_VIEW*>( m_toolMgr->GetView() );
     BOARD*              board = static_cast<BOARD*>( m_toolMgr->GetModel() );
     PCB_BASE_FRAME*     frame = dynamic_cast<PCB_BASE_FRAME*>( m_toolMgr->GetToolHolder() );
     PCB_SELECTION_TOOL* selTool = m_toolMgr->GetTool<PCB_SELECTION_TOOL>();
@@ -173,6 +190,9 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
     bool                     autofillZones = false;
     std::vector<BOARD_ITEM*> staleTeardropPadsAndVias;
     std::set<PCB_TRACK*>     staleTeardropTracks;
+    std::vector<ZONE*>       staleZonesStorage;
+    std::vector<ZONE*>*      staleZones = nullptr;
+    std::vector<BOX2I>       staleRuleAreas;
     PCB_GROUP*               addedGroup = nullptr;
 
     if( Empty() )
@@ -194,6 +214,7 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             && ( frame && frame->GetPcbNewSettings()->m_AutoRefillZones ) )
     {
         autofillZones = true;
+        staleZones = &staleZonesStorage;
 
         for( ZONE* zone : board->Zones() )
             zone->CacheBoundingBox();
@@ -297,8 +318,8 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             if( boardItem->Type() == PCB_GROUP_T || boardItem->Type() == PCB_GENERATOR_T )
                 addedGroup = static_cast<PCB_GROUP*>( boardItem );
 
-            if( autofillZones && boardItem->Type() != PCB_MARKER_T )
-                dirtyIntersectingZones( boardItem, changeType );
+            if( boardItem->Type() != PCB_MARKER_T )
+                propagateDamage( boardItem, staleZones, staleRuleAreas );
 
             if( view && boardItem->Type() != PCB_NETINFO_T )
                 view->Add( boardItem );
@@ -327,8 +348,8 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             if( parentFP && !( parentFP->GetFlags() & STRUCT_DELETED ) )
                 ent.m_parent = parentFP->m_Uuid;
 
-            if( autofillZones )
-                dirtyIntersectingZones( boardItem, changeType );
+            if( boardItem->Type() != PCB_MARKER_T )
+                propagateDamage( boardItem, staleZones, staleRuleAreas );
 
             switch( boardItem->Type() )
             {
@@ -341,7 +362,8 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
             case PCB_SHAPE_T:            // a shape (normally not on copper layers)
             case PCB_REFERENCE_IMAGE_T:  // a bitmap on an associated layer
             case PCB_GENERATOR_T:        // a generator on a layer
-            case PCB_TEXTBOX_T:          // a wrapped text on a layer
+            case PCB_TEXTBOX_T:          // a line-wrapped (and optionally bordered) text item
+            case PCB_TABLE_T:            // rows and columns of tablecells
             case PCB_TRACE_T:            // a track segment (segment on a copper layer)
             case PCB_ARC_T:              // an arced track segment (segment on a copper layer)
             case PCB_VIA_T:              // a via (like track segment on a copper layer)
@@ -439,10 +461,10 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
                 connectivity->Update( boardItem );
             }
 
-            if( m_isBoardEditor && autofillZones )
+            if( boardItem->Type() != PCB_MARKER_T )
             {
-                dirtyIntersectingZones( boardItemCopy, changeType );   // before
-                dirtyIntersectingZones( boardItem, changeType );       // after
+                propagateDamage( boardItemCopy, staleZones, staleRuleAreas );   // before
+                propagateDamage( boardItem, staleZones, staleRuleAreas );       // after
             }
 
             if( view )
@@ -497,6 +519,15 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
                 frame->HideSolderMask();
         }
 
+        PCBNEW_SETTINGS* settings = Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>( "pcbnew" );
+
+        if( !staleRuleAreas.empty() && (   settings->m_Display.m_TrackClearance == SHOW_WITH_VIA_ALWAYS
+                                        || settings->m_Display.m_PadClearance ) )
+        {
+            if( view )
+                view->UpdateCollidingItems( staleRuleAreas, { PCB_TRACE_T, PCB_ARC_T, PCB_VIA_T, PCB_PAD_T } );
+        }
+
         if( !staleTeardropPadsAndVias.empty() || !staleTeardropTracks.empty() )
         {
             teardropMgr.UpdateTeardrops( *this, &staleTeardropPadsAndVias, &staleTeardropTracks );
@@ -543,14 +574,8 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
         }
     }
 
-    if( bulkAddedItems.size() > 0 )
-        board->FinalizeBulkAdd( bulkAddedItems );
-
-    if( bulkRemovedItems.size() > 0 )
-        board->FinalizeBulkRemove( bulkRemovedItems );
-
-    if( itemsChanged.size() > 0 )
-        board->OnItemsChanged( itemsChanged );
+    if( bulkAddedItems.size() > 0 || bulkRemovedItems.size() > 0 || itemsChanged.size() > 0 )
+        board->OnItemsCompositeUpdate( bulkAddedItems, bulkRemovedItems, itemsChanged );
 
     if( frame )
     {
@@ -569,7 +594,14 @@ void BOARD_COMMIT::Push( const wxString& aMessage, int aCommitFlags )
         m_toolMgr->PostEvent( EVENTS::UnselectedEvent );
 
     if( autofillZones )
+    {
+        ZONE_FILLER_TOOL* zoneFillerTool = m_toolMgr->GetTool<ZONE_FILLER_TOOL>();
+
+        for( ZONE* zone : *staleZones )
+            zoneFillerTool->DirtyZone( zone );
+
         m_toolMgr->PostAction( PCB_ACTIONS::zoneFillDirty );
+    }
 
     if( selectedModified )
         m_toolMgr->ProcessEvent( EVENTS::SelectedItemsModified );
@@ -633,16 +665,14 @@ void BOARD_COMMIT::Revert()
     std::vector<BOARD_ITEM*> bulkRemovedItems;
     std::vector<BOARD_ITEM*> itemsChanged;
 
-    for( auto it = m_changes.rbegin(); it != m_changes.rend(); ++it )
+    for( COMMIT_LINE& entry : m_changes )
     {
-        COMMIT_LINE& ent = *it;
-
-        if( !ent.m_item || !ent.m_item->IsBOARD_ITEM() )
+        if( !entry.m_item || !entry.m_item->IsBOARD_ITEM() )
             continue;
 
-        BOARD_ITEM*  boardItem = static_cast<BOARD_ITEM*>( ent.m_item );
-        int          changeType = ent.m_type & CHT_TYPE;
-        int          changeFlags = ent.m_type & CHT_FLAGS;
+        BOARD_ITEM*  boardItem = static_cast<BOARD_ITEM*>( entry.m_item );
+        int          changeType = entry.m_type & CHT_TYPE;
+        int          changeFlags = entry.m_type & CHT_FLAGS;
 
         switch( changeType )
         {
@@ -681,7 +711,9 @@ void BOARD_COMMIT::Revert()
             view->Add( boardItem );
             connectivity->Add( boardItem );
 
-            BOARD_ITEM* parent = board->GetItem( ent.m_parent );
+            // Note: parent can be nullptr, because entry.m_parent is only set for children
+            // of footprints.
+            BOARD_ITEM* parent = board->GetItem( entry.m_parent );
 
             if( parent && parent->Type() == PCB_FOOTPRINT_T )
             {
@@ -701,8 +733,8 @@ void BOARD_COMMIT::Revert()
             view->Remove( boardItem );
             connectivity->Remove( boardItem );
 
-            wxASSERT( ent.m_copy && ent.m_copy->IsBOARD_ITEM() );
-            BOARD_ITEM* boardItemCopy = static_cast<BOARD_ITEM*>( ent.m_copy );
+            wxASSERT( entry.m_copy && entry.m_copy->IsBOARD_ITEM() );
+            BOARD_ITEM* boardItemCopy = static_cast<BOARD_ITEM*>( entry.m_copy );
             boardItem->SwapItemData( boardItemCopy );
 
             if( PCB_GROUP* group = dynamic_cast<PCB_GROUP*>( boardItem ) )
@@ -718,7 +750,7 @@ void BOARD_COMMIT::Revert()
             connectivity->Add( boardItem );
             itemsChanged.push_back( boardItem );
 
-            delete ent.m_copy;
+            delete entry.m_copy;
             break;
         }
 
@@ -730,14 +762,8 @@ void BOARD_COMMIT::Revert()
         boardItem->ClearEditFlags();
     }
 
-    if( bulkAddedItems.size() > 0 )
-        board->FinalizeBulkAdd( bulkAddedItems );
-
-    if( bulkRemovedItems.size() > 0 )
-        board->FinalizeBulkRemove( bulkRemovedItems );
-
-    if( itemsChanged.size() > 0 )
-        board->OnItemsChanged( itemsChanged );
+    if( bulkAddedItems.size() > 0 || bulkRemovedItems.size() > 0 || itemsChanged.size() > 0 )
+        board->OnItemsCompositeUpdate( bulkAddedItems, bulkRemovedItems, itemsChanged );
 
     if( m_isBoardEditor )
     {
