@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2018-2022 KiCad Developers, see change_log.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +27,6 @@
 #include <bitmaps.h>
 #include <widgets/wx_grid.h>
 #include <widgets/std_bitmap_button.h>
-#include <wx/treebook.h>
 #include <grid_tricks.h>
 
 #include <panel_setup_tracks_and_vias.h>
@@ -52,30 +51,29 @@ enum DIFF_VAR_GRID_COLUMNS
 };
 
 
-PANEL_SETUP_TRACKS_AND_VIAS::PANEL_SETUP_TRACKS_AND_VIAS( PAGED_DIALOG* aParent,
+PANEL_SETUP_TRACKS_AND_VIAS::PANEL_SETUP_TRACKS_AND_VIAS( wxWindow* aParentWindow,
                                                           PCB_EDIT_FRAME* aFrame ) :
-    PANEL_SETUP_TRACKS_AND_VIAS_BASE( aParent->GetTreebook() )
+    PANEL_SETUP_TRACKS_AND_VIAS_BASE( aParentWindow )
 {
-    m_Parent = aParent;
     m_Frame = aFrame;
     m_Pcb = m_Frame->GetBoard();
     m_BrdSettings = &m_Pcb->GetDesignSettings();
 
-    m_trackWidthsAddButton->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_trackWidthsSortButton->SetBitmap( KiBitmap( BITMAPS::small_sort_desc ) );
-    m_trackWidthsRemoveButton->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_viaSizesAddButton->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_viaSizesSortButton->SetBitmap( KiBitmap( BITMAPS::small_sort_desc ) );
-    m_viaSizesRemoveButton->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_diffPairsAddButton->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_diffPairsSortButton->SetBitmap( KiBitmap( BITMAPS::small_sort_desc ) );
-    m_diffPairsRemoveButton->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
+    m_trackWidthsAddButton->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_trackWidthsSortButton->SetBitmap( KiBitmapBundle( BITMAPS::small_sort_desc ) );
+    m_trackWidthsRemoveButton->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_viaSizesAddButton->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_viaSizesSortButton->SetBitmap( KiBitmapBundle( BITMAPS::small_sort_desc ) );
+    m_viaSizesRemoveButton->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_diffPairsAddButton->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_diffPairsSortButton->SetBitmap( KiBitmapBundle( BITMAPS::small_sort_desc ) );
+    m_diffPairsRemoveButton->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
 
     // Membership combobox editors require a bit more room, so increase the row size of
     // all our grids for consistency
-    m_trackWidthsGrid->SetDefaultRowSize( m_trackWidthsGrid->GetDefaultRowSize() + 4 );
-    m_viaSizesGrid->SetDefaultRowSize(    m_viaSizesGrid->GetDefaultRowSize()    + 4 );
-    m_diffPairsGrid->SetDefaultRowSize(   m_diffPairsGrid->GetDefaultRowSize()   + 4 );
+    m_trackWidthsGrid->SetDefaultRowSize( m_trackWidthsGrid->GetDefaultRowSize() + FromDIP( 4 ) );
+    m_viaSizesGrid->SetDefaultRowSize( m_viaSizesGrid->GetDefaultRowSize() + FromDIP( 4 ) );
+    m_diffPairsGrid->SetDefaultRowSize( m_diffPairsGrid->GetDefaultRowSize() + FromDIP( 4 ) );
 
     m_trackWidthsGrid->PushEventHandler( new GRID_TRICKS( m_trackWidthsGrid,
                                                           [this]( wxCommandEvent& aEvent )
@@ -105,6 +103,10 @@ PANEL_SETUP_TRACKS_AND_VIAS::PANEL_SETUP_TRACKS_AND_VIAS( PAGED_DIALOG* aParent,
     m_viaSizesGrid->SetAutoEvalCols( { 0, 1 } );
     m_diffPairsGrid->SetAutoEvalCols( { 0, 1, 2 } );
 
+    m_trackWidthsGrid->SetUseNativeColLabels();
+    m_viaSizesGrid->SetUseNativeColLabels();
+    m_diffPairsGrid->SetUseNativeColLabels();
+
     // Ensure width of columns is enough to enter any reasonable value
     WX_GRID* grid_list[] = { m_trackWidthsGrid, m_viaSizesGrid, m_diffPairsGrid, nullptr };
     int min_linesize = m_trackWidthsGrid->GetTextExtent( wxT( "000.000000 mm " ) ).x;
@@ -122,7 +124,7 @@ PANEL_SETUP_TRACKS_AND_VIAS::PANEL_SETUP_TRACKS_AND_VIAS( PAGED_DIALOG* aParent,
         }
     }
 
-    m_Frame->Bind( UNITS_CHANGED, &PANEL_SETUP_TRACKS_AND_VIAS::onUnitsChanged, this );
+    m_Frame->Bind( EDA_EVT_UNITS_CHANGED, &PANEL_SETUP_TRACKS_AND_VIAS::onUnitsChanged, this );
 }
 
 
@@ -133,7 +135,7 @@ PANEL_SETUP_TRACKS_AND_VIAS::~PANEL_SETUP_TRACKS_AND_VIAS()
     m_viaSizesGrid->PopEventHandler( true );
     m_diffPairsGrid->PopEventHandler( true );
 
-    m_Frame->Unbind( UNITS_CHANGED, &PANEL_SETUP_TRACKS_AND_VIAS::onUnitsChanged, this );
+    m_Frame->Unbind( EDA_EVT_UNITS_CHANGED, &PANEL_SETUP_TRACKS_AND_VIAS::onUnitsChanged, this );
 }
 
 
@@ -142,6 +144,9 @@ void PANEL_SETUP_TRACKS_AND_VIAS::OnSortTrackWidthsClick( wxCommandEvent& aEvent
     std::vector<int> trackWidths;
     wxString         msg;
 
+    m_trackWidthsGrid->ClearSelection();
+    wxSafeYield();
+
     wxGridUpdateLocker lock( m_trackWidthsGrid );
 
     for( int row = 0; row < m_trackWidthsGrid->GetNumberRows();  ++row )
@@ -153,7 +158,7 @@ void PANEL_SETUP_TRACKS_AND_VIAS::OnSortTrackWidthsClick( wxCommandEvent& aEvent
     }
 
     std::sort( trackWidths.begin(), trackWidths.end() );
-    m_trackWidthsGrid->DeleteRows(0, m_trackWidthsGrid->GetNumberRows(), false);
+    m_trackWidthsGrid->DeleteRows( 0, m_trackWidthsGrid->GetNumberRows(), false );
 
     for( int width : trackWidths )
         AppendTrackWidth( width );
@@ -165,6 +170,9 @@ void PANEL_SETUP_TRACKS_AND_VIAS::OnSortViaSizesClick( wxCommandEvent& aEvent )
     std::vector<VIA_DIMENSION> vias;
     wxString                   msg;
 
+    m_viaSizesGrid->ClearSelection();
+    wxSafeYield();
+
     wxGridUpdateLocker lock( m_viaSizesGrid );
 
     for( int row = 0; row < m_viaSizesGrid->GetNumberRows();  ++row )
@@ -186,7 +194,7 @@ void PANEL_SETUP_TRACKS_AND_VIAS::OnSortViaSizesClick( wxCommandEvent& aEvent )
     }
 
     std::sort( vias.begin(), vias.end() );
-    m_viaSizesGrid->DeleteRows(0, m_viaSizesGrid->GetNumberRows(), false );
+    m_viaSizesGrid->DeleteRows( 0, m_viaSizesGrid->GetNumberRows(), false );
 
     for( const VIA_DIMENSION& via : vias )
         AppendViaSize( via.m_Diameter, via.m_Drill );
@@ -198,6 +206,9 @@ void PANEL_SETUP_TRACKS_AND_VIAS::OnSortDiffPairsClick( wxCommandEvent& aEvent )
     wxString                         msg;
     std::vector<DIFF_PAIR_DIMENSION> diffPairs;
 
+    m_diffPairsGrid->ClearSelection();
+    wxSafeYield();
+
     wxGridUpdateLocker lock( m_diffPairsGrid );
 
     for( int row = 0; row < m_diffPairsGrid->GetNumberRows();  ++row )
@@ -222,7 +233,7 @@ void PANEL_SETUP_TRACKS_AND_VIAS::OnSortDiffPairsClick( wxCommandEvent& aEvent )
     }
 
     std::sort( diffPairs.begin(), diffPairs.end() );
-    m_diffPairsGrid->DeleteRows(0, m_diffPairsGrid->GetNumberRows(), false );
+    m_diffPairsGrid->DeleteRows( 0, m_diffPairsGrid->GetNumberRows(), false );
 
     for( const DIFF_PAIR_DIMENSION& dp : diffPairs )
         AppendDiffPairs( dp.m_Width, dp.m_Gap, dp.m_ViaGap );
@@ -369,7 +380,7 @@ bool PANEL_SETUP_TRACKS_AND_VIAS::Validate()
         if( !viaDia.IsEmpty() && viaDrill.IsEmpty() )
         {
             msg = _( "No via hole size defined." );
-            m_Parent->SetError( msg, this, m_viaSizesGrid, row, VIA_DRILL_COL );
+            PAGED_DIALOG::GetDialog( this )->SetError( msg, this, m_viaSizesGrid, row, VIA_DRILL_COL );
             return false;
         }
     }
@@ -383,7 +394,7 @@ bool PANEL_SETUP_TRACKS_AND_VIAS::Validate()
         if( !dpWidth.IsEmpty() && dpGap.IsEmpty() )
         {
             msg = _( "No differential pair gap defined." );
-            m_Parent->SetError( msg, this, m_diffPairsGrid, row, 1 );
+            PAGED_DIALOG::GetDialog( this )->SetError( msg, this, m_diffPairsGrid, row, 1 );
             return false;
         }
     }
