@@ -1,4 +1,3 @@
-
 /*
     * Copyright The KiCad Developers.
     * Copyright (C) 2024 Fabien Corona f.corona<at>laposte.net
@@ -720,9 +719,9 @@ void BE_SHAPE_CIRCLE::ShortenChildDueToGV( std::shared_ptr<GRAPH_NODE>& a1,
     EDA_ANGLE angle1 = EDA_ANGLE( a1->m_pos - m_pos );
     EDA_ANGLE angle2 = EDA_ANGLE( a2->m_pos - m_pos );
 
-    while( angle1 < 0 )
+    while( angle1 < ANGLE_0 )
         angle1 += ANGLE_360;
-    while( angle2 < 0 )
+    while( angle2 < ANGLE_0 )
         angle2 += ANGLE_360;
     while( angle1 > ANGLE_360 )
         angle1 -= ANGLE_360;
@@ -783,7 +782,7 @@ void BE_SHAPE_CIRCLE::ConnectChildren( std::shared_ptr<GRAPH_NODE>& a1,
         PATH_CONNECTION pc;
         pc.a1 = a1->m_pos;
         pc.a2 = a2->m_pos;
-        pc.weight = weight;
+        pc.weight = std::max( weight, 0.0 );
 
         aG.AddConnection( a1, a2, pc );
         return;
@@ -1213,7 +1212,7 @@ std::vector<PATH_CONNECTION> CU_SHAPE_CIRCLE::Paths( const BE_SHAPE_POINT& aS2,
         return result;
 
     PATH_CONNECTION pc;
-    pc.weight = weight;
+    pc.weight = std::max( weight, 0.0 );
     pc.a2 = point;
     pc.a1 = center + ( point - center ).Resize( R );
 
@@ -1244,7 +1243,7 @@ std::vector<PATH_CONNECTION> CU_SHAPE_CIRCLE::Paths( const CU_SHAPE_CIRCLE& aS2,
         return result;
 
     PATH_CONNECTION pc;
-    pc.weight = weight;
+    pc.weight = std::max( weight, 0.0 );
     pc.a1 = ( C2 - C1 ).Resize( R1 ) + C1;
     pc.a2 = ( C1 - C2 ).Resize( R2 ) + C2;
     result.push_back( pc );
@@ -1362,7 +1361,7 @@ std::vector<PATH_CONNECTION> CU_SHAPE_CIRCLE::Paths( const CU_SHAPE_ARC& aS2, do
 
             if( testAngle < aS2.GetEndAngle() )
             {
-                pc3.weight = arcRadius - ( circlePos - arcPos ).EuclideanNorm() - circleRadius;
+                pc3.weight = std::max( arcRadius - ( circlePos - arcPos ).EuclideanNorm() - circleRadius, 0.0 );
                 pc3.a1 = circlePos + ( circlePos - arcPos ).Resize( circleRadius );
                 pc3.a2 = arcPos + ( circlePos - arcPos ).Resize( arcRadius - aS2.GetWidth() / 2 );
 
@@ -1550,7 +1549,7 @@ std::vector<PATH_CONNECTION> CU_SHAPE_SEGMENT::Paths( const CU_SHAPE_SEGMENT& aS
     PATH_CONNECTION pc;
     pc.a1 = closest1 + ( closest2 - closest1 ).Resize( halfWidth1 );
     pc.a2 = closest2 + ( closest1 - closest2 ).Resize( halfWidth2 );
-    pc.weight = sqrt( min_dist ) - halfWidth1 - halfWidth2;
+    pc.weight = std::max( sqrt( min_dist ) - halfWidth1 - halfWidth2, 0.0 );
 
     if( pc.weight <= aMaxWeight )
     {
@@ -1586,7 +1585,7 @@ std::vector<PATH_CONNECTION> CU_SHAPE_CIRCLE::Paths( const BE_SHAPE_CIRCLE& aS2,
     }
 
     PATH_CONNECTION pc;
-    pc.weight = weight;
+    pc.weight = std::max( weight, 0.0 );
 
     double circleAngle = EDA_ANGLE( center2 - center1 ).AsRadians();
 
@@ -1643,7 +1642,7 @@ std::vector<PATH_CONNECTION> CU_SHAPE_ARC::Paths( const BE_SHAPE_POINT& aS2, dou
         else
         {
             PATH_CONNECTION pc;
-            pc.weight = ( radius - width / 2 ) - ( point - arcCenter ).EuclideanNorm();
+            pc.weight = std::max( ( radius - width / 2 ) - ( point - arcCenter ).EuclideanNorm(), 0.0 );
             pc.a1 = ( point - arcCenter ).Resize( radius - width / 2 ) + arcCenter;
             pc.a2 = point;
 
@@ -1988,11 +1987,11 @@ std::vector<PATH_CONNECTION> GetPaths( CREEP_SHAPE* aS1, CREEP_SHAPE* aS2, doubl
 
 
     if( cuarc2 && bearc1 )
-        return bearc1->Paths( *bearc2, maxWeight, maxWeightSquared );
+        return bearc1->Paths( *cuarc2, maxWeight, maxWeightSquared );
     if( cuarc2 && becircle1 )
-        return becircle1->Paths( *bearc2, maxWeight, maxWeightSquared );
+        return becircle1->Paths( *cuarc2, maxWeight, maxWeightSquared );
     if( cuarc2 && bepoint1 )
-        return bepoint1->Paths( *bearc2, maxWeight, maxWeightSquared );
+        return bepoint1->Paths( *cuarc2, maxWeight, maxWeightSquared );
     if( cucircle2 && bearc1 )
         return bearc1->Paths( *cucircle2, maxWeight, maxWeightSquared );
     if( cucircle2 && becircle1 )
@@ -2047,9 +2046,12 @@ double CREEPAGE_GRAPH::Solve(
 
     auto cmp = [&distances]( GRAPH_NODE* left, GRAPH_NODE* right )
     {
-        if( distances[left] == distances[right] )
+        double distLeft = distances[left];
+        double distRight = distances[right];
+
+        if( distLeft == distRight )
             return left > right; // Compare addresses to avoid ties.
-        return distances[left] > distances[right];
+        return distLeft > distRight;
     };
     std::priority_queue<GRAPH_NODE*, std::vector<GRAPH_NODE*>, decltype( cmp )> pq( cmp );
 
@@ -2083,6 +2085,15 @@ double CREEPAGE_GRAPH::Solve(
             if( !neighbor )
                 continue;
 
+            // Ignore connections with negative weights as Dijkstra doesn't support them.
+            if( connection->m_path.weight < 0.0 )
+            {
+                wxLogTrace( "CREEPAGE",
+                            "Negative weight connection found. Ignoring connection." );
+                continue;
+            }
+
+
             double alt = distances[current]
                          + connection->m_path.weight; // Calculate alternative path cost
 
@@ -2110,7 +2121,7 @@ double CREEPAGE_GRAPH::Solve(
     {
         GRAPH_NODE* prevNode = previous[step];
 
-        for( std::shared_ptr<GRAPH_CONNECTION> node_conn : step->m_node_conns )
+        for( const std::shared_ptr<GRAPH_CONNECTION>& node_conn : step->m_node_conns )
         {
             if( ( ( node_conn->n1 ).get() == prevNode && ( node_conn->n2 ).get() == step )
                 || ( ( node_conn->n1 ).get() == step && ( node_conn->n2 ).get() == prevNode ) )
@@ -2268,7 +2279,7 @@ void CREEPAGE_GRAPH::Addshape( const SHAPE& aShape, std::shared_ptr<GRAPH_NODE>&
     }
 }
 
-void CREEPAGE_GRAPH::GeneratePaths( double aMaxWeight, PCB_LAYER_ID aLayer, bool aClearance )
+void CREEPAGE_GRAPH::GeneratePaths( double aMaxWeight, PCB_LAYER_ID aLayer )
 {
     std::vector<std::shared_ptr<GRAPH_NODE>> nodes;
     std::mutex                               nodes_lock;
@@ -2288,102 +2299,134 @@ void CREEPAGE_GRAPH::GeneratePaths( double aMaxWeight, PCB_LAYER_ID aLayer, bool
                                                                 && gn1->m_net < gn2->m_net );
               } );
 
-    auto processNodes = [&]( size_t i, size_t j ) -> bool
+    // Build parent -> net -> nodes mapping for efficient filtering
+    std::unordered_map<const BOARD_ITEM*, std::unordered_map<int, std::vector<std::shared_ptr<GRAPH_NODE>>>> parent_net_groups;
+    std::vector<const BOARD_ITEM*> parent_keys;
+
+    for( const auto& gn : nodes )
     {
-            for( size_t ii = i; ii < j; ii++ )
+        const BOARD_ITEM* parent = gn->m_parent->GetParent();
+
+        if( parent_net_groups[parent].empty() )
+            parent_keys.push_back( parent );
+
+        parent_net_groups[parent][gn->m_net].push_back( gn );
+    }
+
+    // Generate work items: compare nodes between different parents only
+    struct WorkItem {
+        std::shared_ptr<GRAPH_NODE> node1, node2;
+    };
+    std::vector<WorkItem> work_items;
+
+    for( size_t i = 0; i < parent_keys.size(); ++i )
+    {
+        for( size_t j = i + 1; j < parent_keys.size(); ++j )
+        {
+            const auto& group1_nets = parent_net_groups[parent_keys[i]];
+            const auto& group2_nets = parent_net_groups[parent_keys[j]];
+
+            for( const auto& [net1, nodes1] : group1_nets )
             {
-                std::shared_ptr<GRAPH_NODE> gn1 = nodes[ii];
-
-                for( size_t jj = ii + 1; jj < nodes.size(); jj++ )
+                for( const auto& [net2, nodes2] : group2_nets )
                 {
-                    std::shared_ptr<GRAPH_NODE> gn2 = nodes[jj];
-
-                    if( gn1->m_parent->GetParent() == gn2->m_parent->GetParent() )
-                        continue;
-
-                    if( ( gn1->m_net == gn2->m_net ) && ( gn1->m_parent->IsConductive() )
-                        && ( gn2->m_parent->IsConductive() ) )
-                        continue;
-
-                    for( PATH_CONNECTION pc : GetPaths( gn1->m_parent, gn2->m_parent, aMaxWeight ) )
+                    // Skip if same net and both nets have only conductive nodes
+                    if( net1 == net2 )
                     {
-                        std::vector<const BOARD_ITEM*> IgnoreForTest;
-                        IgnoreForTest.push_back( gn1->m_parent->GetParent() );
-                        IgnoreForTest.push_back( gn2->m_parent->GetParent() );
+                        bool all_conductive_1 = std::all_of( nodes1.begin(), nodes1.end(),
+                            []( const auto& n ) { return n->m_parent->IsConductive(); } );
+                        bool all_conductive_2 = std::all_of( nodes2.begin(), nodes2.end(),
+                            []( const auto& n ) { return n->m_parent->IsConductive(); } );
 
-                        if( !pc.isValid( m_board, aLayer, m_boardEdge, IgnoreForTest, m_boardOutline,
-                                        { false, true }, m_minGrooveWidth ) )
+                        if( all_conductive_1 && all_conductive_2 )
                             continue;
-
-                        std::shared_ptr<GRAPH_NODE>* connect1 = &gn1;
-                        std::shared_ptr<GRAPH_NODE>* connect2 = &gn2;
-                        std::shared_ptr<GRAPH_NODE>  gnt1 = nullptr;
-                        std::shared_ptr<GRAPH_NODE>  gnt2 = nullptr;
-                        std::lock_guard<std::mutex> lock( nodes_lock );
-
-                        if( gn1->m_parent->GetType() != CREEP_SHAPE::TYPE::POINT )
-                        {
-                            gnt1 = AddNode( GRAPH_NODE::POINT, gn1->m_parent, pc.a1 );
-                            gnt1->m_connectDirectly = false;
-
-                            if( gn1->m_parent->IsConductive() )
-                            {
-                                std::shared_ptr<GRAPH_CONNECTION> gc = AddConnection( gn1, gnt1 );
-
-                                if( gc )
-                                    gc->m_path.m_show = false;
-                            }
-                            connect1 = &gnt1;
-                        }
-
-                        if( gn2->m_parent->GetType() != CREEP_SHAPE::TYPE::POINT )
-                        {
-                            gnt2 = AddNode( GRAPH_NODE::POINT, gn2->m_parent, pc.a2 );
-                            gnt2->m_connectDirectly = false;
-
-                            if( gn2->m_parent->IsConductive() )
-                            {
-                                std::shared_ptr<GRAPH_CONNECTION> gc = AddConnection( gn2, gnt2 );
-
-                                if( gc )
-                                    gc->m_path.m_show = false;
-                            }
-                            connect2 = &gnt2;
-                        }
-
-                        AddConnection( *connect1, *connect2, pc );
                     }
-                } // for jj
-            } // for ii
 
-            return true;
-        };
+                    // Add all node pairs from these net groups
+                    for( const auto& gn1 : nodes1 )
+                        for( const auto& gn2 : nodes2 )
+                            work_items.push_back( { gn1, gn2 } );
+                }
+            }
+        }
+    }
 
-    // Running in the clearance test, we are already in a parallelized loop, so parallelizing again
-    // will run into deadlock as all threads start waiting
-    if( aClearance )
-        processNodes( 0, nodes.size() );
+    auto processWorkItems = [&]( size_t start_idx, size_t end_idx ) -> bool
+    {
+        for( size_t idx = start_idx; idx < end_idx; ++idx )
+        {
+            auto [gn1, gn2] = work_items[idx];
+
+            for( PATH_CONNECTION pc : GetPaths( gn1->m_parent, gn2->m_parent, aMaxWeight ) )
+            {
+                std::vector<const BOARD_ITEM*> IgnoreForTest = {
+                    gn1->m_parent->GetParent(), gn2->m_parent->GetParent()
+                };
+
+                if( !pc.isValid( m_board, aLayer, m_boardEdge, IgnoreForTest, m_boardOutline,
+                                { false, true }, m_minGrooveWidth ) )
+                    continue;
+
+                std::shared_ptr<GRAPH_NODE> connect1 = gn1, connect2 = gn2;
+                std::lock_guard<std::mutex> lock( nodes_lock );
+
+                // Handle non-point node1
+                if( gn1->m_parent->GetType() != CREEP_SHAPE::TYPE::POINT )
+                {
+                    auto gnt1 = AddNode( GRAPH_NODE::POINT, gn1->m_parent, pc.a1 );
+                    gnt1->m_connectDirectly = false;
+                    connect1 = gnt1;
+
+                    if( gn1->m_parent->IsConductive() )
+                    {
+                        if( auto gc = AddConnection( gn1, gnt1 ) )
+                            gc->m_path.m_show = false;
+                    }
+                }
+
+                // Handle non-point node2
+                if( gn2->m_parent->GetType() != CREEP_SHAPE::TYPE::POINT )
+                {
+                    auto gnt2 = AddNode( GRAPH_NODE::POINT, gn2->m_parent, pc.a2 );
+                    gnt2->m_connectDirectly = false;
+                    connect2 = gnt2;
+
+                    if( gn2->m_parent->IsConductive() )
+                    {
+                        if( auto gc = AddConnection( gn2, gnt2 ) )
+                            gc->m_path.m_show = false;
+                    }
+                }
+
+                AddConnection( connect1, connect2, pc );
+            }
+        }
+        return true;
+    };
+
+    // If the number of tasks is high enough, this indicates that the calling process
+    // has already parallelized the work, so we can process all items in one go.
+    if( tp.get_tasks_total() >= tp.get_thread_count() - 4 )
+    {
+        processWorkItems( 0, work_items.size() );
+    }
     else
     {
-        auto ret = tp.parallelize_loop( nodes.size(), processNodes );
+        auto ret = tp.parallelize_loop( work_items.size(), processWorkItems );
 
         for( size_t ii = 0; ii < ret.size(); ii++ )
         {
             std::future<bool>& r = ret[ii];
 
-            if( r.valid() )
-            {
-                std::future_status status = r.wait_for( std::chrono::seconds( 0 ) );
+            if( !r.valid() )
+                continue;
 
-                while( status != std::future_status::ready )
-                {
-                    status = r.wait_for( std::chrono::milliseconds( 100 ) );
-                }
-            }
+            while( r.wait_for( std::chrono::milliseconds( 100 ) ) != std::future_status::ready ){}
         }
     }
 }
 
+
 void CREEPAGE_GRAPH::Trim( double aWeightLimit )
 {
     std::vector<std::shared_ptr<GRAPH_CONNECTION>> toRemove;
