@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2023 KiCad Developers.
+ * Copyright The KiCad Developers.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -129,17 +129,26 @@ static bool commonParallelProjection( const PCB_ARC& p, const PCB_ARC& n, SHAPE_
     VECTOR2I n_center = n.GetCenter();
     double p_radius = p.GetRadius();
     double n_radius = n.GetRadius();
+    bool p_is_ccw = p.IsCCW();
+    bool n_is_ccw = n.IsCCW();
+
+    // Quick check to ensure arcs are of similar size and close enough to be considered coupled
+    double radiusDiffRatio = std::abs(p_radius - n_radius) / std::max(p_radius, n_radius);
+    double centerDistance = (p_center - n_center).EuclideanNorm();
+
+    if (radiusDiffRatio > 0.5 || centerDistance > std::max(p_radius, n_radius) * 0.5)
+        return false;
 
     VECTOR2I p_start( p.GetStart() );
     VECTOR2I p_end( p.GetEnd() );
 
-    if( !p.IsCCW() )
+    if( !p_is_ccw )
         std::swap( p_start, p_end );
 
     VECTOR2I n_start( n.GetStart() );
     VECTOR2I n_end( n.GetEnd() );
 
-    if( !n.IsCCW() )
+    if( !n_is_ccw )
         std::swap( n_start, n_end );
 
     SHAPE_ARC p_arc( p_start, p.GetMid(), p_end, 0 );
@@ -148,53 +157,70 @@ static bool commonParallelProjection( const PCB_ARC& p, const PCB_ARC& n, SHAPE_
     EDA_ANGLE p_start_angle = p_arc.GetStartAngle();
 
     // Rotate the arcs to a common 0 starting angle
-    p_arc.Rotate( -p_start_angle, p_center );
-    n_arc.Rotate( -p_start_angle, n_center );
+    p_arc.Rotate( p_start_angle, p_center );
+    n_arc.Rotate( p_start_angle, n_center );
 
     EDA_ANGLE p_end_angle = p_arc.GetEndAngle();
     EDA_ANGLE n_start_angle = n_arc.GetStartAngle();
     EDA_ANGLE n_end_angle = n_arc.GetEndAngle();
 
+    // Determine overlap region
+    EDA_ANGLE clip_start_angle, clip_end_angle;
 
-    EDA_ANGLE clip_total_angle;
-    EDA_ANGLE clip_start_angle;
+    // No overlap when n starts after p ends or n ends before p starts
+    if( n_start_angle >= p_end_angle || n_end_angle <= EDA_ANGLE(0) )
+        return false;
 
-    if( n_start_angle > p_end_angle )
-    {
-        // n is fully outside of p
-        if( n_end_angle > p_end_angle )
-            return false;
+    // Calculate the start and end angles of the overlap
+    clip_start_angle = std::max( EDA_ANGLE(0), n_start_angle );
+    clip_end_angle = std::min( p_end_angle, n_end_angle );
 
-        // n starts before angle 0 and ends in the middle of p
-        clip_total_angle = n_end_angle + p_start_angle;
-        clip_start_angle = p_start_angle;
-    }
-    else
-    {
-        clip_start_angle = n_start_angle + p_start_angle;
+    // Calculate the total angle of the overlap
+    EDA_ANGLE clip_total_angle = clip_end_angle - clip_start_angle;
 
-        // n is fully inside of p
-        if( n_end_angle < p_end_angle )
-            clip_total_angle = n_end_angle - n_start_angle;
-        else // n starts after 0 and ends after p
-            clip_total_angle = p_end_angle - n_start_angle;
-    }
+    // Now we reset the angles.  However, note that the convention here for adding angles
+    // is OPPOSITE the Rotate convention above.  So this undoes the rotation.
+    clip_start_angle += p_start_angle;
+    clip_end_angle += p_start_angle;
+    clip_start_angle.Normalize();
+    clip_end_angle.Normalize();
 
-    // One arc starts approximately where the other ends
+    // One arc starts approximately where the other ends or overlap is too small
     if( clip_total_angle <= EDA_ANGLE( ADVANCED_CFG::GetCfg().m_MinParallelAngle ) )
         return false;
 
-    VECTOR2I n_start_pt = n_center + VECTOR2I( KiROUND( n_radius ), 0 );
-    VECTOR2I p_start_pt = p_center + VECTOR2I( KiROUND( p_radius ), 0 );
+    // For CCW arcs, we start at clip_start_angle and sweep through clip_total_angle
+    // For CW arcs, we start at clip_end_angle and sweep through -clip_total_angle
+    VECTOR2I p_clip_point, n_clip_point;
 
-    RotatePoint( n_start_pt, n_center, clip_start_angle );
-    RotatePoint( p_start_pt, p_center, clip_start_angle );
+    if( p_is_ccw )
+    {
+        p_clip_point = p_center + VECTOR2I( KiROUND( p_radius * clip_start_angle.Cos() ),
+                                            KiROUND( p_radius * clip_start_angle.Sin() ) );
+        pClip = SHAPE_ARC( p_center, p_clip_point, clip_total_angle );
+    }
+    else
+    {
+        p_clip_point = p_center + VECTOR2I( KiROUND( p_radius * clip_end_angle.Cos() ),
+                                            KiROUND( p_radius * clip_end_angle.Sin() ) );
+        pClip = SHAPE_ARC( p_center, p_clip_point, -clip_total_angle );
+    }
 
-    pClip = SHAPE_ARC( p_center, p_start_pt, clip_total_angle );
-    nClip = SHAPE_ARC( n_center, n_start_pt, clip_total_angle );
+    if( n_is_ccw )
+    {
+        n_clip_point = n_center + VECTOR2I( KiROUND( n_radius * clip_start_angle.Cos() ),
+                                            KiROUND( n_radius * clip_start_angle.Sin() ) );
+        nClip = SHAPE_ARC( n_center, n_clip_point, clip_total_angle );
+    }
+    else
+    {
+        n_clip_point = n_center + VECTOR2I( KiROUND( n_radius * clip_end_angle.Cos() ),
+                                            KiROUND( n_radius * clip_end_angle.Sin() ) );
+        nClip = SHAPE_ARC( n_center, n_clip_point, -clip_total_angle );
+    }
 
-    if( std::abs( pClip.GetP0().x - pClip.GetArcMid().x ) < 10
-     || std::abs( nClip.GetP0().x - nClip.GetArcMid().x ) < 10 )
+    // Ensure the resulting arcs are not degenerate
+    if( pClip.GetLength() < 1.0 || nClip.GetLength() < 1.0 )
         return false;
 
     return true;
@@ -277,24 +303,24 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
 {
     for( BOARD_CONNECTED_ITEM* itemP : aDp.itemsP )
     {
-        PCB_TRACK* sp = dyn_cast<PCB_TRACK*>( itemP );
-        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
-
-        if( !sp )
+        if( !itemP || itemP->Type() != PCB_TRACE_T )
             continue;
 
-        for ( BOARD_CONNECTED_ITEM* itemN : aDp.itemsN )
-        {
-            PCB_TRACK* sn = dyn_cast<PCB_TRACK*> ( itemN );
+        PCB_TRACK* sp = static_cast<PCB_TRACK*>( itemP );
+        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
 
-            if( !sn )
+        for( BOARD_CONNECTED_ITEM* itemN : aDp.itemsN )
+        {
+            if( !itemN || itemN->Type() != PCB_TRACE_T )
                 continue;
 
+            PCB_TRACK* sn = static_cast<PCB_TRACK*>( itemN );
+
             if( ( sn->GetLayerSet() & sp->GetLayerSet() ).none() )
                 continue;
 
-            SEG ssp ( sp->GetStart(), sp->GetEnd() );
-            SEG ssn ( sn->GetStart(), sn->GetEnd() );
+            SEG ssp( sp->GetStart(), sp->GetEnd() );
+            SEG ssn( sn->GetStart(), sn->GetEnd() );
 
             // Segments that are ~ 1 IU in length per side are approximately parallel (tolerance is 1 IU)
             // with everything and their parallel projection is < 1 IU, leading to bad distance calculations
@@ -320,28 +346,28 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
 
         for( auto coupled : coupled_vec )
         {
-            auto excludeSelf = [&]( BOARD_ITEM* aItem )
-            {
-                if( aItem == coupled->parentN || aItem == coupled->parentP )
-                    return false;
-
-                if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T
-                    || aItem->Type() == PCB_ARC_T )
-                {
-                    PCB_TRACK* bci = static_cast<PCB_TRACK*>( aItem );
-
-                    // Directly connected items don't count
-                    if( bci->HitTest( coupled->coupledN.A, 0 )
-                        || bci->HitTest( coupled->coupledN.B, 0 )
-                        || bci->HitTest( coupled->coupledP.A, 0 )
-                        || bci->HitTest( coupled->coupledP.B, 0 ) )
+            auto excludeSelf =
+                    [&]( BOARD_ITEM* aItem )
                     {
-                        return false;
-                    }
-                }
+                        if( aItem == coupled->parentN || aItem == coupled->parentP )
+                            return false;
 
-                return true;
-            };
+                        if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T || aItem->Type() == PCB_ARC_T )
+                        {
+                            PCB_TRACK* bci = static_cast<PCB_TRACK*>( aItem );
+
+                            // Directly connected items don't count
+                            if( bci->HitTest( coupled->coupledN.A, 0 )
+                                || bci->HitTest( coupled->coupledN.B, 0 )
+                                || bci->HitTest( coupled->coupledP.A, 0 )
+                                || bci->HitTest( coupled->coupledP.B, 0 ) )
+                            {
+                                return false;
+                            }
+                        }
+
+                        return true;
+                    };
 
             SHAPE_SEGMENT checkSeg( coupled->nearestN, coupled->nearestP );
             DRC_RTREE*    tree = coupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
@@ -350,27 +376,25 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
             // there's nothing, assume they are really coupled.
 
             if( !tree->CheckColliding( &checkSeg, sp->GetLayer(), 0, excludeSelf ) )
-            {
                 aDp.coupled.push_back( *coupled );
-            }
         }
     }
 
     for( BOARD_CONNECTED_ITEM* itemP : aDp.itemsP )
     {
-        PCB_ARC* sp = dyn_cast<PCB_ARC*>( itemP );
-        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
-
-        if( !sp )
+        if( !itemP || itemP->Type() != PCB_ARC_T )
             continue;
 
+        PCB_ARC* sp = static_cast<PCB_ARC*>( itemP );
+        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
+
         for ( BOARD_CONNECTED_ITEM* itemN : aDp.itemsN )
         {
-            PCB_ARC* sn = dyn_cast<PCB_ARC*> ( itemN );
-
-            if( !sn )
+            if( !itemN || itemN->Type() != PCB_ARC_T )
                 continue;
 
+            PCB_ARC* sn = dyn_cast<PCB_ARC*> ( itemN );
+
             if( ( sn->GetLayerSet() & sp->GetLayerSet() ).none() )
                 continue;
 
@@ -378,7 +402,9 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
             // with everything and their parallel projection is < 1 IU, leading to bad distance calculations
             int64_t sqWidth = static_cast<int64_t>( sp->GetWidth() ) * sp->GetWidth();
 
-            if( sp->GetLength() > 2 && sn->GetLength() > 2 && sp->GetCenter().SquaredDistance( sn->GetCenter() ) < sqWidth )
+            if( sp->GetLength() > 2
+                    && sn->GetLength() > 2
+                    && sp->GetCenter().SquaredDistance( sn->GetCenter() ) < sqWidth )
             {
                 DIFF_PAIR_COUPLED_SEGMENTS cpair;
                 cpair.isArc = true;
@@ -390,7 +416,7 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
                     cpair.parentN = sn;
                     cpair.layer = sp->GetLayer();
                     cpair.coupledArcP.NearestPoints( cpair.coupledArcN, cpair.nearestP, cpair.nearestN,
-                                                  cpair.computedGap );
+                                                     cpair.computedGap );
                     cpair.computedGap = std::sqrt( cpair.computedGap );  // NearestPoints returns squared distance
                     cpair.computedGap -= ( sp->GetWidth() + sn->GetWidth() ) / 2;
                     coupled_vec.push_back( cpair );
@@ -408,10 +434,10 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
 
                         if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T || aItem->Type() == PCB_ARC_T )
                         {
-                            auto bci = static_cast<BOARD_CONNECTED_ITEM*>( aItem );
+                            BOARD_CONNECTED_ITEM* bci = static_cast<BOARD_CONNECTED_ITEM*>( aItem );
 
                             if( bci->GetNetCode() == coupled->parentN->GetNetCode()
-                            ||  bci->GetNetCode() == coupled->parentP->GetNetCode() )
+                                    ||  bci->GetNetCode() == coupled->parentP->GetNetCode() )
                             {
                                 return false;
                             }
@@ -427,9 +453,7 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
             // there's nothing, assume they are really coupled.
 
             if( !tree->CheckColliding( &checkArcMid, sp->GetLayer(), 0, excludeSelf ) )
-            {
                 aDp.coupled.push_back( *coupled );
-            }
         }
     }
 }
@@ -440,7 +464,8 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 {
     m_board = m_drcEngine->GetBoard();
 
-    int epsilon = m_board->GetDesignSettings().GetDRCEpsilon();
+    int  epsilon = m_board->GetDesignSettings().GetDRCEpsilon();
+    LSET boardCopperLayers = LSET::AllCuMask( m_board->GetCopperLayerCount() );
 
     std::map<DIFF_PAIR_KEY, DIFF_PAIR_ITEMS> dpRuleMatches;
 
@@ -455,26 +480,22 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                 {
                     drc_dbg( 10, wxT( "eval dp %p\n" ), item );
 
-                    const DRC_CONSTRAINT_T constraintsToCheck[] = {
-                            DIFF_PAIR_GAP_CONSTRAINT,
-                            MAX_UNCOUPLED_CONSTRAINT
-                    };
-
-                    for( int i = 0; i < 2; i++ )
+                    for( DRC_CONSTRAINT_T constraintType : { DIFF_PAIR_GAP_CONSTRAINT,
+                                                             MAX_UNCOUPLED_CONSTRAINT } )
                     {
-                        DRC_CONSTRAINT constraint = m_drcEngine->EvalRules( constraintsToCheck[ i ],
+                        DRC_CONSTRAINT constraint = m_drcEngine->EvalRules( constraintType,
                                                                             item, nullptr,
                                                                             item->GetLayer() );
 
                         if( constraint.IsNull() || constraint.GetSeverity() == RPT_SEVERITY_IGNORE )
                             continue;
 
-                        drc_dbg( 10, wxT( "cns %d item %p\n" ), constraintsToCheck[i], item );
+                        drc_dbg( 10, wxT( "cns %d item %p\n" ), (int) constraintType, item );
 
                         DRC_RULE* parentRule = constraint.GetParentRule();
                         wxString ruleName = parentRule ? parentRule->m_Name : constraint.GetName();
 
-                        switch( constraintsToCheck[i] )
+                        switch( constraintType )
                         {
                         case DIFF_PAIR_GAP_CONSTRAINT:
                             key.gapConstraint = constraint.GetValue();
@@ -504,8 +525,7 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
     m_board->GetConnectivity()->GetFromToCache()->Rebuild( m_board );
 
-    forEachGeometryItem( { PCB_TRACE_T, PCB_VIA_T, PCB_ARC_T }, LSET::AllCuMask(),
-                         evaluateDpConstraints );
+    forEachGeometryItem( { PCB_TRACE_T, PCB_VIA_T, PCB_ARC_T }, boardCopperLayers, evaluateDpConstraints );
 
     drc_dbg( 10, wxT( "dp rule matches %d\n" ), (int) dpRuleMatches.size() );
 
@@ -573,8 +593,8 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
             drc_dbg( 10, wxT( "               len %d gap %ld l %d\n" ),
                      length,
-                     dp.computedGap,
-                     dp.parentP->GetLayer() );
+                     (long int) dp.computedGap,
+                     (int) dp.parentP->GetLayer() );
 
             if( key.gapConstraint )
             {
@@ -711,7 +731,7 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
                     drcItem->SetViolatingRule( key.gapRule );
 
-                    reportViolation( drcItem, item->GetPosition(), item->GetLayer() );
+                    reportViolation( drcItem, item->GetFocusPosition(), item->GetLayer() );
                 }
             }
         }
