@@ -1044,7 +1044,11 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
             continue;
         }
 
-        SGNODE* mod3d = (SGNODE*) m_Cache3Dmodels->Load( sM->m_Filename, footprintBasePath, aFootprint );
+        std::vector<const EMBEDDED_FILES*> embeddedFilesStack;
+        embeddedFilesStack.push_back( aFootprint->GetEmbeddedFiles() );
+        embeddedFilesStack.push_back( m_board->GetEmbeddedFiles() );
+
+        SGNODE* mod3d = (SGNODE*) m_Cache3Dmodels->Load( sM->m_Filename, footprintBasePath, embeddedFilesStack );
 
         if( nullptr == mod3d )
         {
@@ -1109,8 +1113,12 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
             int old_precision = aOutputFile->precision();
             aOutputFile->precision( m_precision );
 
-            wxFileName srcFile =
-                    m_Cache3Dmodels->GetResolver()->ResolvePath( sM->m_Filename, wxEmptyString, aFootprint );
+            std::vector<const EMBEDDED_FILES*> embeddedFilesStack;
+            embeddedFilesStack.push_back( aFootprint->GetEmbeddedFiles() );
+            embeddedFilesStack.push_back( m_board->GetEmbeddedFiles() );
+
+            wxFileName srcFile = m_Cache3Dmodels->GetResolver()->ResolvePath( sM->m_Filename, wxEmptyString,
+                                                                              embeddedFilesStack );
             wxFileName dstFile;
             dstFile.SetPath( m_Subdir3DFpModels );
             dstFile.SetName( srcFile.GetName() );
@@ -1118,9 +1126,7 @@ void EXPORTER_PCB_VRML::ExportVrmlFootprint( FOOTPRINT* aFootprint, std::ostream
 
             // copy the file if necessary
             wxDateTime srcModTime = srcFile.GetModificationTime();
-            wxDateTime destModTime = srcModTime;
-
-            destModTime.SetToCurrent();
+            wxDateTime destModTime = wxDateTime();
 
             if( dstFile.FileExists() )
                 destModTime = dstFile.GetModificationTime();
