@@ -29,14 +29,18 @@
 #include <board_design_settings.h>
 #include <footprint.h>
 #include <pcb_textbox.h>
+#include <pcb_table.h>
+#include <pcb_tablecell.h>
 #include <pcb_track.h>
 #include <pcb_shape.h>
+#include <pcb_painter.h>
 #include <pad.h>
 #include <zone.h>
 #include <fp_lib_table.h>
 #include "step_pcb_model.h"
 
 #include <pgm_base.h>
+#include <reporter.h>
 #include <base_units.h>
 #include <filename_resolver.h>
 #include <trace_helpers.h>
@@ -60,16 +64,12 @@
 #endif
 
 
-void ReportMessage( const wxString& aMessage )
-{
-    wxPrintf( aMessage );
-    fflush( stdout ); // Force immediate printing (needed on mingw)
-}
-
-class KiCadPrinter : public Message_Printer
+class KICAD_PRINTER : public Message_Printer
 {
 public:
-    KiCadPrinter( EXPORTER_STEP* aConverter ) : m_converter( aConverter ) {}
+    KICAD_PRINTER( REPORTER* aReporter ) :
+            m_reporter( aReporter )
+    {}
 
 protected:
 #if OCC_VERSION_HEX < OCC_VERSION_MIN
@@ -85,44 +85,48 @@ protected:
                        const Standard_Boolean theToPutEol ) const override
 #else
     virtual void send( const TCollection_AsciiString& theString,
-                        const Message_Gravity theGravity ) const override
+                       const Message_Gravity theGravity ) const override
 #endif
     {
-        if( theGravity >= Message_Warning
-            || ( wxLog::IsAllowedTraceMask( traceKiCad2Step ) && theGravity == Message_Info ) )
-      {
-          ReportMessage( theString.ToCString() );
+        wxString msg( theString.ToCString() );
 
 #if OCC_VERSION_HEX < OCC_VERSION_MIN
-          if( theToPutEol )
-              ReportMessage( wxT( "\n" ) );
+        if( theToPutEol )
+            msg += wxT( "\n" );
 #else
-          ReportMessage( wxT( "\n" ) );
+        msg += wxT( "\n" );
 #endif
-      }
 
-      if( theGravity == Message_Warning )
-          m_converter->SetWarn();
-
-      if( theGravity >= Message_Alarm )
-          m_converter->SetError();
-
-      if( theGravity == Message_Fail )
-          m_converter->SetFail();
+        m_reporter->Report( msg, getSeverity( theGravity ) );
     }
 
 private:
-    EXPORTER_STEP* m_converter;
+    SEVERITY getSeverity( const Message_Gravity theGravity ) const
+    {
+        switch( theGravity )
+        {
+        case Message_Trace:   return RPT_SEVERITY_DEBUG;
+        case Message_Info:    return RPT_SEVERITY_DEBUG;
+        case Message_Warning: return RPT_SEVERITY_WARNING;
+        case Message_Alarm:   return RPT_SEVERITY_ERROR;
+        case Message_Fail:    return RPT_SEVERITY_ERROR;
+
+        // There are no other values, but gcc doesn't appear to be able to work that out.
+        default:              return RPT_SEVERITY_UNDEFINED;
+        }
+    }
+
+private:
+    REPORTER* m_reporter;
 };
 
 
-EXPORTER_STEP::EXPORTER_STEP( BOARD* aBoard, const EXPORTER_STEP_PARAMS& aParams ) :
-    m_params( aParams ),
-    m_error( false ),
-    m_fail( false ),
-    m_warn( false ),
-    m_board( aBoard ),
-    m_pcbModel( nullptr )
+EXPORTER_STEP::EXPORTER_STEP( BOARD* aBoard, const EXPORTER_STEP_PARAMS& aParams,
+                              REPORTER* aReporter ) :
+        m_params( aParams ),
+        m_reporter( aReporter ),
+        m_board( aBoard ),
+        m_pcbModel( nullptr )
 {
     m_copperColor = COLOR4D( 0.7, 0.61, 0.0, 1.0 );
 
@@ -155,7 +159,8 @@ EXPORTER_STEP::~EXPORTER_STEP()
 }
 
 
-bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOrigin )
+bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOrigin,
+                                            SHAPE_POLY_SET* aClipPolygon )
 {
     bool              hasdata = false;
     std::vector<PAD*> padsMatchingNetFilter;
@@ -164,11 +169,15 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
     // Dump the pad holes into the PCB
     for( PAD* pad : aFootprint->Pads() )
     {
+        bool castellated = pad->GetProperty() == PAD_PROP::CASTELLATED;
         std::shared_ptr<SHAPE_SEGMENT> holeShape = pad->GetEffectiveHoleShape();
 
         SHAPE_POLY_SET holePoly;
         holeShape->TransformToPolygon( holePoly, maxError, ERROR_INSIDE );
 
+        // This helps with fusing
+        holePoly.Deflate( m_platingThickness / 2, CORNER_STRATEGY::ROUND_ALL_CORNERS, maxError );
+
         for( PCB_LAYER_ID pcblayer : pad->GetLayerSet().Seq() )
         {
             if( pad->IsOnLayer( pcblayer ) )
@@ -198,7 +207,8 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
 
         if( m_params.m_ExportPads )
         {
-            if( m_pcbModel->AddPadShape( pad, aOrigin, false ) )
+            if( m_pcbModel->AddPadShape( pad, aOrigin, false,
+                                         castellated ? aClipPolygon : nullptr) )
                 hasdata = true;
 
             if( m_params.m_ExportSoldermask )
@@ -214,7 +224,7 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
                                                   pad->GetSolderMaskExpansion( cuLayer ), maxError,
                                                   ERROR_INSIDE );
 
-                    m_poly_shapes[pcblayer].Append( poly );
+                    m_poly_shapes[pcblayer][wxEmptyString].Append( poly );
                 }
             }
         }
@@ -235,13 +245,15 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
                                                 true, /* include shapes */
                                                 false /* include private items */ );
 
-        if( m_params.m_NetFilter.IsEmpty() || !IsCopperLayer( pcblayer ) )
+        if( !IsCopperLayer( pcblayer ) )
         {
-            m_poly_shapes[pcblayer].Append( buffer );
+            m_poly_shapes[pcblayer][wxEmptyString].Append( buffer );
         }
         else
         {
-            // Only add shapes colliding with any matching pads
+            std::map<const SHAPE_POLY_SET::POLYGON*, PAD*> polyPadMap;
+
+            // Only add polygons colliding with any matching pads
             for( const SHAPE_POLY_SET::POLYGON& poly : buffer.CPolygons() )
             {
                 for( PAD* pad : padsMatchingNetFilter )
@@ -254,11 +266,24 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
 
                     if( padPoly->Collide( &gfxPoly ) )
                     {
-                        m_poly_shapes[pcblayer].Append( gfxPoly );
+                        polyPadMap[&poly] = pad;
+                        m_poly_shapes[pcblayer][pad->GetNetname()].Append( gfxPoly );
                         break;
                     }
                 }
             }
+
+            if( m_params.m_NetFilter.empty() )
+            {
+                // Add polygons with no net
+                for( const SHAPE_POLY_SET::POLYGON& poly : buffer.CPolygons() )
+                {
+                    auto it = polyPadMap.find( &poly );
+
+                    if( it == polyPadMap.end() )
+                        m_poly_shapes[pcblayer][wxEmptyString].Append( poly );
+                }
+            }
         }
     }
 
@@ -336,8 +361,11 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
             continue;
 
         std::vector<wxString> searchedPaths;
-        wxString mname = m_resolver->ResolvePath( fp_model.m_Filename, footprintBasePath, aFootprint );
+        std::vector<const EMBEDDED_FILES*> embeddedFilesStack;
+        embeddedFilesStack.push_back( aFootprint->GetEmbeddedFiles() );
+        embeddedFilesStack.push_back( m_board->GetEmbeddedFiles() );
 
+        wxString mname = m_resolver->ResolvePath( fp_model.m_Filename, footprintBasePath, embeddedFilesStack );
 
         if( mname.empty() || !wxFileName::FileExists( mname ) )
         {
@@ -345,9 +373,11 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
             if( mname.empty() )
                 mname = fp_model.m_Filename;
 
-            ReportMessage( wxString::Format( wxT( "Could not add 3D model to %s.\n"
-                                                  "File not found: %s\n" ),
-                                             aFootprint->GetReference(), mname ) );
+            m_reporter->Report( wxString::Format( _( "Could not add 3D model for %s.\n"
+                                                     "File not found: %s\n" ),
+                                                  aFootprint->GetReference(),
+                                                  mname ),
+                                RPT_SEVERITY_ERROR );
             continue;
         }
 
@@ -373,9 +403,11 @@ bool EXPORTER_STEP::buildFootprint3DShapes( FOOTPRINT* aFootprint, VECTOR2D aOri
         }
         catch( const Standard_Failure& e )
         {
-            ReportMessage( wxString::Format( wxT( "Could not add 3D model to %s.\n"
-                                                  "OpenCASCADE error: %s\n" ),
-                                             aFootprint->GetReference(), e.GetMessageString() ) );
+            m_reporter->Report( wxString::Format( _( "Could not add 3D model for %s.\n"
+                                                     "OpenCASCADE error: %s\n" ),
+                                                  aFootprint->GetReference(),
+                                                  e.GetMessageString() ),
+                                RPT_SEVERITY_ERROR );
         }
 
     }
@@ -399,7 +431,7 @@ bool EXPORTER_STEP::buildTrack3DShape( PCB_TRACK* aTrack, VECTOR2D aOrigin )
             SHAPE_POLY_SET poly;
             aTrack->TransformShapeToPolygon( poly, F_Mask, 0, maxError, ERROR_INSIDE );
 
-            m_poly_shapes[F_Mask].Append( poly );
+            m_poly_shapes[F_Mask][wxEmptyString].Append( poly );
         }
 
         if( aTrack->IsOnLayer( B_Mask ) )
@@ -407,7 +439,7 @@ bool EXPORTER_STEP::buildTrack3DShape( PCB_TRACK* aTrack, VECTOR2D aOrigin )
             SHAPE_POLY_SET poly;
             aTrack->TransformShapeToPolygon( poly, B_Mask, 0, maxError, ERROR_INSIDE );
 
-            m_poly_shapes[B_Mask].Append( poly );
+            m_poly_shapes[B_Mask][wxEmptyString].Append( poly );
         }
     }
 
@@ -419,6 +451,9 @@ bool EXPORTER_STEP::buildTrack3DShape( PCB_TRACK* aTrack, VECTOR2D aOrigin )
         SHAPE_POLY_SET                 holePoly;
         holeShape->TransformToPolygon( holePoly, maxError, ERROR_INSIDE );
 
+        // This helps with fusing
+        holePoly.Deflate( m_platingThickness / 2, CORNER_STRATEGY::ROUND_ALL_CORNERS, maxError );
+
         LSET layers( via->GetLayerSet() & m_layersToExport );
 
         PCB_LAYER_ID top_layer, bot_layer;
@@ -432,11 +467,12 @@ bool EXPORTER_STEP::buildTrack3DShape( PCB_TRACK* aTrack, VECTOR2D aOrigin )
 
                 SHAPE_POLY_SET poly;
                 shape->TransformToPolygon( poly, maxError, ERROR_INSIDE );
-                m_poly_shapes[pcblayer].Append( poly );
+                m_poly_shapes[pcblayer][via->GetNetname()].Append( poly );
                 m_poly_holes[pcblayer].Append( holePoly );
             }
 
-            m_pcbModel->AddBarrel( *holeShape, top_layer, bot_layer, true, aOrigin );
+            m_pcbModel->AddBarrel( *holeShape, top_layer, bot_layer, true, aOrigin,
+                                   via->GetNetname() );
         }
 
         //// Cut holes in silkscreen (buggy: insufficient polyset self-intersection checking)
@@ -460,7 +496,8 @@ bool EXPORTER_STEP::buildTrack3DShape( PCB_TRACK* aTrack, VECTOR2D aOrigin )
     if( !m_layersToExport.Contains( pcblayer ) )
         return false;
 
-    aTrack->TransformShapeToPolygon( m_poly_shapes[pcblayer], pcblayer, 0, maxError, ERROR_INSIDE );
+    aTrack->TransformShapeToPolygon( m_poly_shapes[pcblayer][aTrack->GetNetname()], pcblayer, 0,
+                                     maxError, ERROR_INSIDE );
 
     return true;
 }
@@ -486,7 +523,7 @@ void EXPORTER_STEP::buildZones3DShape( VECTOR2D aOrigin )
 
             fill_shape.SimplifyOutlines( ADVANCED_CFG::GetCfg().m_TriangulateSimplificationLevel );
 
-            m_poly_shapes[layer].Append( fill_shape );
+            m_poly_shapes[layer][zone->GetNetname()].Append( fill_shape );
         }
     }
 }
@@ -519,8 +556,37 @@ bool EXPORTER_STEP::buildGraphic3DShape( BOARD_ITEM* aItem, VECTOR2D aOrigin )
             return true;
         }
 
-        graphic->TransformShapeToPolygon( m_poly_shapes[pcblayer], pcblayer, 0, maxError,
-                                          ERROR_INSIDE );
+        LINE_STYLE lineStyle = graphic->GetLineStyle();
+
+        if( lineStyle == LINE_STYLE::SOLID )
+        {
+            graphic->TransformShapeToPolygon( m_poly_shapes[pcblayer][graphic->GetNetname()],
+                                              pcblayer, 0, maxError, ERROR_INSIDE );
+        }
+        else
+        {
+            std::vector<SHAPE*>        shapes = graphic->MakeEffectiveShapes( true );
+            const PCB_PLOT_PARAMS&     plotParams = m_board->GetPlotOptions();
+            KIGFX::PCB_RENDER_SETTINGS renderSettings;
+
+            renderSettings.SetDashLengthRatio( plotParams.GetDashedLineDashRatio() );
+            renderSettings.SetGapLengthRatio( plotParams.GetDashedLineGapRatio() );
+
+            for( SHAPE* shape : shapes )
+            {
+                STROKE_PARAMS::Stroke( shape, lineStyle, graphic->GetWidth(), &renderSettings,
+                                       [&]( const VECTOR2I& a, const VECTOR2I& b )
+                                       {
+                                           SHAPE_SEGMENT seg( a, b, graphic->GetWidth() );
+                                           seg.TransformToPolygon(
+                                                   m_poly_shapes[pcblayer][graphic->GetNetname()],
+                                                   maxError, ERROR_INSIDE );
+                                       } );
+            }
+
+            for( SHAPE* shape : shapes )
+                delete shape;
+        }
 
         break;
     }
@@ -529,7 +595,8 @@ bool EXPORTER_STEP::buildGraphic3DShape( BOARD_ITEM* aItem, VECTOR2D aOrigin )
     {
         PCB_TEXT* text = static_cast<PCB_TEXT*>( aItem );
 
-        text->TransformTextToPolySet( m_poly_shapes[pcblayer], 0, maxError, ERROR_INSIDE );
+        text->TransformTextToPolySet( m_poly_shapes[pcblayer][wxEmptyString], 0, maxError,
+                                      ERROR_INSIDE );
         break;
     }
 
@@ -537,13 +604,39 @@ bool EXPORTER_STEP::buildGraphic3DShape( BOARD_ITEM* aItem, VECTOR2D aOrigin )
     {
         PCB_TEXTBOX* textbox = static_cast<PCB_TEXTBOX*>( aItem );
 
-        textbox->TransformTextToPolySet( m_poly_shapes[pcblayer], 0, maxError, ERROR_INSIDE );
+        // border
+        if( textbox->IsBorderEnabled() )
+        {
+            textbox->PCB_SHAPE::TransformShapeToPolygon( m_poly_shapes[pcblayer][wxEmptyString],
+                                                         pcblayer, 0, maxError, ERROR_INSIDE );
+        }
+
+        // text
+        textbox->TransformTextToPolySet( m_poly_shapes[pcblayer][wxEmptyString], 0, maxError,
+                                         ERROR_INSIDE );
         break;
     }
 
     case PCB_TABLE_T:
-        // JEY TODO: tables
+    {
+        PCB_TABLE* table = static_cast<PCB_TABLE*>( aItem );
+
+        for( PCB_TABLECELL* cell : table->GetCells() )
+        {
+            cell->TransformTextToPolySet( m_poly_shapes[pcblayer][wxEmptyString], 0, maxError,
+                                          ERROR_INSIDE );
+        }
+
+        table->DrawBorders(
+                [&]( const VECTOR2I& ptA, const VECTOR2I& ptB, const STROKE_PARAMS& stroke )
+                {
+                    SHAPE_SEGMENT seg( ptA, ptB, stroke.GetWidth() );
+                    seg.TransformToPolygon( m_poly_shapes[pcblayer][wxEmptyString], maxError,
+                                            ERROR_INSIDE );
+                } );
+
         break;
+    }
 
     default: wxFAIL_MSG( "buildGraphic3DShape: unhandled item type" );
     }
@@ -603,6 +696,9 @@ bool EXPORTER_STEP::buildBoard3DShapes()
         wxLogWarning( _( "Board outline is malformed. Run DRC for a full analysis." ) );
     }
 
+    SHAPE_POLY_SET pcbOutlinesNoArcs = pcbOutlines;
+    pcbOutlinesNoArcs.ClearArcs();
+
     VECTOR2D origin;
 
     // Determine the coordinate system reference:
@@ -614,7 +710,7 @@ bool EXPORTER_STEP::buildBoard3DShapes()
     else
         origin = m_params.m_Origin;
 
-    m_pcbModel = std::make_unique<STEP_PCB_MODEL>( m_pcbBaseName );
+    m_pcbModel = std::make_unique<STEP_PCB_MODEL>( m_pcbBaseName, m_reporter );
 
     initOutputVariant();
 
@@ -640,7 +736,7 @@ bool EXPORTER_STEP::buildBoard3DShapes()
     // For copper layers, only pads and tracks are added, because adding everything on copper
     // generate unreasonable file sizes and take a unreasonable calculation time.
     for( FOOTPRINT* fp : m_board->Footprints() )
-        buildFootprint3DShapes( fp, origin );
+        buildFootprint3DShapes( fp, origin, &pcbOutlinesNoArcs );
 
     for( PCB_TRACK* track : m_board->Tracks() )
         buildTrack3DShape( track, origin );
@@ -653,51 +749,59 @@ bool EXPORTER_STEP::buildBoard3DShapes()
         buildZones3DShape( origin );
     }
 
-    SHAPE_POLY_SET pcbOutlinesNoArcs = pcbOutlines;
-    pcbOutlinesNoArcs.ClearArcs();
-
     for( PCB_LAYER_ID pcblayer : m_layersToExport.Seq() )
     {
-        SHAPE_POLY_SET poly = m_poly_shapes[pcblayer];
-        poly.Simplify();
-
-        poly.SimplifyOutlines( pcbIUScale.mmToIU( 0.003 ) );
-        poly.Simplify();
-
         SHAPE_POLY_SET holes = m_poly_holes[pcblayer];
         holes.Simplify();
 
-        // Mask layer is negative
         if( pcblayer == F_Mask || pcblayer == B_Mask )
         {
+            // Mask layer is negative
             SHAPE_POLY_SET mask = pcbOutlinesNoArcs;
 
-            mask.BooleanSubtract( poly );
+            for( auto& [netname, poly] : m_poly_shapes[pcblayer] )
+            {
+                poly.Simplify();
+
+                poly.SimplifyOutlines( pcbIUScale.mmToIU( 0.003 ) );
+                poly.Simplify();
+
+                mask.BooleanSubtract( poly );
+            }
+
             mask.BooleanSubtract( holes );
 
-            poly = mask;
+            m_pcbModel->AddPolygonShapes( &mask, pcblayer, origin, wxEmptyString );
         }
         else
         {
-            // Subtract holes
-            poly.BooleanSubtract( holes );
+            for( auto& [netname, poly] : m_poly_shapes[pcblayer] )
+            {
+                poly.Simplify();
 
-            // Clip to board outline
-            poly.BooleanIntersection( pcbOutlinesNoArcs );
+                poly.SimplifyOutlines( pcbIUScale.mmToIU( 0.003 ) );
+                poly.Simplify();
+
+                // Subtract holes
+                poly.BooleanSubtract( holes );
+
+                // Clip to board outline
+                poly.BooleanIntersection( pcbOutlinesNoArcs );
+
+                m_pcbModel->AddPolygonShapes( &poly, pcblayer, origin, netname );
+            }
         }
-
-        m_pcbModel->AddPolygonShapes( &poly, pcblayer, origin );
     }
 
-    ReportMessage( wxT( "Create PCB solid model\n" ) );
+    m_reporter->Report( wxT( "Create PCB solid model.\n" ), RPT_SEVERITY_DEBUG );
 
-    wxString msg;
-    msg.Printf( wxT( "Board outline: find %d initial points\n" ), pcbOutlines.FullPointCount() );
-    ReportMessage( msg );
+    m_reporter->Report( wxString::Format( wxT( "Board outline: found %d initial points.\n" ),
+                                          pcbOutlines.FullPointCount() ),
+                        RPT_SEVERITY_DEBUG );
 
     if( !m_pcbModel->CreatePCB( pcbOutlines, origin, m_params.m_ExportBoardBody ) )
     {
-        ReportMessage( wxT( "could not create PCB solid model\n" ) );
+        m_reporter->Report( _( "Could not create PCB solid model.\n" ), RPT_SEVERITY_ERROR );
         return false;
     }
 
@@ -712,9 +816,9 @@ bool EXPORTER_STEP::Export()
 
     // setup opencascade message log
     Message::DefaultMessenger()->RemovePrinters( STANDARD_TYPE( Message_PrinterOStream ) );
-    Message::DefaultMessenger()->AddPrinter( new KiCadPrinter( this ) );
+    Message::DefaultMessenger()->AddPrinter( new KICAD_PRINTER( m_reporter ) );
 
-    ReportMessage( _( "Determining PCB data\n" ) );
+    m_reporter->Report( wxT( "Determining PCB data.\n" ), RPT_SEVERITY_DEBUG );
 
     if( m_params.m_OutputFile.IsEmpty() )
     {
@@ -746,15 +850,19 @@ bool EXPORTER_STEP::Export()
 
     try
     {
-        ReportMessage( wxString::Format( _( "Build %s data\n" ), m_params.GetFormatName() ) );
+        m_reporter->Report( wxString::Format( wxT( "Build %s data.\n" ), m_params.GetFormatName() ),
+                            RPT_SEVERITY_DEBUG );
 
         if( !buildBoard3DShapes() )
         {
-            ReportMessage( _( "\n** Error building STEP board model. Export aborted. **\n" ) );
+            m_reporter->Report( _( "\n"
+                                   "** Error building STEP board model. Export aborted. **\n" ),
+                                RPT_SEVERITY_ERROR );
             return false;
         }
 
-        ReportMessage( wxString::Format( _( "Writing %s file\n" ), m_params.GetFormatName() ) );
+        m_reporter->Report( wxString::Format( wxT( "Writing %s file.\n" ), m_params.GetFormatName() ),
+                            RPT_SEVERITY_DEBUG );
 
         bool success = true;
         if( m_params.m_Format == EXPORTER_STEP_PARAMS::FORMAT::STEP )
@@ -772,52 +880,44 @@ bool EXPORTER_STEP::Export()
 
         if( !success )
         {
-            ReportMessage( wxString::Format( _( "\n** Error writing %s file. **\n" ),
-                                             m_params.GetFormatName() ) );
+            m_reporter->Report( wxString::Format( _( "\n"
+                                                     "** Error writing %s file. **\n" ),
+                                                  m_params.GetFormatName() ),
+                                RPT_SEVERITY_ERROR );
             return false;
         }
         else
         {
-            ReportMessage( wxString::Format( _( "%s file '%s' created.\n" ),
-                                             m_params.GetFormatName(), m_outputFile ) );
+            m_reporter->Report( wxString::Format( wxT( "%s file '%s' created.\n" ),
+                                                  m_params.GetFormatName(),
+                                                  m_outputFile ),
+                                RPT_SEVERITY_ACTION );
         }
     }
     catch( const Standard_Failure& e )
     {
-        ReportMessage( e.GetMessageString() );
-        ReportMessage( wxString::Format( _( "\n** Error exporting %s file. Export aborted. **\n" ),
-                                         m_params.GetFormatName() ) );
+        m_reporter->Report( e.GetMessageString(), RPT_SEVERITY_ERROR );
+        m_reporter->Report( wxString::Format( _( "\n"
+                                                 "** Error exporting %s file. Export aborted. **\n" ),
+                                              m_params.GetFormatName() ),
+                            RPT_SEVERITY_ERROR );
         return false;
     }
     catch( ... )
     {
-        ReportMessage( wxString::Format( _( "\n** Error exporting %s file. Export aborted. **\n" ),
-                                         m_params.GetFormatName() ) );
+        m_reporter->Report( wxString::Format( _( "\n"
+                                                 "** Error exporting %s file. Export aborted. **\n" ),
+                                              m_params.GetFormatName() ),
+                            RPT_SEVERITY_ERROR );
         return false;
     }
 
-    if( m_fail || m_error )
-    {
-        wxString msg;
-
-        if( m_fail )
-        {
-            msg = wxString::Format( _( "Unable to create %s file.\n"
-                                       "Check that the board has a valid outline and models." ),
-                                    m_params.GetFormatName() );
-        }
-        else if( m_error || m_warn )
-        {
-            msg = wxString::Format( _( "%s file has been created, but there are warnings." ),
-                                    m_params.GetFormatName() );
-        }
-
-        ReportMessage( msg );
-    }
-
     // Display calculation time in seconds
     double calculation_time = (double)( GetRunningMicroSecs() - stats_startExportTime) / 1e6;
-    ReportMessage( wxString::Format( _( "\nExport time %.3f s\n" ), calculation_time ) );
+    m_reporter->Report( wxString::Format( _( "\n"
+                                             "Export time %.3f s\n" ),
+                                          calculation_time ),
+                        RPT_SEVERITY_INFO );
 
-    return true;
+    return !m_reporter->HasMessageOfSeverity( RPT_SEVERITY_ERROR );
 }
