@@ -59,6 +59,8 @@ BOARD_NETLIST_UPDATER::BOARD_NETLIST_UPDATER( PCB_EDIT_FRAME* aFrame, BOARD* aBo
     m_replaceFootprints = true;
     m_lookupByTimestamp = false;
     m_overrideLocks = false;
+    m_updateFields = false;
+    m_removeExtraFields = false;
 
     m_warningCount = 0;
     m_errorCount = 0;
@@ -479,9 +481,18 @@ bool BOARD_NETLIST_UPDATER::updateFootprintParameters( FOOTPRINT* aPcbFootprint,
     // in the order they are stored in the symbol.
     bool same = true;
 
-    for( std::pair<wxString, wxString> field : compFields )
+    for( const auto& [name, value] : compFields )
     {
-        if( fpFieldsAsMap.count( field.first ) == 0 || fpFieldsAsMap[field.first] != field.second )
+        if( fpFieldsAsMap.count( name ) == 0 || fpFieldsAsMap[name] != value )
+        {
+            same = false;
+            break;
+        }
+    }
+
+    for( const auto& [name, value] : fpFieldsAsMap )
+    {
+        if( compFields.count( name ) == 0 )
         {
             same = false;
             break;
@@ -492,8 +503,11 @@ bool BOARD_NETLIST_UPDATER::updateFootprintParameters( FOOTPRINT* aPcbFootprint,
     {
         if( m_isDryRun )
         {
-            msg.Printf( _( "Update %s fields." ), aPcbFootprint->GetReference() );
-            m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+            if( m_updateFields )
+            {
+                msg.Printf( _( "Update %s fields." ), aPcbFootprint->GetReference() );
+                m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+            }
 
             // Remove fields that aren't present in the symbol
             for( PCB_FIELD* field : aPcbFootprint->GetFields() )
@@ -503,69 +517,78 @@ bool BOARD_NETLIST_UPDATER::updateFootprintParameters( FOOTPRINT* aPcbFootprint,
 
                 if( compFields.count( field->GetName() ) == 0 )
                 {
-                    msg.Printf( _( "Remove %s footprint fields not in symbol." ),
-                                aPcbFootprint->GetReference() );
-                    m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+                    if( m_removeExtraFields )
+                    {
+                        msg.Printf( _( "Remove %s footprint fields not in symbol." ),
+                                    aPcbFootprint->GetReference() );
+                        m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+                    }
+
                     break;
                 }
             }
         }
         else
         {
-            msg.Printf( _( "Updated %s fields." ), aPcbFootprint->GetReference() );
-            m_reporter->Report( msg, RPT_SEVERITY_ACTION );
-
-            changed = true;
-
-            // Add or change field value
-            for( auto& [name, value] : compFields )
+            if( m_updateFields )
             {
-                if( aPcbFootprint->HasFieldByName( name ) )
+                msg.Printf( _( "Updated %s fields." ), aPcbFootprint->GetReference() );
+                m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+
+                changed = true;
+
+                // Add or change field value
+                for( auto& [name, value] : compFields )
                 {
-                    aPcbFootprint->GetFieldByName( name )->SetText( value );
-                }
-                else
-                {
-                    int        idx = aPcbFootprint->GetNextFieldId();
-                    PCB_FIELD* newField = aPcbFootprint->AddField( PCB_FIELD( aPcbFootprint, idx ) );
+                    if( aPcbFootprint->HasFieldByName( name ) )
+                    {
+                        aPcbFootprint->GetFieldByName( name )->SetText( value );
+                    }
+                    else
+                    {
+                        int        idx = aPcbFootprint->GetNextFieldId();
+                        PCB_FIELD* newField = aPcbFootprint->AddField( PCB_FIELD( aPcbFootprint, idx ) );
 
-                    newField->SetName( name );
-                    newField->SetText( value );
-                    newField->SetVisible( false );
-                    newField->SetLayer( aPcbFootprint->GetLayer() == F_Cu ? F_Fab : B_Fab );
+                        newField->SetName( name );
+                        newField->SetText( value );
+                        newField->SetVisible( false );
+                        newField->SetLayer( aPcbFootprint->GetLayer() == F_Cu ? F_Fab : B_Fab );
 
-                    // Give the relative position (0,0) in footprint
-                    newField->SetPosition( aPcbFootprint->GetPosition() );
-                    // Give the footprint orientation
-                    newField->Rotate( aPcbFootprint->GetPosition(), aPcbFootprint->GetOrientation() );
+                        // Give the relative position (0,0) in footprint
+                        newField->SetPosition( aPcbFootprint->GetPosition() );
+                        // Give the footprint orientation
+                        newField->Rotate( aPcbFootprint->GetPosition(), aPcbFootprint->GetOrientation() );
 
-                    if( m_frame )
-                        newField->StyleFromSettings( m_frame->GetDesignSettings() );
+                        if( m_frame )
+                            newField->StyleFromSettings( m_frame->GetDesignSettings() );
+                    }
                 }
             }
 
-            // Remove fields that aren't present in the symbol
-            bool warned = false;
-
-            for( PCB_FIELD* field : aPcbFootprint->GetFields() )
+            if( m_removeExtraFields )
             {
-                if( field->IsMandatory() )
-                    continue;
+                bool warned = false;
 
-                if( compFields.count( field->GetName() ) == 0 )
+                for( PCB_FIELD* field : aPcbFootprint->GetFields() )
                 {
-                    if( !warned )
+                    if( field->IsMandatory() )
+                        continue;
+
+                    if( compFields.count( field->GetName() ) == 0 )
                     {
-                        warned = true;
-                        msg.Printf( _( "Removed %s footprint fields not in symbol." ),
-                                    aPcbFootprint->GetReference() );
-                        m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+                        if( !warned )
+                        {
+                            warned = true;
+                            msg.Printf( _( "Removed %s footprint fields not in symbol." ),
+                                        aPcbFootprint->GetReference() );
+                            m_reporter->Report( msg, RPT_SEVERITY_ACTION );
+                        }
+
+                        aPcbFootprint->RemoveField( field->GetCanonicalName() );
+
+                        if( m_frame )
+                            m_frame->GetCanvas()->GetView()->Remove( field );
                     }
-
-                    aPcbFootprint->RemoveField( field->GetCanonicalName() );
-
-                    if( m_frame )
-                        m_frame->GetCanvas()->GetView()->Remove( field );
                 }
             }
         }
@@ -727,7 +750,7 @@ bool BOARD_NETLIST_UPDATER::updateFootprintParameters( FOOTPRINT* aPcbFootprint,
 
     if( changed && copy )
         m_commit.Modified( aPcbFootprint, copy );
-    else if( copy )
+    else
         delete copy;
 
     return true;
@@ -1131,21 +1154,23 @@ bool BOARD_NETLIST_UPDATER::updateCopperZoneNets( NETLIST& aNetlist )
                 else
                 {
                     PCB_LAYER_ID layer = zone->GetLayer();
-                    VECTOR2I     pos = zone->GetPosition();
+                    VECTOR2I     pt = zone->GetPosition();
 
                     if( m_frame && m_frame->GetPcbNewSettings() )
                     {
                         if( m_frame->GetPcbNewSettings()->m_Display.m_DisplayInvertXAxis )
-                            pos.x *= -1;
+                            pt.x *= -1;
 
                         if( m_frame->GetPcbNewSettings()->m_Display.m_DisplayInvertYAxis )
-                            pos.y *= -1;
+                            pt.y *= -1;
                     }
 
                     msg.Printf( _( "Copper zone on layer %s at (%s, %s) has no pads connected." ),
                                 EscapeHTML( m_board->GetLayerName( layer ) ),
-                                m_frame->MessageTextFromValue( pos.x ),
-                                m_frame->MessageTextFromValue( pos.y ) );
+                                m_frame ? m_frame->MessageTextFromValue( pt.x )
+                                        : EDA_UNIT_UTILS::UI::MessageTextFromValue( pcbIUScale, EDA_UNITS::MM, pt.x ),
+                                m_frame ? m_frame->MessageTextFromValue( pt.y )
+                                        : EDA_UNIT_UTILS::UI::MessageTextFromValue( pcbIUScale, EDA_UNITS::MM, pt.y ) );
                 }
 
                 m_reporter->Report( msg, RPT_SEVERITY_WARNING );
@@ -1282,6 +1307,9 @@ bool BOARD_NETLIST_UPDATER::UpdateNetlist( NETLIST& aNetlist )
                 if( m_replaceFootprints && component->GetFPID() != footprint->GetFPID() )
                     tmp = replaceFootprint( aNetlist, footprint, component );
 
+                if( !tmp )
+                    tmp = footprint;
+
                 if( tmp )
                 {
                     footprintMap[ component ] = tmp;
@@ -1426,7 +1454,8 @@ bool BOARD_NETLIST_UPDATER::UpdateNetlist( NETLIST& aNetlist )
 
         // Although m_commit will probably also set this, it's not guaranteed, and we need to make
         // sure any modification to netclasses gets persisted to project settings through a save.
-        m_frame->OnModify();
+        if( m_frame )
+            m_frame->OnModify();
     }
 
     if( m_isDryRun )
