@@ -4,7 +4,7 @@
  * Copyright (C) 2018 Jean-Pierre Charras, jean-pierre.charras@ujf-grenoble.fr
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * Copyright (C) 2022 CERN
  *
  * This program is free software; you can redistribute it and/or
@@ -33,63 +33,65 @@
 #include <boost/uuid/entropy_error.hpp>
 #endif
 
-#include <kiface_base.h>
-#include <confirm.h>
-#include <pcb_base_frame.h>
-#include <base_units.h>
-#include <widgets/msgpanel.h>
-#include <pgm_base.h>
 #include <3d_viewer/eda_3d_viewer_frame.h>          // To include VIEWER3D_FRAMENAME
-#include <footprint_editor_settings.h>
-#include <pcbnew_settings.h>
-#include <fp_lib_table.h>
-#include <pcbnew_id.h>
+#include <advanced_config.h>
+#include <base_units.h>
 #include <board.h>
-#include <footprint.h>
+#include <cleanup_item.h>
 #include <collectors.h>
-#include <pcb_draw_panel_gal.h>
-#include <math/vector2d.h>
-#include <pcb_group.h>
-
+#include <confirm.h>
+#include <footprint.h>
+#include <footprint_editor_settings.h>
+#include <fp_lib_table.h>
+#include <lset.h>
+#include <kiface_base.h>
 #include <pcb_painter.h>
+#include <pcbnew_id.h>
+#include <pcbnew_settings.h>
+#include <pcb_base_frame.h>
+#include <pcb_draw_panel_gal.h>
+#include <pgm_base.h>
+#include <project_pcb.h>
+#include <wildcards_and_files_ext.h>
+
+#include <math/vector2d.h>
+#include <math/vector2wx.h>
+#include <widgets/msgpanel.h>
+#include <wx/fswatcher.h>
+
 #include <settings/settings_manager.h>
 #include <settings/cvpcb_settings.h>
 #include <tool/tool_manager.h>
 #include <tool/tool_dispatcher.h>
 #include <tools/pcb_actions.h>
 #include <tool/grid_menu.h>
-#include "cleanup_item.h"
-#include <zoom_defines.h>
 #include <ratsnest/ratsnest_view_item.h>
 
-#if defined( KICAD_USE_3DCONNEXION )
 #include <navlib/nl_pcbnew_plugin.h>
-#endif
 
 using KIGFX::RENDER_SETTINGS;
 using KIGFX::PCB_RENDER_SETTINGS;
 
-wxDEFINE_EVENT( BOARD_CHANGED, wxCommandEvent );
+wxDEFINE_EVENT( EDA_EVT_BOARD_CHANGED, wxCommandEvent );
 
 PCB_BASE_FRAME::PCB_BASE_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrameType,
                                 const wxString& aTitle, const wxPoint& aPos, const wxSize& aSize,
                                 long aStyle, const wxString& aFrameName ) :
-        EDA_DRAW_FRAME( aKiway, aParent, aFrameType, aTitle, aPos, aSize, aStyle, aFrameName, pcbIUScale ),
+        EDA_DRAW_FRAME( aKiway, aParent, aFrameType, aTitle, aPos, aSize, aStyle, aFrameName,
+                        pcbIUScale ),
         m_pcb( nullptr ),
         m_originTransforms( *this ),
-        m_spaceMouse( nullptr )
+        m_inFpChangeTimerEvent( false )
 {
+    m_watcherDebounceTimer.Bind( wxEVT_TIMER, &PCB_BASE_FRAME::OnFpChangeDebounceTimer, this );
 }
 
 
 PCB_BASE_FRAME::~PCB_BASE_FRAME()
 {
-#if defined( KICAD_USE_3DCONNEXION )
-    delete m_spaceMouse;
-#endif
-
     // Ensure m_canvasType is up to date, to save it in config
-    m_canvasType = GetCanvas()->GetBackend();
+    if( GetCanvas() )
+        m_canvasType = GetCanvas()->GetBackend();
 
     delete m_pcb;
     m_pcb = nullptr;
@@ -107,7 +109,7 @@ bool PCB_BASE_FRAME::canCloseWindow( wxCloseEvent& aEvent )
 
     // Similarly, wxConvBrokenFileNames uses some statically allocated variables that make it
     // crash when run later from a d'tor.
-    Prj().Cleanup3DCache();
+    PROJECT_PCB::Cleanup3DCache( &Prj() );
 
     return true;
 }
@@ -117,12 +119,8 @@ void PCB_BASE_FRAME::handleActivateEvent( wxActivateEvent& aEvent )
 {
     EDA_DRAW_FRAME::handleActivateEvent( aEvent );
 
-#if defined( KICAD_USE_3DCONNEXION )
-    if( m_spaceMouse != nullptr )
-    {
+    if( m_spaceMouse )
         m_spaceMouse->SetFocus( aEvent.GetActive() );
-    }
-#endif
 }
 
 
@@ -130,12 +128,8 @@ void PCB_BASE_FRAME::handleIconizeEvent( wxIconizeEvent& aEvent )
 {
     EDA_DRAW_FRAME::handleIconizeEvent( aEvent );
 
-#if defined( KICAD_USE_3DCONNEXION )
-    if( m_spaceMouse != nullptr && aEvent.IsIconized() )
-    {
+    if( m_spaceMouse && aEvent.IsIconized() )
         m_spaceMouse->SetFocus( false );
-    }
-#endif
 }
 
 
@@ -164,46 +158,6 @@ void PCB_BASE_FRAME::Update3DView( bool aMarkDirty, bool aRefresh, const wxStrin
 }
 
 
-FP_LIB_TABLE* PROJECT::PcbFootprintLibs()
-{
-    // This is a lazy loading function, it loads the project specific table when
-    // that table is asked for, not before.
-
-    FP_LIB_TABLE*   tbl = (FP_LIB_TABLE*) GetElem( ELEM_FPTBL );
-
-    // its gotta be NULL or a FP_LIB_TABLE, or a bug.
-    wxASSERT( !tbl || tbl->Type() == FP_LIB_TABLE_T );
-
-    if( !tbl )
-    {
-        // Stack the project specific FP_LIB_TABLE overlay on top of the global table.
-        // ~FP_LIB_TABLE() will not touch the fallback table, so multiple projects may
-        // stack this way, all using the same global fallback table.
-        tbl = new FP_LIB_TABLE( &GFootprintTable );
-
-        SetElem( ELEM_FPTBL, tbl );
-
-        wxString projectFpLibTableFileName = FootprintLibTblName();
-
-        try
-        {
-            tbl->Load( projectFpLibTableFileName );
-        }
-        catch( const IO_ERROR& ioe )
-        {
-            DisplayErrorMessage( nullptr, _( "Error loading project footprint libraries." ),
-                                 ioe.What() );
-        }
-        catch( ... )
-        {
-            DisplayErrorMessage( nullptr, _( "Error loading project footprint library table." ) );
-        }
-    }
-
-    return tbl;
-}
-
-
 void PCB_BASE_FRAME::SetBoard( BOARD* aBoard, PROGRESS_REPORTER* aReporter )
 {
     if( m_pcb != aBoard )
@@ -225,12 +179,46 @@ void PCB_BASE_FRAME::SetBoard( BOARD* aBoard, PROGRESS_REPORTER* aReporter )
             }
         }
 
-        wxCommandEvent e( BOARD_CHANGED );
+        wxCommandEvent e( EDA_EVT_BOARD_CHANGED );
         ProcessEventLocally( e );
+
+        for( wxEvtHandler* listener : m_boardChangeListeners )
+        {
+            wxCHECK2( listener, continue );
+
+            // Use the windows variant when handling event messages in case there is any special
+            // event handler pre and/or post processing specific to windows.
+            wxWindow* win = dynamic_cast<wxWindow*>( listener );
+
+            if( win )
+                win->HandleWindowEvent( e );
+            else
+                listener->SafelyProcessEvent( e );
+        }
     }
 }
 
 
+void PCB_BASE_FRAME::AddBoardChangeListener( wxEvtHandler* aListener )
+{
+    auto it = std::find( m_boardChangeListeners.begin(), m_boardChangeListeners.end(), aListener );
+
+    // Don't add duplicate listeners.
+    if( it == m_boardChangeListeners.end() )
+        m_boardChangeListeners.push_back( aListener );
+}
+
+
+void PCB_BASE_FRAME::RemoveBoardChangeListener( wxEvtHandler* aListener )
+{
+    auto it = std::find( m_boardChangeListeners.begin(), m_boardChangeListeners.end(), aListener );
+
+    // Don't add duplicate listeners.
+    if( it != m_boardChangeListeners.end() )
+        m_boardChangeListeners.erase( it );
+}
+
+
 void PCB_BASE_FRAME::AddFootprintToBoard( FOOTPRINT* aFootprint )
 {
     if( aFootprint )
@@ -238,12 +226,12 @@ void PCB_BASE_FRAME::AddFootprintToBoard( FOOTPRINT* aFootprint )
         GetBoard()->Add( aFootprint, ADD_MODE::APPEND );
 
         aFootprint->SetFlags( IS_NEW );
-        aFootprint->SetPosition( wxPoint( 0, 0 ) ); // cursor in GAL may not be initialized yet
+        aFootprint->SetPosition( VECTOR2I( 0, 0 ) ); // cursor in GAL may not be initialized yet
 
         // Put it on FRONT layer (note that it might be stored flipped if the lib is an archive
         // built from a board)
         if( aFootprint->IsFlipped() )
-            aFootprint->Flip( aFootprint->GetPosition(), GetPcbNewSettings()->m_FlipLeftRight );
+            aFootprint->Flip( aFootprint->GetPosition(), GetPcbNewSettings()->m_FlipDirection );
 
         // Place it in orientation 0 even if it is not saved with orientation 0 in lib (note that
         // it might be stored in another orientation if the lib is an archive built from a board)
@@ -275,6 +263,7 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
 {
     static std::vector<KIID> lastBrightenedItemIDs;
 
+    bool itemsUnbrightened = false;
     BOARD_ITEM* lastItem = nullptr;
 
     for( KIID lastBrightenedItemID : lastBrightenedItemIDs )
@@ -298,30 +287,14 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
         if( lastItem && lastItem != DELETED_BOARD_ITEM::GetInstance() )
         {
             lastItem->ClearBrightened();
-
-            if( lastItem->Type() == PCB_FOOTPRINT_T )
-            {
-                static_cast<FOOTPRINT*>( lastItem )->RunOnChildren(
-                        [&]( BOARD_ITEM* child )
-                        {
-                            child->ClearBrightened();
-                        } );
-            }
-            else if( lastItem->Type() == PCB_GROUP_T )
-            {
-                static_cast<PCB_GROUP*>( lastItem )->RunOnChildren(
-                        [&]( BOARD_ITEM* child )
-                        {
-                            child->ClearBrightened();
-                        } );
-            }
-
             GetCanvas()->GetView()->Update( lastItem );
-            lastBrightenedItemID = niluuid;
-            GetCanvas()->Refresh();
+            itemsUnbrightened = true;
         }
     }
 
+    if( itemsUnbrightened )
+        GetCanvas()->Refresh();
+
     lastBrightenedItemIDs.clear();
 
     if( aItems.empty() )
@@ -334,18 +307,16 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
     for( wxWindow* dialog : findDialogs() )
     {
         wxPoint dialogPos = GetCanvas()->ScreenToClient( dialog->GetScreenPosition() );
-        SHAPE_POLY_SET dialogPoly( BOX2D( view->ToWorld( dialogPos, true ),
-                                          view->ToWorld( dialog->GetSize(), false ) ) );
+        SHAPE_POLY_SET dialogPoly( BOX2D( view->ToWorld( ToVECTOR2D( dialogPos ), true ),
+                                          view->ToWorld( ToVECTOR2D( dialog->GetSize() ), false ) ) );
 
         try
         {
-            viewportPoly.BooleanSubtract( dialogPoly, SHAPE_POLY_SET::PM_FAST );
+            viewportPoly.BooleanSubtract( dialogPoly );
         }
-        catch( const ClipperLib::clipperException& exc )
+        catch( const std::exception& e )
         {
-            // This may be overkill and could be an assertion but we are more likely to
-            // find any clipper errors this way.
-            wxLogError( wxT( "Clipper library exception '%s' occurred." ), exc.what() );
+            wxFAIL_MSG( wxString::Format( wxT( "Clipper exception occurred: %s" ), e.what() ) );
         }
     }
 
@@ -356,33 +327,23 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
         if( item && item != DELETED_BOARD_ITEM::GetInstance() )
         {
             item->SetBrightened();
+            lastBrightenedItemIDs.push_back( item->m_Uuid );
 
-            if( item->Type() == PCB_FOOTPRINT_T )
-            {
-                static_cast<FOOTPRINT*>( item )->RunOnChildren(
-                        [&]( BOARD_ITEM* child )
-                        {
-                            child->SetBrightened();
-                        });
-            }
-            else if( item->Type() == PCB_GROUP_T )
-            {
-                static_cast<PCB_GROUP*>( item )->RunOnChildren(
-                        [&]( BOARD_ITEM* child )
-                        {
-                            child->SetBrightened();
-                        });
-            }
+            item->RunOnDescendants(
+                    [&]( BOARD_ITEM* child )
+                    {
+                        child->SetBrightened();
+                        lastBrightenedItemIDs.push_back( child->m_Uuid );
+                    } );
 
             GetCanvas()->GetView()->Update( item );
-            lastBrightenedItemIDs.push_back( item->m_Uuid );
 
             // Focus on the object's location.  Prefer a visible part of the object to its anchor
             // in order to keep from scrolling around.
 
             focusPt = item->GetPosition();
 
-            if( aLayer == UNDEFINED_LAYER )
+            if( aLayer == UNDEFINED_LAYER && item->GetLayerSet().any() )
                 aLayer = item->GetLayerSet().Seq()[0];
 
             switch( item->Type() )
@@ -392,11 +353,9 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
                 {
                     itemPoly = static_cast<FOOTPRINT*>( item )->GetBoundingHull();
                 }
-                catch( const ClipperLib::clipperException& exc )
+                catch( const std::exception& e )
                 {
-                    // This may be overkill and could be an assertion but we are more likely to
-                    // find any clipper errors this way.
-                    wxLogError( wxT( "Clipper library exception '%s' occurred." ), exc.what() );
+                    wxFAIL_MSG( wxString::Format( wxT( "Clipper exception occurred: %s" ), e.what() ) );
                 }
 
                 break;
@@ -409,12 +368,9 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
                 return;
 
             case PCB_SHAPE_T:
+            case PCB_FIELD_T:
             case PCB_TEXT_T:
             case PCB_TEXTBOX_T:
-            case PCB_FP_TEXT_T:
-            case PCB_FP_TEXTBOX_T:
-            case PCB_FP_SHAPE_T:
-            case PCB_FP_ZONE_T:
             case PCB_TRACE_T:
             case PCB_ARC_T:
             case PCB_DIM_ALIGNED_T:
@@ -422,32 +378,25 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
             case PCB_DIM_CENTER_T:
             case PCB_DIM_RADIAL_T:
             case PCB_DIM_ORTHOGONAL_T:
-            case PCB_FP_DIM_ALIGNED_T:
-            case PCB_FP_DIM_LEADER_T:
-            case PCB_FP_DIM_CENTER_T:
-            case PCB_FP_DIM_RADIAL_T:
-            case PCB_FP_DIM_ORTHOGONAL_T:
-                item->TransformShapeToPolygon( itemPoly, aLayer, 0, pcbIUScale.mmToIU( 0.1 ),
-                                               ERROR_INSIDE );
+                item->TransformShapeToPolygon( itemPoly, aLayer, 0, pcbIUScale.mmToIU( 0.1 ), ERROR_INSIDE );
                 break;
 
             case PCB_ZONE_T:
             {
                 ZONE* zone = static_cast<ZONE*>( item );
-                #if 0
+#if 0
                 // Using the filled area shapes to find a Focus point can give good results, but
                 // unfortunately the calculations are highly time consuming, even for not very
                 // large areas (can be easily a few minutes for large areas).
                 // so we used only the zone outline that usually do not have too many vertices.
-                zone->TransformShapeToPolygon( itemPoly, aLayer, 0, pcbIUScale.mmToIU( 0.1 ),
-                                               ERROR_INSIDE );
+                zone->TransformShapeToPolygon( itemPoly, aLayer, 0, pcbIUScale.mmToIU( 0.1 ), ERROR_INSIDE );
 
                 if( itemPoly.IsEmpty() )
                     itemPoly = *zone->Outline();
-                #else
+#else
                 // much faster calculation time when using only the zone outlines
                 itemPoly = *zone->Outline();
-                #endif
+#endif
 
                 break;
             }
@@ -467,13 +416,13 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
 
             try
             {
-                clippedPoly.BooleanIntersection( itemPoly, viewportPoly, SHAPE_POLY_SET::PM_FAST );
+                itemPoly.ClearArcs();
+                viewportPoly.ClearArcs();
+                clippedPoly.BooleanIntersection( itemPoly, viewportPoly );
             }
-            catch( const ClipperLib::clipperException& exc )
+            catch( const std::exception& e )
             {
-                // This may be overkill and could be an assertion but we are more likely to
-                // find any clipper errors this way.
-                wxLogError( wxT( "Clipper library exception '%s' occurred." ), exc.what() );
+                wxFAIL_MSG( wxString::Format( wxT( "Clipper exception occurred: %s" ), e.what() ) );
             }
 
             if( !clippedPoly.IsEmpty() )
@@ -494,13 +443,11 @@ void PCB_BASE_FRAME::FocusOnItems( std::vector<BOARD_ITEM*> aItems, PCB_LAYER_ID
 
         try
         {
-            itemPoly.Deflate( step, 4, SHAPE_POLY_SET::CHAMFER_ACUTE_CORNERS );
+            itemPoly.Deflate( step, CORNER_STRATEGY::ALLOW_ACUTE_CORNERS, ARC_LOW_DEF );
         }
-        catch( const ClipperLib::clipperException& exc )
+        catch( const std::exception& e )
         {
-            // This may be overkill and could be an assertion but we are more likely to
-            // find any clipper errors this way.
-            wxLogError( wxT( "Clipper library exception '%s' occurred." ), exc.what() );
+            wxFAIL_MSG( wxString::Format( wxT( "Clipper exception occurred: %s" ), e.what() ) );
         }
     }
 
@@ -514,8 +461,8 @@ void PCB_BASE_FRAME::HideSolderMask()
 {
     KIGFX::PCB_VIEW* view = GetCanvas()->GetView();
 
-    if( view && GetBoard()->m_SolderMask && view->HasItem( GetBoard()->m_SolderMask ) )
-        view->Remove( GetBoard()->m_SolderMask );
+    if( view && GetBoard()->m_SolderMaskBridges && view->HasItem( GetBoard()->m_SolderMaskBridges ) )
+        view->Remove( GetBoard()->m_SolderMaskBridges );
 }
 
 
@@ -523,12 +470,12 @@ void PCB_BASE_FRAME::ShowSolderMask()
 {
     KIGFX::PCB_VIEW* view = GetCanvas()->GetView();
 
-    if( view && GetBoard()->m_SolderMask )
+    if( view && GetBoard()->m_SolderMaskBridges )
     {
-        if( view->HasItem( GetBoard()->m_SolderMask ) )
-            view->Remove( GetBoard()->m_SolderMask );
+        if( view->HasItem( GetBoard()->m_SolderMaskBridges ) )
+            view->Remove( GetBoard()->m_SolderMaskBridges );
 
-        view->Add( GetBoard()->m_SolderMask );
+        view->Add( GetBoard()->m_SolderMaskBridges );
     }
 }
 
@@ -548,7 +495,7 @@ const PAGE_INFO& PCB_BASE_FRAME::GetPageSettings() const
 }
 
 
-const wxSize PCB_BASE_FRAME::GetPageSizeIU() const
+const VECTOR2I PCB_BASE_FRAME::GetPageSizeIU() const
 {
     // this function is only needed because EDA_DRAW_FRAME is not compiled
     // with either -DPCBNEW or -DEESCHEMA, so the virtual is used to route
@@ -621,18 +568,6 @@ void PCB_BASE_FRAME::SetDrawBgColor( const COLOR4D& aColor )
 }
 
 
-const ZONE_SETTINGS& PCB_BASE_FRAME::GetZoneSettings() const
-{
-    return m_pcb->GetDesignSettings().GetDefaultZoneSettings();
-}
-
-
-void PCB_BASE_FRAME::SetZoneSettings( const ZONE_SETTINGS& aSettings )
-{
-    m_pcb->GetDesignSettings().SetDefaultZoneSettings( aSettings );
-}
-
-
 const PCB_PLOT_PARAMS& PCB_BASE_FRAME::GetPlotSettings() const
 {
     return m_pcb->GetPlotOptions();
@@ -642,6 +577,30 @@ const PCB_PLOT_PARAMS& PCB_BASE_FRAME::GetPlotSettings() const
 void PCB_BASE_FRAME::SetPlotSettings( const PCB_PLOT_PARAMS& aSettings )
 {
     m_pcb->SetPlotOptions( aSettings );
+
+    // Plot Settings can also change the "tent vias" setting, which can affect the solder mask.
+    LSET visibleLayers = GetBoard()->GetVisibleLayers();
+
+    if( visibleLayers.test( F_Mask ) || visibleLayers.test( B_Mask ) )
+    {
+        GetCanvas()->GetView()->UpdateAllItemsConditionally(
+                [&]( KIGFX::VIEW_ITEM* aItem ) -> int
+                {
+                    BOARD_ITEM* item = nullptr;
+
+                    if( aItem->IsBOARD_ITEM() )
+                        item = static_cast<BOARD_ITEM*>( aItem );
+
+                    // Note: KIGFX::REPAINT isn't enough for things that go from invisible to
+                    // visible as they won't be found in the view layer's itemset for re-painting.
+                    if( item && item->Type() == PCB_VIA_T )
+                        return KIGFX::ALL;
+
+                    return 0;
+                } );
+
+        GetCanvas()->Refresh();
+    }
 }
 
 
@@ -651,7 +610,7 @@ BOX2I PCB_BASE_FRAME::GetBoardBoundingBox( bool aBoardEdgesOnly ) const
 
     if( area.GetWidth() == 0 && area.GetHeight() == 0 )
     {
-        wxSize pageSize = GetPageSizeIU();
+        VECTOR2I pageSize = GetPageSizeIU();
 
         if( m_showBorderAndTitleBlock )
         {
@@ -669,6 +628,27 @@ BOX2I PCB_BASE_FRAME::GetBoardBoundingBox( bool aBoardEdgesOnly ) const
 }
 
 
+const BOX2I PCB_BASE_FRAME::GetDocumentExtents( bool aIncludeAllVisible ) const
+{
+    /* "Zoom to Fit" calls this with "aIncludeAllVisible" as true.  Since that feature
+         * always ignored the page and border, this function returns a bbox without them
+         * as well when passed true.  This technically is not all things visible, but it
+         * keeps behavior consistent.
+         *
+         * When passed false, this function returns a bbox of just the board edge. This
+         * allows things like fabrication text or anything else outside the board edge to
+         * be ignored, and just zooms up to the board itself.
+         *
+         * Calling "GetBoardBoundingBox(true)" when edge cuts are turned off will return
+         * the entire page and border, so we call "GetBoardBoundingBox(false)" instead.
+         */
+    if( aIncludeAllVisible || !m_pcb->IsLayerVisible( Edge_Cuts ) )
+        return GetBoardBoundingBox( false );
+    else
+        return GetBoardBoundingBox( true );
+}
+
+
 // Virtual function
 void PCB_BASE_FRAME::doReCreateMenuBar()
 {
@@ -682,6 +662,11 @@ void PCB_BASE_FRAME::ShowChangedLanguage()
 
     // tooltips in toolbars
     RecreateToolbars();
+
+    EDA_3D_VIEWER_FRAME* viewer3D = Get3DViewerFrame();
+
+    if( viewer3D )
+        viewer3D->ShowChangedLanguage();
 }
 
 
@@ -730,22 +715,8 @@ void PCB_BASE_FRAME::SwitchLayer( PCB_LAYER_ID layer )
     // currently enabled needs to be checked.
     if( IsCopperLayer( layer ) )
     {
-        // If only one copper layer is enabled, the only such layer that can be selected to
-        // is the "Copper" layer (so the selection of any other copper layer is disregarded).
-        if( m_pcb->GetCopperLayerCount() < 2 )
-        {
-            if( layer != B_Cu )
-                return;
-        }
-
-        // If more than one copper layer is enabled, the "Copper" and "Component" layers
-        // can be selected, but the total number of copper layers determines which internal
-        // layers are also capable of being selected.
-        else
-        {
-            if( layer != B_Cu && layer != F_Cu && layer >= ( m_pcb->GetCopperLayerCount() - 1 ) )
-                return;
-        }
+        if( layer > m_pcb->GetCopperLayerStackMaxId() )
+            return;
     }
 
     // Is yet more checking required? E.g. when the layer to be selected is a non-copper
@@ -765,16 +736,13 @@ GENERAL_COLLECTORS_GUIDE PCB_BASE_FRAME::GetCollectorsGuide()
                                     GetCanvas()->GetView() );
 
     // account for the globals
-    guide.SetIgnoreMTextsMarkedNoShow( ! m_pcb->IsElementVisible( LAYER_MOD_TEXT_INVISIBLE ) );
-    guide.SetIgnoreMTextsOnBack( ! m_pcb->IsElementVisible( LAYER_MOD_TEXT ) );
-    guide.SetIgnoreMTextsOnFront( ! m_pcb->IsElementVisible( LAYER_MOD_TEXT ) );
-    guide.SetIgnoreModulesOnBack( ! m_pcb->IsElementVisible( LAYER_MOD_BK ) );
-    guide.SetIgnoreModulesOnFront( ! m_pcb->IsElementVisible( LAYER_MOD_FR ) );
-    guide.SetIgnorePadsOnBack( ! m_pcb->IsElementVisible( LAYER_PAD_BK ) );
-    guide.SetIgnorePadsOnFront( ! m_pcb->IsElementVisible( LAYER_PAD_FR ) );
-    guide.SetIgnoreThroughHolePads( ! m_pcb->IsElementVisible( LAYER_PADS_TH ) );
-    guide.SetIgnoreModulesVals( ! m_pcb->IsElementVisible( LAYER_MOD_VALUES ) );
-    guide.SetIgnoreModulesRefs( ! m_pcb->IsElementVisible( LAYER_MOD_REFERENCES ) );
+    guide.SetIgnoreFPTextOnBack( !m_pcb->IsElementVisible( LAYER_FP_TEXT ) );
+    guide.SetIgnoreFPTextOnFront( !m_pcb->IsElementVisible( LAYER_FP_TEXT ) );
+    guide.SetIgnoreFootprintsOnBack( !m_pcb->IsElementVisible( LAYER_FOOTPRINTS_BK ) );
+    guide.SetIgnoreFootprintsOnFront( !m_pcb->IsElementVisible( LAYER_FOOTPRINTS_FR ) );
+    guide.SetIgnoreThroughHolePads( ! m_pcb->IsElementVisible( LAYER_PADS ) );
+    guide.SetIgnoreFPValues( !m_pcb->IsElementVisible( LAYER_FP_VALUES ) );
+    guide.SetIgnoreFPReferences( !m_pcb->IsElementVisible( LAYER_FP_REFERENCES ) );
     guide.SetIgnoreThroughVias( ! m_pcb->IsElementVisible( LAYER_VIAS ) );
     guide.SetIgnoreBlindBuriedVias( ! m_pcb->IsElementVisible( LAYER_VIAS ) );
     guide.SetIgnoreMicroVias( ! m_pcb->IsElementVisible( LAYER_VIAS ) );
@@ -784,19 +752,6 @@ GENERAL_COLLECTORS_GUIDE PCB_BASE_FRAME::GetCollectorsGuide()
 }
 
 
-void PCB_BASE_FRAME::DisplayGridMsg()
-{
-    VECTOR2D gridSize = GetCanvas()->GetGAL()->GetGridSize();
-    wxString line;
-
-    line.Printf( wxT( "grid X %s  Y %s" ),
-                 MessageTextFromValue( gridSize.x, false ),
-                 MessageTextFromValue( gridSize.y, false ) );
-
-    SetStatusText( line, 4 );
-}
-
-
 void PCB_BASE_FRAME::UpdateStatusBar()
 {
     EDA_DRAW_FRAME::UpdateStatusBar();
@@ -869,14 +824,13 @@ void PCB_BASE_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 {
     EDA_DRAW_FRAME::LoadSettings( aCfg );
 
-    if( aCfg->m_Window.grid.sizes.empty() )
-        aCfg->m_Window.grid.sizes = aCfg->DefaultGridSizeList();
-
-    // Currently values read from config file are not used because the user cannot
-    // change this config
-    // if( aCfg->m_Window.zoom_factors.empty() )
+    // Move legacy user grids to grid list
+    if( !aCfg->m_Window.grid.user_grid_x.empty() )
     {
-        aCfg->m_Window.zoom_factors = { ZOOM_LIST_PCBNEW };
+        aCfg->m_Window.grid.grids.emplace_back( GRID{ "User Grid", aCfg->m_Window.grid.user_grid_x,
+                                                      aCfg->m_Window.grid.user_grid_y } );
+        aCfg->m_Window.grid.user_grid_x = wxEmptyString;
+        aCfg->m_Window.grid.user_grid_y = wxEmptyString;
     }
 
     // Some, but not all, derived classes have a PCBNEW_SETTINGS.
@@ -924,13 +878,13 @@ void PCB_BASE_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 
 PCBNEW_SETTINGS* PCB_BASE_FRAME::GetPcbNewSettings() const
 {
-    return Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>();
+    return Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>( "pcbnew" );
 }
 
 
 FOOTPRINT_EDITOR_SETTINGS* PCB_BASE_FRAME::GetFootprintEditorSettings() const
 {
-    return Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+    return Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( "fpedit" );
 }
 
 
@@ -939,37 +893,49 @@ PCB_VIEWERS_SETTINGS_BASE* PCB_BASE_FRAME::GetViewerSettingsBase() const
     switch( GetFrameType() )
     {
     case FRAME_PCB_EDITOR:
-    case FRAME_FOOTPRINT_EDITOR:
-    case FRAME_FOOTPRINT_WIZARD:
     case FRAME_PCB_DISPLAY3D:
     default:
-        return Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>();
+        return Pgm().GetSettingsManager().GetAppSettings<PCBNEW_SETTINGS>( "pcbnew" );
+
+    case FRAME_FOOTPRINT_EDITOR:
+    case FRAME_FOOTPRINT_WIZARD:
+        return Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( "fpedit" );
 
     case FRAME_FOOTPRINT_VIEWER:
-    case FRAME_FOOTPRINT_VIEWER_MODAL:
+    case FRAME_FOOTPRINT_CHOOSER:
     case FRAME_FOOTPRINT_PREVIEW:
     case FRAME_CVPCB:
     case FRAME_CVPCB_DISPLAY:
-        return Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
+        return Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>( "cvpcb" );
     }
 }
 
 
 MAGNETIC_SETTINGS* PCB_BASE_FRAME::GetMagneticItemsSettings()
 {
-    return &GetPcbNewSettings()->m_MagneticItems;
+    static MAGNETIC_SETTINGS fallback;
+
+    if( PCBNEW_SETTINGS* cfg = GetPcbNewSettings() )
+        return &cfg->m_MagneticItems;
+
+    return &fallback;
 }
 
 
-void PCB_BASE_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVarsChanged )
+void PCB_BASE_FRAME::CommonSettingsChanged( int aFlags )
 {
-    EDA_DRAW_FRAME::CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
+    EDA_DRAW_FRAME::CommonSettingsChanged( aFlags );
 
-    RENDER_SETTINGS*     settings = GetCanvas()->GetView()->GetPainter()->GetSettings();
-    PCB_RENDER_SETTINGS* renderSettings = static_cast<KIGFX::PCB_RENDER_SETTINGS*>( settings );
+    KIGFX::VIEW*         view = GetCanvas()->GetView();
+    KIGFX::PCB_PAINTER*  painter = static_cast<KIGFX::PCB_PAINTER*>( view->GetPainter() );
+    PCB_RENDER_SETTINGS* settings = painter->GetSettings();
 
-    renderSettings->LoadColors( GetColorSettings( true ) );
-    renderSettings->LoadDisplayOptions( GetDisplayOptions() );
+    settings->LoadColors( GetColorSettings( true ) );
+    settings->LoadDisplayOptions( GetDisplayOptions() );
+    settings->m_ForceShowFieldsWhenFPSelected = GetPcbNewSettings()->m_Display.m_ForceShowFieldsWhenFPSelected;
+
+    if( aFlags & TEXTVARS_CHANGED )
+        GetBoard()->SynchronizeProperties();
 
     // Note: KIGFX::REPAINT isn't enough for things that go from invisible to visible as
     // they won't be found in the view layer's itemset for re-painting.
@@ -988,11 +954,20 @@ void PCB_BASE_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVars
                 {
                     return KIGFX::REPAINT;    // pad clearance display
                 }
+                else if( EDA_TEXT* text = dynamic_cast<EDA_TEXT*>( aItem ) )
+                {
+                    if( text->HasTextVars() )
+                    {
+                        text->ClearRenderCache();
+                        text->ClearBoundingBoxCache();
+                        return KIGFX::GEOMETRY | KIGFX::REPAINT;
+                    }
+                }
 
                 return 0;
             } );
 
-    GetCanvas()->GetView()->UpdateAllItems( KIGFX::COLOR );
+    view->UpdateAllItems( KIGFX::COLOR );
 
     RecreateToolbars();
 
@@ -1000,7 +975,7 @@ void PCB_BASE_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextVars
     EDA_3D_VIEWER_FRAME* viewer = Get3DViewerFrame();
 
     if( viewer )
-        viewer->CommonSettingsChanged( aEnvVarsChanged, aTextVarsChanged );
+        viewer->CommonSettingsChanged( aFlags );
 }
 
 
@@ -1009,10 +984,11 @@ void PCB_BASE_FRAME::OnModify()
     EDA_BASE_FRAME::OnModify();
 
     GetScreen()->SetContentModified();
-    m_autoSaveRequired = true;
-
     GetBoard()->IncrementTimeStamp();
 
+    if( m_isClosing )
+        return;
+
     UpdateStatusBar();
     UpdateMsgPanel();
 }
@@ -1052,24 +1028,24 @@ void PCB_BASE_FRAME::ActivateGalCanvas()
 
     settings->LoadDisplayOptions( displ_opts );
     settings->LoadColors( GetColorSettings() );
+    settings->m_ForceShowFieldsWhenFPSelected = GetPcbNewSettings()->m_Display.m_ForceShowFieldsWhenFPSelected;
 
     view->RecacheAllItems();
     canvas->SetEventDispatcher( m_toolDispatcher );
     canvas->StartDrawing();
 
-#if defined( KICAD_USE_3DCONNEXION )
     try
+
     {
-        if( m_spaceMouse == nullptr )
+        if( !m_spaceMouse )
         {
-            m_spaceMouse = new NL_PCBNEW_PLUGIN( GetCanvas() );
+            m_spaceMouse = std::make_unique<NL_PCBNEW_PLUGIN>( GetCanvas() );
         }
     }
     catch( const std::system_error& e )
     {
         wxLogTrace( wxT( "KI_TRACE_NAVLIB" ), e.what() );
     }
-#endif
 }
 
 
@@ -1090,6 +1066,11 @@ void PCB_BASE_FRAME::SetDisplayOptions( const PCB_DISPLAY_OPTIONS& aOptions, boo
     // Vias on a restricted layer set must be redrawn when high contrast mode is changed
     if( hcChanged )
     {
+        bool showNetNames = false;
+
+        if( PCBNEW_SETTINGS* config = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
+            showNetNames = config->m_Display.m_NetNames > 0;
+
         // Note: KIGFX::REPAINT isn't enough for things that go from invisible to visible as
         // they won't be found in the view layer's itemset for re-painting.
         GetCanvas()->GetView()->UpdateAllItemsConditionally(
@@ -1097,17 +1078,20 @@ void PCB_BASE_FRAME::SetDisplayOptions( const PCB_DISPLAY_OPTIONS& aOptions, boo
                 {
                     if( PCB_VIA* via = dynamic_cast<PCB_VIA*>( aItem ) )
                     {
-                        if( via->GetViaType() == VIATYPE::BLIND_BURIED
-                                || via->GetViaType() == VIATYPE::MICROVIA
-                                || via->GetRemoveUnconnected() )
+                        if( via->GetViaType() != VIATYPE::THROUGH
+                                || via->GetRemoveUnconnected()
+                                || showNetNames )
                         {
                             return hcVisChanged ? KIGFX::ALL : KIGFX::REPAINT;
                         }
                     }
                     else if( PAD* pad = dynamic_cast<PAD*>( aItem ) )
                     {
-                        if( pad->GetRemoveUnconnected() )
+                        if( pad->GetRemoveUnconnected()
+                                || showNetNames )
+                        {
                             return hcVisChanged ? KIGFX::ALL : KIGFX::REPAINT;
+                        }
                     }
 
                     return 0;
@@ -1117,3 +1101,172 @@ void PCB_BASE_FRAME::SetDisplayOptions( const PCB_DISPLAY_OPTIONS& aOptions, boo
     if( aRefresh )
         canvas->Refresh();
 }
+
+
+void PCB_BASE_FRAME::setFPWatcher( FOOTPRINT* aFootprint )
+{
+    wxLogTrace( "KICAD_LIB_WATCH", "setFPWatcher" );
+
+    Unbind( wxEVT_FSWATCHER, &PCB_BASE_FRAME::OnFPChange, this );
+
+    if( m_watcher )
+    {
+        wxLogTrace( "KICAD_LIB_WATCH", "Remove watch" );
+        m_watcher->RemoveAll();
+        m_watcher->SetOwner( nullptr );
+        m_watcher.reset();
+    }
+
+    wxString libfullname;
+    FP_LIB_TABLE* tbl = PROJECT_PCB::PcbFootprintLibs( &Prj() );
+
+    if( !aFootprint || !tbl )
+        return;
+
+    try
+    {
+        const FP_LIB_TABLE_ROW* row = tbl->FindRow( aFootprint->GetFPID().GetLibNickname() );
+
+        if( !row )
+            return;
+
+        libfullname = row->GetFullURI( true );
+    }
+    catch( const std::exception& e )
+    {
+        DisplayInfoMessage( this, e.what() );
+        return;
+    }
+    catch( const IO_ERROR& error )
+    {
+        wxLogTrace( "KICAD_LIB_WATCH", "Error: %s", error.What() );
+        return;
+    }
+
+    m_watcherFileName.Assign( libfullname, aFootprint->GetFPID().GetLibItemName(),
+                              FILEEXT::KiCadFootprintFileExtension );
+
+    if( !m_watcherFileName.FileExists() )
+        return;
+
+    m_watcherLastModified = m_watcherFileName.GetModificationTime();
+
+    Bind( wxEVT_FSWATCHER, &PCB_BASE_FRAME::OnFPChange, this );
+    m_watcher = std::make_unique<wxFileSystemWatcher>();
+    m_watcher->SetOwner( this );
+
+    wxFileName fn;
+    fn.AssignDir( m_watcherFileName.GetPath() );
+    fn.DontFollowLink();
+
+    wxLogTrace( "KICAD_LIB_WATCH", "Add watch: %s", fn.GetPath() );
+
+    {
+        // Silence OS errors that come from the watcher
+        wxLogNull silence;
+        m_watcher->Add( fn );
+    }
+}
+
+
+void PCB_BASE_FRAME::OnFPChange( wxFileSystemWatcherEvent& aEvent )
+{
+    if( aEvent.GetPath() != m_watcherFileName.GetFullPath() )
+        return;
+
+    // Start the debounce timer (set to 1 second)
+    if( !m_watcherDebounceTimer.StartOnce( 1000 ) )
+    {
+        wxLogTrace( "KICAD_LIB_WATCH", "Failed to start the debounce timer" );
+        return;
+    }
+}
+
+
+void PCB_BASE_FRAME::OnFpChangeDebounceTimer( wxTimerEvent& aEvent )
+{
+    if( aEvent.GetId() != m_watcherDebounceTimer.GetId() )
+    {
+        aEvent.Skip();
+        return;
+    }
+
+    if( m_inFpChangeTimerEvent )
+    {
+        wxLogTrace( "KICAD_LIB_WATCH", "Restarting debounce timer" );
+        m_watcherDebounceTimer.StartOnce( 3000 );
+    }
+
+    wxLogTrace( "KICAD_LIB_WATCH", "OnFpChangeDebounceTimer" );
+
+    // Disable logging to avoid spurious messages and check if the file has changed
+    wxLog::EnableLogging( false );
+    wxDateTime lastModified = m_watcherFileName.GetModificationTime();
+    wxLog::EnableLogging( true );
+
+    if( lastModified == m_watcherLastModified || !lastModified.IsValid() )
+        return;
+
+    m_watcherLastModified = lastModified;
+
+    FOOTPRINT* fp = GetBoard()->GetFirstFootprint();
+    FP_LIB_TABLE* tbl = PROJECT_PCB::PcbFootprintLibs( &Prj() );
+
+    // When loading a footprint from a library in the footprint editor
+    // the items UUIDs must be keep and not reinitialized
+    bool keepUUID = IsType( FRAME_FOOTPRINT_EDITOR );
+
+    if( !fp || !tbl )
+        return;
+
+    m_inFpChangeTimerEvent = true;
+
+    if( !GetScreen()->IsContentModified()
+        || IsOK( this, _( "The library containing the current footprint has changed.\n"
+                          "Do you want to reload the footprint?" ) ) )
+    {
+        wxString fpname = fp->GetFPID().GetLibItemName();
+        wxString nickname = fp->GetFPID().GetLibNickname();
+
+        try
+        {
+            FOOTPRINT* newfp = tbl->FootprintLoad( nickname, fpname, keepUUID );
+
+            if( newfp )
+            {
+                std::vector<KIID> selectedItems;
+
+                for( const EDA_ITEM* item : GetCurrentSelection() )
+                    selectedItems.emplace_back( item->m_Uuid );
+
+                m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
+
+                ReloadFootprint( newfp );
+
+                newfp->ClearAllNets();
+                GetCanvas()->UpdateColors();
+                GetCanvas()->DisplayBoard( GetBoard() );
+                m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
+
+                std::vector<EDA_ITEM*> sel;
+
+                for( const KIID& uuid : selectedItems )
+                {
+                    BOARD_ITEM* item = GetBoard()->GetItem( uuid );
+
+                    if( item != DELETED_BOARD_ITEM::GetInstance() )
+                        sel.push_back( item );
+                }
+
+                if( !sel.empty() )
+                    m_toolManager->RunAction( PCB_ACTIONS::selectItems, &sel );
+            }
+        }
+        catch( const IO_ERROR& ioe )
+        {
+            DisplayError( this, ioe.What() );
+        }
+    }
+
+    m_inFpChangeTimerEvent = false;
+}
