@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2012 CERN.
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@
 #include <richio.h>
 #include <string>
 #include <layer_ids.h>
+#include <lset.h>
 #include <boost/ptr_container/ptr_map.hpp>
 #include <wx_filename.h>
 #include "widgets/report_severity.h"
@@ -39,6 +40,7 @@
 class BOARD;
 class BOARD_ITEM;
 class FP_CACHE;
+class LSET;
 class PCB_IO_KICAD_SEXPR_PARSER;
 class NETINFO_MAPPING;
 class BOARD_DESIGN_SETTINGS;
@@ -47,12 +49,14 @@ class PCB_SHAPE;
 class PCB_REFERENCE_IMAGE;
 class PCB_TARGET;
 class PAD;
+class PADSTACK;
 class PCB_GROUP;
 class PCB_GENERATOR;
 class PCB_TRACK;
 class ZONE;
 class PCB_TEXT;
 class PCB_TEXTBOX;
+class PCB_TABLE;
 class EDA_TEXT;
 class SHAPE_LINE_CHAIN;
 class TEARDROP_PARAMETERS;
@@ -149,7 +153,27 @@ class PCB_IO_KICAD_SEXPR;   // forward decl
 //#define SEXPR_BOARD_FILE_VERSION    20231014  // V8 file format normalization
 //#define SEXPR_BOARD_FILE_VERSION    20231212  // Reference image locking/UUIDs, footprint boolean format
 //#define SEXPR_BOARD_FILE_VERSION    20231231  // Use 'uuid' rather than 'id' for generators and groups
-#define SEXPR_BOARD_FILE_VERSION      20240108  // Convert teardrop parameters to explicit bools
+//#define SEXPR_BOARD_FILE_VERSION    20240108  // Convert teardrop parameters to explicit bools
+//----------------- Start of 9.0 development -----------------
+//#define SEXPR_BOARD_FILE_VERSION    20240201  // Use nullable properties for overrides
+//#define SEXPR_BOARD_FILE_VERSION    20240202  // Tables
+//#define SEXPR_BOARD_FILE_VERSION    20240225  // Rationalization of solder_paste_margin
+//#define SEXPR_BOARD_FILE_VERSION    20240609  // Add 'tenting' keyword
+//#define SEXPR_BOARD_FILE_VERSION    20240617  // Table angles
+//#define SEXPR_BOARD_FILE_VERSION    20240703  // User layer types
+//#define SEXPR_BOARD_FILE_VERSION    20240706  // Embedded Files
+//#define SEXPR_BOARD_FILE_VERSION    20240819  // Embedded Files - Update hash algorithm to Murmur3
+//#define SEXPR_BOARD_FILE_VERSION    20240928  // Component classes
+//#define SEXPR_BOARD_FILE_VERSION    20240929  // Complex padstacks
+//#define SEXPR_BOARD_FILE_VERSION    20241006  // Via stacks
+//#define SEXPR_BOARD_FILE_VERSION    20241007  // Tracks can have soldermask layer and margin
+//#define SEXPR_BOARD_FILE_VERSION    20241009  // Evolve placement rule areas file format
+//#define SEXPR_BOARD_FILE_VERSION    20241010  // Graphic shapes can have soldermask layer and margin
+//#define SEXPR_BOARD_FILE_VERSION    20241030  // Dimension arrow directions, suppress_zeroes normalization
+//#define SEXPR_BOARD_FILE_VERSION    20241129  // Normalise keep_text_aligned and fill properties
+//#define SEXPR_BOARD_FILE_VERSION    20241228  // Convert teardrop curve points to bool
+#define SEXPR_BOARD_FILE_VERSION      20241229  // Expand User layers to arbitrary count
+
 
 #define BOARD_FILE_HOST_VERSION       20200825  ///< Earlier files than this include the host tag
 #define LEGACY_ARC_FORMATTING         20210925  ///< These were the last to use old arc formatting
@@ -179,32 +203,32 @@ class PCB_IO_KICAD_SEXPR;   // forward decl
  * PLUGIN API, and only for the #PCB_PLUGIN plugin.  It is private to this implementation file so
  * it is not placed into a header.
  */
-class FP_CACHE_ITEM
+class FP_CACHE_ENTRY
 {
     WX_FILENAME                m_filename;
     std::unique_ptr<FOOTPRINT> m_footprint;
 
 public:
-    FP_CACHE_ITEM( FOOTPRINT* aFootprint, const WX_FILENAME& aFileName );
+    FP_CACHE_ENTRY( FOOTPRINT* aFootprint, const WX_FILENAME& aFileName );
 
     const WX_FILENAME& GetFileName() const { return m_filename; }
-    void               SetFilePath( const wxString& aFilePath ) { m_filename.SetPath( aFilePath ); }
-    const FOOTPRINT*   GetFootprint() const { return m_footprint.get(); }
+    void SetFilePath( const wxString& aFilePath ) { m_filename.SetPath( aFilePath ); }
+    std::unique_ptr<FOOTPRINT>& GetFootprint() { return m_footprint; }
 };
 
-typedef boost::ptr_map<wxString, FP_CACHE_ITEM> FP_CACHE_FOOTPRINT_MAP;
-
 class FP_CACHE
 {
-    PCB_IO_KICAD_SEXPR*   m_owner;        // Plugin object that owns the cache.
-    wxFileName    m_lib_path;     // The path of the library.
-    wxString      m_lib_raw_path; // For quick comparisons.
-    FP_CACHE_FOOTPRINT_MAP m_footprints;   // Map of footprint filename to FOOTPRINT*.
+    PCB_IO_KICAD_SEXPR*   m_owner;          // Plugin object that owns the cache.
+    wxFileName            m_lib_path;       // The path of the library.
+    wxString              m_lib_raw_path;   // For quick comparisons.
 
-    bool m_cache_dirty;          // Stored separately because it's expensive to check
-                                 // m_cache_timestamp against all the files.
-    long long m_cache_timestamp; // A hash of the timestamps for all the footprint
-                                 // files.
+    boost::ptr_map<wxString, FP_CACHE_ENTRY> m_footprints;  // Map of footprint filename to
+                                                            //   cache entry.
+
+    bool      m_cache_dirty;       // Stored separately because it's expensive to check
+                                   // m_cache_timestamp against all the files.
+    long long m_cache_timestamp;   // A hash of the timestamps for all the footprint
+                                   // files.
 
 public:
     FP_CACHE( PCB_IO_KICAD_SEXPR* aOwner, const wxString& aLibraryPath );
@@ -215,7 +239,7 @@ public:
 
     bool Exists() const { return m_lib_path.IsOk() && m_lib_path.DirExists(); }
 
-    FP_CACHE_FOOTPRINT_MAP& GetFootprints() { return m_footprints; }
+    boost::ptr_map<wxString, FP_CACHE_ENTRY>& GetFootprints() { return m_footprints; }
 
     // Most all functions in this class throw IO_ERROR exceptions.  There are no
     // error codes nor user interface calls from here, nor in any PLUGIN.
@@ -224,9 +248,9 @@ public:
     /**
      * Save the footprint cache or a single footprint from it to disk
      *
-     * @param aFootprint if set, save only this footprint, otherwise, save the full library
+     * @param aFootprintFilter if set, save only this footprint, otherwise, save the full library
      */
-    void Save( FOOTPRINT* aFootprint = nullptr );
+    void Save( FOOTPRINT* aFootprintFilter = nullptr );
 
     void Load();
 
@@ -297,44 +321,47 @@ public:
     bool CanReadBoard( const wxString& aFileName ) const override;
 
     void SaveBoard( const wxString& aFileName, BOARD* aBoard,
-                    const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                    const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     BOARD* LoadBoard( const wxString& aFileName, BOARD* aAppendToMe,
-                      const STRING_UTF8_MAP* aProperties = nullptr, PROJECT* aProject = nullptr ) override;
+                      const std::map<std::string, UTF8>* aProperties = nullptr,
+                      PROJECT* aProject = nullptr ) override;
 
-    BOARD* DoLoad( LINE_READER& aReader, BOARD* aAppendToMe, const STRING_UTF8_MAP* aProperties,
-                     PROGRESS_REPORTER* aProgressReporter, unsigned aLineCount );
+    BOARD* DoLoad( LINE_READER& aReader, BOARD* aAppendToMe, const std::map<std::string,
+                   UTF8>* aProperties, PROGRESS_REPORTER* aProgressReporter, unsigned aLineCount );
 
     void FootprintEnumerate( wxArrayString& aFootprintNames, const wxString& aLibraryPath,
-                             bool aBestEfforts, const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                             bool aBestEfforts, const std::map<std::string,
+                             UTF8>* aProperties = nullptr ) override;
 
     const FOOTPRINT* GetEnumeratedFootprint( const wxString& aLibraryPath,
                                              const wxString& aFootprintName,
-                                             const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                                             const std::map<std::string,
+                                             UTF8>* aProperties = nullptr ) override;
 
     bool FootprintExists( const wxString& aLibraryPath, const wxString& aFootprintName,
-                          const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                          const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     FOOTPRINT* ImportFootprint( const wxString& aFootprintPath, wxString& aFootprintNameOut,
-                                const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                                const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     FOOTPRINT* FootprintLoad( const wxString& aLibraryPath, const wxString& aFootprintName,
                               bool  aKeepUUID = false,
-                              const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                              const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     void FootprintSave( const wxString& aLibraryPath, const FOOTPRINT* aFootprint,
-                        const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                        const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     void FootprintDelete( const wxString& aLibraryPath, const wxString& aFootprintName,
-                          const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                          const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     long long GetLibraryTimestamp( const wxString& aLibraryPath ) const override;
 
     void CreateLibrary( const wxString& aLibraryPath,
-                        const STRING_UTF8_MAP* aProperties = nullptr) override;
+                        const std::map<std::string, UTF8>* aProperties = nullptr) override;
 
     bool DeleteLibrary( const wxString& aLibraryPath,
-                        const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                        const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     bool IsLibraryWritable( const wxString& aLibraryPath ) override;
 
@@ -346,10 +373,9 @@ public:
      * Output \a aItem to \a aFormatter in s-expression format.
      *
      * @param aItem A pointer the an #BOARD_ITEM object to format.
-     * @param aNestLevel The indentation nest level.
      * @throw IO_ERROR on write error.
      */
-    void Format( const BOARD_ITEM* aItem, int aNestLevel = 0 ) const;
+    void Format( const BOARD_ITEM* aItem ) const;
 
     std::string GetStringOutput( bool doClear )
     {
@@ -369,64 +395,69 @@ protected:
     void validateCache( const wxString& aLibraryPath, bool checkModified = true );
 
     const FOOTPRINT* getFootprint( const wxString& aLibraryPath, const wxString& aFootprintName,
-                                   const STRING_UTF8_MAP* aProperties, bool checkModified );
+                                   const std::map<std::string, UTF8>* aProperties,
+                                   bool checkModified );
 
-    void init( const STRING_UTF8_MAP* aProperties );
+    void init( const std::map<std::string, UTF8>* aProperties );
 
     /// formats the board setup information
-    void formatSetup( const BOARD* aBoard, int aNestLevel = 0 ) const;
+    void formatSetup( const BOARD* aBoard ) const;
 
     /// formats the General section of the file
-    void formatGeneral( const BOARD* aBoard, int aNestLevel = 0 ) const;
+    void formatGeneral( const BOARD* aBoard ) const;
 
     /// formats the board layer information
-    void formatBoardLayers( const BOARD* aBoard, int aNestLevel = 0 ) const;
+    void formatBoardLayers( const BOARD* aBoard ) const;
 
     /// formats the Nets and Netclasses
-    void formatNetInformation( const BOARD* aBoard, int aNestLevel = 0 ) const;
+    void formatNetInformation( const BOARD* aBoard ) const;
 
     /// formats the Nets and Netclasses
-    void formatProperties( const BOARD* aBoard, int aNestLevel = 0 ) const;
+    void formatProperties( const BOARD* aBoard ) const;
 
     /// writes everything that comes before the board_items, like settings and layers etc
-    void formatHeader( const BOARD* aBoard, int aNestLevel = 0 ) const;
+    void formatHeader( const BOARD* aBoard ) const;
 
-    void formatTeardropParameters( const TEARDROP_PARAMETERS& tdParams, int aNestLevel = 0 ) const;
+    void formatTeardropParameters( const TEARDROP_PARAMETERS& tdParams ) const;
 
 private:
-    void format( const BOARD* aBoard, int aNestLevel = 0 ) const;
+    void format( const BOARD* aBoard ) const;
 
-    void format( const PCB_DIMENSION_BASE* aDimension, int aNestLevel = 0 ) const;
+    void format( const PCB_DIMENSION_BASE* aDimension ) const;
 
-    void format( const PCB_REFERENCE_IMAGE* aBitmap, int aNestLevel = 0 ) const;
+    void format( const PCB_REFERENCE_IMAGE* aBitmap ) const;
 
-    void format( const PCB_GROUP* aGroup, int aNestLevel = 0 ) const;
+    void format( const PCB_GROUP* aGroup ) const;
 
-    void format( const PCB_SHAPE* aSegment, int aNestLevel = 0 ) const;
+    void format( const PCB_SHAPE* aSegment ) const;
 
-    void format( const PCB_TARGET* aTarget, int aNestLevel = 0 ) const;
+    void format( const PCB_TARGET* aTarget ) const;
 
-    void format( const FOOTPRINT* aFootprint, int aNestLevel = 0 ) const;
+    void format( const FOOTPRINT* aFootprint ) const;
 
-    void format( const PAD* aPad, int aNestLevel = 0 ) const;
+    void format( const PAD* aPad ) const;
 
-    void format( const PCB_TEXT* aText, int aNestLevel = 0 ) const;
-    void format( const PCB_TEXTBOX* aTextBox, int aNestLevel = 0 ) const;
+    void format( const PCB_TEXT* aText ) const;
+    void format( const PCB_TEXTBOX* aTextBox ) const;
 
-    void format( const PCB_GENERATOR* aGenerator, int aNestLevel = 0 ) const;
+    void format( const PCB_TABLE* aTable ) const;
 
-    void format( const PCB_TRACK* aTrack, int aNestLevel = 0 ) const;
+    void format( const PCB_GENERATOR* aGenerator ) const;
 
-    void format( const ZONE* aZone, int aNestLevel = 0 ) const;
+    void format( const PCB_TRACK* aTrack ) const;
 
-    void formatPolyPts( const SHAPE_LINE_CHAIN& outline, int aNestLevel, bool aCompact,
+    void format( const ZONE* aZone ) const;
+
+    void formatPolyPts( const SHAPE_LINE_CHAIN& outline,
                         const FOOTPRINT* aParentFP = nullptr ) const;
 
-    void formatRenderCache( const EDA_TEXT* aText, int aNestLevel ) const;
+    void formatRenderCache( const EDA_TEXT* aText ) const;
 
     void formatLayer( PCB_LAYER_ID aLayer, bool aIsKnockout = false ) const;
 
-    void formatLayers( LSET aLayerMask, int aNestLevel = 0, bool aEnumerateLayers = false ) const;
+    void formatLayers( LSET aLayerMask, bool aEnumerateLayers ) const;
+
+    void formatTenting( const PADSTACK& aPadstack ) const;
 
     friend class FP_CACHE;
 
