@@ -81,8 +81,8 @@ ODB_MISC_ENTITY::ODB_MISC_ENTITY()
                { wxS( "ODB_VERSION_MAJOR" ), wxS( "8" ) },
                { wxS( "ODB_VERSION_MINOR" ), wxS( "1" ) },
                { wxS( "ODB_SOURCE" ), wxS( "KiCad EDA" ) },
-               { wxS( "CREATION_DATE" ), wxDateTime::Now().FormatISOCombined() },
-               { wxS( "SAVE_DATE" ), wxDateTime::Now().FormatISOCombined() },
+               { wxS( "CREATION_DATE" ), wxDateTime::Now().Format( "%Y%m%d.%H%M%S" ) },
+               { wxS( "SAVE_DATE" ), wxDateTime::Now().Format( "%Y%m%d.%H%M%S" ) },
                { wxS( "SAVE_APP" ), wxString::Format( wxS( "KiCad EDA %s" ), GetBuildVersion() ) } };
 }
 
@@ -347,20 +347,17 @@ void ODB_MATRIX_ENTITY::AddDrillMatrixLayer()
                 std::make_pair( PCB_LAYER_ID::UNDEFINED_LAYER, matrix.m_layerName ) );
     };
 
-    if( drill_layers.find( std::make_pair( F_Cu, B_Cu ) ) != drill_layers.end()
-        || !slot_holes.empty() )
-    {
-        // for pad has hole
-        if( has_pth_layer )
-            InitDrillMatrix( "plated", std::make_pair( F_Cu, B_Cu ) );
-        if( has_npth_layer )
-            InitDrillMatrix( "non-plated", std::make_pair( F_Cu, B_Cu ) );
-    }
+    if( has_npth_layer )
+        InitDrillMatrix( "non-plated", std::make_pair( F_Cu, B_Cu ) );
+        // at least one non plated hole is present.
+
+    if( has_pth_layer && drill_layers.find( std::make_pair( F_Cu, B_Cu ) ) == drill_layers.end() )
+        InitDrillMatrix( "plated", std::make_pair( F_Cu, B_Cu ) );
+        // there is no circular plated dril hole present.
 
     for( const auto& [layer_pair, vec] : drill_layers )
     {
-        if( layer_pair != std::make_pair( F_Cu, B_Cu ) ) // pad has initialized above
-            InitDrillMatrix( "plated", layer_pair );     // for via
+        InitDrillMatrix( "plated", layer_pair );
     }
 }
 
@@ -974,10 +971,9 @@ void ODB_STEP_ENTITY::MakeLayerEntity()
 
             for( PCB_LAYER_ID layer : pad_layers )
             {
-                bool onCopperLayer = ( LSET::AllCuMask() & LSET( { layer } ) ).any();
-                bool onSolderMaskLayer = ( LSET( { F_Mask, B_Mask } ) & LSET( { layer } ) ).any();
-                bool onSolderPasteLayer =
-                        ( LSET( { F_Paste, B_Paste } ) & LSET( { layer } ) ).any();
+                bool onCopperLayer = LSET::AllCuMask().test( layer );
+                bool onSolderMaskLayer = LSET( { F_Mask, B_Mask } ).test( layer );
+                bool onSolderPasteLayer = LSET( { F_Paste, B_Paste } ).test( layer );
 
                 if( onSolderMaskLayer )
                     margin.x = margin.y = pad->GetSolderMaskExpansion( PADSTACK::ALL_LAYERS );
