@@ -96,6 +96,8 @@ public:
 
     // Do not create a copy constructor.  The one generated by the compiler is adequate.
 
+    void CopyFrom( const BOARD_ITEM* aOther ) override;
+
     void Move( const VECTOR2I& aMoveVector ) override
     {
         m_Start += aMoveVector;
@@ -283,6 +285,8 @@ public:
 
     PCB_ARC( BOARD_ITEM* aParent, const SHAPE_ARC* aArc );
 
+    void CopyFrom( const BOARD_ITEM* aOther ) override;
+
     static inline bool ClassOf( const EDA_ITEM *aItem )
     {
         return aItem && PCB_ARC_T == aItem->Type();
@@ -383,6 +387,8 @@ public:
     PCB_VIA( const PCB_VIA& aOther );
     PCB_VIA& operator=( const PCB_VIA &aOther );
 
+    void CopyFrom( const BOARD_ITEM* aOther ) override;
+
     bool IsType( const std::vector<KICAD_T>& aScanTypes ) const override
     {
         if( BOARD_CONNECTED_ITEM::IsType( aScanTypes ) )
@@ -407,7 +413,12 @@ public:
     bool HasValidLayerPair( int aCopperLayerCount );
 
     VIATYPE GetViaType() const { return m_viaType; }
-    void SetViaType( VIATYPE aViaType ) { m_viaType = aViaType; }
+    void SetViaType( VIATYPE aViaType ) 
+    {
+        m_viaType = aViaType;
+        // If someone updates a VIA to TH, we want to kick out any non-outer layers
+        SanitizeLayers();
+    }
 
     const PADSTACK& Padstack() const              { return m_padStack; }
     PADSTACK& Padstack()                          { return m_padStack; }
