@@ -210,8 +210,8 @@ BOARD* LoadBoard( const wxString& aFileName, PCB_IO_MGR::PCB_FILE_T aFormat, boo
         resolver.SetProgramBase( PgmOrNull() );
 
         wxString filename = resolver.ResolvePath( BASE_SCREEN::m_DrawingSheetFileName,
-                                              project->GetProjectPath(),
-                                              brd->GetEmbeddedFiles() );
+                                                  project->GetProjectPath(),
+                                                  { brd->GetEmbeddedFiles() } );
 
         wxString msg;
 
@@ -522,7 +522,8 @@ std::deque<BOARD_ITEM*> GetCurrentSelection()
         std::for_each( selection.begin(), selection.end(),
                        [&items]( EDA_ITEM* item )
                        {
-                           items.push_back( static_cast<BOARD_ITEM*>( item ) );
+                           if( item->IsBOARD_ITEM() )
+                               items.push_back( static_cast<BOARD_ITEM*>( item ) );
                        } );
     }
 
