@@ -33,7 +33,8 @@ namespace PNS
 class LINK_HOLDER : public ITEM
 {
 public:
-    LINK_HOLDER( PnsKind aKind ) : ITEM( aKind )
+    LINK_HOLDER( PnsKind aKind ) :
+        ITEM( aKind )
     {}
 
     ///< Add a reference to an item registered in a #NODE that is a part of this line.
@@ -71,6 +72,7 @@ public:
     {
         if( aIndex < 0 )
             aIndex += m_links.size();
+
         return m_links[aIndex];
     }
 
