@@ -222,13 +222,19 @@ int EDIT_TOOL::Move( const TOOL_EVENT& aEvent )
 
     if( BOARD_COMMIT* commit = dynamic_cast<BOARD_COMMIT*>( aEvent.Commit() ) )
     {
-        wxCHECK( aEvent.SynchronousState(), 0 );
-        aEvent.SynchronousState()->store( STS_RUNNING );
+        // Most moves will be synchronous unless they are coming from the API
+        if( aEvent.SynchronousState() )
+            aEvent.SynchronousState()->store( STS_RUNNING );
 
         if( doMoveSelection( aEvent, commit, true ) )
-            aEvent.SynchronousState()->store( STS_FINISHED );
-        else
+        {
+            if( aEvent.SynchronousState() )
+                aEvent.SynchronousState()->store( STS_FINISHED );
+        }
+        else if( aEvent.SynchronousState() )
+        {
             aEvent.SynchronousState()->store( STS_CANCELLED );
+        }
     }
     else
     {
@@ -295,6 +301,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
     BOARD*                             board = editFrame->GetBoard();
     KIGFX::VIEW_CONTROLS*              controls = getViewControls();
     VECTOR2I                           originalCursorPos = controls->GetCursorPosition();
+    VECTOR2I                           originalMousePos = controls->GetMousePosition();
     std::unique_ptr<STATUS_TEXT_POPUP> statusPopup;
     wxString                           status;
     size_t                             itemIdx = 0;
@@ -314,6 +321,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
     if( m_dragging || selection.Empty() )
         return false;
 
+    TOOL_EVENT pushedEvent = aEvent;
     editFrame->PushTool( aEvent );
     Activate();
 
@@ -394,7 +402,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         if( selection.IsHover() )
             m_toolMgr->RunAction( PCB_ACTIONS::selectionClear );
 
-        editFrame->PopTool( aEvent );
+        editFrame->PopTool( pushedEvent );
         return false;
     }
 
@@ -465,12 +473,16 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         if( evt->IsMotion() || evt->IsDrag( BUT_LEFT ) )
             eatFirstMouseUp = false;
 
-        if( evt->IsAction( &PCB_ACTIONS::move ) || evt->IsMotion() || evt->IsDrag( BUT_LEFT )
+        if( evt->IsAction( &PCB_ACTIONS::move )
+                || evt->IsMotion()
+                || evt->IsDrag( BUT_LEFT )
                 || evt->IsAction( &ACTIONS::refreshPreview )
                 || evt->IsAction( &PCB_ACTIONS::moveWithReference )
                 || evt->IsAction( &PCB_ACTIONS::moveIndividually ) )
         {
-            if( m_dragging && evt->Category() == TC_MOUSE )
+            if( m_dragging && (    evt->IsMotion()
+                                || evt->IsDrag( BUT_LEFT )
+                                || evt->IsAction( &ACTIONS::refreshPreview ) ) )
             {
                 bool redraw3D = false;
 
@@ -528,6 +540,9 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                 // Drag items to the current cursor position
                 for( EDA_ITEM* item : sel_items )
                 {
+                    if( !item->IsBOARD_ITEM() )
+                        continue;
+
                     // Don't double move child items.
                     if( !item->GetParent() || !item->GetParent()->IsSelected() )
                         static_cast<BOARD_ITEM*>( item )->Move( movement );
@@ -607,6 +622,9 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                     // Drag items to the current cursor position
                     for( EDA_ITEM* item : selection )
                     {
+                        if( !item->IsBOARD_ITEM() )
+                            continue;
+
                         // Don't double move footprint pads, fields, etc.
                         if( item->GetParent() && item->GetParent()->IsSelected() )
                             continue;
@@ -636,7 +654,9 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
                         }
                     }
 
-                    m_cursor = grid.BestDragOrigin( originalCursorPos, sel_items,
+                    // Use the mouse position over cursor, as otherwise large grids will allow only
+                    // snapping to items that are closest to grid points
+                    m_cursor = grid.BestDragOrigin( originalMousePos, sel_items,
                                                     grid.GetSelectionGrid( selection ),
                                                     &m_selectionTool->GetFilter() );
 
@@ -696,6 +716,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
         }
         else if( evt->IsAction( &ACTIONS::duplicate ) || evt->IsAction( &ACTIONS::cut ) )
         {
+            wxBell();
         }
         else if( evt->IsAction( &PCB_ACTIONS::rotateCw )
                 || evt->IsAction( &PCB_ACTIONS::rotateCcw )
@@ -824,7 +845,7 @@ bool EDIT_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, BOARD_COMMIT* aCommit
     // Remove the dynamic ratsnest from the screen
     m_toolMgr->RunAction( PCB_ACTIONS::hideLocalRatsnest );
 
-    editFrame->PopTool( aEvent );
+    editFrame->PopTool( pushedEvent );
     editFrame->GetCanvas()->SetCurrentCursor( KICURSOR::ARROW );
 
     return !restore_state;
